@@ -13,12 +13,13 @@ b=${1:?"missing base directory name argument"}
 
 tar -cf $b-binary.tar \
     $b/output/sbcl.core $b/src/runtime/sbcl $b/output/prefix.def \
+    $b/src/runtime/sbcl.mk \
+    `grep '^LIBSBCL=' $b/src/runtime/sbcl.mk | cut -d= -f2- | while read lib; do echo $b/src/runtime/$lib; done` \
     $b/BUGS $b/COPYING $b/CREDITS $b/INSTALL $b/NEWS $b/README \
     $b/install.sh $b/find-gnumake.sh $b/sbcl-pwd.sh $b/run-sbcl.sh \
     $b/doc/sbcl.1 \
     $b/pubring.pgp \
     $b/contrib/asdf-module.mk \
-    $b/contrib/vanilla-module.mk \
     `for contrib in $(cd $b/contrib && echo *); do
          src_dir=$b/contrib/$contrib
          cache_dir=$b/obj/asdf-cache/$contrib
