@@ -5,6 +5,9 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
+;; Undo default contribs mufflage so that DECLARATION-INFORMATION tests pass.
+sb-ext::(declaim (unmuffle-conditions compiler-note))
+
 (defpackage :sb-cltl2-tests
   (:use :sb-cltl2 :cl :sb-rt :sb-ext :sb-kernel :sb-int))
 
@@ -133,6 +136,16 @@
         (equalp (frob (+ x x))
                 '(+ y y))))
   t)
+;; MULTIPLE-VALUE-BIND and -SETQ and were failing to macroexpand-all
+;; because the walker treated them both as special operators.
+(deftest macroexpand-all.mvb
+  (let ((form '(multiple-value-bind (a b) (something) (use-them a b))))
+    (not (equalp form (macroexpand-all form nil))))
+  t)
+(deftest macroexpand-all.mvs
+  (let ((form '(multiple-value-setq (a b) (something))))
+    (not (equalp form (macroexpand-all form nil))))
+  t)
 ;;;; DECLARATION-INFORMATION
 
 (defmacro dinfo (thing &environment env)
@@ -438,6 +451,21 @@
    t
    ((ftype function (integer) (values integer &optional)))))
 
+(deftest function-information.ftype-struct-ctor
+  (fun-info sb-c::make-lvar)
+  (:function nil
+   ((ftype function (&optional (or sb-c::node null)) (values sb-c::lvar &optional)))))
+(deftest function-information.ftype-struct-accessor
+  (fun-info (setf sb-c::sset-vector))
+  (:function nil
+   ((ftype function (simple-vector sset) (values simple-vector &optional)))))
+(deftest function-information.ftype-struct-predicate
+  (fun-info sb-c::sset-p)
+  (:function nil ((ftype function (t) (values boolean &optional)))))
+(deftest function-information.ftype-struct-copier
+  (fun-info sb-c::copy-sset)
+  (:function nil ((ftype function (sset) (values sset &optional)))))
+
 ;;;;; AUGMENT-ENVIRONMENT
 
 (defmacro ct (form &environment env)
