@@ -183,6 +183,8 @@
    #:coerce))
 
 (in-package :sb-mpfr)
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (setf (sb-int:system-package-p *package*) t))
 
 (defvar +mpfr-precision+)
 (defvar *mpfr-version* nil)
