@@ -22,17 +22,16 @@ do
   # to this cross-build test. Some of the provisions are only for C code which
   # we don't compile. Or else it doesn't matter much which lisp code is compiled.
   # Assume that dlopen() is provided so that we don't try to read sbcl.nm though.
-  echo '(lambda (features) (union features (list :crossbuild-test :os-provides-dlopen ' > $ltf
+  echo '(lambda (features) (union features (list :os-provides-dlopen ' > $ltf
   echo ":$arch" >> $ltf
   # x86 and x86-64 are tested as if #+win32. Unix is otherwise plenty tested.
   if [ $arch = x86 -o $arch = x86-64 ]; then
-    echo ':win32 :sb-thread :sb-safepoint :sb-thruption :sb-wtimer' >> $ltf
+    echo ':win32 :sb-thread :sb-safepoint' >> $ltf
   else
     echo ':unix :linux :elf' >> $ltf
   fi
   cat crossbuild-runner/backends/$arch/features >> $ltf
   cat crossbuild-runner/backends/$arch/local-target-features >> $ltf
-  echo ':linkage-table' >> $ltf
   echo ')))' >> $ltf
 
   cp -fv crossbuild-runner/backends/$arch/stuff-groveled-from-headers.lisp \
