@@ -3,14 +3,20 @@
   (let ((*print-pretty* nil)
         (*print-length* nil))
     (dolist (thing '(("SB-XC" "*FEATURES*")
-                     ("SB-COLD" "*SHEBANG-BACKEND-SUBFEATURES*")))
+                     ("SB-COLD" "BACKEND-SUBFEATURES")))
       (let* ((sym (intern (cadr thing) (car thing)))
              (val (symbol-value sym)))
         (when val
           (format t "~&target ~S = ~S~%" sym  val))))))
 (in-package "SB-COLD")
+#+sbcl
+(declaim (sb-ext:muffle-conditions
+          sb-ext:compiler-note
+          (satisfies optional+key-style-warning-p)))
 (progn
-  (setf *host-obj-prefix* "obj/from-host/")
+  (setf *host-obj-prefix* (if (boundp 'cl-user::*sbcl-host-obj-prefix*)
+                              (symbol-value 'cl-user::*sbcl-host-obj-prefix*)
+                              "obj/from-host/"))
   (load "src/cold/set-up-cold-packages.lisp")
   (load "src/cold/defun-load-or-cload-xcompiler.lisp")
 
@@ -28,30 +34,22 @@
     ;; UNDEFINED-VARIABLE does not cause COMPILE-FILE to return warnings-p
     ;; unless outside a compilation unit. You find out about it only upon
     ;; exit of SUMMARIZE-COMPILATION-UNIT. So we set up a handler for that.
-    `(let (in-summary fail)
-       (handler-bind (((and simple-warning (not style-warning))
-                       (lambda (c &aux (fc (simple-condition-format-control c)))
-                         ;; hack for PPC. See 'build-order.lisp-expr'
-                         ;; Ignore the warning, and the warning about the warning.
-                         (unless (and (stringp fc)
-                                      (or (search "not allowed by the operand type" fc)
-                                          (search "ignoring FAILURE-P return" fc)))
-                           (setq fail 'warning))))
-                      ;; Prevent regressions on a few platforms
-                      ;; that are known to build cleanly.
-                      (sb-int:simple-style-warning
-                       (lambda (c &aux (fc (simple-condition-format-control c)))
-                         (when (and (featurep '(:or :x86 :x86-64 :arm64))
-                                    in-summary
-                                    (stringp fc)
-                                    (search "undefined" fc))
-                           (unless (eq fail 'warning)
-                             (setq fail 'style-warning))))))
-         (with-compilation-unit ()
-           (multiple-value-prog1 (progn ,@forms) (setq in-summary t))))
-       (when fail
+    `(let (warnp style-warnp)
+       (handler-bind ((style-warning
+                       ;; Any unmuffled STYLE-WARNING should fail
+                       ;; These would typically be from undefined functions,
+                       ;; or optional-and-key when that was visible.
+                       (lambda (c)
+                         (signal c) ; won't do SETQ if MUFFLE-WARNING is invoked
+                         (setq style-warnp 'style-warning)))
+                      (simple-warning
+                        (lambda (c)
+                          (declare (ignore c))
+                          (setq warnp 'warning))))
+         (with-compilation-unit () ,@forms))
+       (when (and (or warnp style-warnp) *fail-on-warnings*)
          (cerror "Proceed anyway"
-                 "make-host-1 stopped due to unexpected ~A." fail)))
+                 "make-host-1 stopped due to unexpected ~A." (or warnp style-warnp))))
 
     #-(or clisp sbcl) `(with-compilation-unit () ,@forms)))
 
@@ -120,7 +118,16 @@
         (setf (gethash input *ucd-inputs*) 'unused))
       (dolist (output outputs)
         (setf (gethash output *ucd-outputs*) 'unmade))
-      (let ((object (compile-file "tools-for-build/ucd.lisp")))
+      (let ((object (apply #'compile-file "tools-for-build/ucd.lisp"
+                           ;; ECL creates its compiled files beside
+                           ;; the truename of a source; that's bad
+                           ;; when we're in a build tree of symlinks.
+                           #+ecl
+                           (list
+                            :output-file
+                            (compile-file-pathname "tools-for-build/ucd.lisp"))
+                           #-ecl
+                           ())))
         (setf (gethash "tools-for-build/ucd.lisp" *ucd-inputs*) 'used)
         (load object :verbose t)
         (delete-file object))
@@ -144,12 +151,6 @@
                  unused-inputs extra-inputs
                  unused-outputs extra-outputs))))))
 
-;;; I don't know the best combination of OPTIMIZE qualities to produce a correct
-;;; and reasonably fast cross-compiler in ECL. At over half an hour to complete
-;;; make-host-{1,2}, I don't really want to waste any more time finding out.
-;;; These settings work, while the defaults do not.
-#+ecl (proclaim '(optimize (safety 2) (debug 2)))
-
 (maybe-with-compilation-unit
   ;; If make-host-1 is parallelized, it will produce host fasls without loading
   ;; them. The host will have interpreted definitions of most everything,
@@ -161,4 +162,5 @@
  (host-cload-stem "src/compiler/generic/genesis" nil)
 ) ; END with-compilation-unit
 
-(sb-cold:genesis :c-header-dir-name "src/runtime/genesis")
+(unless (member :crossbuild-test sb-xc:*features*)
+  (sb-cold:genesis :c-header-dir-name "src/runtime/genesis"))
