@@ -57,8 +57,8 @@
            ;; Features that are also in *FEATURES-POTENTIALLY-AFFECTING-FASL-FORMAT*
            ;; and would probably mess up something if made non-public,
            ;; though I don't think they should all be public.
-           :MSAN
-           :SB-SAFEPOINT :SB-SAFEPOINT-STRICTLY
+           :MSAN :UBSAN
+           :SB-SAFEPOINT
            :SB-THREAD :SB-UNICODE
            ;; Things which (I think) at least one person has requested be kept around
            :SB-LDB
@@ -70,6 +70,7 @@
            :PACKAGE-LOCAL-NICKNAMES
            ;; Developer mode features. A release build will never have them,
            ;; hence it makes no difference whether they're public or not.
+           :METASPACE
            :SB-FLUID :SB-DEVEL :SB-DEVEL-LOCK-PACKAGES)")))
        (removable-features
         (append non-target-features public-features)))
@@ -132,9 +133,9 @@
     ;; can't be uninterned if referenced by a defstruct-description.
     ;; So loop over all structure classoids and clobber any
     ;; symbol that should be uninternable.
-    (maphash (lambda (classoid layout)
+    (maphash (lambda (classoid wrapper)
                (when (structure-classoid-p classoid)
-                 (let ((dd (layout-info layout)))
+                 (let ((dd (wrapper-%info wrapper)))
                    (setf (dd-constructors dd)
                          (delete-if (lambda (x)
                                       (and (consp x) (uninternable-p (car x))))
@@ -175,7 +176,7 @@
   result)
 
 ;;; Check for potentially bad format-control strings
-(defun !scan-format-control-strings ()
+(defun scan-format-control-strings ()
   (labels ((possibly-ungood-package-reference (string)
              ;; We want to see nothing SB-package-like at all
              (or (search "sb-" string :test #'char-equal)
@@ -210,12 +211,6 @@ Please check that all strings which were not recognizable to the compiler
 (as the first argument to WARN, etc.) are wrapped in SB-FORMAT:TOKENS"))
       wps)))
 
-(progn
-  ;; See the giant comment at the bottom of this file
-  ;; concerning the need for this GC.
-  (gc :full t)
-  (!scan-format-control-strings))
-
 ;;; Either set some more package docstrings, or remove any and all docstrings
 ;;; that snuck in (as can happen with any file compiled in warm load)
 ;;; depending on presence of the :sb-doc internal feature.
@@ -253,7 +248,10 @@ Please check that all strings which were not recognizable to the compiler
        :all)
       ;; 2. Variables, types, and anything else
       (do-all-symbols (s)
-        (dolist (category '(:variable :type :typed-structure :setf))
+        (let ((expander (sb-int:info :setf :expander s)))
+          (when (typep expander '(cons t (cons string)))
+            (setf (second expander) nil)))
+        (dolist (category '(:variable :type :typed-structure))
           (clear-it (sb-int:info category :documentation s)))
         (clear-it (sb-int:info :random-documentation :stuff s))))
     (when (plusp count)
@@ -322,18 +320,18 @@ Please check that all strings which were not recognizable to the compiler
   ;; Except that symbols which existed at SBCL build time must be.
   (do-all-symbols (symbol)
     (when (sb-kernel:immobile-space-obj-p symbol)
-      (sb-kernel:set-header-data
-           symbol (logior (sb-kernel:get-header-data symbol)
-                          (ash 1 sb-vm::+initial-core-symbol-bit+)))))
+      (sb-kernel:logior-header-bits
+       symbol (ash 1 sb-vm::+initial-core-symbol-bit+))))
 
   ;; A symbol whose INFO slot underwent any kind of manipulation
   ;; such that it now has neither properties nor globaldb info,
   ;; can have the slot set back to NIL if it wasn't already.
   (do-all-symbols (symbol)
-    (when (and (sb-kernel:symbol-info symbol)
-               (null (sb-kernel:symbol-info-vector symbol))
+    (when (and (sb-kernel:symbol-%info symbol) ; "raw" value is something
+               ;; but both "cooked" values are empty
+               (null (sb-kernel:symbol-dbinfo symbol))
                (null (symbol-plist symbol)))
-      (setf (sb-kernel:symbol-info symbol) nil)))
+      (setf (sb-kernel:symbol-%info symbol) nil)))
 )
 
 (sb-ext:gc :full t)
@@ -398,7 +396,7 @@ Please check that all strings which were not recognizable to the compiler
    #+sb-devel
    (lambda (symbol accessibility)
      (declare (ignore accessibility))
-     (or (sb-kernel:symbol-info symbol)
+     (or (sb-kernel:symbol-%info symbol)
          (and (boundp symbol) (not (keywordp symbol)))))
    ;; Release mode: retain all symbols satisfying this intricate test
    #-sb-devel
@@ -409,15 +407,13 @@ Please check that all strings which were not recognizable to the compiler
            ;; overapproximate what we need for contribs and tests
            (member symbol '(sb-vm::map-referencing-objects
                             sb-vm::map-stack-references
-                            sb-vm::thread-profile-data-slot
-                            sb-vm::thread-alloc-region-slot
                             sb-vm::reconstitute-object
                             ;; need this for defining a vop which
                             ;; tests the x86-64 allocation profiler
                             sb-vm::pseudo-atomic
                             ;; Naughty outside-world code uses these.
-                            #+x86-64 sb-vm::reg-in-size
-                            sb-vm::thread-control-stack-start-slot))
+                            #+x86-64 sb-vm::reg-in-size))
+           (let ((s (string symbol))) (and (search "THREAD-" s) (search "-SLOT" s)))
            (search "-OFFSET" (string symbol))
            (search "-TN" (string symbol))))
       ((#.(find-package "SB-C")
@@ -453,7 +449,7 @@ Please check that all strings which were not recognizable to the compiler
                sb-assem::*backend-instruction-set-package*)
            (or (eq accessibility :external) (asm-inst-p symbol))
            ;; By default, retain any symbol with any attachments
-           (or (sb-kernel:symbol-info symbol)
+           (or (sb-kernel:symbol-%info symbol)
                (and (boundp symbol) (not (keywordp symbol))))))))
    :verbose nil :print nil)
   (unintern 'sb-impl::shake-packages 'sb-impl)
@@ -474,9 +470,7 @@ Please check that all strings which were not recognizable to the compiler
             sum-delta-ext sum-delta-int
             (+ sum-delta-ext sum-delta-int))))
 
-;;; Use historical (stupid) behavior for storing pathname namestrings
-;;; in fasls.
-(setq sb-c::*name-context-file-path-selector* 'truename)
+(scan-format-control-strings)
 
 ;;; Lock internal packages
 #-(and sb-devel
