@@ -14,11 +14,24 @@
       (floor (+ feature-bit n-fixnum-tag-bits) n-byte-bits)
     (inst test :byte (static-symbol-value-ea '*cpu-feature-bits* byte) (ash 1 bit))))
 
-(defun invoke-asm-routine (inst routine vop)
+(defun uniquify-fixup (name &aux (asmstream *asmstream*))
+  (or (cdr (assoc name (sb-assem::asmstream-indirection-table asmstream)))
+      (let ((label (gen-label)))
+        ;; This has to be separate from the :ELSEWHERE section because we could be
+        ;; emitting code into :ELSEWHERE when requesting a unique label.
+        (assemble (:indirections)
+          (emit-label label)
+          (inst jmp (ea (make-fixup name :assembly-routine*))))
+        (push (cons name label) (sb-assem::asmstream-indirection-table asmstream))
+        label)))
+
+(defun invoke-asm-routine (inst routine vop &optional uniquify)
   (declare (ignorable vop))
   (let ((fixup
          (cond ((sb-c::code-immobile-p vop)
                 (make-fixup routine :assembly-routine))
+               (uniquify
+                (uniquify-fixup routine))
                (t
                 (ea (make-fixup routine :assembly-routine*))))))
     (ecase inst
@@ -62,8 +75,10 @@
     (flet ((gpr-save/restore (operation except)
              (declare (type (member push pop) operation))
              (let ((registers (ecase convention
+                               ;; RBX and R12..R15 are preserved across C call
                                (c '#1=(rax-tn rcx-tn rdx-tn rsi-tn rdi-tn r8-tn r9-tn r10-tn r11-tn))
-                               (lisp '(rbx-tn r12-tn r14-tn r15-tn . #1#)))))
+                               ;; all GPRs are potentially destroyed across lisp call
+                               (lisp '(rbx-tn r12-tn #-sb-thread r13-tn r14-tn r15-tn . #1#)))))
                (when except
                  (setf registers (remove except registers)))
                ;; Preserve alignment
