@@ -130,13 +130,16 @@
   (in-index +ansi-stream-in-buffer-length+
             :type (integer 0 #.+ansi-stream-in-buffer-length+))
 
+  ;; FIXME: declare all these function types more rigorously.
+
   ;; buffered input functions
   (in #'ill-in :type function)                  ; READ-CHAR function
   (bin #'ill-bin :type function)                ; byte input function
   ;; 'n-bin' might not transfer bytes to the consumer.
   ;; A character FD-STREAM uses this method to transfer octets from the
   ;; source buffer into characters of the destination buffer.
-  (n-bin #'ill-bin :type function)              ; n-byte input function
+  (n-bin #'ill-bin :type                        ; n-byte input function
+   (sfunction (stream (simple-unboxed-array (*)) index index t) index))
 
   ;; output functions
   (out #'ill-out :type function)                ; WRITE-CHAR function
@@ -169,6 +172,22 @@
   (symbol nil :type symbol :read-only t))
 (declaim (freeze-type synonym-stream))
 
+(defstruct (broadcast-stream (:include ansi-stream
+                                       (out #'broadcast-out)
+                                       (bout #'broadcast-bout)
+                                       (sout #'broadcast-sout)
+                                       (misc #'broadcast-misc))
+                             (:constructor %make-broadcast-stream
+                                           (streams))
+                             (:copier nil)
+                             (:predicate nil))
+  ;; a list of all the streams we broadcast to
+  (streams () :type list :read-only t))
+(declaim (freeze-type broadcast-stream))
+
+(define-load-time-global *null-broadcast-stream* (make-broadcast-stream))
+(declaim (type stream *null-broadcast-stream*))
+
 (defmethod print-object ((x stream) stream)
   (print-unreadable-object (x stream :type t :identity t)))
 
