@@ -40,10 +40,8 @@
 ;;;; !COLD-INIT
 
 ;;; a list of toplevel things set by GENESIS
-(defvar *!cold-toplevels*)   ; except for DEFUNs and SETF macros
-(defvar *!cold-setf-macros*) ; just SETF macros
-(defvar *!cold-defuns*)      ; just DEFUNs
-(defvar *!cold-defsymbols*)  ; "easy" DEFCONSTANTs and DEFPARAMETERs
+(defvar *!cold-toplevels*)
+(defvar *!cold-defsymbols*)  ; "easy" DEFCONSTANTs
 
 ;;; a SIMPLE-VECTOR set by GENESIS
 (defvar *!load-time-values*)
@@ -58,29 +56,74 @@
 
 (defun !c-runtime-noinform-p () (/= (extern-alien "lisp_startup_options" char) 0))
 
+(defun !format-cold-init ()
+  (sb-format::!late-format-init)
+  (sb-format::!format-directives-init))
+
+;;; Allows the SIGNAL function to be called early.
+(defun !signal-function-cold-init ()
+  #+sb-devel
+  (progn
+    (setq *break-on-signals* nil)
+    (setq sb-kernel::*current-error-depth* 0))
+  (setq *stack-top-hint* nil))
+
+(defun !printer-control-init ()
+  (setq *print-readably* nil
+        *print-escape* t
+        *print-pretty* nil
+        *print-base* 10
+        *print-radix* nil
+        *print-vector-length* nil
+        *print-circle* nil
+        *print-case* :upcase
+        *print-array* t
+        *print-gensym* t
+        *print-lines* nil
+        *print-right-margin* nil
+        *print-miser-width* nil
+        *print-pprint-dispatch* (sb-pretty::make-pprint-dispatch-table #() nil nil)
+        *suppress-print-errors* nil
+        *current-level-in-print* 0))
+
 ;;; called when a cold system starts up
 (defun !cold-init (&aux (real-choose-symbol-out-fun #'choose-symbol-out-fun)
                         (real-failed-aver-fun #'%failed-aver))
   "Give the world a shove and hope it spins."
 
   (/show0 "entering !COLD-INIT")
-  (setf (symbol-function '%failed-aver) #'!cold-failed-aver)
-  (!readtable-cold-init)
-  (setq *readtable* (make-readtable)
-        *print-length* 6 *print-level* 3)
-  (setq *error-output* (!make-cold-stderr-stream)
-                      *standard-output* *error-output*
-                      *trace-output* *error-output*)
+  #+sb-show (setq */show* t)
+  (/show0 "cold-initializing streams")
+  (sb-impl::!cold-stream-init)
+  (show-and-call !signal-function-cold-init)
+  (show-and-call !printer-control-init) ; needed before first instance of FORMAT or WRITE-STRING
+  (setq *unparse-fun-type-simplify* nil) ; needed by TLFs in target-error.lisp
+  (setq sb-unix::*unblock-deferrables-on-enabling-interrupts-p* nil) ; needed by LOAD-LAYOUT called by CLASSES-INIT
+  (setq *print-length* 6
+        *print-level* 3)
   (/show "testing '/SHOW" *print-length* *print-level*) ; show anything
+  ;; This allows FORMAT to work, and can go as early needed for
+  ;; debugging.
+  (show-and-call !format-cold-init)
   (unless (!c-runtime-noinform-p)
     ;; I'd like FORMAT to remain working in cold-init, where it does work,
     ;; hence the conditional.
     #+(or x86 x86-64) (format t "COLD-INIT... ")
     #-(or x86 x86-64) (write-string "COLD-INIT... "))
-  (!cold-init-hash-table-methods)
-  ;; Establish **initial-handler-clusters**
-  (show-and-call sb-kernel::!target-error-cold-init)
-  ;; And now *CURRENT-THREAD* and *HANDLER-CLUSTERS*
+
+  ;; Anyone might call RANDOM to initialize a hash value or something;
+  ;; and there's nothing which needs to be initialized in order for
+  ;; this to be initialized, so we initialize it right away.
+  (show-and-call !random-cold-init)
+
+  ;; All sorts of things need INFO and/or (SETF INFO).
+  (/show0 "about to SHOW-AND-CALL !GLOBALDB-COLD-INIT")
+  (show-and-call !globaldb-cold-init)
+  (show-and-call !function-names-init)
+
+  (setf (symbol-function '%failed-aver) #'!cold-failed-aver)
+
+  ;; And now *CURRENT-THREAD*
   (sb-thread::init-main-thread)
 
   ;; Assert that FBOUNDP doesn't choke when its answer is NIL.
@@ -88,21 +131,17 @@
   ;; And be extra paranoid - ensure that it really gets called.
   (locally (declare (notinline fboundp)) (fboundp '(setf !zzzzzz)))
 
-  ;; Anyone might call RANDOM to initialize a hash value or something;
-  ;; and there's nothing which needs to be initialized in order for
-  ;; this to be initialized, so we initialize it right away.
-  (show-and-call !random-cold-init)
-
-  ;; Putting data in a synchronized hashtable (*PACKAGE-NAMES*)
-  ;; requires that the main thread be properly initialized.
-  (show-and-call thread-init-or-reinit)
   ;; Printing of symbols requires that packages be filled in, because
-  ;; OUTPUT-SYMBOL calls FIND-SYMBOL to determine accessibility.
+  ;; OUTPUT-SYMBOL calls FIND-SYMBOL to determine accessibility. Also
+  ;; allows IN-PACKAGE to work.
   (show-and-call !package-cold-init)
-  (setq *print-pprint-dispatch* (sb-pretty::make-pprint-dispatch-table nil nil nil))
-  ;; Because L-T-V forms have not executed, CHOOSE-SYMBOL-OUT-FUN doesn't work.
-  (setf (symbol-function 'choose-symbol-out-fun)
-        (lambda (&rest args) (declare (ignore args)) #'output-preserve-symbol))
+  ;; debug machinery needed for printing symbols
+  #+sb-devel
+  (progn
+    (!readtable-cold-init)
+    ;; Because L-T-V forms have not executed, CHOOSE-SYMBOL-OUT-FUN doesn't work.
+    (setf (symbol-function 'choose-symbol-out-fun)
+          (lambda (&rest args) (declare (ignore args)) #'output-preserve-symbol)))
 
   ;; *RAW-SLOT-DATA* is essentially a compile-time constant
   ;; but isn't dumpable as such because it has functions in it.
@@ -111,24 +150,14 @@
   ;; Must be done before any non-opencoded array references are made.
   (show-and-call sb-vm::!hairy-data-vector-reffer-init)
 
-  (show-and-call !character-database-cold-init)
-  (show-and-call !character-name-database-cold-init)
-  (show-and-call sb-unicode::!unicode-properties-cold-init)
-
-  ;; All sorts of things need INFO and/or (SETF INFO).
-  (/show0 "about to SHOW-AND-CALL !GLOBALDB-COLD-INIT")
-  (show-and-call !globaldb-cold-init)
-
   ;; Various toplevel forms call MAKE-ARRAY, which calls SUBTYPEP, so
   ;; the basic type machinery needs to be initialized before toplevel
   ;; forms run.
   (show-and-call !type-class-cold-init)
-  (show-and-call sb-kernel::!primordial-type-cold-init)
   (show-and-call !classes-cold-init)
-  (show-and-call !pred-cold-init)
-  (show-and-call !early-type-cold-init)
-  (show-and-call !late-type-cold-init)
-  (show-and-call !alien-type-cold-init)
+  (show-and-call sb-kernel::!primordial-type-cold-init)
+
+  (show-and-call !type-cold-init)
   ;; FIXME: It would be tidy to make sure that that these cold init
   ;; functions are called in the same relative order as the toplevel
   ;; forms of the corresponding source files.
@@ -141,31 +170,24 @@
   ;; to the subclasses of STRUCTURE-OBJECT.
   (show-and-call sb-kernel::!set-up-structure-object-class)
 
-  ;; Genesis is able to perform some of the work of DEFCONSTANT and
-  ;; DEFPARAMETER, but not all of it. It assigns symbol values, but can not
-  ;; manipulate globaldb. Therefore, a subtlety of these macros for bootstrap
-  ;; is that we see each DEFthing twice: once during cold-load and again here.
-  ;; Now it being the case that DEFPARAMETER implies variable assignment
-  ;; unconditionally, you may think it should assign. No! This was logically
-  ;; ONE use of the defining macro, but split into pieces as a consequence
-  ;; of the implementation.
+  ;; Genesis is able to perform some of the work of DEFCONSTANT, but
+  ;; not all of it. It assigns symbol values, but can not manipulate
+  ;; globaldb. Therefore, a subtlety of these macros for bootstrap is
+  ;; that we see each DEFthing twice: once during cold-load and again
+  ;; here.
+  (setq sb-pcl::*!docstrings* nil) ; needed by %DEFCONSTANT
   (dolist (x *!cold-defsymbols*)
     (destructuring-bind (fun name source-loc . docstring) x
       (aver (boundp name)) ; it's a bug if genesis didn't initialize
       (ecase fun
-        (sb-c::%defconstant
-         (apply #'sb-c::%defconstant name (symbol-value name) source-loc docstring))
-        (sb-impl::%defparameter ; use %DEFVAR which will not clobber
-         (apply #'%defvar name source-loc nil docstring)))))
-  (dolist (x *!cold-defuns*)
-    (destructuring-bind (name inline-expansion dxable-args) x
-      (%defun name (fdefinition name) inline-expansion dxable-args)))
+        (%defconstant
+         (apply #'%defconstant name (symbol-value name) source-loc docstring)))))
 
   (unless (!c-runtime-noinform-p)
-    #+(or x86 x86-64) (format t "[Length(TLFs)=~D]~%" (length *!cold-toplevels*))
-    #-(or x86 x86-64)
-    (progn (write `("Length(TLFs)=" ,(length *!cold-toplevels*)) :escape nil)
-           (terpri)))
+    #+(or x86 x86-64) (format t "[Length(TLFs)=~D]" (length *!cold-toplevels*))
+    #-(or x86 x86-64) (write `("Length(TLFs)=" ,(length *!cold-toplevels*)) :escape nil))
+
+  (setq sb-c::*queued-proclaims* nil) ; needed before any proclaims are run
 
   (loop with *package* = *package* ; rebind to self, as if by LOAD
         for index-in-cold-toplevels from 0
@@ -186,16 +208,17 @@
            (aver (typep object 'code-component))
            (aver (unbound-marker-p (code-header-ref object index)))
            (setf (code-header-ref object index) (svref *!load-time-values* value))))
-        ((cons (eql defstruct))
-         (apply 'sb-kernel::%defstruct (cdr toplevel-thing)))
+        ((cons (eql :begin-file))
+         (unless (!c-runtime-noinform-p) (print (cdr toplevel-thing))))
         (t
          (!cold-lose "bogus operation in *!COLD-TOPLEVELS*"))))
   (/show0 "done with loop over cold toplevel forms and fixups")
+  (unless (!c-runtime-noinform-p) (terpri))
 
   ;; Precise GC seems to think these symbols are live during the final GC
   ;; which in turn enlivens a bunch of other "*!foo*" symbols.
   ;; Setting them to NIL helps a little bit.
-  (setq *!cold-defuns* nil *!cold-defsymbols* nil *!cold-toplevels* nil)
+  (setq *!cold-defsymbols* nil *!cold-toplevels* nil)
 
   ;; Now that L-T-V forms have executed, the symbol output chooser works.
   (setf (symbol-function 'choose-symbol-out-fun) real-choose-symbol-out-fun)
@@ -265,6 +288,7 @@
   ;; The system is finally ready for GC.
   (/show0 "enabling GC")
   (setq *gc-inhibit* nil)
+  #+sb-thread (finalizer-thread-start)
 
   ;; The show is on.
   (/show0 "going into toplevel loop")
@@ -330,24 +354,18 @@ process to continue normally."
 
 ;;;; initialization functions
 
-(defun thread-init-or-reinit ()
-  (sb-thread::init-job-control)
-  (sb-thread::get-foreground))
-
 (defun reinit (total)
   ;; WITHOUT-GCING implies WITHOUT-INTERRUPTS.
   (without-gcing
     ;; Until *CURRENT-THREAD* has been set, nothing the slightest bit complicated
     ;; can be called, as pretty much anything can assume that it is set.
     (when total ; newly started process, and not a failed save attempt
-      (sb-thread::init-main-thread)
-      #+sb-thread (setq *finalizer-thread* t))
+      (sb-thread::init-main-thread))
     ;; Initializing the standard streams calls ALLOC-BUFFER which calls FINALIZE
     (finalizers-reinit)
     ;; Initialize streams next, so that any errors can be printed
     (stream-reinit t)
     (os-cold-init-or-reinit)
-    (thread-init-or-reinit)
     #-(and win32 (not sb-thread))
     (signal-cold-init-or-reinit)
     (setf (extern-alien "internal_errors_enabled" int) 1)
@@ -359,7 +377,8 @@ process to continue normally."
   ;; re-disable ldb again.
   (when (eq *invoke-debugger-hook* 'sb-debug::debugger-disabled-hook)
     (sb-debug::disable-debugger))
-  (call-hooks "initialization" *init-hooks*))
+  (call-hooks "initialization" *init-hooks*)
+  #+sb-thread (finalizer-thread-start))
 
 ;;;; some support for any hapless wretches who end up debugging cold
 ;;;; init code
