@@ -54,6 +54,29 @@
 (defun set-symbol-global-value (sym val)
   (error "Can't set symbol-global-value: ~S ~S" sym val))
 
+(defun %defun (name lambda &optional inline-expansion)
+  (declare (ignore inline-expansion))
+  (cl:proclaim `(ftype function ,name))
+  (setf (fdefinition name) lambda))
+
+(defun %defglobal (name value source-location &optional (doc nil docp))
+  (declare (ignore source-location doc docp))
+  (cl:proclaim `(special ,name))
+  (setf (symbol-value name) value))
+
+(defun %defparameter (var val source-location &optional (doc nil docp))
+  (declare (ignore source-location doc docp))
+  (cl:proclaim `(special ,var))
+  (setf (symbol-value var) val))
+
+(defun %defvar (var source-location &optional (val nil valp) (doc nil docp))
+  (declare (ignore source-location doc docp))
+  (cl:proclaim `(special ,var))
+  (when (and valp (not (boundp var)))
+    (setf (symbol-value var) val)))
+
+(defun %boundp (symbol) (boundp symbol))
+
 ;;; The GENESIS function works with fasl code which would, in the
 ;;; target SBCL, work on ANSI-STREAMs (streams which aren't extended
 ;;; Gray streams). In ANSI Common Lisp, an ANSI-STREAM is just a
@@ -222,6 +245,35 @@
   (write structure :stream stream :circle t))
 
 (in-package "SB-KERNEL")
+
+;;; These functions are required to emulate SBCL kernel functions
+;;; in a vanilla ANSI Common Lisp cross-compilation host.
+;;; The emulation doesn't need to be efficient, since it's needed
+;;; only for object dumping.
+
+;; The set of structure types that we access by slot position at cross-compile
+;; time is fairly small:
+;;   - DEFINITION-SOURCE-LOCATION
+;;   - DEFSTRUCT-DESCRIPTION, DEFSTRUCT-SLOT-DESCRIPTION
+;;   - DEBUG-SOURCE, COMPILED-DEBUG-INFO, COMPILED-DEBUG-FUN-{something}
+;;   - HEAP-ALIEN-INFO and ALIEN-{something}-TYPE
+;;   - COMMA
+#-metaspace (defmacro wrapper-friend (x) x)
+(defun %instance-wrapper (instance)
+  (declare (notinline classoid-wrapper))
+  (classoid-wrapper (find-classoid (type-of instance))))
+(defun %instance-length (instance)
+  (declare (notinline wrapper-length))
+  ;; In the target, it is theoretically possible to have %INSTANCE-LENGTH
+  ;; exceeed layout length, but in the cross-compiler they're the same.
+  (wrapper-length (%instance-wrapper instance)))
+(defun %raw-instance-ref/word (instance index)
+  (declare (ignore instance index))
+  (error "No such thing as raw structure access on the host"))
+(defun layout-id (x)
+  (declare (notinline sb-kernel::wrapper-id))
+  (sb-kernel::wrapper-id x))
+
 (defun %find-position (item seq from-end start end key test)
   (let ((position (position item seq :from-end from-end
                             :start start :end end :key key :test test)))
@@ -241,6 +293,11 @@
 ;;; Mainly for the fasl loader
 (defun %fun-name (f) (nth-value 2 (function-lambda-expression f)))
 
+(defun %svset (vector index val) ; stemming from toplevel (SETF SVREF)
+  (setf (aref vector index) val))
+(defun %puthash (key table val) ; stemming from toplevel (SETF GETHASH)
+  (setf (gethash key table) val))
+
 ;;;; Variables which have meaning only to the cross-compiler, defined here
 ;;;; in lieu of #+sb-xc-host elsewere which messes up toplevel form numbers.
 (in-package "SB-C")
@@ -261,9 +318,9 @@
   (let* ((n (length string))
          (a (make-array n :element-type '(unsigned-byte 8))))
     (dotimes (i n a)
-      (let ((code (sb-xc:char-code (char string i))))
+      (let ((code (char-code (char string i))))
         (unless (<= 0 code 127)
-          (setf code (sb-xc:char-code #\?)))
+          (setf code (char-code #\?)))
         (setf (aref a i) code)))))
 
 ;;;; Stubs for host
@@ -273,91 +330,13 @@
                    `(lambda ,@(cddr lambda))
                    lambda)))
 
-(defun sb-impl::%defun (name lambda &optional inline-expansion)
-  (declare (ignore inline-expansion))
-  (proclaim `(ftype function ,name))
-  (setf (fdefinition name) (eval lambda)))
-
-(defun %svset (vector index val) ; stemming from toplevel (SETF SVREF)
-  (setf (aref vector index) val))
-(defun %puthash (key table val) ; stemming from toplevel (SETF GETHASH)
-  (setf (gethash key table) val))
-
 ;;; The compiler calls this with forms in EVAL-WHEN (:COMPILE-TOPLEVEL) situations.
-;;; Since we've already performed macroexpansion using our macros, we can either
-;;; implement target-compatible functions for all things into which we might expand,
-;;; or we can un-macro-expand the form.  This does a little of both,
-;;; mainly for the sake of showing that it's quite easily done.
-;;; Truth be told I'd have preferred to use the anti-expansion technique consistently,
-;;; however occasionally we see things like (LET ((V FROB)) (%SVSET *THING* X V))
-;;; which means that the host is going to do the LET and then call %SVSET.
 (defun eval-tlf (form index &optional lexenv)
   (declare (ignore index lexenv))
-  (flet ((matchp (template form &aux results)
-           (if (named-let recurse ((form form) (template template))
-                 (typecase template
-                   (null (null form))
-                   ((eql ?) (push form results) t) ; match and store anything
-                   ((eql :ignore) t) ; match anything and disregard
-                   ((cons (eql :or))
-                    (some (lambda (template)
-                            (recurse form template))
-                          (cdr template)))
-                   (cons (and (consp form)
-                              (and (recurse (car form) (car template))
-                                   (recurse (cdr form) (cdr template)))))
-                   (t (eql template form)))) ; match template exactly
-               (nreverse results)
-               (error "Pattern match failure: ~S~% ~S~%" template form))))
-    ;; Note that in all cases below, the package lock on CL prevents
-    ;; accidental appearance of a CL symbol as a thing being defined.
-    (named-let recurse ((form form))
-      (case (car form)
-       (progn (mapc #'recurse (cdr form))) ; compiler doesn't care about return value
-       (t
-        (eval
-         (case (car form)
-           (sb-impl::%defglobal
-            (destructuring-bind (symbol value)
-                (matchp '((quote ?) (if (%boundp :ignore) :ignore ?)) (cdr form))
-              `(defvar ,symbol ,value)))
-           (sb-impl::%defparameter
-            (destructuring-bind (symbol value)
-                (matchp '((quote ?) ? :ignore) (cdr form))
-              `(defparameter ,symbol ,value)))
-           (sb-impl::%defvar
-            (destructuring-bind (symbol value) ; always occurs with a value
-                (matchp '((quote ?) (source-location) (:or (unless (%boundp :ignore) ?) ?))
-                        (cdr form))
-              `(defvar ,symbol ,value)))
-           (sb-c::%defconstant
-            ;; There is genuinely ambiguity here - does :COMPILE-TOPLEVEL situation for
-            ;; DEFCONSTANT mean that we want the host compiler to know the constant?
-            ;; It must, because the standard specifies that defconstant need not be within
-            ;; EVAL-WHEN for the compiler to know it. But because of how our defconstant
-            ;; expands - calling %defconstant inside of an eval-when listing all 3
-            ;; situations - we can't discern whether this is our defconstant doing its
-            ;; normal thing, versus inside an explicity written eval-when with intent
-            ;; to convey the constant to the host - perhaps because of a DEFUN also inside
-            ;; an eval-when where we need the host to reference the constant.
-            ;; Therefore every constant has to be made known to the host under the
-            ;; assumption that it needs it, AND the cross-compiler under the assumption
-            ;; that this is our normal :compile-toplevel handling.
-            (destructuring-bind (symbol value) (matchp '((quote ?) ? . :ignore) (cdr form))
-              `(progn (defconstant ,symbol ,value)
-                      (sb-c::%defconstant ',symbol ,symbol nil))))
-           (defconstant ; we see this macro as well. The host expansion will not do,
-            ;; because it calls our %defconstant which does not assign the symbol a value.
-            ;; It might be possible to change that now that we don't use CL: symbols.
-            (destructuring-bind (symbol value) (cdr form)
-              `(progn (defconstant ,symbol ,value)
-                      (sb-c::%defconstant ',symbol ,symbol nil))))
-           (t
-            form))))))))
+  (eval form))
 
 (defmacro sb-format:tokens (string) string)
 
-;;; For a use in EARLY-TYPE
 (defmacro with-system-mutex ((lock) &body body)
   (declare (ignore lock))
   `(progn ,@body))
@@ -376,3 +355,5 @@
   ;; Let's suppose that the host lisp knows what it's doing to efficiently
   ;; compile the standard macro. Don't try to outdo it.
   `(with-output-to-string (,var) ,@body))
+
+(defun source-location ())
