@@ -87,7 +87,7 @@
 
 (def!struct (compiled-debug-fun (:include debug-fun)
                                 (:copier nil)
-                                #-sb-xc-host (:pure t))
+                                (:pure t))
   ;; KLUDGE: Courtesy of more than a decade of, ah, organic growth in
   ;; CMU CL, there are two distinct -- but coupled -- mechanisms to
   ;; finding the name of a function. The slot here is one mechanism
@@ -285,23 +285,23 @@
 ;;; If you add more subtypes here, be sure to amend the set of
 ;;; predefined layout FOP codes in src/code/fop
 (def!struct (compiled-debug-fun-optional (:include compiled-debug-fun)
-                                         #-sb-xc-host (:pure t)
+                                         (:pure t)
                                          (:copier nil)
                                          (:predicate nil)))
 (def!struct (compiled-debug-fun-more (:include compiled-debug-fun)
-                                     #-sb-xc-host (:pure t)
+                                     (:pure t)
                                      (:copier nil)
                                      (:predicate nil)))
 (def!struct (compiled-debug-fun-external (:include compiled-debug-fun)
-                                         #-sb-xc-host (:pure t)
+                                         (:pure t)
                                          (:copier nil)
                                          (:predicate nil)))
 (def!struct (compiled-debug-fun-toplevel (:include compiled-debug-fun)
-                                         #-sb-xc-host (:pure t)
+                                         (:pure t)
                                          (:copier nil)
                                          (:predicate nil)))
 (def!struct (compiled-debug-fun-cleanup (:include compiled-debug-fun)
-                                        #-sb-xc-host (:pure t)
+                                        (:pure t)
                                         (:copier nil)
                                         (:predicate nil)))
 
@@ -380,7 +380,7 @@
 
 ;;; There is one per compiled file and one per function compiled at
 ;;; toplevel or loaded from source.
-(def!struct (debug-source #-sb-xc-host (:pure t)
+(def!struct (debug-source (:pure t)
                           (:copier nil))
   ;; When the DEBUG-SOURCE describes a file, the file's namestring.
   ;; Otherwise, NIL.
@@ -390,14 +390,6 @@
   (created nil :type (or unsigned-byte null))
   ;; Additional information from (WITH-COMPILATION-UNIT (:SOURCE-PLIST ...))
   (plist *source-plist* :read-only t))
-(def!struct (core-debug-source #-sb-xc-host (:pure t)
-                               (:copier nil)
-                               (:include debug-source))
-  ;; Compilation to memory stores each toplevel form given to %COMPILE.
-  ;; That form can generate multiple functions, and those functions can
-  ;; be in one or more code components. They all point at the same form.
-  form
-  (function nil :read-only t))
 
 ;;;; DEBUG-INFO structures
 
@@ -412,13 +404,15 @@
 (def!struct (compiled-debug-info
              (:include debug-info)
              (:copier nil)
-             #-sb-xc-host (:pure t))
+             (:pure t))
   ;; COMPILED-DEBUG-FUNs linked through COMPILED-DEBUG-FUN-NEXT
   (fun-map (missing-arg) :type compiled-debug-fun)
   ;; Location contexts
   ;; A (simple-array * (*)) or a context if there's only one context.
   (contexts nil :type t :read-only t)
-  (tlf-num+offset nil :type integer))
+  ;; Packed integers. Also can be a cons of that plus an alist which
+  ;; maps SB-C::COMPILED-DEBUG-FUN to SB-DI::COMPILED-DEBUG-FUN instances.
+  (tlf-num+offset (missing-arg) :type (or integer cons)))
 
 ;;; The TLF-NUMBER and CHAR-OFFSET of a compiled-debug-info can each be NIL,
 ;;; but aren't often. However, to allow that, convert NIL to 0 and non-nil
@@ -445,17 +439,18 @@
              (:copier nil)
              (:print-object (lambda (s stream)
                               (print-unreadable-object (s stream :type t)
-                                (princ (file-info-name s) stream)))))
+                                (princ (file-info-truename s) stream)))))
   ;; If a file, the truename of the corresponding source file. If from
-  ;; a Lisp form, :LISP. If from a stream, :STREAM.
-  (name (missing-arg) :type (or pathname (eql :lisp)) :read-only t)
+  ;; a Lisp form, :LISP. In COMPILE-FILE, this gets filled lazily
+  ;; after the file gets opened.
+  (truename nil :type (or pathname null (eql :lisp)))
   ;; the external format that we'll call OPEN with, if NAME is a file.
   (external-format nil  :read-only t)
   ;; the defaulted, but not necessarily absolute file name (i.e. prior
   ;; to TRUENAME call.) Null if not a file. This is used to set
   ;; *COMPILE-FILE-PATHNAME*, and if absolute (a harmful constraint to be sure),
   ;; is dumped in the debug-info.
-  (untruename nil :type (or pathname null) :read-only t)
+  (pathname nil :type (or pathname null) :read-only t)
   ;; the file's write date (if relevant)
   (write-date nil :type (or unsigned-byte null)  :read-only t)
   ;; parallel vectors containing the forms read out of the file and
@@ -475,10 +470,7 @@
   ;; the "&OPTIONAL and &KEY" warning is quite annoying to see repeated.
   ;; And I doubt it changes anyone's mind about coding style anyway.
   ;; Typically this matters for DEFTYPE and DEFMACRO.
-  (style-warning-tracker nil :type list)
-  ;; A list of forms that are pending compilation. Delaying compilation helps
-  ;; with defstruct constructors that involve type dependency cycles.
-  (queued-tlfs nil))
+  (style-warning-tracker nil :type list))
 
 ;;; The SOURCE-INFO structure provides a handle on all the source
 ;;; information for an entire compilation.
