@@ -13,10 +13,42 @@
 
 (in-package "SB-IMPL")
 
+;;; FIXME: a lot of places in the code that should use ARRAY-RANGE
+;;; instead use INDEX and vice-versa, but the thing is, we have
+;;; a ton of fenceposts errors all over the place regardless.
+;;; CLHS glosary reference:
+;;; "array total size n. the total number of elements in an array,
+;;;  computed by taking the product of the dimensions of the array."
+;;; and ARRAY-TOTAL-SIZE-LIMIT:
+;;; "The upper exclusive bound on the array total size of an array."
+
+;;; Consider a reduced example for the sake of argument in which
+;;; ARRAY-TOTAL-SIZE-LIMIT were 21.
+;;; - the largest vector would have LENGTH 20.
+;;; - the largest legal index for AREF on it would be 19.
+;;; - the largest legal :END on sequence functions would be 20.
+;;; Nothing should allow 21. So DEFTYPE ARRAY-RANGE is wrong
+;;; and must never be used for anything.
+;;; Unfortunately, it's all over mb-util and enc-utf etc.
+;;;
+;;; *Also* INDEX is wrong, as the comment there says.
+;;; To fix these and also SEQUENCE-END:
+;;; - INDEX should be `(integer 0 (,1- array-dimension-limit))
+;;; - ARRAY-RANGE should be `(integer 0 (,array-dimension-limit))
+;;; - SEQUENCE-END (in deftypes-for-target) should be
+;;;   `(OR NULL ARRAY-RANGE)
+;;;
+;;; A further consideration for INDEX on 64-bit machines is that
+;;; if INDEX were equivalent to (UNSIGNED-BYTE n) for some 'n'
+;;; then (TYPEP X 'INDEX) can combines FIXNUMP and the range test into
+;;; one bit-masking test, depending on the architecture.
+;;; Probably it should be (UNSIGNED-BYTE 61) except on ppc64
+;;; where it would have to be (UNSIGNED-BYTE 59).
+
 ;;; A number that can represent an index into a vector, including
 ;;; one-past-the-end
 (deftype array-range ()
-  `(integer 0 ,sb-xc:array-dimension-limit))
+  `(integer 0 ,array-dimension-limit))
 
 ;;; a type used for indexing into sequences, and for related
 ;;; quantities like lengths of lists and other sequences.
@@ -35,13 +67,13 @@
 ;;; MOST-POSITIVE-FIXNUM lets the system know it can increment a value
 ;;; of type INDEX without having to worry about using a bignum to
 ;;; represent the result.
-(def!type index () `(integer 0 (,sb-xc:array-dimension-limit)))
+(def!type index () `(integer 0 (,array-dimension-limit)))
 
 ;;; like INDEX, but augmented with -1 (useful when using the index
 ;;; to count downwards to 0, e.g. LOOP FOR I FROM N DOWNTO 0, with
 ;;; an implementation which terminates the loop by testing for the
 ;;; index leaving the loop range)
-(def!type index-or-minus-1 () `(integer -1 (,sb-xc:array-dimension-limit)))
+(def!type index-or-minus-1 () `(integer -1 (,array-dimension-limit)))
 
 ;;; The smallest power of two that is equal to or greater than X.
 (declaim (inline power-of-two-ceiling))
@@ -203,49 +235,17 @@
          ,@(if ignores `((declare (ignorable ,@ignores))))
          ,@body))))
 
-;;;; some old-fashioned functions. (They're not just for old-fashioned
-;;;; code, they're also used as optimized forms of the corresponding
-;;;; general functions when the compiler can prove that they're
-;;;; equivalent.)
+;;; Functions for compatibility sake:
 
-;;; like (MEMBER ITEM LIST :TEST #'EQ)
 (defun memq (item list)
   "Return tail of LIST beginning with first element EQ to ITEM."
-  (declare (explicit-check))
-  ;; KLUDGE: These could be and probably should be defined as
-  ;;   (MEMBER ITEM LIST :TEST #'EQ)),
-  ;; but when I try to cross-compile that, I get an error from
-  ;; LTN-ANALYZE-KNOWN-CALL, "Recursive known function definition". The
-  ;; comments for that error say it "is probably a botched interpreter stub".
-  ;; Rather than try to figure that out, I just rewrote this function from
-  ;; scratch. -- WHN 19990512
-  (do ((i list (cdr i)))
-      ((null i))
-    (when (eq (car i) item)
-      (return i))))
+  (declare (inline member))
+  (member item list :test #'eq))
 
-;;; like (ASSOC ITEM ALIST :TEST #'EQ):
-;;;   Return the first pair of ALIST where ITEM is EQ to the key of
-;;;   the pair.
 (defun assq (item alist)
-  (declare (explicit-check))
-  ;; KLUDGE: CMU CL defined this with
-  ;;   (DECLARE (INLINE ASSOC))
-  ;;   (ASSOC ITEM ALIST :TEST #'EQ))
-  ;; which is pretty, but which would have required adding awkward
-  ;; build order constraints on SBCL (or figuring out some way to make
-  ;; inline definitions installable at build-the-cross-compiler time,
-  ;; which was too ambitious for now). Rather than mess with that, we
-  ;; just define ASSQ explicitly in terms of more primitive
-  ;; operations:
-  (dolist (pair alist)
-    ;; though it may look more natural to write this as
-    ;;   (AND PAIR (EQ (CAR PAIR) ITEM))
-    ;; the temptation to do so should be resisted, as pointed out by PFD
-    ;; sbcl-devel 2003-08-16, as NIL elements are rare in association
-    ;; lists.  -- CSR, 2003-08-16
-    (when (and (eq (car pair) item) (not (null pair)))
-      (return pair))))
+  "Return the first pair of alist where item is EQ to the key of pair."
+  (declare (inline assoc))
+  (assoc item alist :test #'eq))
 
 ;;; Delete just one item
 (defun delq1 (item list)
@@ -309,7 +309,7 @@
                        ,result)))
     `(let ((,n-table ,table))
        ,(if locked
-            `(with-locked-system-table (,n-table) ,iter-form)
+            `(with-system-mutex ((hash-table-lock ,n-table)) ,iter-form)
             iter-form))))
 
 ;;; Executes BODY for all entries of PLIST with KEY and VALUE bound to
@@ -317,6 +317,7 @@
 (defmacro doplist ((key val) plist &body body)
   (with-unique-names (tail)
     `(let ((,tail ,plist) ,key ,val)
+       (declare (ignorable ,key ,val))
        (loop (when (null ,tail) (return nil))
              (setq ,key (pop ,tail))
              (when (null ,tail)
@@ -467,6 +468,7 @@ NOTE: This interface is experimental and subject to change."
 
 ;; Make a new hash-cache and optionally create the statistics vector.
 (defun alloc-hash-cache (size symbol)
+  (declare (type index size))
   (let (cache)
     ;; It took me a while to figure out why infinite recursion could occur
     ;; in VALUES-SPECIFIER-TYPE. It's because SET calls VALUES-SPECIFIER-TYPE.
@@ -565,7 +567,7 @@ NOTE: This interface is experimental and subject to change."
          (line-type (let ((n (+ nargs nvalues)))
                       (if (<= n 3) 'cons `(simple-vector ,n))))
          (bind-hashval
-          `((,hashval (the (signed-byte #.sb-vm:n-fixnum-bits)
+          `((,hashval (the sb-xc:fixnum
                            (funcall ,hash-function ,@arg-vars)))
             (,cache ,var-name)))
          (probe-it
@@ -574,7 +576,7 @@ NOTE: This interface is experimental and subject to change."
                (let ((,hashval ,hashval) ; gets clobbered in probe loop
                      (,cache (truly-the ,cache-type ,cache)))
                  ;; FIXME: redundant?
-                 (declare (type (signed-byte #.sb-vm:n-fixnum-bits) ,hashval))
+                 (declare (type sb-xc:fixnum ,hashval))
                  (loop repeat 2
                     do (let ((,entry
                               (svref ,cache
@@ -1071,6 +1073,12 @@ NOTE: This interface is experimental and subject to change."
 (defun print-type (stream type &optional colon at)
   (print-type-specifier stream (type-specifier type) colon at)))
 
+(defun print-lambda-list (stream lambda-list &optional colon at)
+  (declare (ignore colon at))
+  (let ((sb-pretty:*pprint-quote-with-syntactic-sugar* nil)
+        (*package* *cl-package*))
+    (format stream "~:A" lambda-list)))
+
 
 ;;;; Deprecating stuff
 
@@ -1158,7 +1166,7 @@ NOTE: This interface is experimental and subject to change."
     ((or symbol cons)
      (%check-deprecated-type type-specifier))
     (class
-     (let ((name (class-name type-specifier)))
+     (let ((name (cl:class-name type-specifier)))
        (when (and name (symbolp name)
                   (eq type-specifier (find-class name nil)))
          (%check-deprecated-type name))))))
@@ -1228,7 +1236,7 @@ NOTE: This interface is experimental and subject to change."
 (defun setup-type-in-final-deprecation
     (software version name replacement-spec)
   (declare (ignore software version replacement-spec))
-  (%compiler-deftype name (constant-type-expander name t) nil))
+  (%deftype name (constant-type-expander name t) nil))
 
 ;; Given DECLS as returned by from parse-body, and SYMBOLS to be bound
 ;; (with LET, MULTIPLE-VALUE-BIND, etc) return two sets of declarations:
@@ -1248,7 +1256,9 @@ NOTE: This interface is experimental and subject to change."
                                 ((or (listp id) ; must be a type-specifier
                                      (memq id '(special ignorable ignore
                                                 dynamic-extent
-                                                truly-dynamic-extent))
+                                                truly-dynamic-extent
+                                                sb-c::constant-value
+                                                sb-c::no-constraints))
                                      (info :type :kind id))
                                  (cdr decl))))))
            (partition (spec)
@@ -1367,33 +1377,6 @@ NOTE: This interface is experimental and subject to change."
                                 (length cache))))
                   short-name))))))
 
-;;; Just like WITH-OUTPUT-TO-STRING but doesn't close the stream,
-;;; producing more compact code.
-(defmacro with-simple-output-to-string
-    ((var &optional string (element-type :default)) &body body)
-  ;; Don't need any fancy type-specifier parsing. Uses of this macro
-  ;; are confined to our own code. So the element-type is literally
-  ;; either :DEFAULT or BASE-CHAR.
-  (aver (member element-type '(base-char :default)))
-  (multiple-value-bind (forms decls) (parse-body body nil)
-    (if string
-        `(let ((,var (make-fill-pointer-output-stream ,string)))
-           ,@decls
-           ,@forms)
-        ;; Why not dxify this stream?
-        `(let ((,var #+sb-xc-host (make-string-output-stream)
-                     #-sb-xc-host
-                     ,(case element-type
-                        ;; Non-unicode doesn't have %MAKE-DEFAULT-STRING-OSTREAM
-                        #+sb-unicode
-                        (:default '(%make-default-string-ostream))
-                        ;; and this macro is never employed to make streams
-                        ;; that can only return CHARACTER string.
-                        (t '(%make-base-string-ostream)))))
-           ,@decls
-           ,@forms
-           (get-output-stream-string ,var)))))
-
 (in-package "SB-KERNEL")
 
 (defun fp-zero-p (x)
