@@ -16,7 +16,7 @@
 ;;; The current level we are printing at, to be compared against
 ;;; *PRINT-LEVEL*. See the macro DESCEND-INTO for a handy interface to
 ;;; depth abbreviation.
-(defparameter *current-level-in-print* 0) ; initialized by genesis
+(defvar *current-level-in-print* 0)
 (declaim (index *current-level-in-print*))
 
 ;;; Automatically handle *PRINT-LEVEL* abbreviation. If we are too
@@ -189,8 +189,6 @@
             (write-char #\= stream)
             t)))))
 
-(define-load-time-global *null-broadcast-stream* (make-broadcast-stream))
-(declaim (type stream *null-broadcast-stream*))
 (defmacro with-circularity-detection ((object stream) &body body)
   (with-unique-names (marker body-name)
     `(labels ((,body-name ()
