@@ -20,12 +20,12 @@
   (declare (explicit-check)
            (dynamic-extent arguments))
   (cond ((and (%instancep datum)
-              (let ((layout (%instance-layout datum)))
-                (and (logtest +condition-layout-flag+ (layout-flags layout))
+              (let ((wrapper (%instance-wrapper datum)))
+                (and (logtest (wrapper-flags wrapper) +condition-layout-flag+)
                      ;; An invalid layout will drop into the (MAKE-CONDITION) branch
                      ;; which rightly fails because ALLOCATE-CONDITION asserts that
                      ;; the first argument is a condition-designator, which it won't be.
-                     (not (layout-invalid layout)))))
+                     (not (wrapper-invalid wrapper)))))
          (when (and arguments (not (eq fun-name 'cerror)))
            (cerror "Ignore the additional arguments."
                    'simple-type-error
@@ -69,7 +69,7 @@
   ;; is one dx cons cell for the whole cluster.
   ;; Otherwise it takes 1+2N cons cells where N is the number of bindings.
   ;;
-  (collect ((local-functions) (cluster-entries) (dummy-forms))
+  (collect ((local-functions) (cluster-entries) (dummy-forms) (complex-initforms))
     (flet ((const-cons (test handler)
              ;; If possible, render HANDLER as a load-time constant so that
              ;; consing the test and handler is also load-time constant.
@@ -166,34 +166,41 @@
                 ;; Unless the expression is ({FUNCTION|QUOTE} <sym>), then create a
                 ;; new local function. If the supplied handler is spelled
                 ;; (LAMBDA ...) or #'(LAMBDA ...), then the local function is the
-                ;; lambda but named. If not spelled as such, the function funcalls
-                ;; the user's sexpr so that the compiler enforces callable-ness.
+                ;; lambda but named.  If not spelled as such, the function funcalls
+                ;; the user's sexpr through a variable binding so that the compiler
+                ;; enforces callable-ness but evaluates the supplied form only once.
                 (if (typep handler '(cons (member function quote) (cons symbol null)))
                     handler
-                    (let ((lexpr
-                           (typecase handler
-                             ;; These two are merely expansion prettifiers,
-                             ;; and not strictly necessary.
-                             ((cons (eql function) (cons (cons (eql lambda)) null))
-                              (cadr handler))
-                             ((cons (eql lambda))
-                              handler)
-                             (t
-                              ;; Should be (THE (FUNCTION-DESIGNATOR (CONDITION)))
-                              ;; but the cast kills DX allocation.
-                              `(lambda (c) (funcall ,handler c)))))
-                          (name (let ((*gensym-counter*
-                                       (length (cluster-entries))))
-                                  (sb-xc:gensym "H"))))
-                      (local-functions `(,name ,@(rest lexpr)))
+                    (let* ((name (let ((*gensym-counter*
+                                        (length (cluster-entries))))
+                                   (sb-xc:gensym "H")))
+                           (lexpr
+                            (typecase handler
+                              ;; These two are merely expansion prettifiers,
+                              ;; and not strictly necessary.
+                              ((cons (eql function) (cons (cons (eql lambda)) null))
+                               (cadr handler))
+                              ((cons (eql lambda))
+                               handler)
+                              (t
+                               (complex-initforms `(,name ,handler))
+                               ;; Should be (THE (FUNCTION-DESIGNATOR (CONDITION)))
+                               ;; but the cast kills DX allocation.
+                               `(lambda (c) (funcall ,name c))))))
+                      (local-functions
+                       `(,name ,(cadr lexpr)
+                               ,@(when (typep (cadr lexpr) '(cons t null))
+                                   '((declare (sb-c::local-optimize (sb-c::verify-arg-count 0)))))
+                               ,@(cddr lexpr)))
                       `#',name))))))))
 
-      `(dx-flet ,(local-functions)
-         ,@(dummy-forms)
-         (dx-let ((*handler-clusters*
-                   (cons ,(const-list (cluster-entries))
-                         *handler-clusters*)))
-           ,form)))))
+      `(let ,(complex-initforms)
+         (dx-flet ,(local-functions)
+           ,@(dummy-forms)
+           (dx-let ((*handler-clusters*
+                     (cons ,(const-list (cluster-entries))
+                           *handler-clusters*)))
+             ,form))))))
 
 (sb-xc:defmacro handler-bind (bindings &body forms)
   "(HANDLER-BIND ( {(type handler)}* ) body)
@@ -235,7 +242,7 @@ specification."
                (annotated-cases
                  (mapcar (lambda (case)
                            (with-current-source-form (case)
-                             (with-unique-names (tag fun)
+                             (with-unique-names (block fun)
                                (destructuring-bind (type ll &body body) case
                                  (unless (and (listp ll)
                                               (symbolp (car ll))
@@ -245,47 +252,43 @@ specification."
                                  (multiple-value-bind (body declarations)
                                      (parse-body body nil)
                                    (push `(,fun ,ll ,@declarations (progn ,@body)) local-funs))
-                                 (list tag type ll fun)))))
+                                 (list block type ll fun)))))
                          cases)))
-          (with-unique-names (block cell form-fun)
-            `(dx-flet ((,form-fun ()
-                         #-x86 (progn ,form) ;; no declarations are accepted
-                         ;; Need to catch FP errors here!
-                         #+x86 (multiple-value-prog1 ,form (float-wait)))
-                       ,@(reverse local-funs))
-               (declare (optimize (sb-c::check-tag-existence 0)))
-               (block ,block
-                 ;; KLUDGE: We use a dx CONS cell instead of just assigning to
-                 ;; the variable directly, so that we can stack allocate
-                 ;; robustly: dx value cells don't work quite right, and it is
-                 ;; possible to construct user code that should loop
-                 ;; indefinitely, but instead eats up some stack each time
-                 ;; around.
-                 (dx-let ((,cell (cons :condition nil)))
-                   (declare (ignorable ,cell))
-                   (tagbody
-                      (%handler-bind
-                       ,(mapcar (lambda (annotated-case)
-                                  (destructuring-bind (tag type ll fun-name) annotated-case
-                                    (declare (ignore fun-name))
-                                    (list type
-                                          `(lambda (temp)
-                                             ,(if ll
-                                                  `(setf (cdr ,cell) temp)
-                                                  '(declare (ignore temp)))
-                                             (go ,tag)))))
-                                annotated-cases)
-                       (return-from ,block (,form-fun)))
-                      ,@(mapcan
-                         (lambda (annotated-case)
-                           (destructuring-bind (tag type ll fun-name) annotated-case
-                             (declare (ignore type))
-                             (list tag
-                                   `(return-from ,block
-                                      ,(if ll
-                                           `(,fun-name (cdr ,cell))
-                                           `(,fun-name))))))
-                         annotated-cases))))))))))
+          (with-unique-names (block form-fun)
+            (let ((body `(%handler-bind
+                          ,(mapcar (lambda (annotated-case)
+                                     (destructuring-bind (block type ll fun-name) annotated-case
+                                       (declare (ignore fun-name))
+                                       (list type
+                                             `(lambda (temp)
+                                                ,@(unless ll
+                                                    `((declare (ignore temp))))
+                                                (return-from ,block
+                                                  ,@(and ll '(temp)))))))
+                                   annotated-cases)
+                          (return-from ,block (,form-fun)))))
+              (labels ((wrap (cases)
+                         (if cases
+                             (destructuring-bind (fun-block type ll fun-name) (car cases)
+                               (declare (ignore type))
+                               `(return-from ,block
+                                  ,(if ll
+                                       `(,fun-name (block ,fun-block
+                                                     ,(wrap (cdr cases))))
+                                       `(progn (block ,fun-block
+                                                 ,(wrap (cdr cases)))
+                                               (,fun-name)))))
+                             body)))
+                `(flet ((,form-fun ()
+                          #-x86 (progn ,form) ;; no declarations are accepted
+                          ;; Need to catch FP errors here!
+                          #+x86 (multiple-value-prog1 ,form (float-wait)))
+                        ,@(reverse local-funs))
+                   (declare (optimize (sb-c::check-tag-existence 0))
+                            (inline ,form-fun
+                                    ,@(mapcar #'car local-funs)))
+                   (block ,block
+                     ,(wrap annotated-cases))))))))))
 
 (sb-xc:defmacro ignore-errors (&rest forms)
   "Execute FORMS handling ERROR conditions, returning the result of the last
