@@ -1107,7 +1107,11 @@
       (dolist (elt list1)
         (unless (funcall member-test elt list2 key test)
           (push elt result)))
-      (dx-flet ((test (x y) (funcall (truly-the function test) y x)))
+      (dx-flet ((test (x y)
+                      ;; This local function is never called if TEST is NIL,
+                      ;; but the compiler doesn't know that, suppress the warning.
+                      (funcall (the* (function :truly t :silent-conflict t) test)
+                               y x)))
         (dolist (elt list2)
           (unless (funcall member-test elt list1 key #'test)
             (push elt result)))))
@@ -1190,7 +1194,11 @@
 
 (defun acons (key datum alist)
   "Construct a new alist by adding the pair (KEY . DATUM) to ALIST."
-  (cons (cons key datum) alist))
+  ;; This function is maybe-inline, so can't use vop-existsp
+  ;; which does not remain in the image post-build.
+  #.(if (gethash 'acons sb-c::*backend-template-names*)
+        '(acons key datum alist) ; vop translated
+        '(cons (cons key datum) alist)))
 
 (defun pairlis (keys data &optional (alist '()))
   "Construct an association list from KEYS and DATA (adding to ALIST)."
