@@ -12,11 +12,9 @@
 
 (in-package "SB-IMPL")
 
-(defun sb-xc:lisp-implementation-type ()
+(defun lisp-implementation-type ()
   "SBCL")
 
-(defun sb-xc:lisp-implementation-version ()
-  ;; Read the version file once and once only
-  #+sb-xc-host #.(sb-cold:read-from-file "version.lisp-expr")
-  #-sb-xc-host #.(sb-xc:lisp-implementation-version))
-
+(defun lisp-implementation-version ()
+  #+sb-xc-host #.sb-cold:*target-sbcl-version*
+  #-sb-xc-host #.(lisp-implementation-version))
