@@ -57,7 +57,7 @@
 
 (sb-xc:defstruct (package
                   (:constructor %make-package
-                                (%name internal-symbols external-symbols))
+                                (internal-symbols external-symbols))
                   (:copier nil)
                   (:predicate packagep))
   "the standard structure for the description of a package"
@@ -93,17 +93,19 @@
   (%local-nicknames nil :type (or null (cons simple-vector simple-vector)))
   ;; Definition source location
   (source-location nil :type (or null sb-c:definition-source-location)))
-(sb-xc:proclaim '(freeze-type package-hashtable package))
-(!set-load-form-method package (:xc)
-  (lambda (obj env)
-    (declare (ignore env))
-    ;; the target code will use FIND-UNDELETED-PACKAGE-OR-LOSE
-    `(find-package ,(package-name obj))))
+(proclaim '(freeze-type package-hashtable package))
 
 (defconstant +initial-package-bits+ 2) ; for genesis
 
+#-sb-xc-host
 (defmacro system-package-p (package) ; SBCL stuff excluding CL and KEYWORD
-  #+sb-xc-host `(eql (mismatch "SB-" (package-name ,package)) 3)
-  #-sb-xc-host `(logbitp 1 (package-%bits ,package)))
+  `(logbitp 1 (package-%bits ,package)))
 
 (defmacro package-lock (package) `(logbitp 0 (package-%bits ,package)))
+
+;;;; IN-PACKAGE
+(proclaim '(special *package*))
+(sb-xc:defmacro in-package (string-designator)
+  (let ((string (string string-designator)))
+    `(eval-when (:compile-toplevel :load-toplevel :execute)
+       (setq *package* (find-undeleted-package-or-lose ,string)))))
