@@ -13,50 +13,49 @@
 
 ;;;; exported printer control variables
 
-;; NB: all of the following are initialized during genesis
-(defparameter *print-readably* nil
+(defvar *print-readably* nil
   "If true, all objects will be printed readably. If readable printing
   is impossible, an error will be signalled. This overrides the value of
   *PRINT-ESCAPE*.")
-(defparameter *print-escape* t
+(defvar *print-escape* t
   "Should we print in a reasonably machine-readable way? (possibly
   overridden by *PRINT-READABLY*)")
-(defparameter *print-pretty* nil ; (set later when pretty-printer is initialized)
+(defvar *print-pretty* nil ; (set later when pretty-printer is initialized)
   "Should pretty printing be used?")
-(defparameter *print-base* 10.
+(defvar *print-base* 10.
   "The output base for RATIONALs (including integers).")
-(defparameter *print-radix* nil
+(defvar *print-radix* nil
   "Should base be verified when printing RATIONALs?")
-(defparameter *print-level* nil
+(defvar *print-level* nil
   "How many levels should be printed before abbreviating with \"#\"?")
-(defparameter *print-length* nil
+(defvar *print-length* nil
   "How many elements at any level should be printed before abbreviating
   with \"...\"?")
-(defparameter *print-vector-length* nil
+(defvar *print-vector-length* nil
   "Like *PRINT-LENGTH* but works on strings and bit-vectors.
 Does not affect the cases that are already controlled by *PRINT-LENGTH*")
-(defparameter *print-circle* nil
+(defvar *print-circle* nil
   "Should we use #n= and #n# notation to preserve uniqueness in general (and
   circularity in particular) when printing?")
-(defparameter *print-case* :upcase
+(defvar *print-case* :upcase
   "What case should the printer should use default?")
-(defparameter *print-array* t
+(defvar *print-array* t
   "Should the contents of arrays be printed?")
-(defparameter *print-gensym* t
+(defvar *print-gensym* t
   "Should #: prefixes be used when printing symbols with null SYMBOL-PACKAGE?")
-(defparameter *print-lines* nil
+(defvar *print-lines* nil
   "The maximum number of lines to print per object.")
-(defparameter *print-right-margin* nil
+(defvar *print-right-margin* nil
   "The position of the right margin in ems (for pretty-printing).")
-(defparameter *print-miser-width* nil
+(defvar *print-miser-width* nil
   "If the remaining space between the current column and the right margin
    is less than this, then print using ``miser-style'' output. Miser
    style conditional newlines are turned on, and all indentations are
    turned off. If NIL, never use miser mode.")
 (defvar *print-pprint-dispatch*
-  (sb-pretty::make-pprint-dispatch-table nil nil nil) ; for type-correctness
+  (sb-pretty::make-pprint-dispatch-table #() nil nil)
   "The pprint-dispatch-table that controls how to pretty-print objects.")
-(defparameter *suppress-print-errors* nil
+(defvar *suppress-print-errors* nil
   "Suppress printer errors when the condition is of the type designated by this
 variable: an unreadable object representing the error is printed instead.")
 
@@ -378,15 +377,24 @@ variable: an unreadable object representing the error is printed instead.")
         (return-from output-ugly-object
           (print-unreadable-object (object stream :identity t)
             (prin1 'instance stream))))
-      (let ((classoid (layout-classoid layout)))
-        ;; Additionally, if the object is an obsolete CONDITION, don't crash.
-        ;; (There is no update-instance protocol for conditions)
+      (let* ((wrapper (layout-friend layout))
+             (classoid (wrapper-classoid wrapper)))
+        ;; Additionally, don't crash if the object is an obsolete thing with
+        ;; no update protocol.
         (when (or (sb-kernel::undefined-classoid-p classoid)
-                  (and (layout-invalid layout)
-                       (logtest (layout-flags layout) +condition-layout-flag+)))
+                  (and (wrapper-invalid wrapper)
+                       (logtest (layout-flags layout)
+                                (logior +structure-layout-flag+
+                                        +condition-layout-flag+))))
           (return-from output-ugly-object
             (print-unreadable-object (object stream :identity t)
               (format stream "UNPRINTABLE instance of ~W" classoid)))))))
+  (when (funcallable-instance-p object)
+    (let ((layout (%fun-layout object)))
+      (unless (logtest (get-lisp-obj-address layout) sb-vm:widetag-mask)
+        (return-from output-ugly-object
+          (print-unreadable-object (object stream :identity t)
+            (prin1 'funcallable-instance stream))))))
   (print-object object stream))
 
 ;;;; symbols
@@ -495,7 +503,7 @@ variable: an unreadable object representing the error is printed instead.")
                                :initial-element 36)))
       (dotimes (i 36 a)
         (let ((char (digit-char i 36)))
-          (setf (aref a (sb-xc:char-code char)) i))))
+          (setf (aref a (char-code char)) i))))
   #'equalp)
 
 (defconstant-eqx +character-attributes+
@@ -504,7 +512,7 @@ variable: an unreadable object representing the error is printed instead.")
                                :element-type '(unsigned-byte 16)
                                :initial-element 0)))
       (flet ((set-bit (char bit)
-               (let ((code (sb-xc:char-code char)))
+               (let ((code (char-code char)))
                  (setf (aref a code) (logior bit (aref a code))))))
 
         (dolist (char '(#\! #\@ #\$ #\% #\& #\* #\= #\~ #\[ #\] #\{ #\}
@@ -514,12 +522,12 @@ variable: an unreadable object representing the error is printed instead.")
         (dotimes (i 10)
           (set-bit (digit-char i) number-attribute))
 
-        (do ((code (sb-xc:char-code #\A) (1+ code))
-             (end (sb-xc:char-code #\Z)))
+        (do ((code (char-code #\A) (1+ code))
+             (end (char-code #\Z)))
             ((> code end))
           (declare (fixnum code end))
-          (set-bit (sb-xc:code-char code) uppercase-attribute)
-          (set-bit (char-downcase (sb-xc:code-char code)) lowercase-attribute))
+          (set-bit (code-char code) uppercase-attribute)
+          (set-bit (char-downcase (code-char code)) lowercase-attribute))
 
         (set-bit #\- sign-attribute)
         (set-bit #\+ sign-attribute)
@@ -1115,7 +1123,7 @@ variable: an unreadable object representing the error is printed instead.")
   ;;   "The function SB-KERNEL:SIMPLE-CHARACTER-STRING-P is undefined."
   ;; but a symbol-macrolet is ok. This is a FIXME except I don't care.
   (symbol-macrolet ((chars "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"))
-    (declare (optimize (sb-c::insert-array-bounds-checks 0) speed))
+    (declare (optimize (sb-c:insert-array-bounds-checks 0) speed))
     (macrolet ((iterative-algorithm ()
                  `(loop (multiple-value-bind (q r)
                             (truncate (truly-the word integer) base)
@@ -1356,8 +1364,14 @@ variable: an unreadable object representing the error is printed instead.")
             (double-float double-float-min-e)
             #+long-float
             (long-float long-float-min-e))))
-    (multiple-value-bind (f e)
-        (integer-decode-float float)
+    (multiple-value-bind (f e) (integer-decode-float float)
+      ;; An extra step became necessary here for subnormals because the
+      ;; algorithm assumes that the fraction is left-aligned in a field
+      ;; that is FLOAT-DIGITS wide.
+      (when (< (float-precision float) float-digits)
+        (let ((shift (- float-digits (integer-length f))))
+          (setq f (ash f shift)
+                e (- e shift))))
       (let ( ;; FIXME: these even tests assume normal IEEE rounding
             ;; mode.  I wonder if we should cater for non-normal?
             (high-ok (evenp f))
@@ -1697,7 +1711,7 @@ variable: an unreadable object representing the error is printed instead.")
                       (write-string ", " stream)
                       (output-object (sb-c::debug-info-name dinfo) stream)))))))))
 
-#-(or x86 x86-64)
+#-(or x86 x86-64 arm64)
 (defmethod print-object ((lra lra) stream)
   (print-unreadable-object (lra stream :identity t)
     (write-string "return PC object" stream)))
@@ -1822,6 +1836,9 @@ variable: an unreadable object representing the error is printed instead.")
     ;; as opposed to any other unbound marker.
     (print-unreadable-object (object stream) (write-string "unbound" stream))
     (return-from print-object))
+  (when (eql (get-lisp-obj-address object) sb-vm:no-tls-value-marker-widetag)
+    (print-unreadable-object (object stream) (write-string "novalue" stream))
+    (return-from print-object))
   (print-unreadable-object (object stream :identity t)
     (let ((lowtag (lowtag-of object)))
       (case lowtag
