@@ -188,64 +188,8 @@
                (ash header-word (- hash-slot-present-flag))
                1))))
 
-;;; * If symbols are 7 words incl header, as they are on 32-bit w/threads,
-;;;   then the part of NIL that manifests as symbol slots consumes 6 words
-;;;   (less the header) because NIL's symbol widetag precedes it by 1 word.
-;;;   So, there are 6 post-header words and 2 pre-header: one containing
-;;;   the widetag (not that it is ever read), and one 0 word.
-;;; * If symbols are 6 words incl header, as they are on 64-bit and
-;;;   32-bit w/o threads, then the symbol-like part of NIL is 5 words,
-;;;   which aligns up to 6, plus the two pre-header words.
-;;; So either way, NIL's alignment padding makes it come out to 8 words in total.
-(defconstant sizeof-nil-in-words (+ 2 (sb-int:align-up (1- sb-vm:symbol-size) 2)))
-
-;;; It's unclear to me whether reinventing sizetab[] in lisp is strictly
-;;; an improvement over doing the obvious:
-;;;  (WITH-PINNED-OBJECTS (object) (alien-funcall "ext_lispboj_size" ...))
-;;; It certainly doesn't feel that it's better, but it has significantly less overhead
-;;; at least on cheneygc if not also on the precise gencgc platforms.
-;;; But see WITH-PINNED-OBJECT-ITERATOR in 'target-hash-table' which offers
-;;; a way to mitigate the need to bind/unbind a special var when doing a massive
-;;; numbers of WITH-PINNED-OBJECT operations.
-(defun primitive-object-size (object)
-  "Return number of bytes of heap or stack directly consumed by OBJECT"
-  (unless (is-lisp-pointer (get-lisp-obj-address object))
-    (return-from primitive-object-size 0))
-  (let ((words
-         ;; This should pick off the most frequently-occurring things first.
-         ;; CONS and INSTANCE of course top the list.
-         (typecase object
-           (cons 2)
-           (instance (1+ (instance-length object)))
-           (function
-            (when (= (fun-subtype object) simple-fun-widetag)
-              (return-from primitive-object-size
-                (code-object-size (fun-code-header (truly-the simple-fun object)))))
-            (1+ (get-closure-length object)))
-           (code-component
-            (return-from primitive-object-size (code-object-size object)))
-           ;; Most everything else is an OTHER pointer, except for NIL.
-           ;; Arrays (especially strings and simple-vector) and symbols tend to be the
-           ;; next-most-common heap object, for which we need the ROOM-INFO.
-           (t
-            (let ((room-info (aref *room-info* (%other-pointer-widetag object))))
-              (cond ((typep room-info 'specialized-array-element-type-properties)
-                     (let ((n-data-octets (vector-n-data-octets object room-info)))
-                       (+ (ceiling n-data-octets n-word-bytes) ; N data words
-                          vector-data-offset)))
-                    (t
-                     (typecase object
-                       (fdefn 4) ; constant length not stored in the header
-                       ((satisfies array-header-p)
-                        (+ array-dimensions-offset (array-rank object)))
-                       ((simple-array nil (*)) 2) ; no payload
-                       (null sizeof-nil-in-words)
-                       (t
-                        ;; GET-HEADER-DATA works for everything that's left
-                        (1+ (logand (get-header-data object)
-                                    (room-info-mask room-info))))))))))))
-        (* (align-up words 2) n-word-bytes)))
-
+;;; Macros not needed after this file (and avoids a redefinition warning this way)
+(eval-when (:compile-toplevel)
 (defmacro widetag@baseptr (sap)
   #+big-endian `(sap-ref-8 ,sap ,(1- n-word-bytes))
   #+little-endian `(sap-ref-8 ,sap 0))
@@ -254,7 +198,7 @@
   `(%make-lisp-obj
     (logior (sap-int ,sap)
             (logand (deref (extern-alien "widetag_lowtag" (array char 256)) ,widetag)
-                    lowtag-mask))))
+                    lowtag-mask)))))
 
 ;;; This uses the funny fixnum representation of ADDRESS. I'd like to change this
 ;;; to take a SAP but god forbid people are using it?
@@ -302,25 +246,6 @@
      #-sb-devel
      (aver (sap= start end)))))
 
-;;; Test the sizing function ASAP, because if it's broken, then MAP-ALLOCATED-OBJECTS
-;;; is too, and then creating the initial core will crash because of the various heap
-;;; traversals performed in SAVE-LISP-AND-DIE. Don't delay a crash.
-(dovector (saetp *specialized-array-element-type-properties*)
-  (let ((length 0) (et (saetp-specifier saetp)))
-    (loop (let* ((array (make-array length :element-type et))
-                 (size-from-lisp (primitive-object-size array))
-                 (size-from-c
-                  (with-pinned-objects (array)
-                    (alien-funcall (extern-alien "ext_lispobj_size" (function unsigned unsigned))
-                                   (logandc2 (get-lisp-obj-address array) lowtag-mask)))))
-            (unless (= size-from-lisp size-from-c)
-              (bug "size calculation mismatch on ~S" array))
-            ;; Stop after enough trials to hit all the edge case
-            (when (or (>= size-from-lisp (* 8 sb-vm:n-word-bytes))
-                      (and (eq et nil) (>= length 4))) ; always 2 words
-              (return))
-            (incf length)))))
-
 ;;; Access to the GENCGC page table for better precision in
 ;;; MAP-ALLOCATED-OBJECTS
 #+gencgc
@@ -562,14 +487,14 @@ We could try a few things to mitigate this:
   #+gencgc
   (let ((region-struct
           #+sb-thread (sap+ (sb-thread::current-thread-sap)
-                            (ash thread-alloc-region-slot word-shift))
+                            (ash thread-boxed-tlab-slot word-shift))
           ;; If no threads, the alloc_region struct is in static space.
           #-sb-thread (int-sap boxed-region)))
     ;; The 'start_addr' field is at word index 3 of the structure (see gencgc-alloc-region.h).
     (unless (eql (sap-ref-word region-struct (ash 3 word-shift)) 0)
       (alien-funcall (extern-alien "gc_close_region" (function void system-area-pointer int))
                      region-struct
-                     1))
+                     1)) ; ASSUMPTION: BOXED_PAGE_FLAG = 1
     nil))
 
 ;;;; MEMORY-USAGE
@@ -777,9 +702,9 @@ We could try a few things to mitigate this:
                  (eql type funcallable-instance-widetag))
          (incf total-objects)
          (let* ((layout (if (eql type funcallable-instance-widetag)
-                            (%fun-layout obj)
-                            (%instance-layout obj)))
-                (classoid (layout-classoid layout))
+                            (%fun-wrapper obj)
+                            (%instance-wrapper obj)))
+                (classoid (wrapper-classoid layout))
                 (found (ensure-gethash classoid totals (cons 0 0)))
                 (size size))
            (declare (fixnum size))
@@ -1049,12 +974,9 @@ We could try a few things to mitigate this:
          ;; These two are in fact generally the most frequently occurring type.
          ,.(make-case 'cons `(car ,obj) `(cdr ,obj))
          ,.(make-case* 'instance
-            `(let ((.l. (%instance-layout ,obj)))
-               ;; Though we've bound %INSTANCE-LAYOUT to a variable,
-               ;; pass the form %INSTANCE-LAYOUT to functoid
-               ;; in case it wants to examine the form
+            `(progn
                (,functoid (%instance-layout ,obj) ,@more)
-               (do-instance-tagged-slot (.i. ,obj :layout .l. :pad nil)
+               (do-instance-tagged-slot (.i. ,obj nil)
                  (,functoid (%instance-ref ,obj .i.) ,@more))))
          (function
           (typecase ,obj
@@ -1066,7 +988,7 @@ We could try a few things to mitigate this:
                   ;; if functoid is a macro that does nothing.
                   (,functoid .o. ,@more)))
             ,.(make-case* 'funcallable-instance
-               `(let ((.l. (%fun-layout ,obj)))
+               `(progn
                   ;; As for INSTANCE, allow the functoid to see the access form
                   (,functoid (%fun-layout ,obj) ,@more)
                   (,functoid (%funcallable-instance-fun ,obj) ,@more)
@@ -1076,7 +998,7 @@ We could try a few things to mitigate this:
                   ;; both tricky and unnecessary to generalize iteration.
                   ;; So just hardcode the few cases that exist.
                   #+compact-instance-header
-                  (ecase (layout-bitmap .l.)
+                  (ecase (wrapper-bitmap (%fun-wrapper ,obj))
                     (-1 ; external trampoline, all slots are tagged
                      ;; In this case, the trampoline word is scanned, with no ill effect.
                      (loop for .i. from 0
@@ -1090,7 +1012,7 @@ We could try a few things to mitigate this:
                      (,functoid (%funcallable-instance-info ,obj 0) ,@more)))
                   #-compact-instance-header
                   (progn
-                    (aver (eql (layout-bitmap .l.) -4))
+                    (aver (eql (wrapper-bitmap (%fun-wrapper ,obj)) -4))
                     ;;            v ----trampoline
                     ;; = #b1...1100
                     ;;           ^----- layout
@@ -1117,7 +1039,7 @@ We could try a few things to mitigate this:
             ,.(make-case 'array)
             ,.(make-case* 'symbol
                `(,functoid (%primitive sb-c:fast-symbol-global-value ,obj) ,@more)
-               `(,functoid (symbol-info ,obj) ,@more)
+               `(,functoid (symbol-%info ,obj) ,@more)
                `(,functoid (symbol-name ,obj) ,@more)
                `(,functoid (symbol-package ,obj) ,@more)
                `(when (symbol-extra-slot-p ,obj)
@@ -1160,6 +1082,7 @@ We could try a few things to mitigate this:
 ;;; code-components are considered to reference their embedded
 ;;; simple-funs for this purpose; if THIS is a simple-fun, it is ignored.
 (defun references-p (this that)
+  (declare (optimize (sb-c::aref-trapping 0)))
   (macrolet ((test (x) `(when (eq ,x that) (go win))))
     (tagbody
        (do-referenced-object (this test)
@@ -1274,7 +1197,8 @@ We could try a few things to mitigate this:
                                 &aux (n-code-bytes 0)
                                      (total-pages next-free-page)
                                      (pages
-                                      (make-array total-pages :element-type 'bit)))
+                                      (make-array total-pages :element-type 'bit
+                                                  :initial-element 0)))
   (flet ((dump-page (page-num)
            (format stream "~&Page ~D~%" page-num)
            (let ((where (+ dynamic-space-start (* page-num gencgc-card-bytes)))
@@ -1334,17 +1258,18 @@ We could try a few things to mitigate this:
 
 #+gencgc
 (defun generation-of (object)
-  (let* ((addr (get-lisp-obj-address object))
-         (page (find-page-index addr)))
-    (cond ((>= page 0) (slot (deref page-table page) 'gen))
-          #+immobile-space
-          ((immobile-space-addr-p addr)
-           ;; SIMPLE-FUNs don't contain a generation byte
-           (when (simple-fun-p object)
-             (setq addr (get-lisp-obj-address (fun-code-header object))))
-           (let ((sap (int-sap (logandc2 addr lowtag-mask))))
-             (logand (if (fdefn-p object) (sap-ref-8 sap 1) (sap-ref-8 sap 3))
-                     #xF))))))
+  (with-pinned-objects (object)
+    (let* ((addr (get-lisp-obj-address object))
+           (page (find-page-index addr)))
+      (cond ((>= page 0) (slot (deref page-table page) 'gen))
+            #+immobile-space
+            ((immobile-space-addr-p addr)
+             ;; SIMPLE-FUNs don't contain a generation byte
+             (when (simple-fun-p object)
+               (setq addr (get-lisp-obj-address (fun-code-header object))))
+             (let ((sap (int-sap (logandc2 addr lowtag-mask))))
+               (logand (if (fdefn-p object) (sap-ref-8 sap 1) (sap-ref-8 sap 3))
+                       #xF)))))))
 
 ;;; Show objects in a much simpler way than print-allocated-objects.
 ;;; Probably don't use this for generation 0 of dynamic space. Other spaces are ok.
@@ -1362,9 +1287,12 @@ We could try a few things to mitigate this:
 ;;; Unfortunately this is a near total copy of the test in gc.impure.lisp
 (defun !ensure-genesis-code/data-separation ()
   #+gencgc
-  (let* ((n-bits (+ next-free-page 10))
-         (code-bits (make-array n-bits :element-type 'bit))
-         (data-bits (make-array n-bits :element-type 'bit))
+  (let* ((n-bits
+          (progn
+            (sb-vm::close-current-gc-region)
+            (+ next-free-page 50)))
+         (code-bits (make-array n-bits :element-type 'bit :initial-element 0))
+         (data-bits (make-array n-bits :element-type 'bit :initial-element 0))
          (total-code-size 0))
     (map-allocated-objects
      (lambda (obj type size)
@@ -1472,28 +1400,38 @@ We could try a few things to mitigate this:
           (+ start sb-vm:gencgc-card-bytes)))
     (map-objects-in-range #'print-it (%make-lisp-obj start) (%make-lisp-obj end)))))
 
-(defun code-from-serialno (serial)
-  (dx-flet ((visit (obj type size)
+(defun map-code-objects (fun)
+  (dx-flet ((filter (obj type size)
               (declare (ignore size))
-              #+gencgc (aver (= type sb-vm:code-header-widetag))
-              (when (= (%code-serialno obj) serial)
-                (return-from code-from-serialno obj))))
-    #+cheneygc (map-allocated-objects #'visit :all)
+              (when (= type sb-vm:code-header-widetag)
+                (funcall fun obj)))
+            (nofilter (obj type size)
+              (declare (ignore type size))
+              (funcall fun obj)))
+    #+cheneygc (map-allocated-objects #'filter :all)
     #+gencgc
-    (progn
+    (without-gcing
       #+immobile-code
-      (map-objects-in-range #'visit
+      (map-objects-in-range #'nofilter
                             (ash varyobj-space-start (- n-fixnum-tag-bits))
                             (%make-lisp-obj (sap-int *varyobj-space-free-pointer*)))
-      (walk-dynamic-space #'visit
+      (alien-funcall (extern-alien "close_code_region" (function void)))
+      (walk-dynamic-space #'nofilter
                           #b1111111 ; all generations
                           3 3))))
 
+(export 'code-from-serialno)
+(defun code-from-serialno (serial)
+  (dx-flet ((visit (obj)
+              (when (= (%code-serialno obj) serial)
+                (return-from code-from-serialno obj))))
+    (map-code-objects #'visit)))
+
 (defun show-all-layouts ()
-  (let ((l (sb-vm::list-allocated-objects :all :test #'sb-kernel::layout-p))
+  (let ((l (sb-vm::list-allocated-objects :all :test #'sb-kernel::wrapper-p))
         zero trailing-raw trailing-tagged vanilla)
     (dolist (x l)
-      (let ((m (sb-kernel::layout-bitmap x)))
+      (let ((m (wrapper-bitmap x)))
         (cond ((eql m +layout-all-tagged+) (push x vanilla))
               ((eql m 0) (push x zero))
               ((minusp m) (push x trailing-tagged))
@@ -1501,42 +1439,51 @@ We could try a few things to mitigate this:
     (flet ((legend (newline str list)
              (when newline (terpri))
              (let ((s (format nil str (length list))))
-               (format t "~A~%~A~%" s (make-string (length s) :initial-element #\-))))
-           (name (x) (classoid-name (layout-classoid x))))
+               (format t "~A~%~A~%" s (make-string (length s) :initial-element #\-)))))
       (when zero
         (legend nil "Zero bitmap (~d):" zero)
-        (dolist (x zero) (format t "~a~%" (name x))))
+        (dolist (x zero) (format t "~a~%" (wrapper-classoid-name x))))
       (when trailing-raw
         (legend t "Trailing raw (~d):" trailing-raw)
         (dolist (x trailing-raw)
-          (let ((m (sb-kernel::layout-bitmap x)))
+          (let ((m (wrapper-bitmap x)))
             (format t "~30a 0...~v,'0b~%"
-                    (name x)
-                    (acond ((layout-info x) (1+ (dd-length it))) (t 32))
+                    (wrapper-classoid-name x)
+                    (acond ((wrapper-info x) (1+ (dd-length it))) (t 32))
                     m))))
       (when trailing-tagged
         (legend t "Trailing tagged (~d):" trailing-tagged)
         (dolist (x trailing-tagged)
-          (let ((m (sb-kernel::layout-bitmap x)))
+          (let ((m (wrapper-bitmap x)))
             (format t "~30a 1...~b~%"
-                    (name x)
-                    (acond ((layout-info x) (ldb (byte (dd-length it) 0) m))
+                    (wrapper-classoid-name x)
+                    (acond ((wrapper-info x) (ldb (byte (dd-length it) 0) m))
                            (t (ldb (byte 32 0) m)))))))
       (legend t "Default: (~d) [not shown]" vanilla))))
 
+#+ubsan
+(defun find-poisoned-vectors (&aux result)
+  (dolist (v (sb-vm:list-allocated-objects :all :type sb-vm:simple-vector-widetag)
+             result)
+    (when (dotimes (i (length v))
+            (declare (optimize (sb-c::aref-trapping 0)))
+            (let ((val (svref v i)))
+              (when (= (get-lisp-obj-address val) no-tls-value-marker-widetag)
+                (return t))))
+      (push (make-weak-pointer v) result)
+      (let* ((origin (vector-extra-data v))
+             (code (sb-di::code-header-from-pc (ash origin -3)))
+             (*print-array* nil))
+        (format t "g~d ~a ~a~%" (sb-kernel:generation-of v) v code)))))
+
 (in-package "SB-C")
 ;;; As soon as practical in warm build it makes sense to add
-;;; cold-allocation-point-fixups into the weak hash-table.
+;;; cold-allocation-patch-points into the weak hash-table.
 ;;; FIXME: I suspect that this wants to be just a weak vector
 ;;; (all code objects that have any allocation profiling compiled in),
 ;;; and not a hash-table, and that the list of fixups in the component
 ;;; can be attached to the debug info (in the manner of debug funs).
 ;;; When this was first implemented, weak-vectors weren't a thing. Maybe?
-(defvar *!cold-allocation-point-fixups*)
-(let ((hash-table (make-hash-table :test 'eq)))
-  ;; Group by code component
-  (loop for (code . offset) across *!cold-allocation-point-fixups*
-        do (push offset (gethash code hash-table)))
-  (dohash ((code list) hash-table)
-    (setf (gethash code *allocation-point-fixups*)
-          (convert-alloc-point-fixups code list))))
+(defvar *!cold-allocation-patch-point*)
+(loop for (code . points) in *!cold-allocation-patch-point*
+      do (setf (gethash code *allocation-patch-points*) points))
