@@ -307,7 +307,8 @@ PROCESS."
   "Hand SIGNAL to PROCESS. If WHOM is :PID, use the kill Unix system call. If
    WHOM is :PROCESS-GROUP, use the killpg Unix system call. If WHOM is
    :PTY-PROCESS-GROUP deliver the signal to whichever process group is
-   currently in the foreground."
+   currently in the foreground.
+   Returns T if successful, otherwise returns NIL and error number (two values)."
   (let ((pid (ecase whom
                ((:pid :process-group)
                 (process-pid process))
@@ -543,7 +544,10 @@ status slot."
         ;; Copy string.
         (copy-ub8-to-system-area octets 0 string-sap 0 size)
         ;; NULL-terminate it
-        (system-area-ub8-fill 0 string-sap size 4)
+        ;; (As it says up top: "assume 4 byte is enough for everyone.")
+        (let ((sap (sap+ string-sap size)))
+          (setf (sap-ref-8 sap 0) 0 (sap-ref-8 sap 1) 0
+                (sap-ref-8 sap 2) 0 (sap-ref-8 sap 3) 0))
         ;; Put the pointer in the vector.
         (setf (sap-ref-sap vec-sap vec-index-offset) string-sap)
         ;; Advance string-sap for the next string.
@@ -1323,11 +1327,15 @@ Users Manual for details about the PROCESS structure.
   if not available."
   (or sb-sys::*software-version*
       (setf sb-sys::*software-version*
-            (string-trim '(#\newline)
-                         (%with-output-to-string (stream)
+            (possibly-base-stringize
+             #+linux
+             (with-open-file (f "/proc/sys/kernel/osrelease") (read-line f))
+             #-linux
+             (string-trim '(#\newline)
+                          (%with-output-to-string (stream)
                            (run-program "/bin/uname"
                                         ;; "-r" on haiku just prints "1"
                                         ;; but "-v" prints some detail.
                                         #+haiku '("-v")
                                         #-haiku '("-r")
-                                        :output stream))))))
+                                        :output stream)))))))
