@@ -80,16 +80,22 @@
        sb-thread::*session*
        sb-kernel::*gc-epoch*))
 
-(defun start-lisp (toplevel)
+(defun start-lisp (toplevel callable-exports)
   (named-lambda start-lisp ()
-    (handling-end-of-the-world
-     (reinit t)
-     (funcall toplevel))))
+    (cond (callable-exports
+           (reinit t)
+           (dolist (export callable-exports)
+             (sb-alien::initialize-alien-callable-symbol export)))
+          (t
+           (handling-end-of-the-world
+            (reinit t)
+            (funcall toplevel))))))
 
 (defun save-lisp-and-die (core-file-name &key
-                                         (toplevel #'toplevel-init)
+                                         (toplevel #'toplevel-init toplevel-supplied)
                                          (executable nil)
                                          (save-runtime-options nil)
+                                         (callable-exports ())
                                          (purify t)
                                          (root-structures ())
                                          (environment-name "auxiliary")
@@ -125,6 +131,13 @@ The following &KEY arguments are defined:
      all command line arguments to be passed to the toplevel.
      Meaningless if :EXECUTABLE is NIL.
 
+  :CALLABLE-EXPORTS
+     This should be a list of symbols to be initialized to the
+     appropriate alien callables on startup. All exported symbols should
+     be present as global symbols in the symbol table of the runtime
+     before the saved core is loaded. When this list is non-empty, the
+     :TOPLEVEL argument cannot be supplied.
+
   :PURIFY
      If true (the default on cheneygc), do a purifying GC which moves all
      dynamically allocated objects into static space. This takes
@@ -194,6 +207,8 @@ sufficiently motivated to do lengthy fixes."
   (declare (ignore environment-name))
   #+gencgc
   (declare (ignore purify) (ignorable root-structures))
+  (when (and callable-exports toplevel-supplied)
+    (error ":TOPLEVEL cannot be supplied when there are callable exports."))
   ;; If the toplevel function is not defined, this will signal an
   ;; error before saving, not at startup time.
   (let ((toplevel (%coerce-callable-to-fun toplevel))
@@ -217,7 +232,7 @@ sufficiently motivated to do lengthy fixes."
              (if value 1 0)))
       (let ((name (native-namestring (physicalize-pathname core-file-name)
                                      :as-file t))
-            (startfun (start-lisp toplevel)))
+            (startfun (start-lisp toplevel callable-exports)))
         (deinit)
         ;; FIXME: Would it be possible to unmix the PURIFY logic from this
         ;; function, and just do a GC :FULL T here? (Then if the user wanted
@@ -298,8 +313,7 @@ sufficiently motivated to do lengthy fixes."
 
 (defun deinit ()
   (call-hooks "save" *save-hooks*)
-  #+sb-wtimer
-  (itimer-emulation-deinit)
+  #+win32 (itimer-emulation-deinit)
   #+sb-thread
   (let (err)
     (with-system-mutex (sb-thread::*make-thread-lock*)
@@ -317,7 +331,9 @@ sufficiently motivated to do lengthy fixes."
             (make-condition 'save-with-multiple-threads-error
                             :interactive-threads interactive
                             :other-threads other)))))
-    (when err (error err)))
+    (when err (error err))
+    #+allocator-metrics (setq sb-thread::*allocator-metrics* nil)
+    (setq sb-thread::*sprof-data* nil))
   (tune-image-for-dump)
   (float-deinit)
   (profile-deinit)
@@ -366,6 +382,7 @@ sufficiently motivated to do lengthy fixes."
 ;;; Doing too much consing within MAP-ALLOCATED-OBJECTS can lead to heap
 ;;; exhaustion (due to inhibited GC), so this takes several passes.
 (defun coalesce-ctypes (&optional verbose)
+  (declare (optimize (sb-c::aref-trapping 0)))
   (let* ((table (make-hash-table :test 'equal))
          interned-ctypes
          referencing-objects)
@@ -445,6 +462,7 @@ sufficiently motivated to do lengthy fixes."
                                (%set-symbol-global-value obj (coalesce part)))))
                          ((not (memq accessor
                                      '(%closure-fun
+                                       %fun-layout %instance-layout
                                        symbol-package symbol-name fdefn-name
                                        %numerator %denominator
                                        %realpart %imagpart
@@ -524,8 +542,7 @@ sb-c::
                      (cond ((not foundp)
                             (setf (gethash name name-ht) name))
                            ((neq name new)
-                            (setf (%instance-ref debug-fun
-                                                 (get-dsd-index compiled-debug-fun name))
+                            (%instance-set debug-fun (get-dsd-index compiled-debug-fun name)
                                   new))))
                    while next))
             (sb-lockless::linked-list
@@ -563,7 +580,7 @@ sb-c::
                                        result))
                           (pushnew (code-from-fun fun) result)))))))
              (code-from-fun (fun)
-               (ecase (fun-subtype fun)
+               (ecase (%fun-pointer-widetag fun)
                  (#.simple-fun-widetag
                   (fun-code-header fun))
                  (#.funcallable-instance-widetag
