@@ -192,6 +192,8 @@
 
 ;;;; SERVE-ALL-EVENTS, SERVE-EVENT, and friends
 
+(declaim (start-block wait-until-fd-usable serve-event serve-all-events compute-pollfds))
+
 ;;; When a *periodic-polling-function* is defined the server will not
 ;;; block for more than the maximum event timeout and will call the
 ;;; polling function if it does time out.
@@ -257,7 +259,7 @@ waiting."
                                      (+ (* 1000 to-sec) (truncate to-usec 1000))
                                      -1)
                    when (or #+win32 (eq direction :output)
-                            #+win32 (sb-win32:handle-listen fd)
+                            #+win32 (sb-win32:handle-listen fd (or to-sec 0) (or to-usec 0))
                             #-win32
                             (sb-unix:unix-simple-poll fd direction to-msec))
                    do (return-from wait-until-fd-usable t)
