@@ -27,7 +27,7 @@
   (let ((word
          ;; threads imply gencgc. use the per-thread alloc region pointer
          #+sb-thread
-         (sap-int (sb-vm::current-thread-offset-sap sb-vm::thread-alloc-region-slot))
+         (sap-int (sb-vm::current-thread-offset-sap sb-vm::thread-boxed-tlab-slot))
          #+(and (not sb-thread) cheneygc)
          (sap-int (dynamic-space-free-pointer))
          ;; dynamic-space-free-pointer increments only when a page is full.
@@ -92,9 +92,12 @@
 (defun %instance-sxhash (instance)
   ;; to avoid consing in fmix
   (declare (inline #+64-bit murmur3-fmix64 #-64-bit murmur3-fmix32))
-  ;; FIXME: this special case might be removable, but there are callers
-  ;; of sxhash on layouts due to the expansion of TYPECASE.
-  (when (typep instance 'layout)
+  ;; LAYOUT must not acquire an extra slot for the stable hash,
+  ;; because the bitmap length is derived from the instance length.
+  ;; It would probably be simple to eliminate this as a special case
+  ;; by ensuring that instances of LAYOUT commence life with a trailing
+  ;; hash slot and the SB-VM:HASH-SLOT-PRESENT-FLAG set.
+  (when (typep instance 'sb-vm:layout)
     ;; This might be wrong if the clos-hash was clobbered to 0
     (return-from %instance-sxhash (layout-clos-hash instance)))
   ;; Non-simple cases: no hash slot, and either unhashed or hashed-not-moved.
@@ -196,27 +199,22 @@
            (mix-chunk (word)
              `(setq result (word-mix ,word result)))
            (mix-remaining (word)
-             ;; N-BITS-REMAINING is between 1 inclusive and N-WORD-BITS exclusive
-             (let ((mask
-                    #+little-endian ; if all except 1 bit remain, right-shift by 1, etc
-                    `(ash most-positive-word (- n-bits-remaining sb-vm:n-word-bits))
-                    #+big-endian ; same, except left-shift (modularly)
-                    `(logand most-positive-word
-                             (ash most-positive-word
-                                  (- sb-vm:n-word-bits n-bits-remaining)))))
-               `(setq result (word-mix (logand ,word ,mask) result)))))
+             ;; In the current implementation of bit operations, they may leave random
+             ;; bits in an ignored suffix of bits, hence the need for a masking operation.
+             ;; (See examples above DEF-BIT-ARRAY-OP)
+             ;; N-BITS-REMAINING is between 1 inclusive and N-WORD-BITS exclusive.
+             ;; Produce a mask of 1s spanning the remaining bits, which would be
+             ;; (- n-word-bits n-bits-remaining) and logically AND it with word.
+             ;; The mask is equal mod N-WORD-BITS to (- n-bits-remaining).
+             ;; SHIFT-TOWARDS-START clips the shift count explicitly if the CPU doesn't.
+             `(mix-chunk (logand (shift-towards-start most-positive-word
+                                                      (- n-bits-remaining))
+                                 ,word))))
 (defun %sxhash-simple-bit-vector (x)
   (with-hash (result (length (truly-the simple-bit-vector x)))
     (multiple-value-bind (n-full-words n-bits-remaining) (floor (length x) sb-vm:n-word-bits)
       (dotimes (i n-full-words) (mix-chunk (%vector-raw-bits x i)))
       (when (plusp n-bits-remaining)
-        ;; FIXME: Do we really have to mask off bits of the final word?
-        ;; I don't think so, given that remaining bits are invariantly zero.
-        ;; Maybe this has to do with stack-allocated vectors?
-        ;; Either that, or it anticipates a change wherein we do not always
-        ;; prezero unboxed vectors unless :INITIAL-ELEMENT was specified.
-        ;; (i.e. you'd get random bytes, and so the only known good bits/bytes
-        ;; would appear where you had performed a SETF on them)
         (mix-remaining (%vector-raw-bits x n-full-words))))
     (logand result sb-xc:most-positive-fixnum)))
 (defun %sxhash-bit-vector (bit-vector)
@@ -345,7 +343,7 @@
                 (logxor 72185131
                         (sxhash (char-code x)))) ; through DEFTRANSFORM
                (funcallable-instance
-                (if (layout-for-pcl-obj-p (%fun-layout x))
+                (if (logtest (layout-flags (%fun-layout x)) +pcl-object-layout-flag+)
                     ;; We have a hash code, so might as well use it.
                     (fsc-instance-hash x)
                     ;; funcallable structure, not funcallable-standard-object
@@ -355,62 +353,72 @@
 
 ;;;; the PSXHASH function
 
-;;; To avoid "note: Return type not fixed values ..."
-(declaim (ftype (sfunction (number) hash-code) number-psxhash))
-
 ;;; like SXHASH, but for EQUALP hashing instead of EQUAL hashing
+(macrolet ((hash-float (type key)
+             ;; Floats that represent integers must hash as the integer would.
+             (let ((lo (coerce most-negative-fixnum type))
+                   (hi (coerce most-positive-fixnum type)))
+               `(let ((key ,key))
+                  (cond ( ;; This clause allows FIXNUM-sized integer
+                         ;; values to be handled without consing.
+                         (<= ,lo key ,hi)
+                         (multiple-value-bind (q r) (floor (the (,type ,lo ,hi) key))
+                           (if (zerop (the ,type r))
+                               (sxhash q)
+                               (sxhash (coerce key 'double-float)))))
+                        ((float-infinity-p key)
+                         ;; {single,double}-float infinities are EQUALP
+                         (if (minusp key)
+                             (sxhash sb-ext:single-float-negative-infinity)
+                             (sxhash sb-ext:single-float-positive-infinity)))
+                        (t
+                         (multiple-value-bind (q r) (floor key)
+                           (if (zerop (the ,type r))
+                               (sxhash q)
+                               (sxhash (coerce key 'double-float))))))))))
 (defun psxhash (key)
   (declare (optimize speed))
   (labels
-    ((array-psxhash (key depthoid)
-       (declare (type array key))
-       (declare (type (integer 0 #.+max-hash-depthoid+) depthoid))
-       (if (vectorp key)
-           ;; VECTORs have to be treated specially because ANSI specifies
-           ;; that we must respect fill pointers.
-           (let ((result 572539))
-             (declare (type hash-code result))
-             (mixf result (length key))
-             (when (plusp depthoid)
-               (decf depthoid)
-               (macrolet ((traverse (element-hasher)
-                            `(dotimes (i (length key))
-                               (declare (type index i))
-                               (mixf result
-                                     ,(ecase element-hasher
-                                        (character `(char-code (char-upcase (aref key i))))
-                                        (integer `(sxhash (aref key i)))
-                                        (number `(number-psxhash (aref key i)))
-                                        (:default `(%psxhash (aref key i) depthoid)))))))
-                 (typecase key
-                   ;; There are two effects from the typecase:
-                   ;;  1. using a specialized array reffer
-                   ;;  2. dispatching to a specific hash function
-                   (simple-base-string (traverse character))           ; both effects
-                   ((simple-array character (*)) (traverse character)) ; ""
-                   (simple-vector (traverse :default))                 ; effect #1 only
-                   ((simple-array (unsigned-byte 8) (*)) (traverse integer)) ; both effects
-                   ;; this seems arbitrary. I think (simple-array word (*)) is more popular!
-                   ((simple-array fixnum (*)) (traverse integer))      ; both effects
-                   (string (traverse character))                       ; effect #2 only
-                   ((vector t) (traverse :default)) ; weed out non-simple T vector from final case
-                   (t (traverse number)))))         ; all else - effect #2 only
-             result)
-           ;; Any other array can be hashed by working with its underlying
-           ;; one-dimensional physical representation.
-           (let ((result 60828))
-             (declare (type fixnum result))
-             (dotimes (i (array-rank key))
-               (mixf result (%array-dimension key i)))
-             (when (plusp depthoid)
-               (decf depthoid)
-               (with-array-data ((key key) (start) (end))
-                 (let ((getter (truly-the function (svref %%data-vector-reffers%%
-                                                          (%other-pointer-widetag key)))))
-                   (loop for i from start below end
-                         do (mixf result (%psxhash (funcall getter key i) depthoid))))))
-             result)))
-     (structure-object-psxhash (key depthoid)
+      ((data-vector-hash (data start end depthoid)
+         (declare (optimize (sb-c:insert-array-bounds-checks 0)))
+         (let ((result 572539))
+           (declare (type hash-code result))
+           (when (plusp depthoid)
+             (decf depthoid)
+             (macrolet ((traverse (et &aux (elt '(aref data i)))
+                          `(let ((data (truly-the (simple-array ,et (*)) data)))
+                             (loop for i fixnum from (truly-the fixnum start)
+                                   below (truly-the fixnum end)
+                                   do (mixf result
+                                            ,(case et
+                                               ((t) `(%psxhash ,elt depthoid))
+                                               ((base-char character)
+                                                `(char-code (char-upcase ,elt)))
+                                               (single-float `(sfloat-psxhash ,elt))
+                                               (double-float `(dfloat-psxhash ,elt))
+                                               ;; the remaining types are integers and complex numbers.
+                                               ;; COMPLEX will cons here, as will word-sized
+                                               ;; integers. Nothing else should though.
+                                               (t `(sxhash ,elt)))))))) ; xformed
+               (typecase data
+                 ;; There are two effects of this typecase:
+                 ;;  1. using an optimized array reader
+                 ;;  2. dispatching to a type-specific hash function
+                 (simple-vector (traverse t)) ; effect #1 only
+                 (simple-base-string (traverse base-char)) ; both effects
+                 #+sb-unicode (simple-character-string (traverse character))  ; both
+                 ((simple-array single-float (*)) (traverse single-float)) ; and so on
+                 ((simple-array double-float (*)) (traverse double-float))
+                 ;; (SIMPLE-ARRAY WORD (*)) would be helpful to avoid consing,
+                 ;; but there is no SXHASH transform on word-sized integers.
+                 ;; It might be possible to do something involving WORD-MIX.
+                 ((simple-array fixnum (*)) (traverse fixnum))
+                 (t
+                  (let ((getter (svref %%data-vector-reffers%% (%other-pointer-widetag data))))
+                    (loop for i fixnum from (truly-the fixnum start) below (truly-the fixnum end)
+                          do (mixf result (number-psxhash (funcall getter data i)))))))))
+             result))
+       (structure-object-psxhash (key depthoid)
        ;; Compute a PSXHASH for KEY. Salient points:
        ;; * It's not enough to use the bitmap to figure out how to mix in raw slots.
        ;;   The floating-point types all need special treatment. And we want to avoid
@@ -455,15 +463,16 @@
                          (let ((cplx (%raw-instance-ref/complex-double key i)))
                            ,(mix-float '(realpart cplx) $0d0)
                            ,(mix-float '(imagpart cplx) $0d0)))))))
-         (let* ((layout (%instance-layout key))
-                (result (layout-clos-hash layout)))
+         (let* ((wrapper (%instance-wrapper key))
+                (result (wrapper-clos-hash wrapper)))
            (declare (type fixnum result))
            (when (plusp depthoid)
              (let ((max-iterations depthoid)
-                   (depthoid (1- depthoid)))
+                   (depthoid (1- depthoid))
+                   (dd (wrapper-dd wrapper)))
                (declare (index max-iterations))
-               (if (/= (layout-bitmap layout) +layout-all-tagged+)
-                   (let ((slots (dd-slots (layout-dd layout))))
+               (if (/= (sb-kernel::dd-bitmap dd) +layout-all-tagged+)
+                   (let ((slots (dd-slots dd)))
                      (loop (unless slots (return))
                            (let* ((slot (pop slots))
                                   (rsd-index+1 (rsd-index+1 slot))
@@ -484,97 +493,77 @@
                            (incf i)
                            (if (zerop (decf max-iterations)) (return)))))))
            result)))
-     (%psxhash (key depthoid)
-       (typecase key
-         (array (array-psxhash key depthoid))
-         (structure-object
-          (cond ((hash-table-p key)
-                 ;; This is a purposely not very strong hash so that it does not make any
-                 ;; distinctions that EQUALP does not make. Computing a hash of the k/v pair
-                 ;; vector would incorrectly take insertion order into account.
-                 (mix (mix 103924836 (hash-table-count key))
-                      (sxhash (hash-table-test key))))
-                ((pathnamep key) (pathname-sxhash key))
-                (t
-                 (structure-object-psxhash key depthoid))))
-         (list
-          (cond ((null key)
-                 (the fixnum 480929))
-                ((eql depthoid 0)
-                 (the fixnum 779578))
-                (t
-                 (let ((depthoid (1- (truly-the (integer 0 #.+max-hash-depthoid+)
-                                                depthoid))))
-                   (mix (%psxhash (car key) depthoid)
-                        (%psxhash (cdr key) depthoid))))))
-         (number (number-psxhash key))
-         (character (char-code (char-upcase key)))
-         (t (sxhash key)))))
+       (sfloat-psxhash (key)
+         (declare (single-float key))
+         (hash-float single-float key))
+       (dfloat-psxhash (key)
+         (declare (double-float key))
+         (hash-float double-float key))
+       (number-psxhash (key)
+         (declare (type number key)
+                  (muffle-conditions compiler-note))
+         (macrolet ((hash-complex (hasher)
+                      `(if (zerop (imagpart key))
+                           (,hasher (realpart key))
+                           ;; I'm not sure what the point of an additional mix step
+                           ;; with a constant was. Maybe trying to get it not to hash
+                           ;; like a ratio whose num/den are equal to the real and imag
+                           ;; parts of a complex number? That seems silly.
+                           ;; But sure, let's do something like it, but simpler.
+                           ;; (It might hash like a cons of these integers anyway)
+                           (logand (lognot (mix (,hasher (realpart key)) (,hasher (imagpart key))))
+                                   most-positive-fixnum))))
+           (etypecase key
+             (integer (sxhash key))
+             (single-float (sfloat-psxhash key))
+             (double-float (dfloat-psxhash key))
+             (rational (if (and (<= most-negative-double-float
+                                    key
+                                    most-positive-double-float)
+                                (= (coerce key 'double-float) key))
+                           (sxhash (coerce key 'double-float))
+                           ;; a rational for which '=' does not return T when compared
+                           ;; to itself cast as double-float need to have the same hash
+                           ;; as any float. That's why this case is legitimate.
+                           (sxhash key)))
+             ((complex double-float) (hash-complex dfloat-psxhash))
+             ((complex single-float) (hash-complex sfloat-psxhash))
+             ((complex rational)     (hash-complex number-psxhash)))))
+       (%psxhash (key depthoid)
+         (typecase key
+           (array
+            (if (vectorp key)
+                (with-array-data ((a key) (start) (end) :force-inline t :check-fill-pointer t)
+                  (mix (data-vector-hash a start end depthoid) (length key)))
+                (with-array-data ((a key) (start) (end) :force-inline t :array-header-p t)
+                  (let ((result (data-vector-hash a start end depthoid)))
+                    (dotimes (i (array-rank key) result)
+                      (mixf result (%array-dimension key i)))))))
+           (structure-object
+            (cond ((hash-table-p key)
+                   ;; This is a purposely not very strong hash so that it does not make any
+                   ;; distinctions that EQUALP does not make. Computing a hash of the k/v pair
+                   ;; vector would incorrectly take insertion order into account.
+                   (mix (mix 103924836 (hash-table-count key))
+                        (sxhash (hash-table-test key))))
+                  ((pathnamep key) (pathname-sxhash key))
+                  (t
+                   (structure-object-psxhash key depthoid))))
+           (list
+            (cond ((null key)
+                   (the fixnum 480929))
+                  ((eql depthoid 0)
+                   (the fixnum 779578))
+                  (t
+                   (let ((depthoid (1- (truly-the (integer 0 #.+max-hash-depthoid+)
+                                                  depthoid))))
+                     (mix (%psxhash (car key) depthoid)
+                          (%psxhash (cdr key) depthoid))))))
+           (number (number-psxhash key))
+           (character (char-code (char-upcase key)))
+           (t (sxhash key)))))
     (%psxhash key +max-hash-depthoid+)))
-
-(defun number-psxhash (key)
-  (declare (type number key)
-           (explicit-check)
-           (muffle-conditions compiler-note)
-           (optimize speed))
-  (flet ((sxhash-double-float (val)
-           (declare (type double-float val))
-           ;; FIXME: Check to make sure that the DEFTRANSFORM kicks in and the
-           ;; resulting code works without consing. (In Debian cmucl 2.4.17,
-           ;; it didn't.)
-           (sxhash val)))
-    (macrolet ((hash-float (type key)
-                 (let ((lo (coerce most-negative-fixnum type))
-                       (hi (coerce most-positive-fixnum type)))
-                   `(let ((key ,key))
-                      (cond ( ;; This clause allows FIXNUM-sized integer
-                             ;; values to be handled without consing.
-                             (<= ,lo key ,hi)
-                             (multiple-value-bind (q r)
-                                 (floor (the (,type ,lo ,hi) key))
-                               (if (zerop (the ,type r))
-                                   (sxhash q)
-                                   (sxhash-double-float
-                                    (coerce key 'double-float)))))
-                            ((float-infinity-p key)
-                             ;; {single,double}-float infinities are EQUALP
-                             (if (minusp key)
-                                 (sxhash sb-ext:single-float-negative-infinity)
-                                 (sxhash sb-ext:single-float-positive-infinity)))
-                            (t
-                             (multiple-value-bind (q r) (floor key)
-                               (if (zerop (the ,type r))
-                                   (sxhash q)
-                                   (sxhash-double-float
-                                    (coerce key 'double-float)))))))))
-               (hash-complex (&optional (hasher '(number-psxhash)))
-                 `(if (zerop (imagpart key))
-                      (,@hasher (realpart key))
-                      (let ((result 330231))
-                        (declare (type fixnum result))
-                        (mixf result (,@hasher (realpart key)))
-                        (mixf result (,@hasher (imagpart key)))
-                        result))))
-     (etypecase key
-       (integer (sxhash key))
-       (float (macrolet ()
-                (etypecase key
-                  (single-float (hash-float single-float key))
-                  (double-float (hash-float double-float key))
-                  #+long-float
-                  (long-float (error "LONG-FLOAT not currently supported")))))
-       (rational (if (and (<= most-negative-double-float
-                              key
-                              most-positive-double-float)
-                          (= (coerce key 'double-float) key))
-                     (sxhash-double-float (coerce key 'double-float))
-                     (sxhash key)))
-       ((complex double-float)
-        (hash-complex (hash-float double-float)))
-       ((complex single-float)
-        (hash-complex (hash-float single-float)))
-       ((complex rational)
-        (hash-complex))))))
+) ; end MACROLET
 
 ;;; Semantic equivalent of SXHASH, but better-behaved for function names.
 ;;; It performs more work by not cutting off as soon in the CDR direction.
