@@ -132,11 +132,11 @@ means to wait indefinitely.")
 ;;; cancelation points (system and C library calls) which check for cancelation,
 ;;; perform whatever cleanups were pushed by pthread_cleanup_push() and then stop.
 ;;;
-;;; P.S. To see that #+sb-thruption is no magic fix - suppose you have a stack
+;;; P.S. To see that #+sb-safepoint is no magic fix - suppose you have a stack
 ;;; with Lisp -> C -> -> Lisp where C acquired a resource and the currently
 ;;; top-of-stack Lisp function took a safepoint trap. It would be fine if all it
 ;;; wanted to do was GC, but is not safe in general.
-;;; Hence #+sb-thruption paints a dangerously attractive veneer over an unsafe
+;;; Hence #+sb-safepoint paints a dangerously attractive veneer over an unsafe
 ;;; concept, making it more subtly bad instead of very obviously bad.
 ;;;
 ;;; So this is a bad default. Bad bad bad. But it's backward-compatible.
@@ -157,7 +157,7 @@ means to wait indefinitely.")
                  (if *forcibly-terminate-threads-on-exit*
                      (sb-thread::%exit-other-threads)
                      (with-deadline (:seconds nil :override t)
-                       (sb-impl::finalizer-thread-stop)
+                       #+sb-thread (finalizer-thread-stop)
                        (sb-thread:grab-mutex sb-thread::*make-thread-lock*)))
                  (setf ok t))
             (os-exit code :abort (not ok)))))))
@@ -402,6 +402,13 @@ any non-negative real number."
           args)
   (exit :code 1))
 
+
+(defvar *runtime-options*
+  #("--noinform" "--core" "--help" "--version" "--dynamic-space-size"
+    "--control-stack-size" "--tls"
+    "--debug-environment" "--disable-ldb" "--lose-on-corruption"
+    "--end-runtime-options" "--merge-core-pages" "--no-merge-core-pages"))
+
 ;;; the default system top level function
 (defun toplevel-init ()
   (/show0 "entering TOPLEVEL-INIT")
@@ -497,6 +504,9 @@ any non-negative real number."
                    ((string= option "--end-toplevel-options")
                     (pop-option)
                     (return))
+                   ((find option *runtime-options* :test #'string=)
+                    (startup-error "C runtime option ~a in the middle of Lisp options."
+                                   option))
                    (t
                     ;; Anything we don't recognize as a toplevel
                     ;; option must be the start of user-level
@@ -549,9 +559,9 @@ any non-negative real number."
                                                  (pathname it))
                                              :if-does-not-exist nil)
                        (cond (stream
-                              (dx-flet ((thunk ()
-                                          (load-as-source stream :context kind)))
-                                (sb-fasl::call-with-load-bindings #'thunk stream)))
+                              (sb-fasl::call-with-load-bindings
+                               (lambda (stream kind) (load-as-source stream :context kind))
+                               stream kind stream))
                              (specified-pathname
                               (cerror "Ignore missing init file"
                                       "The specified ~A file ~A was not found."
