@@ -42,31 +42,22 @@
          (make-array 32 :element-type 'bit :initial-element 0)))
 
 #+sb-xc
-(macrolet ((def ()
-             (let ((classes (make-array 32 :initial-element nil))
-                   ;; A curiosity observed with CLISP: if GENERATOR-STATE is not bound
-                   ;; to a variable, but instead inserted with read-time-eval
-                   ;; as "#.(sb-xc:make-array ...)" then DUMP-VECTOR fails to find the
-                   ;; array in SB-COLD::*ARRAY-TO-SPECIALIZATION* even though it's there.
-                   ;; Somehow between the read-time-eval and macroexpansion of DEF,
-                   ;; CLISP seems to copy the array. It's as if READ did not return the
-                   ;; object produced by "#." and I can't imagine how that's legal,
-                   ;; let alone possible.
-                   (generator-state
-                     (sb-xc:make-array 1 :element-type '(and fixnum unsigned-byte)
-                                         :initial-element 0)))
+(macrolet ((def (name init-form)
              `(progn
-                (declaim (type (simple-array (and fixnum unsigned-byte) (1))
-                               *ctype-hash-state*)
-                         (type (simple-vector 32) *type-classes*))
-                ;; Strictly speaking these array are immutable, as with all literals
-                ;; appearing in code. We mutate them anyway. Avoiding that minor infraction
-                ;; would require FOPCOMPILE to emit the MAKE-ARRAY expression to be executed
-                ;; in genesis; which is more of a project than I want to tackle.
-                (!define-load-time-global *ctype-hash-state* ,generator-state)
-                (!define-load-time-global *type-classes* ,classes)
-                (!cold-init-forms (fill *type-classes* (make-type-class :name :bogus)))))))
-  (def))
+                (define-load-time-global ,name ,init-form)
+                (!cold-init-forms (setq ,name ,init-form)))))
+  (declaim (type (simple-array (and fixnum unsigned-byte) (1))
+                 *ctype-hash-state*)
+           (type (simple-vector 32) *type-classes*)
+           (type fixnum *type-cache-nonce*))
+  (def *ctype-hash-state* (make-array 1 :element-type '(and fixnum unsigned-byte)
+                                        :initial-element 0))
+  (def *type-classes* (make-array 32 :initial-element nil))
+  ;; This is for "observers" who want to know if type names have been added.
+  ;; Rather than registering listeners, they can detect changes by comparing
+  ;; their stored nonce to the current nonce. Additionally the observers
+  ;; can detect whether function definitions have occurred.
+  (def *type-cache-nonce* 0))
 
 #-sb-xc-host
 (define-compiler-macro type-class-or-lose (&whole form name)
@@ -265,6 +256,10 @@
   ;; But it's consistent for genesis to treat it always as a raw slot.
   (%bits (missing-arg) :type sb-vm:word :read-only t))
 
+(defmethod print-object ((ctype ctype) stream)
+  (print-unreadable-object (ctype stream :type t)
+    (prin1 (type-specifier ctype) stream)))
+
 ;;; take 27 low bits but exclude bits 20 and 21
 ;;; [Our MASK-FIELD can't be folded, and I didn't feel like fixing that.]
 (defconstant +type-hash-mask+
@@ -364,16 +359,30 @@
 
 (defmacro define-type-method ((class method &rest more-methods)
                                lambda-list &body body)
-  (let ((name (symbolicate class "-" method "-TYPE-METHOD"))
-        (arg-restriction
+  (let* ((name (symbolicate class "-" method "-TYPE-METHOD"))
+         (arg-type
           (case class
-           (classoid 'classoid)
-           (number 'numeric-type)
-           (function 'fun-type)
-           (alien 'alien-type-type)
-           ;; hairy could reparse the specifier into anything
-           (hairy (if (eq method :simple-subtypep) t 'hairy-type))
-           (t (symbolicate class "-TYPE")))))
+            (classoid 'classoid)
+            (number 'numeric-type)
+            (function 'fun-type)
+            (alien 'alien-type-type)
+            (t (symbolicate class "-TYPE"))))
+         (first (car lambda-list))
+         (second (cadr lambda-list))
+         ;; make-host-1 verifies that type methods are invoked correctly,
+         ;; but afterwards we assume that they are
+         (operator #+sb-xc-host 'the #-sb-xc-host 'truly-the)
+         (rebind
+          (unless more-methods
+            (case method
+              ((:complex-subtypep-arg1 :unparse :negate :singleton-p)
+               `((,first (,operator ,arg-type ,first))))
+              ((:complex-subtypep-arg2)
+               `((,first ,first) ; because there might be a DECLARE IGNORE on it
+                 (,second (,operator ,arg-type ,second))))
+              ((:simple-intersection2 :simple-union2 :simple-subtypep :simple-=)
+               `((,first (,operator ,arg-type ,first))
+                 (,second (,operator ,arg-type ,second))))))))
     `(progn
        #+sb-xc-host
        (when (plusp (bit *type-class-was-inherited* (type-class-name->id ',class)))
@@ -381,14 +390,7 @@
          ;; both an ancestor and its descendants on some method.
          ;; Too bad for you- this throws the baby out with the bathwater.
          (error "Can't define-type-method for class ~s: already inherited" ',class))
-       (defun ,name ,lambda-list
-         ,@(cond ((member method '(:unparse :negate :singleton-p))
-                  `((declare (type ,arg-restriction ,(car lambda-list)))))
-                 ((and (member method '(:simple-intersection2 :simple-union2
-                                        :simple-subtypep :simple-=))
-                       (not more-methods))
-                  `((declare (type ,arg-restriction ,(car lambda-list) ,(cadr lambda-list))))))
-         ,@body)
+       (defun ,name ,lambda-list ,@(if rebind `((let ,rebind ,@body)) body))
        (!cold-init-forms
         ,@(mapcar (lambda (method)
                     `(setf (,(type-class-fun-slot method)
@@ -422,16 +424,12 @@
                  (loop for name in type-class-fun-slots
                        append `(,(keywordicate name)
                                 (,(type-class-fun-slot name) parent))))))))
+    ;; Careful: type-classes are very complicated things to redefine.
+    ;; For the sake of parallelized make-host-1 we have to allow
+    ;; redefinition, but it has to be a no-op.
     #+sb-xc-host
-    `(progn
-       ;; Careful: type-classes are very complicated things to redefine.
-       ;; For the sake of parallelized make-host-1 we have to allow
-       ;; redefinition, but it has to be a no-op.
-       (unless (find ',name *type-classes* :key #'type-class-name)
-         (vector-push ,make-it *type-classes*))
-       ;; I have no idea what compiler bug could be worked around by adding a form here,
-       ;; but this certainly achieves something, somehow.
-       #+host-quirks-cmu (print (aref *type-classes* (1- (length *type-classes*)))))
+    `(unless (find ',name *type-classes* :key #'type-class-name)
+       (vector-push ,make-it *type-classes*))
 
     #+sb-xc
     (let ((type-class-index
@@ -532,16 +530,40 @@
         (funcall (the function method-fun) type1 type2)
         (values subtypep win))))
 
-;;; KLUDGE: This function is dangerous, as its overuse could easily
-;;; cause stack exhaustion through unbounded recursion.  We only use
-;;; it in one place; maybe it ought not to be a function at all?
+(defvar *invoked-complex-=-other-method* nil)
 (defun invoke-complex-=-other-method (type1 type2)
   (let* ((type-class (type-class type1))
          (method-fun (type-class-complex-= type-class)))
-    (if method-fun
-        (funcall (the function method-fun) type2 type1)
+    (if (and method-fun (not *invoked-complex-=-other-method*))
+        (let ((*invoked-complex-=-other-method* t))
+          (funcall (the function method-fun) type2 type1))
         (values nil t))))
 
+;; The following macros expand into either constructor calls,
+;; if building the cross-compiler, or forms which reference
+;; previously constructed objects, if running the cross-compiler.
+#+sb-xc-host
+(progn
+  (defmacro literal-ctype (constructor &optional specifier)
+    (declare (ignore specifier))
+    `(load-time-value ,constructor))
+
+  (defmacro literal-ctype-vector (var)
+    `(load-time-value ,var nil)))
+
+;; Omitting the specifier works only if the unparser method has been
+;; defined in time to use it, and you're sure that constructor's result
+;; can be unparsed - some unparsers may be confused if called on a
+;; non-canonical object, such as an instance of (CONS T T) that is
+;; not EQ to the interned instance.
+#-sb-xc-host
+(progn
+  (defmacro literal-ctype (constructor &optional (specifier nil specifier-p))
+    (if specifier-p (specifier-type specifier) (symbol-value constructor)))
+
+  (defmacro literal-ctype-vector (var)
+    (symbol-value var)))
+
 ;;;; miscellany
 
 ;;; Hash two things (types) down to a target fixnum. In CMU CL this was an EQ
@@ -567,8 +589,7 @@
         do (setq res (logxor (ash res -1) (type-hash-value type)))
         finally (return res)))
 
-;;; A few type representations need to be defined slightly earlier than
-;;; 'early-type' is compiled, so they're defined here.
+;;;; representations of types
 
 ;;; The NAMED-TYPE is used to represent *, T and NIL, the standard
 ;;; special cases, as well as other special cases needed to
@@ -584,6 +605,31 @@
                        (:copier nil))
   (name nil :type symbol :read-only t))
 
+;;; A HAIRY-TYPE represents anything too weird to be described
+;;; reasonably or to be useful, such as NOT, SATISFIES, unknown types,
+;;; and unreasonably complicated types involving AND. We just remember
+;;; the original type spec.
+;;; A possible improvement would be for HAIRY-TYPE to have a subtype
+;;; named SATISFIES-TYPE for the hairy types which are specifically
+;;; of the form (SATISFIES pred) so that we don't have to examine
+;;; the sexpr repeatedly to decide whether it takes that form.
+;;; And as a further improvement, we might want a table that maps
+;;; predicates to their exactly recognized type when possible.
+;;; We have such a table in fact - *BACKEND-PREDICATE-TYPES*
+;;; as a starting point. But something like PLUSP isn't in there.
+;;; On the other hand, either of these points may not be sources of
+;;; inefficiency, and the latter if implemented might have undesirable
+;;; user-visible ramifications, though it seems unlikely.
+(defstruct (hairy-type (:include ctype)
+                       (:constructor %make-hairy-type
+                           (specifier &aux (%bits (pack-ctype-bits hairy))))
+                       (:constructor !make-interned-hairy-type
+                           (specifier &aux (%bits (pack-interned-ctype-bits 'hairy))))
+                       (:copier nil))
+  ;; the Common Lisp type-specifier of the type we represent.
+  ;; For other than an unknown type, this must be a (SATISFIES f) expression.
+  (specifier nil :type t :read-only t))
+
 ;;; A MEMBER-TYPE represent a use of the MEMBER type specifier. We
 ;;; bother with this at this level because MEMBER types are fairly
 ;;; important and union and intersection are well defined.
@@ -671,6 +717,23 @@
                                             (enumerable types))
                               (:copier nil)))
 
+(defstruct (alien-type-type
+            (:include ctype (%bits (pack-ctype-bits alien)))
+            (:constructor %make-alien-type-type (alien-type))
+            (:copier nil))
+  (alien-type nil :type alien-type :read-only t))
+
+(defstruct (negation-type (:include ctype (%bits (pack-ctype-bits negation)))
+                          (:copier nil)
+                          (:constructor make-negation-type (type)))
+  (type (missing-arg) :type ctype :read-only t))
+
+;;; An UNKNOWN-TYPE is a type not known to the type system (not yet
+;;; defined). We make this distinction since we don't want to complain
+;;; about types that are hairy but defined.
+(defstruct (unknown-type (:include hairy-type (%bits (pack-ctype-bits hairy)))
+                         (:copier nil)))
+
 ;;; a list of all the float "formats" (i.e. internal representations;
 ;;; nothing to do with #'FORMAT), in order of decreasing precision
 (defglobal *float-formats*
@@ -728,34 +791,6 @@
   (car-type (missing-arg) :type ctype :read-only t)
   (cdr-type (missing-arg) :type ctype :read-only t))
 
-(in-package "SB-ALIEN")
-(def!struct (alien-type
-             (:copier nil)
-             (:constructor make-alien-type
-                           (&key class bits alignment
-                            &aux (alignment
-                                  (or alignment (guess-alignment bits))))))
-  (class 'root :type symbol :read-only t)
-  (bits nil :type (or null unsigned-byte))
-  (alignment nil :type (or null unsigned-byte)))
-(!set-load-form-method alien-type (:xc :target))
-
-(in-package "SB-KERNEL")
-(defstruct (alien-type-type
-            (:include ctype (%bits (pack-ctype-bits alien)))
-            (:constructor %make-alien-type-type (alien-type))
-            (:copier nil))
-  (alien-type nil :type alien-type :read-only t))
-
-;;; the description of a &KEY argument
-(defstruct (key-info #-sb-xc-host (:pure t)
-                     (:copier nil))
-  ;; the key (not necessarily a keyword in ANSI Common Lisp)
-  (name (missing-arg) :type symbol :read-only t)
-  ;; the type of the argument value
-  (type (missing-arg) :type ctype :read-only t))
-(declaim (freeze-type key-info))
-
 ;;; ARGS-TYPE objects are used both to represent VALUES types and
 ;;; to represent FUNCTION types.
 (defstruct (args-type (:include ctype)
@@ -773,6 +808,22 @@
   ;; true if other &KEY arguments are allowed
   (allowp nil :type boolean :read-only t))
 
+;;; the description of a &KEY argument
+(defstruct (key-info #-sb-xc-host (:pure t)
+                     (:copier nil))
+  ;; the key (not necessarily a keyword in ANSI Common Lisp)
+  (name (missing-arg) :type symbol :read-only t)
+  ;; the type of the argument value
+  (type (missing-arg) :type ctype :read-only t))
+(declaim (freeze-type key-info))
+
+(defstruct (values-type
+            (:include args-type (%bits (pack-ctype-bits values)))
+            (:constructor %make-values-type)
+            (:copier nil)))
+
+(declaim (freeze-type values-type))
+
 ;;; (SPECIFIER-TYPE 'FUNCTION) and its subtypes
 (defstruct (fun-type (:include args-type (%bits (pack-ctype-bits function)))
                      (:copier nil)
@@ -795,6 +846,38 @@
             (:constructor make-fun-designator-type
                 (required optional rest
                  keyp keywords allowp wild-args returns))))
+
+;;; The CONSTANT-TYPE structure represents a use of the CONSTANT-ARG
+;;; "type specifier", which is only meaningful in function argument
+;;; type specifiers used within the compiler. (It represents something
+;;; that the compiler knows to be a constant.)
+(defstruct (constant-type
+            (:include ctype (%bits (pack-ctype-bits constant)))
+            (:copier nil))
+  ;; The type which the argument must be a constant instance of for this type
+  ;; specifier to win.
+  (type (missing-arg) :type ctype :read-only t))
+
+
+;;; A SIMD-PACK-TYPE is used to represent a SIMD-PACK type.
+#+sb-simd-pack
+(defstruct (simd-pack-type
+            (:include ctype (%bits (pack-ctype-bits simd-pack)))
+            (:constructor %make-simd-pack-type (element-type))
+            (:copier nil))
+  (element-type (missing-arg)
+   :type (cons #||(member #.*simd-pack-element-types*) ||#)
+   :read-only t))
+
+#+sb-simd-pack-256
+(defstruct (simd-pack-256-type
+            (:include ctype (%bits (pack-ctype-bits simd-pack-256)))
+            (:constructor %make-simd-pack-256-type (element-type))
+            (:copier nil))
+  (element-type (missing-arg)
+   :type (cons #||(member #.*simd-pack-element-types*) ||#)
+   :read-only t))
+
 (declaim (ftype (sfunction (ctype ctype) (values t t)) csubtypep))
 ;;; Look for nice relationships for types that have nice relationships
 ;;; only when one is a hierarchical subtype of the other.
@@ -817,18 +900,6 @@
         ((csubtypep type2 type1) type1)
         (t nil)))
 
-;; KLUDGE: putting this here satisfies CMUCL for an inexplicable reason.
-;; It should suffice to put it anywhere before %MAKE-CHARACTER-SET-TYPE
-;; is actually called.
-;;
-;; all character-set types are enumerable, but it's not possible
-;; for one to be TYPE= to a MEMBER type because (MEMBER #\x)
-;; is not internally represented as a MEMBER type.
-;; So in case it wasn't clear already ENUMERABLE-P does not mean
-;;  "possibly a MEMBER type in the Lisp-theoretic sense",
-;; but means "could be implemented in SBCL as a MEMBER type".
-(define-type-class character-set :enumerable nil
-                    :might-contain-other-types nil)
 (!defun-from-collected-cold-init-forms !type-class-cold-init)
 
 ;;; CAUTION: unhygienic macro specifically designed to expand into body code
@@ -901,7 +972,7 @@
                          (values nil nil)))) ; can't decide
                 (test-character-type (type)
                   (when (characterp ,thing)
-                    (let ((code (sb-xc:char-code ,thing)))
+                    (let ((code (char-code ,thing)))
                       (dolist (pair (character-set-type-pairs type) nil)
                         (destructuring-bind (low . high) pair
                           (when (<= low code high)
@@ -909,7 +980,7 @@
            ;; It should always work to dispatch by class-id, but ALIEN-TYPE-TYPE
            ;; is a problem in the cross-compiler due to not having a type-class-id
            ;; when 'src/code/cross-type' is compiled. I briefly tried moving
-           ;; it later, but then class-init failed to compile.
+           ;; it later, but then type-init failed to compile.
            #+sb-xc-host
            (etypecase type ,@clauses)
            #-sb-xc-host
