@@ -67,6 +67,7 @@
                              "sb_select" ; int-syscall
                              "sb_getitimer" ; syscall*
                              "sb_setitimer" ; syscall*
+                             "sb_clock_gettime" ; alien-funcall
                              "sb_utimes") ; posix
                          :test #'string=)
                  (subseq x 3)
@@ -178,11 +179,13 @@ corresponds to NAME, or NIL if there is none."
   #+win32 (sb-win32:unixlike-open path flags :overlapped overlapped)
   #-win32
   (with-restarted-syscall (value errno)
-    (int-syscall ("open" c-string int int)
-                 path
-                 (logior #+largefile o_largefile
-                         flags)
-                 mode)))
+    (locally
+        (declare (optimize (sb-c::float-accuracy 0)))
+      (let ((result (alien-funcall (extern-alien "open" (function int c-string int &optional int))
+                                   path (logior flags) mode)))
+        (if (minusp result)
+            (values nil (get-errno))
+            (values result 0))))))
 
 ;;; UNIX-CLOSE accepts a file descriptor and attempts to close the file
 ;;; associated with it.
@@ -1049,7 +1052,8 @@ avoiding atexit(3) hooks, etc. Otherwise exit(2) is called."
   (defun clock-gettime (clockid)
     (declare (type (signed-byte 32) clockid))
     (with-alien ((ts (struct timespec)))
-      (alien-funcall (extern-alien "clock_gettime" (function int int (* (struct timespec))))
+      (alien-funcall (extern-alien #.(libc-name-for "sb_clock_gettime")
+                                   (function int int (* (struct timespec))))
                      clockid (addr ts))
       ;; 'seconds' is definitely a fixnum for 64-bit, because most-positive-fixnum
       ;; can express 1E11 years in seconds.
