@@ -83,6 +83,10 @@
   (process handle)
   (exit-code uint))
 
+(defun zero-alien (alien type)
+  `(alien-funcall (extern-alien "memset" (function void system-area-pointer int unsigned))
+                  (alien-sap ,alien) 0 (alien-size ,type :bytes)))
+
 (defun mswin-spawn (program argv stdin stdout stderr searchp envp directory window preserve-handles)
   (let ((std-handles (multiple-value-list (get-std-handles)))
         (inheritp nil))
@@ -95,9 +99,7 @@
               do (setf (inheritable-handle-p handle) t)))
       (with-alien ((process-information process-information)
                    (startup-info startup-info))
-        (sb-kernel:system-area-ub8-fill
-         0 (alien-sap startup-info)
-         0 (alien-size startup-info :bytes))
+        (zero-alien startup-info startup-info)
         (setf (slot startup-info 'cb) (alien-size startup-info :bytes)
               (slot startup-info 'stdin) (maybe-std-handle stdin)
               (slot startup-info 'stdout) (maybe-std-handle stdout)
@@ -270,11 +272,6 @@ true to stop searching)." *console-control-spec*)
 
 (defconstant +copier-buffer+ 256)
 
-(defmacro zero-alien (alien type)
-  `(sb-kernel:system-area-ub8-fill
-    0 (alien-sap ,alien)
-    0 (alien-size ,type :bytes)))
-
 (defun setup-copiers (copiers)
   (let ((result (make-array (length copiers))))
     (loop for copier in copiers
