@@ -19,14 +19,19 @@
   (:use "CL"))
 (in-package "SB-COLD")
 
+(setq *features* (union *features* sb-impl:+internal-features+))
+
 (defstruct package-data name doc shadow export reexport import-from use)
 (export 'package-data)
-(dolist (data (with-open-file (f "package-data-list.lisp-expr") (read f)))
+(dolist (data (with-open-file (f (merge-pathnames "package-data-list.lisp-expr" *load-pathname*))
+                (read f)))
   (labels ((flatten (tree)
              (mapcan (lambda (x) (if (listp x) (flatten x) (list x)))
                      tree)))
     (let ((*package* (find-package (package-data-name data))))
-      (export (mapcar 'intern (flatten (package-data-export data)))))))
+      (dolist (x (flatten (package-data-export data)))
+        (let ((symbol (intern x)))
+          (ignore-errors (export symbol)))))))
 
 (sb-ext:unlock-package "CL")
 (rename-package "COMMON-LISP" "COMMON-LISP"
@@ -37,26 +42,13 @@
     (when (sb-int:system-package-p (find-package name))
       (sb-ext:unlock-package package))))
 
-;;; Define this first to avoid a style-warning from 'shebang'
+;;; Restore target floating-point number syntax
 (defun read-target-float (stream char)
   (declare (ignore stream char))
   (values)) ; ignore the $ as if it weren't there
 (compile 'read-target-float)
-;;; We need the #! readtable modifications.
-(load (merge-pathnames "shebang.lisp" *load-pathname*))
-;;; ... applied to the default readtable
-(set-dispatch-macro-character #\# #\+ #'read-targ-feature-expr)
-(set-dispatch-macro-character #\# #\- #'read-targ-feature-expr)
 (set-macro-character #\$ #'read-target-float t)
 
-;;; Just in case we want to play with the initial value of
-;;; backend-subfeatures
-(setf sb-cold:*shebang-backend-subfeatures* sb-c:*backend-subfeatures*)
-
-;; Restore !DEFINE-LOAD-TIME-GLOBAL macro
-(setf (macro-function 'sb-int::!define-load-time-global)
-      (macro-function 'sb-ext:define-load-time-global))
-
 (unless (fboundp 'sb-int:!cold-init-forms)
   (defmacro sb-int:!cold-init-forms (&rest forms) `(progn ,@forms)))
 
@@ -67,3 +59,5 @@
   (setf (macro-function 'sb-int:/noshow) (macro-function 'sb-int:/show)
         (macro-function 'sb-int:/show0) (macro-function 'sb-int:/show)
         (macro-function 'sb-int:/noshow0) (macro-function 'sb-int:/show)))
+
+(load "SYS:src;compiler;vop-existsp.lisp")
