@@ -9,26 +9,26 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
-(define-vop (debug-cur-sp)
-  (:translate sb!di::current-sp)
+(define-vop ()
+  (:translate current-sp)
   (:policy :fast-safe)
   (:results (res :scs (sap-reg)))
   (:result-types system-area-pointer)
   (:generator 1
     (load-csp res)))
 
-(define-vop (debug-cur-fp)
-  (:translate sb!di::current-fp)
+(define-vop (current-fp-sap)
+  (:translate current-fp)
   (:policy :fast-safe)
   (:results (res :scs (sap-reg)))
   (:result-types system-area-pointer)
   (:generator 1
     (move res cfp-tn)))
 
-(define-vop (read-control-stack)
-  (:translate sb!kernel:stack-ref)
+(define-vop ()
+  (:translate stack-ref)
   (:policy :fast-safe)
   (:args (sap :scs (sap-reg))
          (offset :scs (any-reg)))
@@ -38,18 +38,15 @@
   (:generator 5
     (inst ldr result (@ sap offset))))
 
-(define-vop (write-control-stack)
-  (:translate sb!kernel:%set-stack-ref)
+(define-vop ()
+  (:translate %set-stack-ref)
   (:policy :fast-safe)
   (:args (sap :scs (sap-reg))
          (offset :scs (any-reg))
-         (value :scs (descriptor-reg) :target result))
+         (value :scs (descriptor-reg)))
   (:arg-types system-area-pointer positive-fixnum *)
-  (:results (result :scs (descriptor-reg)))
-  (:result-types *)
   (:generator 5
-    (inst str value (@ sap offset))
-    (move result value)))
+    (inst str value (@ sap offset))))
 
 (define-vop (code-from-mumble)
   (:policy :fast-safe)
@@ -59,6 +56,7 @@
   (:variant-vars lowtag)
   (:generator 5
     (loadw temp thing 0 lowtag)
+    ;; SLEAZY ASSUMPTION: THE WIDETAG HAS TWO HIGH ZERO BITS
     (inst mov temp (lsr temp (- n-widetag-bits word-shift)))
     (inst cmp temp 0)
     (unless (= lowtag other-pointer-lowtag)
@@ -67,11 +65,11 @@
     (inst mov :eq code null-tn)))
 
 (define-vop (code-from-lra code-from-mumble)
-  (:translate sb!di::lra-code-header)
+  (:translate sb-di::lra-code-header)
   (:variant other-pointer-lowtag))
 
 (define-vop (code-from-fun code-from-mumble)
-  (:translate sb!di::fun-code-header)
+  (:translate sb-di::fun-code-header)
   (:variant fun-pointer-lowtag))
 
 (define-vop (%make-lisp-obj)
@@ -85,8 +83,8 @@
 
 (define-vop (get-lisp-obj-address)
   (:policy :fast-safe)
-  (:translate sb!di::get-lisp-obj-address)
-  (:args (thing :scs (descriptor-reg) :target result))
+  (:translate sb-di::get-lisp-obj-address)
+  (:args (thing :scs (descriptor-reg any-reg) :target result))
   (:results (result :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 1
