@@ -14,7 +14,7 @@
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   ;; Imports from this package into SB-VM
-  (import '(conditional-opcode emit-word
+  (import '(conditional-opcode negate-condition emit-word
             composite-immediate-instruction encodable-immediate
             lsl lsr asr ror cpsr @) "SB-VM")
   ;; Imports from SB-VM into this package
@@ -40,7 +40,7 @@
     (:le . 13)
     (:al . 14))
   #'equal)
-(defconstant-eqx sb-vm::+condition-name-vec+
+(defconstant-eqx +condition-name-vec+
   #.(let ((vec (make-array 16 :initial-element nil)))
       (dolist (cond +conditions+ vec)
         (when (null (aref vec (cdr cond)))
@@ -49,6 +49,9 @@
 
 (defun conditional-opcode (condition)
   (cdr (assoc condition +conditions+ :test #'eq)))
+(defun negate-condition (name)
+  (let ((code (logxor 1 (conditional-opcode name))))
+    (aref +condition-name-vec+ code)))
 
 ;;;; disassembler field definitions
 
@@ -716,6 +719,31 @@
 (define-data-processing-instruction movs #x1b t nil)
 (define-data-processing-instruction mvn  #x1e t nil)
 (define-data-processing-instruction mvns #x1f t nil)
+
+(define-instruction-format (movw-format 32
+                            :default-printer '(:name :tab rd ", #" immediate))
+  (cond :field (byte 4 28) :type 'condition-code)
+  (opcode-8 :field (byte 8 20))
+  (immediate :fields (list (byte 4 16) (byte 12 0))
+             :prefilter (lambda (dstate high low)
+                          (declare (ignore dstate))
+                          (logior (ash high 12) low)))
+  (rd :field (byte 4 12) :type 'reg))
+
+(macrolet ((mov-imm-16 (segment rd imm half)
+             `(emit-dp-instruction ,segment 14 #b00 #b1
+                                   ,(ecase half
+                                      (:low  #b10000)
+                                      (:high #b10100))
+                                   (ldb (byte 4 12) ,imm)
+                                   (tn-offset ,rd)
+                                   (ldb (byte 12 0) ,imm))))
+(define-instruction movw (segment rd imm) ; move wide (zero-extend)
+  (:printer movw-format ((opcode-8 #b00110000)))
+  (:emitter (mov-imm-16 segment rd imm :low)))
+(define-instruction movt (segment rd imm) ; move top bits (and keep bottom)
+  (:printer movw-format ((opcode-8 #b00110100)))
+  (:emitter (mov-imm-16 segment rd imm :high))))
 
 ;;;; Exception-generating instructions
 
@@ -1760,3 +1788,22 @@
       (:absolute
        (setf (sap-ref-32 sap offset) value))))
   nil)
+
+(define-instruction store-coverage-mark (segment path-index temp)
+  (:emitter
+   ;; No backpatch is needed to compute the offset into the code header
+   ;; because COMPONENT-HEADER-LENGTH is known at this point.
+   (let* ((offset (+ (component-header-length)
+                     n-word-bytes ; skip over jump table word
+                     path-index
+                     (- other-pointer-lowtag)))
+          (addr
+           (@ sb-vm::code-tn
+              (etypecase offset
+                ((integer 0 4095) offset)
+                ((unsigned-byte 31)
+                 (inst* segment 'movw temp (logand offset #xffff))
+                 (when (ldb-test (byte 16 16) offset)
+                   (inst* segment 'movt temp (ldb (byte 16 16) offset)))
+                 temp)))))
+     (inst* segment 'strb sb-vm::null-tn addr))))
