@@ -103,7 +103,7 @@
 (defmacro lisp-jump (function)
   "Jump to the lisp function FUNCTION."
   `(inst add pc-tn ,function
-         (- (ash simple-fun-code-offset word-shift)
+         (- (ash simple-fun-insts-offset word-shift)
             fun-pointer-lowtag)))
 
 (defmacro lisp-return (return-pc return-style)
@@ -132,7 +132,7 @@
 
 ;;; Move a stack TN to a register and vice-versa.
 (defun load-stack-offset (reg stack stack-tn &optional (predicate :al))
-  (let ((offset (* (tn-offset stack-tn) n-word-bytes)))
+  (let ((offset (tn-byte-offset stack-tn)))
     (cond ((or (tn-p offset)
                (typep offset '(unsigned-byte 12)))
            (inst ldr predicate reg (@ stack offset)))
@@ -148,7 +148,7 @@
         (load-stack-offset reg cfp-tn stack ,predicate)))))
 
 (defun store-stack-offset (reg stack stack-tn &optional (predicate :al))
-  (let ((offset (* (tn-offset stack-tn) n-word-bytes)))
+  (let ((offset (tn-byte-offset stack-tn)))
     (cond ((or (typep offset '(unsigned-byte 12))
                (tn-p offset))
            (inst str predicate reg (@ stack offset)))
@@ -199,80 +199,71 @@
 ;;; surround a call to ALLOCATION anyway), and to indicate that the
 ;;; P-A FLAG-TN is also acceptable here.
 
-#!+gencgc
-(defun allocation-tramp (alloc-tn size back-label)
-  (let ((fixup (gen-label)))
-    (when (integerp size)
-      (load-immediate-word alloc-tn size))
-    (inst word (logior #xe92d0000
-                       (ash 1 (if (integerp size) (tn-offset alloc-tn) (tn-offset size)))
-                       (ash 1 (tn-offset lr-tn))))
-    (inst load-from-label alloc-tn alloc-tn fixup)
-    (inst blx alloc-tn)
-    (inst word (logior #xe8bd0000
-                       (ash 1 (tn-offset alloc-tn))
-                       (ash 1 (tn-offset lr-tn))))
-    (inst b back-label)
-    (emit-label fixup)
-    (inst word (make-fixup "alloc_tramp" :foreign))))
+#+gencgc
+(defun allocation-tramp (type alloc-tn size back-label)
+  (when (integerp size)
+    (load-immediate-word alloc-tn size))
+  ;; Using the native stack is OK - the register values have fixnum nature.
+  (inst word (logior #xe92d0000 ; PUSH {rN, lr}
+                     (ash 1 (if (integerp size) (tn-offset alloc-tn) (tn-offset size)))
+                     (ash 1 (tn-offset lr-tn))))
+  ;; select the C function index as per *LINKAGE-SPACE-PREDEFINED-ENTRIES*
+  (let ((index (if (eq type 'list) 1 0)))
+    (inst ldr alloc-tn (@ null-tn (- (linkage-table-entry-address index)
+                                     nil-value))))
+  (inst blx alloc-tn)
+  (inst word (logior #xe8bd0000 ; POP {rN, lr}
+                     (ash 1 (tn-offset alloc-tn))
+                     (ash 1 (tn-offset lr-tn))))
+  (inst b back-label))
 
-(defmacro allocation (result-tn size lowtag &key flag-tn
-                                                 stack-allocate-p)
+(defun allocation (type size lowtag result-tn &key flag-tn stack-allocate-p)
   ;; Normal allocation to the heap.
-  (once-only ((result-tn result-tn)
-              (size size)
-              (lowtag lowtag)
-              (flag-tn flag-tn)
-              (stack-allocate-p stack-allocate-p))
-    `(cond (,stack-allocate-p
-            (load-csp ,result-tn)
-            (inst tst ,result-tn lowtag-mask)
-            (inst add :ne ,result-tn ,result-tn n-word-bytes)
-            (if (integerp ,size)
-                (composite-immediate-instruction add ,flag-tn ,result-tn ,size)
-                (inst add ,flag-tn ,result-tn ,size))
-            (store-csp ,flag-tn)
-            ;; :ne is from TST above, this needs to be done after the
-            ;; stack pointer has been stored.
-            (storew null-tn ,result-tn -1 0 :ne)
-            (inst orr ,result-tn ,result-tn ,lowtag))
-           #!-gencgc
-           (t
-            (load-symbol-value ,flag-tn *allocation-pointer*)
-            (inst add ,result-tn ,flag-tn ,lowtag)
-            (if (integerp ,size)
-                (composite-immediate-instruction add ,flag-tn ,flag-tn ,size)
-                (inst add ,flag-tn ,flag-tn ,size))
-            (store-symbol-value ,flag-tn *allocation-pointer*))
-           #!+gencgc
-           (t
-            (let ((fixup (gen-label))
-                  (alloc (gen-label))
-                  (back-from-alloc (gen-label)))
-              (inst load-from-label ,flag-tn ,flag-tn FIXUP)
-              (loadw ,result-tn ,flag-tn)
-              (loadw ,flag-tn ,flag-tn 1)
-              (if (integerp ,size)
-                  (composite-immediate-instruction add ,result-tn ,result-tn ,size)
-                  (inst add ,result-tn ,result-tn ,size))
-              (inst cmp ,result-tn ,flag-tn)
-              (inst b :hi ALLOC)
-              (inst load-from-label ,flag-tn ,flag-tn FIXUP)
-              (storew ,result-tn ,flag-tn)
+  (cond (stack-allocate-p
+         (load-csp result-tn)
+         (inst tst result-tn lowtag-mask)
+         (inst add :ne result-tn result-tn n-word-bytes)
+         (if (integerp size)
+             (composite-immediate-instruction add flag-tn result-tn size)
+             (inst add flag-tn result-tn size))
+         (store-csp flag-tn)
+         ;; :ne is from TST above, this needs to be done after the
+         ;; stack pointer has been stored.
+         (storew null-tn result-tn -1 0 :ne)
+         (inst orr result-tn result-tn lowtag))
+        #-gencgc
+        (t
+         (load-symbol-value flag-tn *allocation-pointer*)
+         (inst add result-tn flag-tn lowtag)
+         (if (integerp size)
+             (composite-immediate-instruction add flag-tn flag-tn size)
+             (inst add flag-tn flag-tn size))
+         (store-symbol-value flag-tn *allocation-pointer*))
+        #+gencgc
+        (t
+         (let ((region-disp (- boxed-region nil-value))
+               (alloc (gen-label))
+               (back-from-alloc (gen-label)))
+           (inst ldr result-tn (@ null-tn region-disp)) ; free ptr
+           (inst ldr flag-tn (@ null-tn (+ region-disp n-word-bytes))) ; end ptr
+           (if (integerp size)
+               (composite-immediate-instruction add result-tn result-tn size)
+               (inst add result-tn result-tn size))
+           (inst cmp result-tn flag-tn)
+           (inst b :hi ALLOC)
+           (inst str result-tn (@ null-tn region-disp)) ; free ptr
 
-              (if (integerp ,size)
-                  (composite-immediate-instruction sub ,result-tn ,result-tn ,size)
-                  (inst sub ,result-tn ,result-tn ,size))
+           (if (integerp size)
+               (composite-immediate-instruction sub result-tn result-tn size)
+               (inst sub result-tn result-tn size))
 
-              (emit-label BACK-FROM-ALLOC)
-              (when ,lowtag
-                (inst orr ,result-tn ,result-tn ,lowtag))
+           (emit-label BACK-FROM-ALLOC)
+           (when lowtag
+             (inst orr result-tn result-tn lowtag))
 
-              (assemble (:elsewhere)
-                (emit-label ALLOC)
-                (allocation-tramp ,result-tn ,size BACK-FROM-ALLOC)
-                (emit-label FIXUP)
-                (inst word (make-fixup "gc_alloc_region" :foreign))))))))
+           (assemble (:elsewhere)
+             (emit-label ALLOC)
+             (allocation-tramp type result-tn size BACK-FROM-ALLOC))))))
 
 (defmacro with-fixed-allocation ((result-tn flag-tn type-code size
                                             &key (lowtag other-pointer-lowtag)
@@ -286,12 +277,11 @@
   (once-only ((result-tn result-tn) (flag-tn flag-tn)
               (type-code type-code) (size size) (lowtag lowtag))
     `(pseudo-atomic (,flag-tn)
-       (allocation ,result-tn (pad-data-block ,size) ,lowtag
+       (allocation nil (pad-data-block ,size) ,lowtag ,result-tn
                    :flag-tn ,flag-tn
                    :stack-allocate-p ,stack-allocate-p)
        (when ,type-code
-         (inst mov ,flag-tn (ash (1- ,size) n-widetag-bits))
-         (inst orr ,flag-tn ,flag-tn ,type-code)
+         (load-immediate-word ,flag-tn (compute-object-header ,size ,type-code))
          (storew ,flag-tn ,result-tn 0 ,lowtag))
        ,@body)))
 
@@ -366,15 +356,12 @@
      (:policy :fast-safe)
      (:args (object :scs (descriptor-reg))
             (index :scs (any-reg))
-            (value :scs ,scs :target result))
+            (value :scs ,scs))
      (:arg-types ,type tagged-num ,el-type)
      (:temporary (:scs (interior-reg)) lip)
-     (:results (result :scs ,scs))
-     (:result-types ,el-type)
      (:generator 2
        (inst add lip object index)
-       (storew value lip ,offset ,lowtag)
-       (move result value))))
+       (storew value lip ,offset ,lowtag))))
 
 (defmacro define-partial-reffer (name type size signed offset lowtag scs
                                  el-type &optional translate)
@@ -405,15 +392,12 @@
      (:policy :fast-safe)
      (:args (object :scs (descriptor-reg))
             (index :scs (unsigned-reg))
-            (value :scs ,scs :target result))
+            (value :scs ,scs))
      (:arg-types ,type positive-fixnum ,el-type)
      (:temporary (:scs (interior-reg)) lip)
-     (:results (result :scs ,scs))
-     (:result-types ,el-type)
      (:generator 5
        ,(if (eq size :byte)
             '(inst add lip object index)
             '(inst add lip object (lsl index 1)))
        (inst ,(ecase size (:byte 'strb) (:short 'strh))
-             value (@ lip (- (* ,offset n-word-bytes) ,lowtag)))
-       (move result value))))
+             value (@ lip (- (* ,offset n-word-bytes) ,lowtag))))))
