@@ -15,7 +15,7 @@
 ;;; Make a TN for the argument count passing location for a
 ;;; non-local entry.
 (defun make-nlx-entry-arg-start-location ()
-  (make-wired-tn *fixnum-primitive-type* immediate-arg-scn r8-offset))
+  (make-wired-tn *fixnum-primitive-type* immediate-arg-scn r9-offset))
 
 ;;; Save and restore dynamic environment.
 (define-vop (current-stack-pointer)
@@ -48,14 +48,13 @@
   (:info entry-label)
   (:results (block :scs (any-reg)))
   (:temporary (:scs (descriptor-reg)) temp)
-  (:temporary (:scs (interior-reg)) lip)
   (:vop-var vop)
   (:generator 22
     (inst add block cfp-tn (add-sub-immediate (tn-byte-offset tn)))
     (load-tl-symbol-value temp *current-unwind-protect-block*)
     (storew-pair temp unwind-block-uwp-slot cfp-tn unwind-block-cfp-slot block)
-    (inst compute-lra temp lip entry-label)
-    (storew-pair code-tn unwind-block-code-slot temp unwind-block-entry-pc-slot block)
+    (inst adr temp entry-label)
+    (storew temp block unwind-block-entry-pc-slot)
     #+sb-thread
     (loadw-pair
      temp (/ (info :variable :wired-tls '*binding-stack-pointer*) n-word-bytes)
@@ -67,9 +66,12 @@
       (load-tl-symbol-value tmp-tn *current-catch-block*))
     (storew-pair temp unwind-block-bsp-slot tmp-tn unwind-block-current-catch-slot block)
     (inst mov-sp temp nsp-tn)
-    (storew-pair (or (current-nfp-tn vop) zr-tn) unwind-block-nfp-slot
-                 temp unwind-block-nsp-slot
-                 block)))
+    (let ((nfp (current-nfp-tn vop)))
+      (if nfp
+          (storew-pair nfp unwind-block-nfp-slot
+                       temp unwind-block-nsp-slot
+                       block)
+          (storew temp block unwind-block-nsp-slot)))))
 
 ;;; Like Make-Unwind-Block, except that we also store in the specified tag, and
 ;;; link the block into the Current-Catch list.
@@ -79,14 +81,13 @@
   (:info entry-label)
   (:results (block :scs (any-reg)))
   (:temporary (:scs (descriptor-reg)) temp)
-  (:temporary (:scs (interior-reg)) lip)
   (:vop-var vop)
   (:generator 44
     (inst add block cfp-tn (add-sub-immediate (tn-byte-offset tn)))
     (load-tl-symbol-value temp *current-unwind-protect-block*)
     (storew-pair temp catch-block-uwp-slot cfp-tn catch-block-cfp-slot block)
-    (inst compute-lra temp lip entry-label)
-    (storew-pair code-tn catch-block-code-slot temp catch-block-entry-pc-slot block)
+    (inst adr temp entry-label)
+    (storew temp block catch-block-entry-pc-slot)
 
     #+sb-thread
     (loadw-pair
@@ -100,9 +101,12 @@
     (storew-pair tmp-tn catch-block-previous-catch-slot tag catch-block-tag-slot block)
     (storew temp block catch-block-bsp-slot)
     (inst mov-sp temp nsp-tn)
-    (storew-pair (or (current-nfp-tn vop) zr-tn) unwind-block-nfp-slot
-                 temp unwind-block-nsp-slot
-                 block)
+    (let ((nfp (current-nfp-tn vop)))
+      (if nfp
+          (storew-pair nfp unwind-block-nfp-slot
+                       temp unwind-block-nsp-slot
+                       block)
+          (storew temp block unwind-block-nsp-slot)))
     (store-tl-symbol-value block *current-catch-block*)))
 
 ;;; Just set the current unwind-protect to UWP.  This
@@ -136,7 +140,7 @@
 
 (define-vop (nlx-entry)
   (:args (sp) ; Note: we can't list an sc-restriction, 'cause any load vops
-              ; would be inserted before the LRA.
+              ; would be inserted before the label.
          (start)
          (count))
   (:results (values :more t :from :load))
@@ -145,7 +149,7 @@
   (:save-p :force-to-stack)
   (:vop-var vop)
   (:generator 30
-    (emit-return-pc label)
+    (emit-label label)
     (note-this-location vop :non-local-entry)
     (cond ((zerop nvals))
           ((= nvals 1)
@@ -176,6 +180,19 @@
                     (store-stack-tn tn move-temp))))))))
     (load-stack-tn csp-tn sp)))
 
+(define-vop (nlx-entry-single)
+  (:args (sp)
+         (value))
+  (:results (res :from :load))
+  (:info label)
+  (:save-p :force-to-stack)
+  (:vop-var vop)
+  (:generator 30
+    (emit-label label)
+    (note-this-location vop :non-local-entry)
+    (inst mov res value)
+    (load-stack-tn csp-tn sp)))
+
 (define-vop (nlx-entry-multiple)
   (:args (top :target result)
          (src)
@@ -191,7 +208,7 @@
   (:save-p :force-to-stack)
   (:vop-var vop)
   (:generator 30
-    (emit-return-pc label)
+    (emit-label label)
     (note-this-location vop :non-local-entry)
 
     ;; Setup results, and test for the zero value case.
@@ -223,24 +240,33 @@
 (define-vop (uwp-entry)
   (:info label)
   (:save-p :force-to-stack)
-  (:results (block) (start) (count))
-  (:ignore block start count)
   (:vop-var vop)
   (:generator 0
-    (emit-return-pc label)
+    (emit-label label)
     (note-this-location vop :non-local-entry)))
 
-;;; Doesn't handle NSP and is disabled.
+(define-vop (uwp-entry-block)
+  (:info label)
+  (:save-p :force-to-stack)
+  (:results (block))
+  (:vop-var vop)
+  (:generator 0
+    (emit-label label)
+    (note-this-location vop :non-local-entry)
+    ;; Get the block saved in UNWIND
+    (inst ldr block (@ csp-tn (* -4 n-word-bytes)))))
+
 #+unwind-to-frame-and-call-vop
 (define-vop (unwind-to-frame-and-call)
   (:args (ofp :scs (descriptor-reg))
          (uwp :scs (descriptor-reg))
          (function :scs (descriptor-reg) :to :load :target saved-function)
          (bsp :scs (any-reg descriptor-reg))
+         (nsp :scs (any-reg descriptor-reg))
          (catch-block :scs (any-reg descriptor-reg)))
-  (:arg-types system-area-pointer system-area-pointer t t t)
+  (:arg-types system-area-pointer system-area-pointer t t t t)
   (:temporary (:sc unsigned-reg) temp)
-  (:temporary (:sc descriptor-reg :offset r8-offset) saved-function)
+  (:temporary (:sc descriptor-reg :offset r9-offset) saved-function)
   (:temporary (:sc unsigned-reg :offset r0-offset) block)
   (:temporary (:sc descriptor-reg :offset lexenv-offset) lexenv)
   (:temporary (:scs (interior-reg)) lip)
@@ -250,7 +276,7 @@
     (let ((entry-label (gen-label)))
       ;; Store the function into a non-stack location, since we'll be
       ;; unwinding the stack and destroying register contents before we
-      ;; use it.  It turns out that R8 is preserved as part of the
+      ;; use it.  It turns out that R9 is preserved as part of the
       ;; normal multiple-value handling of an unwind, so use that.
       (move saved-function function)
 
@@ -265,22 +291,21 @@
       (loadw temp ofp sap-pointer-slot other-pointer-lowtag)
       (storew temp block unwind-block-cfp-slot)
       (storew bsp block unwind-block-bsp-slot)
+      (storew nsp block unwind-block-nsp-slot)
       (storew catch-block block unwind-block-current-catch-slot)
-      ;; Don't need to save code at unwind-block-code-slot since
-      ;; it's not going to be used and will be overwritten after the
-      ;; function call
 
-      (inst compute-lra temp lip entry-label)
+      (inst adr temp entry-label)
       (storew temp block catch-block-entry-pc-slot)
 
       ;; Run any required UWPs.
       (load-inline-constant tmp-tn '(:fixup unwind :assembly-routine) lip)
       (inst br tmp-tn)
 
-      (emit-return-pc ENTRY-LABEL)
+      (emit-label ENTRY-LABEL)
       (inst mov nargs 0)
 
       (move lexenv saved-function)
 
       (loadw saved-function lexenv closure-fun-slot fun-pointer-lowtag)
       (lisp-jump saved-function lip))))
+
