@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 
 ;;;; Moves and coercions:
@@ -34,8 +34,9 @@
   (:results (res :scs (descriptor-reg)))
   (:note "SAP to pointer coercion")
   (:generator 20
-    (with-fixed-allocation (res pa-flag sap-widetag sap-size :lip lip)
-      (storew sap res sap-pointer-slot other-pointer-lowtag))))
+    (with-fixed-allocation (res pa-flag sap-widetag sap-size :lip lip
+                            :store-type-code nil)
+      (storew-pair pa-flag 0 sap sap-pointer-slot tmp-tn))))
 
 (define-move-vop move-from-sap :move
   (sap-reg) (descriptor-reg))
@@ -157,39 +158,65 @@
                   (:results (result :scs (,sc)))
                   (:result-types ,type)
                   (:generator 5
-                              (inst ,(case size
-                                       (:byte (if signed 'ldrsb 'ldrb))
-                                       (:short (if signed 'ldrsh 'ldrh))
-                                       (:word (if signed 'ldrsw 'ldr))
-                                       (t 'ldr))
-                                    ,(if (eq size :word)
-                                         '(32-bit-reg result)
-                                         'result)
-                                    (@ sap offset))))
+                    (inst ,(case size
+                             (:byte (if signed 'ldrsb 'ldrb))
+                             (:short (if signed 'ldrsh 'ldrh))
+                             (:word (if signed 'ldrsw 'ldr))
+                             (t 'ldr))
+                      ,(if (eq size :word)
+                           '(32-bit-reg result)
+                           'result)
+                      (@ sap offset))))
                 (define-vop (,set-name)
                   (:translate ,set-name)
                   (:policy :fast-safe)
-                  (:args (sap :scs (sap-reg))
-                         (offset :scs (signed-reg))
-                         (value :scs (,sc) :target result))
-                  (:arg-types system-area-pointer signed-num ,type)
-                  (:results (result :scs (,sc)))
-                  (:result-types ,type)
+                  (:args (value :scs (,sc zero))
+                         (sap :scs (sap-reg))
+                         (offset :scs (signed-reg)))
+                  (:arg-types ,type system-area-pointer signed-num)
                   (:generator 5
-                              (inst ,(case size
-                                       (:byte 'strb)
-                                       (:short 'strh)
-                                       (t 'str))
-                                    ,(if (eq size :word)
-                                         '(32-bit-reg value)
-                                         'value)
-                                    (@ sap offset))
-                              (unless (location= result value)
-                                ,@(case size
-                                    ((:single :double)
-                                     '((inst fmov result value)))
-                                    (t
-                                     '((inst mov result value))))))))))
+                    (inst ,(case size
+                             (:byte 'strb)
+                             (:short 'strh)
+                             (t 'str))
+                      ,(if (eq size :word)
+                           '(32-bit-reg value)
+                           'value)
+                      (@ sap offset))))
+                (define-vop (,(symbolicate ref-name "-C"))
+                  (:translate ,ref-name)
+                  (:policy :fast-safe)
+                  (:args (sap :scs (sap-reg)))
+                  (:info offset)
+                  (:arg-types system-area-pointer (:constant (satisfies ldr-str-offset-encodable)))
+                  (:RESULTS (result :scs (,sc)))
+                  (:result-types ,type)
+                  (:generator 4
+                    (inst ,(case size
+                             (:byte (if signed 'ldrsb 'ldrb))
+                             (:short (if signed 'ldrsh 'ldrh))
+                             (:word (if signed 'ldrsw 'ldr))
+                             (t 'ldr))
+                      ,(if (eq size :word)
+                           '(32-bit-reg result)
+                           'result)
+                      (@ sap offset))))
+                (define-vop (,(symbolicate set-name "-C"))
+                  (:translate ,set-name)
+                  (:policy :fast-safe)
+                  (:args (value :scs (,sc zero))
+                         (sap :scs (sap-reg)))
+                  (:info offset)
+                  (:arg-types ,type system-area-pointer (:constant (satisfies ldr-str-offset-encodable)))
+                  (:generator 4
+                    (inst ,(case size
+                             (:byte 'strb)
+                             (:short 'strh)
+                             (t 'str))
+                      ,(if (eq size :word)
+                           '(32-bit-reg value)
+                           'value)
+                      (@ sap offset)))))))
   (def-system-ref-and-set sap-ref-8 %set-sap-ref-8
     unsigned-reg positive-fixnum :byte :signed nil)
   (def-system-ref-and-set signed-sap-ref-8 %set-signed-sap-ref-8
