@@ -70,10 +70,41 @@
             (load-store-offset
              (- (+ #+little-endian 4
                    (ash (+ instance-slots-offset
-                           (get-dsd-index layout sb-kernel::%bits))
+                           (get-dsd-index layout sb-kernel::flags))
                         word-shift))
                 instance-pointer-lowtag))))))
 
+;;; This could be split into two vops to avoid wasting an allocation for 'temp'
+;;; when the immediate form is used.
+(define-vop ()
+  (:translate sb-c::%structure-is-a)
+  (:args (x :scs (descriptor-reg)))
+  (:arg-types * (:constant t))
+  (:policy :fast-safe)
+  (:conditional :eq)
+  (:info test-layout)
+  (:temporary (:sc unsigned-reg) this-id temp)
+  (:generator 4
+    (let ((test-id (layout-id test-layout))
+          (offset (+ (id-bits-offset)
+                     (ash (- (wrapper-depthoid test-layout) 2) 2)
+                     (- instance-pointer-lowtag))))
+      (declare (ignorable test-id))
+      (inst ldr (32-bit-reg this-id) (@ x offset))
+      ;; 8-bit IDs are permanently assigned, so no fixup ever needed for those.
+      (cond ((typep test-id '(and (signed-byte 8) (not (eql 0))))
+             (if (minusp test-id)
+                 (inst cmn (32-bit-reg this-id) (- test-id))
+                 (inst cmp (32-bit-reg this-id) test-id)))
+            (t
+             (destructuring-bind (size . label)
+                 ;; This uses the bogus definition of :dword, the one which
+                 ;; emits 4 bytes. _technically_ dword should be 8 bytes.
+                 (register-inline-constant :dword `(:layout-id ,test-layout))
+               (declare (ignore size))
+               (inst load-from-label (32-bit-reg temp) label))
+             (inst cmp (32-bit-reg this-id) (32-bit-reg temp)))))))
+
 (define-vop (%other-pointer-widetag)
   (:translate %other-pointer-widetag)
   (:policy :fast-safe)
@@ -83,8 +114,8 @@
   (:generator 6
     (load-type result object (- other-pointer-lowtag))))
 
-(define-vop (fun-subtype)
-  (:translate fun-subtype)
+(define-vop ()
+  (:translate %fun-pointer-widetag)
   (:policy :fast-safe)
   (:args (function :scs (descriptor-reg)))
   (:results (result :scs (unsigned-reg)))
@@ -92,16 +123,6 @@
   (:generator 6
     (load-type result function (- fun-pointer-lowtag))))
 
-(define-vop (fun-header-data)
-  (:translate fun-header-data)
-  (:policy :fast-safe)
-  (:args (x :scs (descriptor-reg)))
-  (:results (res :scs (unsigned-reg)))
-  (:result-types positive-fixnum)
-  (:generator 6
-    (loadw res x 0 fun-pointer-lowtag)
-    (inst lsr res res n-widetag-bits)))
-
 (define-vop (get-header-data)
   (:translate get-header-data)
   (:policy :fast-safe)
@@ -115,10 +136,9 @@
 (define-vop (set-header-data)
   (:translate set-header-data)
   (:policy :fast-safe)
-  (:args (x :scs (descriptor-reg) :target res)
+  (:args (x :scs (descriptor-reg))
          (data :scs (any-reg immediate)))
   (:arg-types * positive-fixnum)
-  (:results (res :scs (descriptor-reg)))
   (:temporary (:scs (non-descriptor-reg)) t1)
   (:generator 6
     (load-type t1 x (- other-pointer-lowtag))
@@ -127,8 +147,7 @@
        (inst orr t1 t1 (lsl data (- n-widetag-bits n-fixnum-tag-bits))))
       (immediate
        (inst orr t1 t1 (logical-mask (ash (tn-value data) n-widetag-bits)))))
-    (storew t1 x 0 other-pointer-lowtag)
-    (move res x)))
+    (storew t1 x 0 other-pointer-lowtag)))
 
 (define-vop (pointer-hash)
   (:translate pointer-hash)
@@ -205,33 +224,25 @@
     (inst sub ndescr ndescr (- other-pointer-lowtag fun-pointer-lowtag))
     (inst add func code ndescr)))
 ;;;
-(define-vop (symbol-info-vector)
+
+(defun load-symbol-dbinfo (result symbol temp)
+  (assemble ()
+    (loadw result symbol symbol-info-slot other-pointer-lowtag)
+    ;; If RESULT has list-pointer-lowtag, take its CDR. If not, use it as-is.
+    (inst and temp result lowtag-mask)
+    (inst cmp temp list-pointer-lowtag)
+    (inst b :ne NE)
+    (loadw result result cons-cdr-slot list-pointer-lowtag)
+    NE))
+
+(define-vop (symbol-dbinfo)
   (:policy :fast-safe)
-  (:translate symbol-info-vector)
+  (:translate symbol-dbinfo)
   (:args (x :scs (descriptor-reg)))
   (:results (res :scs (descriptor-reg)))
   (:temporary (:sc unsigned-reg) temp)
   (:generator 1
-    (loadw res x symbol-info-slot other-pointer-lowtag)
-    ;; If RES has list-pointer-lowtag, take its CDR. If not, use it as-is.
-    (inst and temp res lowtag-mask)
-    (inst cmp temp list-pointer-lowtag)
-    (inst b :ne NE)
-    (loadw res res cons-cdr-slot list-pointer-lowtag)
-    NE))
-
-(define-vop (symbol-plist)
-  (:policy :fast-safe)
-  (:translate symbol-plist)
-  (:args (x :scs (descriptor-reg)))
-  (:results (res :scs (descriptor-reg)))
-  (:generator 1
-    (loadw res x symbol-info-slot other-pointer-lowtag)
-    ;; Instruction pun: (CAR x) is the same as (VECTOR-LENGTH x)
-    ;; so if the info slot holds a vector, this gets a fixnum- it's not a plist.
-    (loadw res res cons-car-slot list-pointer-lowtag)
-    (inst tst res fixnum-tag-mask)
-    (inst csel res null-tn res :eq)))
+    (load-symbol-dbinfo res x temp)))
 
 ;;;; other miscellaneous VOPs
 
@@ -264,10 +275,10 @@
     (:result-types system-area-pointer)
     (:translate current-thread-offset-sap)
     (:info n)
-    (:arg-types (:constant (satisfies ldr-str-offset-encodable)))
+    (:arg-types (:constant (satisfies ldr-str-word-offset-encodable)))
     (:policy :fast-safe)
     (:generator 1
-                (inst ldr sap (@ thread-tn (ash n word-shift)))))
+      (inst ldr sap (@ thread-tn (ash n word-shift)))))
 
   (define-vop (current-thread-offset-sap)
     (:results (sap :scs (sap-reg)))
@@ -277,7 +288,7 @@
     (:arg-types signed-num)
     (:policy :fast-safe)
     (:generator 2
-                (inst ldr sap (@ thread-tn (extend n :lsl word-shift))))))
+      (inst ldr sap (@ thread-tn (extend n :lsl word-shift))))))
 
 ;;; Barriers
 (define-vop (%compiler-barrier)
@@ -307,3 +318,12 @@
   (:policy :fast-safe)
   (:translate %data-dependency-barrier)
   (:generator 3))
+
+(define-vop (sb-c::mark-covered)
+ (:info index)
+ (:temporary (:sc unsigned-reg) tmp)
+ (:temporary (:sc descriptor-reg) vector)
+ (:generator 4
+   ;; Can't compute code-tn-relative index until the boxed header length
+   ;; is known. Some vops emit new boxed words via EMIT-CONSTANT.
+   (inst store-coverage-mark index tmp vector)))
