@@ -35,6 +35,66 @@
 #-sb-xc-host
 (declaim (ftype (sfunction (t) ctype) global-ftype))
 
+;;; A bit about the physical representation of the packed info format:
+;;; With #+compact-instance-header it is possible to represent a vector of N things
+;;; in a structure using (ALIGN-UP (1+ N) 2) words of memory. This is a saving
+;;; of 1 word on average when compared to SIMPLE-VECTOR which needs
+;;; (ALIGN-UP (+ N 2) 2) words.  Granted that either might have a padding word,
+;;; but I've observed 5% to 10% space reduction by eliminating one slot.
+;;; Without compact-instance-header, we'e indifferent, in terms of space,
+;;; as to whether this is an INSTANCE or a SIMPLE-VECTOR. For consistency,
+;;; we use an INSTANCE regardless of presence of the compact-header feature.
+;;; This makes assembly routines (e.g. CALL-SYMBOL) slightly less sensitive
+;;; to the feature's absence.
+
+;;; Since variable-length instances aren't portably a thing,
+;;; we use a structure of one slot holding a vector.
+#+sb-xc-host (defstruct (packed-info
+                          (:constructor %make-packed-info (cells))
+                          (:copier nil))
+               ;; These objects are immutable.
+               (cells #() :type simple-vector :read-only t))
+;;; Some abstractions for host/target compatibility of all the defuns.
+#+sb-xc-host
+(progn
+  (defmacro make-packed-info (n) `(%make-packed-info (make-array ,n)))
+  (defmacro copy-packed-info (info)
+    `(%make-packed-info (copy-seq (packed-info-cells ,info))))
+  (defmacro packed-info-len (info)
+    `(length (packed-info-cells ,info)))
+  (defmacro  %info-ref (info index) `(svref (packed-info-cells ,info) ,index)))
+#-sb-xc-host
+(progn
+  #+nil
+  (defmethod print-object ((obj packed-info) stream)
+    (format stream "[~{~W~^ ~}]"
+            (loop for i below (%instance-length obj)
+               collect (%instance-ref obj i))))
+  (defmethod print-object ((self sb-int:packed-info) stream)
+    (print-unreadable-object (self stream :type t :identity t)
+      (format stream "len=~d" (sb-kernel:%instance-length self))))
+  (eval-when (:compile-toplevel)
+    (sb-xc:defmacro make-packed-info (n)
+      ;; this file is earlier than early-vm, so we have to take
+      ;; INSTANCE-DATA-START from the value set in make-host-1.
+      `(let ((new (%make-instance (+ ,n #.sb-vm:instance-data-start))))
+         (setf (%instance-wrapper new) #.(find-layout 'packed-info))
+         new))
+    ;; We can't merely call COPY-STRUCTURE due to two issues:
+    ;; 1. bootstrapping- the DEFSTRUCT-DESCRIPTION is not available
+    ;;    in cold-init by the first call to COPY-PACKED-INFO,
+    ;;    but COPY-STRUCTURE needs it, because of raw slots.
+    ;; 2. the transform for COPY-STRUCTURE would think that
+    ;;    it needs to copy exactly 1 slot. Well, it would think that,
+    ;;    if the FREEZE-TYPE wasn't commented out.
+    (sb-xc:defmacro copy-packed-info (info)
+      ;; not bothering with ONCE-ONLY here (doesn't matter)
+      `(%copy-instance (%make-instance (%instance-length ,info)) ,info)))
+  (defmacro packed-info-len (info)
+      `(- (%instance-length ,info) sb-vm:instance-data-start))
+  (defmacro %info-ref (v i)
+    `(%instance-ref ,v (+ ,i #.sb-vm:instance-data-start))))
+
 ;;; At run time, we represent the type of a piece of INFO in the globaldb
 ;;; by a small integer between 1 and 63.  [0 is reserved for internal use.]
 ;;; CLISP, and maybe others, need EVAL-WHEN because without it, the constant
@@ -46,9 +106,7 @@
 ;;; A map from info-number to its META-INFO object.
 ;;; The reverse mapping is obtained by reading the META-INFO.
 (declaim (type (simple-vector #.(ash 1 info-number-bits)) *info-types*))
-(!define-load-time-global *info-types*
-            ;; Must be dumped as a literal for cold-load.
-            #.(make-array (ash 1 info-number-bits) :initial-element nil))
+(define-load-time-global *info-types* (make-array (ash 1 info-number-bits) :initial-element nil))
 
 (defstruct (meta-info
             (:constructor
@@ -87,25 +145,26 @@
 ;;; presented at the definition of SYMBOL-PLIST, if the object in SYMBOL's
 ;;; info slot is LISTP, it is in state 1 or 3. Either way, take the CDR.
 ;;; Otherwise, it is in state 2 so return the value as-is.
-;;; In terms of this function being named "-vector", implying always a vector,
-;;; it is understood that NIL is a proxy for +NIL-PACKED-INFOS+, a vector.
+;;; NIL is an acceptable substitute for +NIL-PACKED-INFOS+,
+;;; but I might change that.
 ;;;
-;;; Define SYMBOL-INFO-VECTOR as an inline function unless a vop translates it.
+;;; Define SYMBOL-INFO as an inline function unless a vop translates it.
 ;;; (Inlining occurs first, which would cause the vop not to be used.)
 #-sb-xc-host
-(sb-c::unless-vop-existsp (:translate sb-kernel:symbol-info-vector)
-  (declaim (inline symbol-info-vector))
-  (defun symbol-info-vector (symbol)
-    (let ((info-holder (symbol-info symbol)))
-      (truly-the (or null simple-vector)
+(sb-c::unless-vop-existsp (:translate sb-kernel:symbol-dbinfo)
+  (declaim (inline symbol-dbinfo))
+  (defun symbol-dbinfo (symbol)
+    (let ((info-holder (symbol-%info symbol)))
+      (truly-the (or null instance)
                  (if (listp info-holder) (cdr info-holder) info-holder)))))
 
-;;; SYMBOL-INFO is a primitive object accessor defined in 'objdef.lisp'
-;;; But in the host Lisp, there is no such thing as a symbol-info slot.
-;;; Instead, symbol-info is kept in the host symbol's plist.
-;;; This must be a SETFable place.
+;;; %SYMBOL-INFO is a primitive object accessor defined in 'objdef.lisp'
+;;; But in the host Lisp, there is no such thing. Instead, SYMBOL-%INFO
+;;; is kept as a property on the host symbol.
+;;; The compatible "primitive" accessor must be a SETFable place.
 #+sb-xc-host
-(defmacro symbol-info-vector (symbol) `(get ,symbol :sb-xc-globaldb-info))
+(progn (defmacro symbol-%info (symbol) `(get ,symbol :sb-xc-globaldb-info))
+       (defun symbol-dbinfo (symbol) (symbol-%info symbol)))
 
 ;; Perform the equivalent of (GET-INFO-VALUE KIND +INFO-METAINFO-TYPE-NUM+)
 ;; but skipping the defaulting logic.
@@ -113,19 +172,21 @@
 ;; - though not always - a unique identifier for the (:TYPE :KIND) pair.
 ;; Note that bypassing of defaults is critical for bootstrapping,
 ;; since INFO is used to retrieve its own META-INFO at system-build time.
-(defmacro !get-meta-infos (kind)
-  `(let* ((info-vector (symbol-info-vector ,kind))
-          (index (if info-vector
-                     (packed-info-value-index info-vector +no-auxiliary-key+
+(defmacro get-meta-infos (kind)
+  `(let* ((packed-info (symbol-dbinfo ,kind))
+          (index (if packed-info
+                     (packed-info-value-index packed-info +no-auxiliary-key+
                                               +info-metainfo-type-num+))))
-     (if index (svref info-vector index))))
+     (if index (%info-ref packed-info index))))
 
-;; (UNSIGNED-BYTE 16) is an arbitrarily generous limit on the number of
-;; cells in an info-vector. Most vectors have a fewer than a handful of things,
+;; (UNSIGNED-BYTE 11) is an arbitrarily generous limit on the number of
+;; cells in a packed-info. Most packed-infos have fewer than a handful of things,
 ;; and performance would need to be re-thought if more than about a dozen
 ;; cells were in use. (It would want to become hash-based probably)
-(declaim (ftype (function (simple-vector (or (eql 0) symbol) info-number)
-                          (or null (unsigned-byte 16)))
+;; It has to be smaller than INSTANCE_LENGTH_MASK certainly,
+;; plus leaving room for a layout slot if #-compact-instance-header.
+(declaim (ftype (function (packed-info (or (eql 0) symbol) info-number)
+                          (or null (unsigned-byte 11)))
                 packed-info-value-index))
 
 ;; Return the META-INFO object for CATEGORY and KIND, signaling an error
@@ -138,7 +199,7 @@
 ;; through, whereas typically no more than 3 or 4 items have the same KIND.
 ;;
 (defun meta-info (category kind &optional (errorp t))
-  (or (let ((metadata (!get-meta-infos kind)))
+  (or (let ((metadata (get-meta-infos kind)))
         (cond ((listp metadata) ; conveniently handles NIL
                (dolist (info metadata nil) ; FIND is slower :-(
                  (when (eq (meta-info-category (truly-the meta-info info))
@@ -146,7 +207,7 @@
                    (return info))))
               ((eq (meta-info-category (truly-the meta-info metadata)) category)
                metadata)))
-      ;; !GET-META-INFOS enforces that KIND is a symbol, therefore
+      ;; GET-META-INFOS enforces that KIND is a symbol, therefore
       ;; if a metaobject was found, CATEGORY was necessarily a symbol too.
       ;; Otherwise, if the caller wants no error to be signaled on missing info,
       ;; we must nevertheless enforce that CATEGORY was actually a symbol.
@@ -184,10 +245,6 @@
                             .whole.)))
                     .whole.))))
 
-  ;; ECL bug workaround: INFO ceases to be a valid macrolet name
-  ;; because it tries to run the compiler-macro before the local macro.
-  ;; In particular, "(collect ((info)) ...)" will not compile correctly.
-  #-host-quirks-ecl
   (def info (category kind name)
     `(truly-the (values ,(meta-info-type-spec meta-info) boolean)
                 (get-info-value ,name ,(meta-info-number meta-info))))
