@@ -14,11 +14,7 @@
 (in-package "SB-C")
 
 ;;; This phase runs before IR2 conversion, initializing each XEP's
-;;; ENTRY-INFO structure. We call the VM-supplied
-;;; SELECT-COMPONENT-FORMAT function to make VM-dependent
-;;; initializations in the IR2-COMPONENT. This includes setting the
-;;; IR2-COMPONENT-KIND and allocating fixed implementation overhead in
-;;; the constant pool. If there was a forward reference to a function,
+;;; ENTRY-INFO structure.  If there was a forward reference to a function,
 ;;; then the ENTRY-INFO will already exist, but will be uninitialized.
 (defun entry-analyze (component)
   (let ((2comp (component-info component)))
@@ -28,7 +24,6 @@
                         (setf (leaf-info fun) (make-entry-info)))))
           (compute-entry-info fun info)
           (push info (ir2-component-entries 2comp))))))
-  (select-component-format component)
   (values))
 
 ;;; An "effectively" null environment captures at most
@@ -57,7 +52,7 @@
   (let ((bind (lambda-bind fun))
         (internal-fun (functional-entry-fun fun)))
     (setf (entry-info-closure-tn info)
-          (if (physenv-closure (lambda-physenv fun))
+          (if (environment-closure (lambda-environment fun))
               (make-normal-tn *backend-t-primitive-type*)
               nil))
     (setf (entry-info-offset info) (gen-label))
@@ -66,6 +61,8 @@
     (setf (entry-info-xref info) (pack-xref-data (functional-xref internal-fun)))
     (let* ((inline-expansion (functional-inline-expansion internal-fun))
            (form  (if (fasl-output-p *compile-object*)
+                      ;; If compiling to a file, we only store sources if the STORE-SOURCE
+                      ;; quality value is 3. If to memory, any nonzero value will do.
                       (and (policy bind (= store-source-form 3))
                            ;; Downgrade the error to a warning if this was signaled
                            ;; by SB-PCL::DONT-KNOW-HOW-TO-DUMP.
@@ -81,15 +78,19 @@
                                      ;; on MAKE-LOAD-FORM?  Why did we choose to further obfuscate
                                      ;; a condition that was reflectable and instead turn it
                                      ;; into a dumb text string?
-                                     (when (and (typep c 'simple-error)
-                                                (search "know how to dump"
-                                                        (simple-condition-format-control c)))
-                                       ;; This might be worth a full warning. Dunno.
-                                       ;; After all, the user asked to do what can't be done.
-                                       (compiler-style-warn
+                                     (if (and (typep c 'simple-error)
+                                              (stringp (simple-condition-format-control c))
+                                              (search "know how to dump"
+                                                      (simple-condition-format-control c)))
+                                         ;; This might be worth a full warning. Dunno.
+                                         ;; After all, the user asked to do what can't be done.
+                                         (compiler-style-warn
                                         "Can't preserve function source - ~
 missing MAKE-LOAD-FORM methods?")
-                                       (return nil)))))
+                                         (compiler-style-warn
+                                          "Can't preserve function source: ~A"
+                                          (princ-to-string c)))
+                                       (return nil))))
                                (constant-value (find-constant inline-expansion)))))
                       (and (policy bind (> store-source-form 0))
                            inline-expansion)))
@@ -111,12 +112,14 @@ missing MAKE-LOAD-FORM methods?")
       (let ((spec (type-specifier (leaf-type internal-fun)))
             (result))
         (setf (entry-info-type info)
-              (if (and (listp spec)
-                       (typep (setq result (third spec))
-                              '(cons (eql values)
-                                     (cons t (cons (eql &optional) null)))))
-                  `(sfunction ,(cadr spec) ,(cadr result))
-                  spec)))))
+              (constant-value
+               (find-constant
+                (if (and (listp spec)
+                         (typep (setq result (third spec))
+                                '(cons (eql values)
+                                  (cons t (cons (eql &optional) null)))))
+                    `(sfunction ,(cadr spec) ,(cadr result))
+                    spec)))))))
   (values))
 
 ;;; Replace all references to COMPONENT's non-closure XEPs that appear
@@ -153,7 +156,7 @@ missing MAKE-LOAD-FORM methods?")
                             ;; It may have been deleted due to none of
                             ;; the optional entries reaching it.
                             (neq (functional-kind main-entry) :deleted)
-                            (physenv-closure (lambda-physenv main-entry)))))
+                            (environment-closure (lambda-environment main-entry)))))
              (dolist (ref (leaf-refs lambda))
                (let ((ref-component (node-component ref)))
                  (cond ((eq ref-component component))
