@@ -15,10 +15,10 @@
 
 ;;;; coercions
 
-(deftransform float ((n f) (* single-float) *)
+(deftransform float ((n f) (t single-float) *)
   '(%single-float n))
 
-(deftransform float ((n f) (* double-float) *)
+(deftransform float ((n f) (t double-float) *)
   '(%double-float n))
 
 (deftransform float ((n) *)
@@ -35,7 +35,7 @@
 ;;; RANDOM
 (macrolet ((frob (fun type)
              `(deftransform random ((num &optional state)
-                                    (,type &optional *) *)
+                                    (,type &optional t) *)
                 "Use inline float operations."
                 '(,fun num (or state *random-state*)))))
   (frob %random-single-float single-float)
@@ -67,7 +67,7 @@
 ;;; a division to get the random value into the desired range.
 (deftransform random ((num &optional state)
                       ((constant-arg (integer 1 #.(expt 2 sb-vm:n-word-bits)))
-                       &optional *)
+                       &optional t)
                       *
                       :policy (and (> speed compilation-speed)
                                    (> speed space)))
@@ -220,7 +220,7 @@
 (deftransform integer-decode-float ((x) (double-float) *)
   '(integer-decode-double-float x))
 
-(deftransform scale-float ((f ex) (single-float *) *)
+(deftransform scale-float ((f ex) (single-float t) *)
   (cond #+x86
         ((csubtypep (lvar-type ex)
                     (specifier-type '(signed-byte 32)))
@@ -228,7 +228,7 @@
         (t
          '(scale-single-float f ex))))
 
-(deftransform scale-float ((f ex) (double-float *) *)
+(deftransform scale-float ((f ex) (double-float t) *)
   (cond #+x86
         ((csubtypep (lvar-type ex)
                     (specifier-type '(signed-byte 32)))
@@ -338,18 +338,20 @@
 
 ;;; Do some stuff to recognize when the loser is doing mixed float and
 ;;; rational arithmetic, or different float types, and fix it up. If
-;;; we don't, he won't even get so much as an efficiency note.
+;;; we don't, we won't even get so much as an efficiency note.
 ;;; Unfortunately this produces unnecessarily bad code for something
 ;;; as simple as (ZEROP (THE FLOAT X)) because we _know_ that the thing
 ;;; is a float, but the ZEROP transform injected a rational 0,
 ;;; which we then go to the trouble of coercing to a float.
 (progn
   (deftransform real-float-contagion-arg1 ((x y) * * :defun-only t :node node)
+    "open-code float conversion in mixed numeric operation"
     (if (or (not (types-equal-or-intersect (lvar-type y) (specifier-type 'single-float)))
             (safe-ctype-for-single-coercion-p (lvar-type x)))
         `(,(lvar-fun-name (basic-combination-fun node)) (float x y) y)
         (give-up-ir1-transform #1="the first argument cannot safely be converted to SINGLE-FLOAT")))
   (deftransform complex-float-contagion-arg1 ((x y) * * :defun-only t :node node)
+    "open-code float conversion in mixed numeric operation"
     (if (or (not (types-equal-or-intersect
                   (lvar-type y) (specifier-type '(complex single-float))))
             (safe-ctype-for-single-coercion-p (lvar-type x)))
@@ -357,11 +359,13 @@
           (float x (realpart y)) y)
         (give-up-ir1-transform #1#)))
   (deftransform real-float-contagion-arg2 ((x y) * * :defun-only t :node node)
+    "open-code float conversion in mixed numeric operation"
     (if (or (not (types-equal-or-intersect (lvar-type x) (specifier-type 'single-float)))
             (safe-ctype-for-single-coercion-p (lvar-type y)))
         `(,(lvar-fun-name (basic-combination-fun node)) x (float y x))
         (give-up-ir1-transform #2="the second argument cannot safely be converted to SINGLE-FLOAT")))
   (deftransform complex-float-contagion-arg2 ((x y) * * :defun-only t :node node)
+    "open-code float conversion in mixed numeric operation"
     (if (or (not (types-equal-or-intersect (lvar-type x) (specifier-type '(complex single-float))))
             (safe-ctype-for-single-coercion-p (lvar-type y)))
         `(,(lvar-fun-name (basic-combination-fun node))
@@ -377,11 +381,7 @@
                (2 (if complexp
                       (values `(complex ,float-type) other-type #'complex-float-contagion-arg2)
                       (values float-type other-type #'real-float-contagion-arg2))))
-           (let ((note (format nil "open-code float conversion of ~:R ~
-                                    argument in mixed ~S-~S numeric ~
-                                    operation"
-                               argument type1 type2)))
-             (%deftransform operator `(function (,type1 ,type2) *) function note :slightly)))))
+             (%deftransform operator nil `(function (,type1 ,type2) *) function))))
 
   (dolist (operator '(+ * / -))
     (def operator 'float 'rational nil 1)
@@ -479,10 +479,17 @@
                   (give-up-ir1-transform
                    "The RATIONAL value isn't known at compile time."))
                 (let ((val (lvar-value y)))
-                  (unless (eql (rational (float val)) val)
-                    (give-up-ir1-transform
-                     "~S doesn't have a precise float representation."
-                     val)))
+                  (multiple-value-bind (low high type)
+                      (if (csubtypep (lvar-type x) (specifier-type 'double-float))
+                          (values most-negative-double-float most-positive-double-float
+                                  'double-float)
+                          (values most-negative-single-float most-positive-single-float
+                                  'single-float))
+                      (unless (and (sb-xc:<= low val high)
+                                   (eql (rational (coerce val type)) val))
+                        (give-up-ir1-transform
+                         "~S doesn't have a precise float representation."
+                         val))))
                 `(,',op x (float y ,',(if complex
                                           '(realpart x)
                                           'x))))))
@@ -619,15 +626,21 @@
 ;;; should be the right kind of float. Allow bounds for the float
 ;;; part too.
 (defun float-or-complex-float-type (arg &optional lo hi)
-  (declare (type numeric-type arg))
-  (let* ((format (case (numeric-type-class arg)
-                   ((integer rational) 'single-float)
-                   (t (numeric-type-format arg))))
-         (float-type (or format 'float))
-         (lo (coerce-numeric-bound lo float-type))
-         (hi (coerce-numeric-bound hi float-type)))
-    (specifier-type `(or (,float-type ,(or lo '*) ,(or hi '*))
-                         (complex ,float-type)))))
+  (cond
+    ((numeric-type-p arg)
+     (let* ((format (case (numeric-type-class arg)
+                      ((integer rational) 'single-float)
+                      (t (numeric-type-format arg))))
+            (float-type (or format 'float))
+            (lo (coerce-numeric-bound lo float-type))
+            (hi (coerce-numeric-bound hi float-type)))
+       (specifier-type `(or (,float-type ,(or lo '*) ,(or hi '*))
+                            (complex ,float-type)))))
+    ((union-type-p arg)
+     (apply #'type-union
+            (loop for type in (union-type-types arg)
+                  collect (float-or-complex-float-type type lo hi))))
+    (t (specifier-type 'number))))
 
 (eval-when (:compile-toplevel :execute)
   ;; So the problem with this hack is that it's actually broken.  If
@@ -831,7 +844,7 @@
        (list (interval-expt-> x y-)
              (interval-expt-> x y+))))))
 
-;;; Handle the case when x <= 1
+;;; Handle the case when 0 <= x <= 1
 (defun interval-expt-< (x y)
   (case (interval-range-info x $0d0)
     (+
@@ -862,9 +875,13 @@
                 (interval-expt-< x y+))))))
     (-
      ;; The case where x <= 0. Y MUST be an INTEGER for this to work!
-     ;; The calling function must insure this! For now we'll just
-     ;; return the appropriate unbounded float type.
-     (list (make-interval :low nil :high nil)))
+     ;; The calling function must insure this!
+     (loop for interval in (flatten-list (interval-expt (interval-neg x) y))
+           for low = (interval-low interval)
+           for high = (interval-high interval)
+           collect interval
+           when (or high low)
+           collect (interval-neg interval)))
     (t
      (destructuring-bind (neg pos)
          (interval-split 0 x t t)
@@ -1516,7 +1533,7 @@
 
 (macrolet ((define-frobs (fun ufun)
              `(deftransform ,fun ((x &optional by)
-                                  (* &optional (constant-arg (member 1))))
+                                  (t &optional (constant-arg (member 1))))
                   '(let ((res (,ufun x)))
                     (values res (locally
                                     (declare (flushable %single-float
@@ -1550,7 +1567,8 @@
                                         * :result result)
                   (let* ((result-type (and result
                                            (lvar-derived-type result)))
-                         (compute-all (and (values-type-p result-type)
+                         (compute-all (and (or (eq result-type *wild-type*)
+                                               (values-type-p result-type))
                                            (not (type-single-value-p result-type)))))
                     (if (or (not y)
                             (and (constant-lvar-p y) (= 1 (lvar-value y))))
