@@ -226,8 +226,8 @@
   funcallable-instance-widetag              ;  36   3D  36   3D
 
   ;; x86[-64] does not have objects with this widetag,
-  #-(or x86 x86-64) return-pc-widetag       ;  3A   41  3A   41
-  #+(or x86 x86-64) lra-widetag-notused
+  #-(or x86 x86-64 arm64) return-pc-widetag ;  3A   41  3A   41
+  #+(or x86 x86-64 arm64) lra-widetag-notused
 
   value-cell-widetag                        ;  3E   45  3E   45
   character-widetag                         ;  42   49  42   49
@@ -252,6 +252,9 @@
   #-64-bit unused09-widetag                 ;  7E       7E
 
   simple-array-widetag                      ;  82   81  82   81
+  ;; NIL element type is not in the contiguous range of widetags
+  ;; corresponding to SIMPLE-UNBOXED-ARRAY
+  simple-array-nil-widetag
   simple-vector-widetag                     ;
   simple-bit-vector-widetag                 ;
   simple-array-unsigned-byte-2-widetag      ;
@@ -285,9 +288,6 @@
   simple-array-complex-single-float-widetag ;
   simple-array-complex-double-float-widetag ;
 
-  ;; Not a string type
-  simple-array-nil-widetag                  ;
-
   simple-base-string-widetag                ;  D6   E1  D6   E1       \
   #+sb-unicode                              ;                          |
   simple-character-string-widetag           ;  DA   E5                 | Strings
@@ -310,12 +310,21 @@
 
 ;;; Byte index:           3           2           1           0
 ;;;                 +-----------------------------------+-----------+
-;;;                 |  unused   |   rank    |   flags   |  widetag  |
+;;;                 |  unused   |   rank   <|>   flags  |  widetag  |
 ;;;                 +-----------+-----------+-----------+-----------+
 ;;;                 |<---------- HEADER DATA ---------->|
 
+;;; Having contiguous rank and widetag allows
+;;; SIMPLE-ARRAY-HEADER-OF-RANK-P to be done with just one comparison.
+;;; Other backends may not be ready to switch the order yet.
+(defconstant array-rank-position  #-arm64 16 #+arm64 8)
+(defconstant array-flags-position #-arm64 8  #+arm64 16)
+
+(defconstant array-flags-data-position (- array-flags-position n-widetag-bits))
 (defconstant +array-fill-pointer-p+    #x80)
 
+(defconstant +vector-dynamic-extent+   #x40)
+
 ;; A vector tagged as +VECTOR-SHAREABLE+ is logically readonly,
 ;; and permitted to be shared with another vector per the CLHS standard
 ;; under the concept of similarity as constant. A vector so tagged is
