@@ -55,8 +55,9 @@
                             (type-specifier
                              (specifier-type
                               `(simple-array ,(sb-vm:saetp-specifier saetp) (*)))))
-                          (remove t sb-vm:*specialized-array-element-type-properties*
-                                  :key 'sb-vm:saetp-specifier))))
+                          (remove-if (lambda (x) (member x '(nil t)))
+                                     sb-vm:*specialized-array-element-type-properties*
+                                     :key 'sb-vm:saetp-specifier))))
                 `(((integer 0 ,array-dimension-limit)
                    object-not-array-dimension)
                   ;; Union of all unboxed array specializations,
@@ -109,6 +110,8 @@
    ("odd number of &KEY arguments" odd-key-args 0)
    ("unknown &KEY argument" unknown-key-arg 1)
    ("invalid array index" invalid-array-index 3)
+   ("invalid vector index" invalid-vector-index 2)
+   ("uninitialized element" uninitialized-element 2)
    ("A function with declared result type NIL returned." nil-fun-returned 1)
    ("An array with element-type NIL was accessed." nil-array-accessed 1)
    ("Object layout is invalid. (indicates obsolete instance)" layout-invalid 2)
@@ -185,7 +188,7 @@
   sb-c::vop
   sb-c::basic-combination
   sb-sys:fd-stream
-  layout
+  wrapper
   (sb-assem:segment object-not-assem-segment)
   sb-c::cblock
   sb-disassem:disassem-state
@@ -231,20 +234,3 @@
   (if (array-in-bounds-p sb-c:+backend-internal-errors+ error-number)
       (cddr (svref sb-c:+backend-internal-errors+ error-number))
       0))
-
-#-sb-xc-host ; no SB-C:SAP-READ-VAR-INTEGERF
-(defun decode-internal-error-args (sap trap-number &optional error-number)
-  (let ((error-number (cond (error-number)
-                            ((>= trap-number sb-vm:error-trap)
-                             (prog1
-                                 (- trap-number sb-vm:error-trap)
-                               (setf trap-number sb-vm:error-trap)))
-                            (t
-                             (prog1 (sap-ref-8 sap 0)
-                               (setf sap (sap+ sap 1)))))))
-    (let ((length (sb-kernel::error-length error-number)))
-      (declare (type (unsigned-byte 8) length))
-      (values error-number
-              (loop repeat length with index = 0
-                    collect (sb-c:sap-read-var-integerf sap index))
-              trap-number))))
