@@ -79,11 +79,15 @@
                )))
     (let*
         ((spaces (append `((read-only ,ro-space-size)
+                           #+(and win32 x86-64)
+                           (seh-data ,(symbol-value '+backend-page-bytes+) win64-seh-data-addr)
                            (linkage-table ,small-space-size)
                            #+sb-safepoint
                            ;; Must be just before NIL.
                            (safepoint ,(symbol-value '+backend-page-bytes+) gc-safepoint-page-addr)
-                           (static ,small-space-size))
+                           (static ,small-space-size)
+                           #+darwin-jit
+                           (static-code ,small-space-size))
                          #+immobile-space
                          `((fixedobj ,fixedobj-space-size*)
                            (varyobj ,varyobj-space-size*))))
@@ -127,7 +131,20 @@
            (or ,(or (!read-dynamic-space-size) dynamic-space-size*)
                (ecase n-word-bits
                  (32 (expt 2 29))
-                 (64 (expt 2 30)))))))))
+                 (64 (expt 2 30)))))
+         (defconstant gencgc-card-shift
+           (integer-length (1- sb-vm:gencgc-card-bytes)))
+         ;; This is a constant during build, but a different value
+         ;; can be patched directly into the affected machine code
+         ;; when the core is loaded based on dynamic-space-size.
+         ;; I think the C runtime does a floor operation rather than ceiling,
+         ;; but firstly there's probably no difference, and secondly it's better
+         ;; to be safe than sorry - using too many bits rather than too few.
+         (defconstant gencgc-card-table-index-nbits
+           (integer-length (1- (ceiling sb-vm::default-dynamic-space-size
+                                        sb-vm::gencgc-card-bytes))))
+         (defconstant gencgc-card-table-index-mask
+           (1- (ash 1 gencgc-card-table-index-nbits)))))))
 
 (defconstant-eqx +c-callable-fdefns+
   '(sub-gc
@@ -145,11 +162,9 @@
     sb-di::handle-breakpoint
     sb-di::handle-single-step-trap
     #+win32 sb-kernel::handle-win32-exception
-    #+sb-thruption sb-thread::run-interruption
+    #+sb-safepoint sb-thread::run-interruption
     enter-alien-callback
-    #+sb-thread sb-thread::enter-foreign-callback
-    #+(and sb-safepoint-strictly (not win32))
-    sb-unix::signal-handler-callback)
+    #+sb-thread sb-thread::enter-foreign-callback)
   #'equal)
 
 ;;; (potentially) static symbols that C code must be able to set/get
@@ -163,9 +178,8 @@
   `((*free-interrupt-context-index* 0)
     (sb-sys:*allow-with-interrupts* t)
     (sb-sys:*interrupts-enabled* t)
-    *alloc-signal*
     sb-sys:*interrupt-pending*
-    #+sb-thruption sb-sys:*thruption-pending*
+    #+sb-safepoint sb-sys:*thruption-pending*
     *in-without-gcing*
     *gc-inhibit*
     *gc-pending*
@@ -194,6 +208,7 @@
         *current-unwind-protect-block*)
 
     #+immobile-space *immobile-freelist* ; not per-thread (yet...)
+    #+metaspace *metaspace-tracts*
 
     ;; stack pointers
     #-sb-thread *binding-stack-start* ; a thread slot if #+sb-thread
@@ -245,10 +260,12 @@
   #-sb-thread 1024) ; crazy value
 
 ;;; Thread slots accessed at negative indices relative to struct thread.
-;;; sb-safepoint needs to access one word below thread-base-tn for the
-;;; foreign call safepoint trap word, so this trick doesn't work.
 (defconstant thread-header-slots
-  (+ #+(and x86-64 (not sb-safepoint)) 16))
+  ;; This seems to need to be an even number.
+  ;; I'm not sure what the constraint on that stems from.
+  #+(and x86-64 sb-safepoint) 14 ; the safepoint trap page is at word index -15
+  #+(and x86-64 (not sb-safepoint)) 16
+  #-x86-64 0)
 
 #+gencgc
 (progn
@@ -258,5 +275,47 @@
 (defparameter *runtime-asm-routines* nil)
 (defparameter *linkage-space-predefined-entries* nil)
 
+;;; Floating-point related constants, both format descriptions and FPU
+;;; control register descriptions.  These don't exactly match up with
+;;; what the machine manuals say because the Common Lisp standard
+;;; defines floating-point values somewhat differently than the IEEE
+;;; standard does.
+
+;;; We can currently manipulate only IEEE single and double precision.
+;;; Machine-specific formats (such as x86 80-bit and PPC double-double)
+;;; are unsupported.
+
+#+ieee-floating-point
+(progn
+(defconstant float-sign-shift 31)
+
+;;; The exponent bias is the amount up by which the true exponent is
+;;; incremented for storage purposes.
+;;; (Wikipedia entry for single-float: "an exponent value of 127 represents the actual zero")
+;;; 126 works for us because we actually want an exponent of -1 and not 0
+;;; when using DECODE-FLOAT. -1 is correct because the implied 1 bit in a normalized
+;;; float is the _left_ of of the binary point, so the powers of 2 for the first represented
+;;; bit is 2^-1. Similarly for double-float.
+(defconstant single-float-bias 126)
+(defconstant-eqx single-float-exponent-byte (byte 8 23) #'equalp)
+(defconstant-eqx single-float-significand-byte (byte 23 0) #'equalp)
+(defconstant single-float-normal-exponent-min 1)
+(defconstant single-float-normal-exponent-max 254)
+(defconstant single-float-hidden-bit (ash 1 23))
+
+(defconstant double-float-bias 1022)
+(defconstant-eqx double-float-exponent-byte (byte 11 20) #'equalp)
+(defconstant-eqx double-float-significand-byte (byte 20 0) #'equalp)
+(defconstant double-float-normal-exponent-min 1)
+(defconstant double-float-normal-exponent-max #x7FE)
+(defconstant double-float-hidden-bit (ash 1 20))
+
+(defconstant single-float-digits
+  (+ (byte-size single-float-significand-byte) 1))
+
+(defconstant double-float-digits
+  (+ (byte-size double-float-significand-byte) 32 1))
+)
+
 (push '("SB-VM" +c-callable-fdefns+ +common-static-symbols+)
       *!removable-symbols*)
