@@ -50,13 +50,11 @@
       0))
 
 ;;; the address of the linkage table entry for table index I.
-#+linkage-table
 (defun linkage-table-entry-address (i)
   (ecase linkage-table-growth-direction
     (:up   (+ (* i linkage-table-entry-size) linkage-table-space-start))
     (:down (- linkage-table-space-end (* (1+ i) linkage-table-entry-size)))))
 
-#+linkage-table
 (defun linkage-table-index-from-address (addr)
   (ecase linkage-table-growth-direction
     (:up
@@ -132,7 +130,7 @@
 
 ;;; Make an environment-live stack TN for saving the SP for NLX entry.
 (defun make-nlx-sp-tn (env)
-  (physenv-live-tn
+  (environment-live-tn
    (make-representation-tn *fixnum-primitive-type* any-reg-sc-number)
    env))
 
@@ -212,13 +210,112 @@
        (not (types-equal-or-intersect (tn-ref-type tn-ref)
                                       (specifier-type '(eql nil))))))
 
+(defun stack-consed-p (object)
+  (let ((write (sb-c::tn-writes object))) ; list of write refs
+    (when (or (not write) ; grrrr, the only write is from a LOAD tn
+                          ; and we don't know the corresponding normal TN?
+              (tn-ref-next write)) ; can't determine if > 1 write
+      (return-from stack-consed-p nil))
+    (let ((vop (tn-ref-vop write)))
+      (when (not vop) ; wat?
+        (return-from stack-consed-p nil))
+      (when (eq (vop-name vop) 'allocate-vector-on-stack)
+        (return-from stack-consed-p t))
+      (when (and (eq (vop-name vop) 'fixed-alloc)
+                 (fifth (vop-codegen-info vop))) ; STACK-ALLOCATE-P
+        (return-from stack-consed-p t))
+      ;; Should we try to detect a stack-consed LIST also?
+      ;; I don't think that will work.
+      ;; (And is there anything else interesting to try?)
+      (unless (member (vop-name vop) '(splat-word splat-small splat-any))
+        (return-from stack-consed-p nil))
+      (let* ((splat-input (vop-args vop))
+             (splat-input-source
+              (tn-ref-vop (sb-c::tn-writes (tn-ref-tn splat-input)))))
+        ;; How in the heck can there NOT be a vop??? Well, sometimes there isn't.
+        (when (and splat-input-source
+                   (eq (vop-name splat-input-source)
+                       'allocate-vector-on-stack))
+          (return-from stack-consed-p t)))))
+  nil)
+
+;;; Just gathering some data to see where we can improve
+(define-load-time-global *store-barriers-potentially-emitted* 0)
+(define-load-time-global *store-barriers-emitted* 0)
+
+(defun require-gc-store-barrier-p (object value-tn-ref value-tn)
+  (incf *store-barriers-potentially-emitted*)
+  ;; If OBJECT is stack-allocated, elide the barrier
+  (when (stack-consed-p object)
+    (return-from require-gc-store-barrier-p nil))
+  (flet ((potential-heap-pointer-p (tn tn-ref)
+           (when (sc-is tn any-reg) ; must be fixnum
+             (return-from potential-heap-pointer-p nil))
+           ;; If stack-allocated, elide the barrier
+           (when (stack-consed-p tn)
+             (return-from potential-heap-pointer-p nil))
+           ;; If immediate non-pointer, elide the barrier
+           (when (sc-is tn immediate)
+             (let ((value (tn-value tn)))
+               (when (sb-xc:typep value '(or character sb-xc:fixnum boolean
+                                             #+64-bit single-float))
+                 (return-from potential-heap-pointer-p nil))))
+           (when (sb-c::unbound-marker-tn-p tn)
+             (return-from potential-heap-pointer-p nil))
+           ;; And elide for things like (OR FIXNUM NULL)
+           (let ((type (tn-ref-type tn-ref)))
+             (when (csubtypep type (specifier-type '(or character sb-xc:fixnum boolean
+                                                        #+64-bit single-float)))
+               (return-from potential-heap-pointer-p nil)))
+           t))
+    (cond (value-tn
+           (unless (eq (tn-ref-tn value-tn-ref) value-tn)
+             (aver (eq (tn-ref-load-tn value-tn-ref) value-tn)))
+           (unless (potential-heap-pointer-p value-tn value-tn-ref)
+             (return-from require-gc-store-barrier-p nil)))
+          (value-tn-ref ; a list of refs linked through TN-REF-ACROSS
+           ;; (presumably from INSTANCE-SET-MULTIPLE)
+           (let ((any-pointer
+                  (do ((ref value-tn-ref (tn-ref-across ref)))
+                      ((null ref))
+                    (when (potential-heap-pointer-p (tn-ref-tn ref) ref)
+                      (return t)))))
+             (unless any-pointer
+               (return-from require-gc-store-barrier-p nil))))))
+  (incf *store-barriers-emitted*)
+  t)
+
+(defun vop-nth-arg (n vop)
+  (let ((ref (vop-args vop)))
+    (dotimes (i n ref) (setq ref (tn-ref-across ref)))))
+
 (defun length-field-shift (widetag)
   (if (= widetag instance-widetag)
       instance-length-shift
       n-widetag-bits))
 
-(defun compute-object-header (size widetag)
-  (logior (case widetag
-            (#.fdefn-widetag 0)
-            (t (ash (1- size) (length-field-shift widetag))))
-          widetag))
+(defconstant array-rank-mask 255)
+;;; Rank is encoded as a (UNSIGNED-BYTE 8) minus one.
+;;; Initialization of simple rank 1 array header words is completely unaffected-
+;;; they store 0 for the rank, which is the correct encoding for 1.
+;;; The encoding 1 means 2, encoding 2 means 3, and so on.
+;;; Decoding is just an addition and bitwise AND.
+(defun encode-array-rank (rank)
+  (declare (type (unsigned-byte 8) rank))
+  (logand (1- rank) array-rank-mask))
+
+(defun compute-object-header (nwords widetag)
+  (let ((array-header-p
+         (or (= widetag simple-array-widetag)
+             (>= widetag complex-base-string-widetag))))
+    (logior (if array-header-p
+                (let ((rank (- nwords array-dimensions-offset)))
+                  (ash (encode-array-rank rank) array-rank-position))
+                (case widetag
+                  (#.fdefn-widetag 0)
+                  (t (ash (1- nwords) (length-field-shift widetag)))))
+            widetag)))
+
+(defmacro id-bits-offset ()
+  (let ((slot (get-dsd-index layout sb-kernel::id-word0)))
+    (ash (+ sb-vm:instance-slots-offset slot) sb-vm:word-shift)))
