@@ -123,7 +123,12 @@
 (defknown %set-symbol-hash (symbol hash-code)
   t ())
 
-(defknown symbol-info-vector (symbol) (or null simple-vector))
+;;; TODOD: I'd like to eliminate the (OR NULL) from this return type.
+;;; For that to happen, I probably need +nil-packed-infos+ to become
+;;; placed in static space because assembly routines may need it.
+;;; On the other hand, they may not, because there is no special case
+;;; code needed when reading from it, which is entire point.
+(defknown symbol-dbinfo (symbol) (or null packed-info))
 
 (defknown initialize-vector ((simple-array * (*)) &rest t)
   (simple-array * (*))
@@ -168,7 +173,7 @@
 ;;; ASSIGN-VECTOR-FLAGSS assign all and only the flags byte.
 ;;; RESET- performs LOGANDC2 and returns no value.
 (defknown (assign-vector-flags reset-header-bits)
-  (t (unsigned-byte 8)) (values)
+  (t (unsigned-byte 16)) (values)
   (#+x86-64 always-translatable))
 (defknown (test-header-bit)
   (t (unsigned-byte #.(- sb-vm:n-word-bits sb-vm:n-widetag-bits))) (boolean)
@@ -181,10 +186,12 @@
 (defknown %array-rank (array) array-rank
   (flushable))
 
-#+(or x86 x86-64)
+#+(or x86 x86-64 arm64)
 (defknown (%array-rank= widetag=) (t t) boolean
   (flushable))
 
+(defknown simple-array-header-of-rank-p (t array-rank) boolean
+  (flushable))
 (defknown sb-kernel::check-array-shape (simple-array list)
   (simple-array)
   (flushable)
@@ -208,7 +215,6 @@
   (foldable flushable))
 (defknown %set-instance-layout (instance sb-vm:layout) (values) ())
 ;;; %SET-FUN-LAYOUT should only called on FUNCALLABLE-INSTANCE
-;;; (but %set-funcallable-instance-layout is too long a name)
 (defknown %set-fun-layout (funcallable-instance sb-vm:layout) (values) ())
 ;;; Layout getter that accepts any object, and if it has INSTANCE- or FUN-
 ;;; POINTER-LOWTAG returns the layout, otherwise some agreed-upon layout.
@@ -256,8 +262,7 @@
 ;;; These two are mostly used for bit-bashing operations.
 (defknown %vector-raw-bits (t index) sb-vm:word
   (flushable))
-(defknown (%set-vector-raw-bits) (t index sb-vm:word) sb-vm:word
-  ())
+(defknown (%set-vector-raw-bits) (t index sb-vm:word) (values) ())
 
 
 ;;; Allocate an unboxed, non-fancy vector with type code TYPE, length LENGTH,
@@ -274,7 +279,7 @@
 ;;; vector type will usuallly end up calling allocate-vector-with-widetag
 ;;; via %MAKE-ARRAY.
 (defknown allocate-vector (#+ubsan boolean
-                           (unsigned-byte 9) index
+                           word index
                            ;; The number of words is later converted
                            ;; to bytes, make sure it fits.
                            (and index
@@ -584,7 +589,7 @@
 (defknown fdefn-name (fdefn) t (foldable flushable))
 (defknown fdefn-fun (fdefn) (or function null) (flushable))
 (defknown (setf fdefn-fun) (function fdefn) t ())
-(defknown fdefn-makunbound (fdefn) t ())
+(defknown fdefn-makunbound (fdefn) (values) ())
 ;;; FDEFN -> FUNCTION, trapping if not FBOUNDP
 (defknown safe-fdefn-fun (fdefn) function ())
 
@@ -595,6 +600,7 @@
 
 (defknown %closure-index-ref (function index) t
   (flushable))
+(defknown %closure-index-set (function index t) (values) ())
 
 ;; T argument is for the 'fun' slot.
 (defknown sb-vm::%alloc-closure (index t) function (flushable))
@@ -605,7 +611,8 @@
   ())
 
 (defknown %funcallable-instance-info (function index) t (flushable))
-(defknown %set-funcallable-instance-info (function index t) t ())
+(defknown (setf %funcallable-instance-info) (t function index) t ())
+(defknown %set-funcallable-instance-info (function index t) (values) ())
 
 #+sb-fasteval
 (defknown sb-interpreter:fun-proto-fn (interpreted-function)
