@@ -53,10 +53,11 @@
 
 ;;; For each named function with an XEP, note the definition of that
 ;;; name, and add derived type information to the INFO environment. We
-;;; also delete the FUNCTIONAL from *FREE-FUNS* to eliminate the
+;;; also delete the FUNCTIONAL from (FREE-FUNS *IR1-NAMESPACE*) to eliminate the
 ;;; possibility that new references might be converted to it.
 (defun finalize-xep-definition (fun)
   (let* ((leaf (functional-entry-fun fun))
+         (ns *ir1-namespace*)
          (defined-ftype (definition-type leaf)))
     (setf (leaf-type leaf) defined-ftype)
     (when (and (leaf-has-source-name-p leaf)
@@ -65,11 +66,11 @@
       (let ((source-name (leaf-source-name leaf)))
         (let* ((where (info :function :where-from source-name))
                (*compiler-error-context* (lambda-bind (main-entry leaf)))
-               (global-def (gethash source-name *free-funs*))
+               (global-def (gethash source-name (free-funs ns)))
                (global-p (defined-fun-p global-def)))
           (note-name-defined source-name :function)
           (when global-p
-            (remhash source-name *free-funs*))
+            (remhash source-name (free-funs ns)))
           (ecase where
             (:assumed
              (let ((approx-type (info :function :assumed-type source-name)))
@@ -84,7 +85,7 @@
                (setf (info :function :type source-name) defined-ftype))
              (setf (info :function :where-from source-name) :defined))
             ((:declared :defined-method)
-             (let ((declared-ftype (proclaimed-ftype source-name)))
+             (let ((declared-ftype (global-ftype source-name)))
                (unless (defined-ftype-matches-declared-ftype-p
                          defined-ftype declared-ftype)
                  (compiler-style-warn
@@ -104,7 +105,7 @@
 (defun note-assumed-types (component name var)
   (when (and (eq (leaf-where-from var) :assumed)
              (not (and (defined-fun-p var)
-                       (eq (defined-fun-inlinep var) :notinline)))
+                       (eq (defined-fun-inlinep var) 'notinline)))
              (eq (info :function :where-from name) :assumed)
              (eq (info :function :kind name) :function))
     (let ((atype (info :function :assumed-type name)))
@@ -143,10 +144,10 @@
                (delete-filter node lvar (cast-value node))))))))
 
 (defglobal *two-arg-functions*
-    '((* two-arg-*)
+    `((* two-arg-* (,(specifier-type 'fixnum) ,(specifier-type 'fixnum)) multiply-fixnums)
       (+ two-arg-+)
       (- two-arg--)
-      (/ two-arg-/)
+      (/ two-arg-/ (,(specifier-type 'integer) ,(specifier-type 'integer)) sb-kernel::integer-/-integer)
       (< two-arg-<)
       (= two-arg-=)
       (> two-arg->)
@@ -193,15 +194,38 @@
 (def-two-arg-funs (number number)
   >= <= /=)
 
+;;; A list of function which always call their functional argument correctly,
+;;; meaning that no arg-count-error can occur in the callee
+;;; and therefore the callee can skip the check.
+(dolist (fun '(sb-thread::call-with-mutex
+               sb-thread::call-with-recursive-lock
+               sb-thread::call-with-system-mutex
+               sb-thread::call-with-system-mutex/allow-with-interrupts
+               sb-thread::call-with-system-mutex/without-gcing
+               sb-thread::call-with-recursive-system-lock
+               ;; wish we had a little more commonality to these names
+               sb-impl::%with-standard-io-syntax
+               sb-impl::%with-rebound-io-syntax
+               sb-debug::funcall-with-debug-io-syntax
+               sb-impl::call-with-sane-io-syntax
+               ;; there are others, maybe %with-compilation-unit
+               sb-impl::%print-unreadable-object
+               ))
+  (let ((info (info :function :info fun)))
+    (when info
+      (setf (ir1-attributep (fun-info-attributes info) callee-omit-arg-count-check)
+            t))))
+
 ;;; Convert function designators to functions in calls to known functions
 ;;; Also convert to TWO-ARG- variants
 (defun ir1-optimize-functional-arguments (component)
   (do-blocks (block component)
     (do-nodes (node nil block)
       (when (and (combination-p node)
-                 (eq (combination-kind node) :known))
-        (let ((comination-name (lvar-fun-name (combination-fun node) t)))
-          (map-callable-arguments
+                 (eq (combination-kind node) :known)
+                 ;; REDUCE can call with zero arguments.
+                 (neq (lvar-fun-name (combination-fun node) t) 'reduce))
+        (map-callable-arguments
            (lambda (lvar args results &key no-function-conversion &allow-other-keys)
              (declare (ignore results))
              (unless no-function-conversion
@@ -209,17 +233,15 @@
                      (arg-count (length args)))
                  (labels ((translate-two-args (name)
                             (and (eql arg-count 2)
-                                 (neq comination-name 'reduce)
+                                 (not (fun-lexically-notinline-p name (node-lexenv node)))
                                  (cadr (assoc name *two-arg-functions*))))
                           (translate (ref)
                             (let* ((leaf (ref-leaf ref))
                                    (fun-name (and (constant-p leaf)
                                                   (constant-value leaf)))
                                    (replacement
-                                     (cond ((and fun-name
-                                                 (symbolp fun-name))
-                                            (or (translate-two-args fun-name)
-                                                fun-name))
+                                     (cond ((and fun-name (symbolp fun-name))
+                                            (translate-two-args fun-name))
                                            ((and (global-var-p leaf)
                                                  (eq (global-var-kind leaf) :global-function))
                                             (translate-two-args (global-var-%source-name leaf)))))
@@ -239,19 +261,48 @@
                                   (change-ref-leaf ref replacement :recklessly t)
                                   (setf (node-derived-type cast)
                                         (lvar-derived-type (cast-value cast)))))))))))))
-           node))))))
+           node)
+        ;; One more thing: builtin higher-order functions utilized by builtin macros
+        ;; can impart a policy change to the callee, but it can't (easily) be done
+        ;; strictly lexically, because the policy would leak downward.
+        ;; e.g. (WITH-SOME-STUFF () ..) ->
+        ;; -> (DX-FLET ((THUNK () <user-code>)) (CALL-WITH-STUFF #'THUNK))
+        ;; should not inject (OPTIMIZE (VERIFY-ARG-COUNT 0)) at the top of <user-code>
+        ;; because every lambda therein would omit the arg-count check.
+        (when (ir1-attributep (fun-info-attributes (combination-fun-info node))
+                              callee-omit-arg-count-check)
+          (let* ((args (combination-args node))
+                 (arg (if (eq (lvar-fun-name (combination-fun node) t)
+                              'sb-impl::%print-unreadable-object)
+                          (fourth args)
+                          (first args)))
+                 (ref (and arg (lvar-uses arg))))
+            (when (and (ref-p ref) (lambda-p (ref-leaf ref)))
+              ;; It seems impolite (un-debuggable too) to alter the lexenv-policy
+              ;; of functional-lexenv, so annotate this differently.
+              (setf (getf (functional-plist (ref-leaf ref)) 'verify-arg-count)
+                    nil))))))))
 
 (defun rewrite-full-call (combination)
-  (let ((comination-name (lvar-fun-name (combination-fun combination) t))
+  (let ((combination-name (lvar-fun-name (combination-fun combination) t))
         (args (combination-args combination)))
-    (let ((two-arg (cadr (assoc comination-name *two-arg-functions*)))
+    (let ((two-arg (assoc combination-name *two-arg-functions*))
           (ref (lvar-uses (combination-fun combination))))
       (when (and two-arg
                  (ref-p ref)
-                 (= (length args) 2))
-        (change-ref-leaf
-         ref
-         (find-free-fun two-arg "rewrite-full-call"))))))
+                 (= (length args) 2)
+                 (not (fun-lexically-notinline-p combination-name
+                                                 (node-lexenv combination))))
+        (destructuring-bind (name two-arg &optional types typed-two-arg) two-arg
+          (declare (ignore name))
+          (when (and types
+                     (loop for arg in args
+                           for type in types
+                           always (csubtypep (lvar-type arg) type)))
+            (setf two-arg typed-two-arg))
+          (change-ref-leaf
+           ref
+           (find-free-fun two-arg "rewrite-full-call")))))))
 
 ;;; Do miscellaneous things that we want to do once all optimization
 ;;; has been done:
@@ -273,7 +324,7 @@
 
   (maphash (lambda (k v)
              (note-assumed-types component k v))
-           *free-funs*)
+           (free-funs *ir1-namespace*))
 
   (ir1-merge-casts component)
   (ir1-optimize-functional-arguments component)
