@@ -20,7 +20,6 @@
 ;;;; Note: Take a look at the compiler-overview.tex section on "Hairy
 ;;;; function representation" before you seriously mess with this
 ;;;; stuff.
-#-sb-devel
 (declaim (start-block ir1-convert-lambda ir1-convert-lambda-body
                       ir1-convert-aux-bindings varify-lambda-arg
                       ir1-convert-lambdalike))
@@ -201,7 +200,8 @@
                                 debug-name
                                 (note-lexical-bindings t)
                                 post-binding-lexenv
-                                system-lambda)
+                                system-lambda
+                                local-policy)
   (declare (list body vars aux-vars aux-vals))
 
   ;; We're about to try to put new blocks into *CURRENT-COMPONENT*.
@@ -209,10 +209,13 @@
 
   (let* ((bind (make-bind))
          (lambda (make-lambda :vars vars
-                  :bind bind
-                  :%source-name source-name
-                  :%debug-name debug-name
-                  :system-lambda-p system-lambda))
+                              :bind bind
+                              :%source-name source-name
+                              :%debug-name debug-name
+                              :system-lambda-p system-lambda
+                              :lexenv (if local-policy
+                                          (make-lexenv :policy local-policy)
+                                          *lexenv*)))
          (result-ctran (make-ctran))
          (result-lvar (make-lvar)))
     ;; just to check: This function should fail internal assertions if
@@ -480,7 +483,9 @@
               (found-allow-p nil))
 
           (temps #-stack-grows-downward-not-upward
-                 `(,n-index (1- ,n-count))
+                 `(,n-index (+ ,n-count ,(if (vop-existsp :translate %more-keyword-pair)
+                                             0
+                                             -1)))
                  #+stack-grows-downward-not-upward
                  `(,n-index (- (1- ,n-count)))
                  #-stack-grows-downward-not-upward n-value-temp
@@ -536,7 +541,9 @@
                        (setq ,n-lose ,n-key))))
 
             (body
-             `(when (oddp ,n-count)
+             `(when (oddp ,(if (vop-existsp :translate %more-keyword-pair)
+                               n-index
+                               n-count))
                 (%odd-key-args-error)))
 
             (body
@@ -544,12 +551,19 @@
              `(locally
                 (declare (optimize (safety 0)))
                 (loop
-                  (when (minusp ,n-index) (return))
-                  (setf ,n-value-temp (%more-arg ,n-context ,n-index))
-                  (decf ,n-index)
-                  (setq ,n-key (%more-arg ,n-context ,n-index))
-                  (decf ,n-index)
-                  (case ,n-key ,@(tests))))
+                 ,@(cond ((vop-existsp :translate %more-keyword-pair)
+                          `((when (zerop ,n-index) (return))
+                            (decf ,n-index 2)
+                            (multiple-value-bind (key value)
+                                (%more-keyword-pair ,n-context ,n-index)
+                              (setf ,n-value-temp value ,n-key key))))
+                         (t
+                          `((when (minusp ,n-index) (return))
+                            (setf ,n-value-temp (%more-arg ,n-context ,n-index))
+                            (decf ,n-index)
+                            (setq ,n-key (%more-arg ,n-context ,n-index))
+                            (decf ,n-index))))
+                 (case ,n-key ,@(tests))))
              #+stack-grows-downward-not-upward
              `(locally (declare (optimize (safety 0)))
                 (loop
@@ -884,7 +898,8 @@
       (multiple-value-bind (vars keyp allow-other-keys aux-vars aux-vals)
           (make-lambda-vars (cadr form))
         (binding* (((*lexenv* result-type post-binding-lexenv
-                              lambda-list explicit-check source-form)
+                              lambda-list explicit-check source-form
+                              local-policy)
                     (process-decls decls (append aux-vars vars) nil
                                    :binding-form-p t :allow-lambda-list t))
                    (debug-catch-p (and maybe-add-debug-catch
@@ -913,7 +928,8 @@
                                                         :post-binding-lexenv post-binding-lexenv
                                                         :source-name source-name
                                                         :debug-name debug-name
-                                                        :system-lambda system-lambda)))))
+                                                        :system-lambda system-lambda
+                                                        :local-policy local-policy)))))
           (when explicit-check
             (setf (getf (functional-plist res) 'explicit-check) explicit-check))
           (setf (functional-inline-expansion res) (or source-form form))
@@ -1055,7 +1071,120 @@
             (allow (when (ll-kwds-allowp llks) '(&allow-other-keys))))
         (careful-specifier-type `(function (,@reqs ,@opts ,@rest ,@keys ,@allow) *))))))
 
-#-sb-devel
+(declaim (start-block maybe-inline-syntactic-closure))
+
+;;; Take the lexenv surrounding an inlined function and extract things
+;;; needed for the inline expansion suitable for dumping into fasls.
+;;; Right now it's MACROLET, SYMBOL-MACROLET, SPECIAL and
+;;; INLINE/NOTINLINE declarations. Upon encountering something else return NIL.
+;;; This is later used by PROCESS-INLINE-LEXENV to reproduce the lexenv.
+;;;
+;;; Previously it just used the functions and vars of the innermost
+;;; lexenv, but the body of macrolet can refer to other macrolets
+;;; defined earlier, so it needs to process all the parent lexenvs to
+;;; recover the proper order.
+(defun reconstruct-lexenv (lexenv)
+  (let (shadowed-funs
+        shadowed-vars
+        result)
+    (loop for env = lexenv then parent
+          for parent = (lexenv-parent env)
+          for vars = (lexenv-vars env)
+          for funs = (lexenv-funs env)
+          for declarations = nil
+          for symbol-macros = nil
+          for macros = nil
+          do
+          (loop for binding in vars
+                for (name . what) = binding
+                unless (and parent
+                            (find binding (lexenv-vars parent)))
+                do (typecase what
+                     (cons
+                      (aver (eq (car what) 'macro))
+                      (push name shadowed-vars)
+                      (push (list name (cdr what)) symbol-macros))
+                     (global-var
+                      (aver (eq (global-var-kind what) :special))
+                      (push `(special ,name) declarations))
+                     (t
+                      (unless (memq name shadowed-vars)
+                        (return-from reconstruct-lexenv)))))
+          (loop for binding in funs
+                for (name . what) = binding
+                unless (and parent
+                            (find binding (lexenv-funs parent)))
+                do
+                (typecase what
+                  (cons
+                   (push name shadowed-funs)
+                   (let ((expression (function-lambda-expression (cdr what))))
+                     (aver expression)
+                     (push (cons name expression) macros)))
+                  ;; FIXME: Is there a good reason for this not to be
+                  ;; DEFINED-FUN (which :INCLUDEs GLOBAL-VAR, in case
+                  ;; you're wondering how this ever worked :-)? Maybe
+                  ;; in conjunction with an AVERrance that it's not an
+                  ;; (AND GLOBAL-VAR (NOT GLOBAL-FUN))? -- CSR,
+                  ;; 2002-07-08
+                  (global-var
+                   (unless (defined-fun-p what)
+                     (return-from reconstruct-lexenv))
+                   (push `(,(car (defined-fun-inlinep what))
+                           ,name)
+                         declarations))
+                  (t
+                   (unless (memq name shadowed-funs)
+                     (return-from reconstruct-lexenv)))))
+          (when declarations
+            (setf result (list* :declare declarations (and result (list result)))))
+          (when symbol-macros
+            (setf result (list* :symbol-macro symbol-macros (and result (list result)))))
+          (when macros
+            (setf result (list* :macro macros (and result (list result)))))
+          while (and parent
+                     (not (null-lexenv-p parent))))
+    result))
+
+;;; Return a sexpr for LAMBDA in LEXENV such that loading it from fasl
+;;; preserves the original lexical environment for inlining.
+;;; Return NIL if the lexical environment is too complicated.
+(defun maybe-inline-syntactic-closure (lambda lexenv)
+  (declare (type list lambda) (type lexenv-designator lexenv))
+  (aver (eql (first lambda) 'lambda))
+  ;; We used to have a trivial implementation, verifying that lexenv
+  ;; was effectively null. However, this fails to take account of the
+  ;; idiom
+  ;;
+  ;; (declaim (inline foo))
+  ;; (macrolet ((def (x) `(defun ,x () ...)))
+  ;;   (def foo))
+  ;;
+  ;; which, while too complicated for the cross-compiler to handle in
+  ;; unfriendly foreign lisp environments, would be good to support in
+  ;; the target compiler. -- CSR, 2002-05-13 and 2002-11-02
+  (typecase lexenv
+   (lexenv
+    (let ((vars (lexenv-vars lexenv))
+          (funs (lexenv-funs lexenv)))
+      (acond ((or (lexenv-blocks lexenv) (lexenv-tags lexenv)) nil)
+             ((and (null vars) (null funs)) lambda)
+             ;; If the lexenv is too hairy for cross-compilation,
+             ;; you'll find out later, when trying to perform inlining.
+             ;; This is fine, because if the inline expansion is only
+             ;; for the target, it's totally OK to cross-compile this
+             ;; defining form. The syntactic env is correctly captured.
+             ((reconstruct-lexenv lexenv)
+              `(lambda-with-lexenv ,it ,@(cdr lambda))))))
+   #+(and sb-fasteval (not sb-xc-host))
+   (sb-interpreter:basic-env
+    (awhen (sb-interpreter::reconstruct-syntactic-closure-env lexenv)
+      `(lambda-with-lexenv ,it ,@(cdr lambda))))
+   #+sb-fasteval
+   (null lambda))) ; trivial case. Never occurs in the compiler.
+
+(declaim (end-block))
+
 (declaim (start-block ir1-convert-inline-lambda))
 
 ;;; Convert the forms produced by RECONSTRUCT-LEXENV to LEXENV
@@ -1124,6 +1253,7 @@
                 (lexenv-flushable *lexenv*)
                 lexenv-lambda
                 *lexenv*)))
+         (*inlining* (1+ *inlining*))
          (clambda (ir1-convert-lambda body
                                       :source-name source-name
                                       :debug-name debug-name
@@ -1138,40 +1268,38 @@
 ;;; previous references.
 (defun get-defined-fun (name &optional (lambda-list nil lp))
   (proclaim-as-fun-name name)
-  (when #+sb-xc-host (not *compile-time-eval*)
-        #-sb-xc-host (boundp '*ir1-namespace*)
-    (let ((found (find-free-fun name "shouldn't happen! (defined-fun)"))
-          (free-funs (free-funs *ir1-namespace*)))
-      (note-name-defined name :function)
-      (cond ((not (defined-fun-p found))
-             ;; This assertion is wrong in block compilation mode, for
-             ;; instance
-             ;;
-             ;; (defun foo (x) (bar x))
-             ;; (declaim (inline bar))
-             ;; (defun bar (x) x)
-             (aver (or (block-compile *compilation*)
-                       (not (info :function :inlinep name))))
-             (let* ((where-from (leaf-where-from found))
-                    (res (make-defined-fun
-                          :%source-name name
-                          :where-from (if (eq where-from :declared)
-                                          :declared
-                                          :defined-here)
-                          :type (if (eq :declared where-from)
-                                    (leaf-type found)
-                                    (or (and lp
-                                             (ignore-errors
-                                              (ftype-from-lambda-list lambda-list)))
-                                        (specifier-type 'function))))))
-               (substitute-leaf res found)
-               (setf (gethash name free-funs) res)))
-            ;; If FREE-FUNS has a previously converted definition
-            ;; for this name, then blow it away and try again.
-            ((defined-fun-functionals found)
-             (remhash name free-funs)
-             (get-defined-fun name lambda-list))
-            (t found)))))
+  (let ((found (find-free-fun name "shouldn't happen! (defined-fun)"))
+        (free-funs (free-funs *ir1-namespace*)))
+    (note-name-defined name :function)
+    (cond ((not (defined-fun-p found))
+           ;; This assertion is wrong in block compilation mode, for
+           ;; instance
+           ;;
+           ;; (defun foo (x) (bar x))
+           ;; (declaim (inline bar))
+           ;; (defun bar (x) x)
+           (aver (or (block-compile *compilation*)
+                     (not (info :function :inlinep name))))
+           (let* ((where-from (leaf-where-from found))
+                  (res (make-defined-fun
+                        :%source-name name
+                        :where-from (if (eq where-from :declared)
+                                        :declared
+                                        :defined-here)
+                        :type (if (eq :declared where-from)
+                                  (leaf-type found)
+                                  (or (and lp
+                                           (ignore-errors
+                                            (ftype-from-lambda-list lambda-list)))
+                                      (specifier-type 'function))))))
+             (substitute-leaf res found)
+             (setf (gethash name free-funs) res)))
+          ;; If FREE-FUNS has a previously converted definition
+          ;; for this name, then blow it away and try again.
+          ((defined-fun-functionals found)
+           (remhash name free-funs)
+           (get-defined-fun name lambda-list))
+          (t found))))
 
 ;;; Check a new global function definition for consistency with
 ;;; previous declaration or definition, and assert argument/result
@@ -1375,27 +1503,3 @@ is potentially harmful to any already-compiled callers using (SAFETY 0)."
             (specifier-type 'function))))
 
   (values))
-
-;; Similar to above, detect duplicate definitions within a file,
-;; but the package lock check is unnecessary - it's handled elsewhere.
-;;
-;; Additionally, this is a STYLE-WARNING, not a WARNING, because there is
-;; meaningful behavior that can be ascribed to some redefinitions, e.g.
-;;  (defmacro foo () first-definition)
-;;  (defun f () (use-it (foo )))
-;;  (defmacro foo () other-definition)
-;; will use the first definition when compiling F, but make the second available
-;; in the loaded fasl. In this usage it would have made sense to wrap the
-;; respective definitions with EVAL-WHEN for different situations,
-;; but as long as the compile-time behavior is deterministic, it's just bad style
-;; and not flat-out wrong, though there is indeed some waste in the fasl.
-;;
-;; KIND is the globaldb KIND of this NAME
-(defun %compiler-defmacro (kind name)
-  (let ((name-key `(,kind ,name)))
-    (when (boundp '*lexenv*)
-      (aver (producing-fasl-file))
-      ;; a slight OAOO issue here wrt %COMPILER-DEFUN
-      (if (member name-key (fun-names-in-this-file *compilation*) :test #'equal)
-          (compiler-style-warn 'same-file-redefinition-warning :name name)
-          (push name-key (fun-names-in-this-file *compilation*))))))
