@@ -14,6 +14,10 @@
 
 (declaim (special *compiler-error-bailout*))
 
+;;; the lexical environment we are currently converting in
+(defvar *lexenv*)
+(declaim (type lexenv *lexenv*))
+
 ;;; *CURRENT-FORM-NUMBER* is used in FIND-SOURCE-PATHS to compute the
 ;;; form number to associate with a source path. This should be bound
 ;;; to an initial value of 0 before the processing of each truly
@@ -23,7 +27,8 @@
 
 ;;; Report as we try each transform? (either source or IR)
 ;;; Bind to T to see the ones that didn't abort,
-;;; or :ALL if you want to see each attempted transform.
+;;; :ALL if you want to see each attempted transform.
+;;; :DERIVE-TYPE for newly derived types of combinations.
 (defvar *show-transforms-p* nil)
 
 (declaim (inline source-form-has-path-p))
@@ -34,18 +39,29 @@
   (when (source-form-has-path-p form)
     (gethash form *source-paths*)))
 
-(defvar *transforming* nil)
+(defvar *transforming* 0)
+(defvar *inlining* 0)
 
 (defun ensure-source-path (form)
-  (or (get-source-path form)
-      (if (and *transforming*
-               (not (memq 'transformed *current-path*)))
-          ;; Don't hide all the transformed paths, since we might want
-          ;; to look at the final form for error reporting.
-          (list* (simplify-source-path-form form)
-                 'transformed *current-path*)
-          (cons (simplify-source-path-form form)
-                *current-path*))))
+  (flet ((level> (value kind)
+           (let ((x (memq kind *current-path*)))
+             (> value
+                (if x
+                    (cadr x)
+                    0)))))
+    (or (get-source-path form)
+        (cond ((level> *transforming* 'transformed)
+               ;; Don't hide all the transformed paths, since we might want
+               ;; to look at the final form for error reporting.
+               (list* (simplify-source-path-form form)
+                      'transformed *transforming* *current-path*))
+              ;; Avoids notes about inlined code leaking out
+              ((level> *inlining* 'inlined)
+               (list* (simplify-source-path-form form)
+                      'inlined *inlining* *current-path*))
+              (t
+               (cons (simplify-source-path-form form)
+                     *current-path*))))))
 
 (defun simplify-source-path-form (form)
   (if (consp form)
@@ -64,6 +80,11 @@
     (setf (gethash form *source-paths*)
           (apply #'list* 'original-source-start *current-form-number* arguments))))
 
+(defun proper-list (form)
+  (if (proper-list-p form)
+      form
+      (compiler-error "~@<~S is not a proper list.~@:>" form)))
+
 ;;; *CURRENT-COMPONENT* is the COMPONENT structure which we link
 ;;; blocks into as we generate them. This just serves to glue the
 ;;; emitted blocks together until local call analysis and flow graph
@@ -100,6 +121,34 @@
 
 ;;;; namespace management utilities
 
+;;; Unpack (INFO :FUNCTION :INLINING-DATA FUN-NAME). If an explicit expansion
+;;; is not stored but FUN-NAME is a structure constructor, reconstitute the
+;;; expansion from the defstruct description. Secondary value is T if the expansion
+;;; was explicit. See SAVE-INLINE-EXPANSION-P for why we care.
+;;; (Essentially, once stored then always stored, lest inconsistency result)
+;;; If we have just a DXABLE-ARGS, or nothing at all, return NIL and NIL.
+;;; If called on a string or anything that is not a function designator,
+;;; return NIL and NIL.
+(declaim (ftype (sfunction (t) (values list boolean))
+                fun-name-inline-expansion))
+(defun fun-name-inline-expansion (fun-name)
+  (multiple-value-bind (answer winp) (info :function :inlining-data fun-name)
+    (typecase answer
+      ;; an INLINING-DATA is a DXABLE-ARGS, so test it first
+      (inlining-data (setq answer (inlining-data-expansion answer)))
+      (dxable-args   (setq answer nil winp nil)))
+    (when (and (not winp) (symbolp fun-name))
+      (let ((info (info :function :source-transform fun-name)))
+        (when (typep info '(cons defstruct-description (eql :constructor)))
+          (let* ((dd (car info)) (spec (assq fun-name (dd-constructors dd))))
+            (aver spec)
+            (setq answer `(lambda ,@(structure-ctor-lambda-parts dd (cdr spec))))))))
+    (values answer winp)))
+(defun fun-name-dx-args (fun-name)
+  (let ((answer (info :function :inlining-data fun-name)))
+    (when (typep answer 'dxable-args)
+      (dxable-args-list answer))))
+
 ;; As with LEXENV-FIND, we assume use of *LEXENV*, but macroexpanders
 ;; receive an explicit environment and should pass it.
 ;; A declaration will trump a proclamation.
@@ -119,9 +168,56 @@
     ;; If ANSWER is NIL, go for the global value
     (eq (or answer (info :function :inlinep name)) 'notinline)))
 
-#-sb-devel
+
+;;;; code coverage
+
+;;; Check the policy for whether we should generate code coverage
+;;; instrumentation. If not, just return the original START
+;;; ctran. Otherwise insert code coverage instrumentation after
+;;; START, and return the new ctran.
+(defun instrument-coverage (start mode form
+                            &aux (metadata (coverage-metadata *compilation*)))
+  ;; We don't actually use FORM for anything, it's just convenient to
+  ;; have around when debugging the instrumentation.
+  (declare (ignore form))
+  (if (and metadata
+           (policy *lexenv* (> store-coverage-data 0))
+           *allow-instrumenting*)
+      (let ((path (source-path-original-source *current-path*)))
+        (when mode
+          (push mode path))
+        (if (member (ctran-block start)
+                    (gethash path (code-coverage-blocks metadata)))
+            ;; If this source path has already been instrumented in
+            ;; this block, don't instrument it again.
+            start
+            (let ((next (make-ctran))
+                  (*allow-instrumenting* nil))
+              (ensure-gethash path (code-coverage-records metadata)
+                              (cons path +code-coverage-unmarked+))
+              (push (ctran-block start)
+                    (gethash path (code-coverage-blocks metadata)))
+              (ir1-convert start next nil `(%primitive mark-covered ',path))
+              next)))
+      start))
+
+;;; In contexts where we don't have a source location for FORM
+;;; e.g. due to it not being a cons, but where we have a source
+;;; location for the enclosing cons, use the latter source location if
+;;; available. This works pretty well in practice, since many PROGNish
+;;; macroexpansions will just directly splice a block of forms into
+;;; some enclosing form with `(progn ,@body), thus retaining the
+;;; EQness of the conses.
+(defun maybe-instrument-progn-like (start forms form)
+  (or (when (and *allow-instrumenting*
+                 (not (get-source-path form)))
+        (let ((*current-path* (get-source-path forms)))
+          (when *current-path*
+            (instrument-coverage start nil form))))
+      start))
+
 (declaim (start-block find-free-fun find-lexically-apparent-fun
-                      ;; needed by with-fun-name-leaf
+                      ;; needed by ir1-translators
                       find-global-fun))
 
 (defun maybe-defined-here (name where)
@@ -228,7 +324,17 @@
   (or (let ((old-free-fun (gethash name free-funs)))
         (when old-free-fun
           (clear-invalid-functionals old-free-fun)
-          old-free-fun))
+          (when (or (not (defined-fun-p old-free-fun))
+                    (not (block-compile *compilation*))
+                    ;; When block-compiling, it is the case that we
+                    ;; could proclaim an inline, define some things,
+                    ;; then proclaim a notinline, and in the case that
+                    ;; the function's :inlinep info changes, the
+                    ;; old-free-fun is stale and we should make a new
+                    ;; one.
+                    (eq (info :function :inlinep name)
+                        (defined-fun-inlinep old-free-fun)))
+            old-free-fun)))
       (let ((kind (info :function :kind name)))
         (ecase kind
           ((:macro :special-form)
@@ -324,67 +430,99 @@
 ;;; processed with MAKE-LOAD-FORM. We have to be careful, because
 ;;; CONSTANT might be circular. We also check that the constant (and
 ;;; any subparts) are dumpable at all.
-(defun maybe-emit-make-load-forms (constant &optional (name nil namep))
-  (let ((xset (alloc-xset)))
-    (labels ((grovel (value)
-               ;; Unless VALUE is an object which which obviously
-               ;; can't contain other objects
-               (unless (dumpable-leaflike-p value)
-                 (if (xset-member-p value xset)
-                     (return-from grovel nil)
-                     (add-to-xset value xset))
-                 (typecase value
-                   (cons
-                    (grovel (car value))
-                    (grovel (cdr value)))
-                   (simple-vector
-                    (dotimes (i (length value))
-                      (grovel (svref value i))))
-                   ((vector t)
-                    (dotimes (i (length value))
-                      (grovel (aref value i))))
-                   ((simple-array t)
-                    ;; Even though the (ARRAY T) branch does the exact
-                    ;; same thing as this branch we do this separately
-                    ;; so that the compiler can use faster versions of
-                    ;; array-total-size and row-major-aref.
-                    (dotimes (i (array-total-size value))
-                      (grovel (row-major-aref value i))))
-                   ((array t)
-                    (dotimes (i (array-total-size value))
-                      (grovel (row-major-aref value i))))
-                   (instance
-                    ;; In the target SBCL, we can dump any instance, but
-                    ;; in the cross-compilation host, %INSTANCE-FOO
-                    ;; functions don't work on general instances, only on
-                    ;; STRUCTURE!OBJECTs.
-                    ;;
-                    ;; Behold the wonderfully clear sense of this-
-                    ;;  WHEN (EMIT-MAKE-LOAD-FORM VALUE)
-                    ;; meaning "when you're _NOT_ using a custom load-form"
-                    ;;
-                    ;; FIXME: What about funcallable instances with
-                    ;; user-defined MAKE-LOAD-FORM methods?
-                    (when (emit-make-load-form value)
-                      #+sb-xc-host
-                      (aver (eql (layout-bitmap (%instance-layout value))
-                                 sb-kernel:+layout-all-tagged+))
-                      (do-instance-tagged-slot (i value)
-                        (grovel (%instance-ref value i)))))
-                   (t
-                    (compiler-error
-                      "Objects of type ~/sb-impl:print-type-specifier/ ~
-                       can't be dumped into fasl files."
-                     (type-of value)))))))
-      ;; Dump all non-trivial named constants using the name.
-      (if (and namep (not (sb-xc:typep constant '(or symbol character fixnum
-                                                  #+64-bit single-float))))
-          (emit-make-load-form constant name)
-          (grovel constant))))
+(defun maybe-emit-make-load-forms (constant)
+  (declare #-sb-xc-host (inline alloc-xset))
+  (dx-let ((xset (alloc-xset)))
+    (named-let grovel ((value constant))
+      ;; Unless VALUE is an object which which can't contain other objects,
+      ;; or was visited, or is acccessible in a named constant ...
+      (unless (or (dumpable-leaflike-p value)
+                  (xset-member-p value xset)
+                  (gethash value (eql-constants *ir1-namespace*)))
+        (add-to-xset value xset)
+        ;; FIXME: shouldn't this be something like SB-XC:TYPECASE ?
+        (typecase value
+          (cons
+           (grovel (car value))
+           (grovel (cdr value)))
+          (simple-vector
+           (dotimes (i (length value))
+             (grovel (svref value i))))
+          ((vector t)
+           (dotimes (i (length value))
+             (grovel (aref value i))))
+          ((simple-array t)
+           ;; Even though the (ARRAY T) branch does the exact
+           ;; same thing as this branch we do this separately
+           ;; so that the compiler can use faster versions of
+           ;; array-total-size and row-major-aref.
+           (dotimes (i (array-total-size value))
+             (grovel (row-major-aref value i))))
+          ((array t)
+           (dotimes (i (array-total-size value))
+             (grovel (row-major-aref value i))))
+          (instance
+           ;; In the target SBCL, we can dump any instance, but
+           ;; in the cross-compilation host, %INSTANCE-FOO
+           ;; functions don't work on general instances, only on
+           ;; STRUCTURE!OBJECTs.
+           ;;
+           ;; Behold the wonderfully clear sense of this-
+           ;;  WHEN (EMIT-MAKE-LOAD-FORM VALUE)
+           ;; meaning "when you're _NOT_ using a custom load-form"
+           ;;
+           ;; FIXME: What about funcallable instances with
+           ;; user-defined MAKE-LOAD-FORM methods?
+           (when (emit-make-load-form value)
+             #+sb-xc-host
+             (aver (eql (wrapper-bitmap (%instance-wrapper value))
+                        sb-kernel:+layout-all-tagged+))
+             (do-instance-tagged-slot (i value)
+               (grovel (%instance-ref value i)))))
+          (t
+           (compiler-error
+            "Objects of type ~/sb-impl:print-type-specifier/ can't be dumped into fasl files."
+            (type-of value)))))))
   (values))
+
+;;; A constant is trivially externalizable if it involves no INSTANCE types
+;;; or any un-dumpable object.
+(defun trivially-externalizable-p (constant)
+  (declare #-sb-xc-host (inline alloc-xset))
+  (dx-let ((xset (alloc-xset)))
+    (named-let ok ((value constant))
+      (if (or (dumpable-leaflike-p value) (xset-member-p value xset))
+          t
+          (progn
+            (add-to-xset value xset)
+            (typecase value
+             (cons (and (ok (car value)) (ok (cdr value))))
+             (vector
+              (dotimes (i (length value) t)
+                (unless (ok (aref value i)) (return nil))))
+             (array
+              (dotimes (i (array-total-size value) t)
+                (unless (ok (row-major-aref value i)) (return nil))))))))))
 
 ;;;; some flow-graph hacking utilities
 
+;; Bind *COMPILER-ERROR-BAILOUT* to a function that throws out of the
+;; body and converts a condition signalling form instead. The source
+;; form is converted to a string since it may contain arbitrary
+;; non-externalizable objects.
+(defmacro ir1-error-bailout ((start next result form) &body body)
+  (with-unique-names (skip condition)
+    `(block ,skip
+       (let ((,condition (catch 'ir1-error-abort
+                           (let ((*compiler-error-bailout*
+                                   (lambda (&optional e)
+                                     (throw 'ir1-error-abort e))))
+                             ,@body
+                             (return-from ,skip nil)))))
+         (ir1-convert ,start ,next ,result
+                      (make-compiler-error-form ,condition
+                                                ,form))))))
+
 ;;; This function sets up the back link between the node and the
 ;;; ctran which continues at it.
 (defun link-node-to-previous-ctran (node ctran)
@@ -424,21 +562,12 @@
 
 ;;; Insert NEW before OLD in the flow-graph.
 (defun insert-node-before (old new)
-  (let ((prev (node-prev old))
-        (temp (make-ctran)))
-    (ensure-block-start prev)
-    (setf (ctran-next prev) nil)
-    (link-node-to-previous-ctran new prev)
-    (use-ctran new temp)
-    (link-node-to-previous-ctran old temp))
-  (values))
-
-(defun insert-node-before-no-split (old new)
   (let ((prev (node-prev old))
         (temp (make-ctran)))
     (setf (ctran-next prev) nil)
     (link-node-to-previous-ctran new prev)
     (use-ctran new temp)
+    (setf (ctran-source-path temp) (ctran-source-path prev))
     (link-node-to-previous-ctran old temp))
   (values))
 
@@ -553,67 +682,55 @@
          (frob)
          (frob)
          (setq trail (cdr trail)))))))
+
 
 ;;;; IR1-CONVERT, macroexpansion and special form dispatching
-(declaim (ftype (sfunction (ctran ctran (or lvar null) t)
-                           (values))
-                ir1-convert))
-(macrolet (;; Bind *COMPILER-ERROR-BAILOUT* to a function that throws
-           ;; out of the body and converts a condition signalling form
-           ;; instead. The source form is converted to a string since it
-           ;; may contain arbitrary non-externalizable objects.
-           (ir1-error-bailout ((start next result form) &body body)
-             (with-unique-names (skip condition)
-               `(block ,skip
-                  (let ((,condition (catch 'ir1-error-abort
-                                      (let ((*compiler-error-bailout*
-                                              (lambda (&optional e)
-                                                (throw 'ir1-error-abort e))))
-                                        ,@body
-                                        (return-from ,skip nil)))))
-                    (ir1-convert ,start ,next ,result
-                                 (make-compiler-error-form ,condition
-                                                           ,form)))))))
 
-  ;; Translate FORM into IR1. The code is inserted as the NEXT of the
-  ;; CTRAN START. RESULT is the LVAR which receives the value of the
-  ;; FORM to be translated. The translators call this function
-  ;; recursively to translate their subnodes.
-  ;;
-  ;; As a special hack to make life easier in the compiler, a LEAF
-  ;; IR1-converts into a reference to that LEAF structure. This allows
-  ;; the creation using backquote of forms that contain leaf
-  ;; references, without having to introduce dummy names into the
-  ;; namespace.
-  (defun ir1-convert (start next result form)
-    (let* ((*current-path* (ensure-source-path form))
-           (start (instrument-coverage start nil form)))
-      (ir1-error-bailout (start next result form)
-        (cond ((atom form)
-               (cond ((and (symbolp form) (not (keywordp form)))
-                      (ir1-convert-var start next result form))
-                     ((leaf-p form)
-                      (reference-leaf start next result form))
-                     (t
-                      (reference-constant start next result form))))
-              ((not (proper-list-p form))
-               (compiler-error "~@<~S is not a proper list.~@:>" form))
-              (t
-               (ir1-convert-functoid start next result form)))))
-    (values))
+(declaim (start-block ir1-convert ir1-convert-progn-body
+                      ir1-convert-combination-args reference-leaf
+                      reference-constant
+                      expand-compiler-macro
+                      maybe-reanalyze-functional))
 
-  ;; Generate a reference to a manifest constant, creating a new leaf
-  ;; if necessary.
-  (defun reference-constant (start next result value)
-    (declare (type ctran start next)
-             (type (or lvar null) result))
-    (ir1-error-bailout (start next result value)
-      (let* ((leaf (find-constant value))
-             (res (make-ref leaf)))
-        (push res (leaf-refs leaf))
-        (link-node-to-previous-ctran res start)
-        (use-continuation res next result)))
-    (values)))
+;;; Translate FORM into IR1. The code is inserted as the NEXT of the
+;;; CTRAN START. RESULT is the LVAR which receives the value of the
+;;; FORM to be translated. The translators call this function
+;;; recursively to translate their subnodes.
+;;;
+;;; As a special hack to make life easier in the compiler, a LEAF
+;;; IR1-converts into a reference to that LEAF structure. This allows
+;;; the creation using backquote of forms that contain leaf
+;;; references, without having to introduce dummy names into the
+;;; namespace.
+(defun ir1-convert (start next result form)
+  (declare (type ctran start next)
+           (type (or lvar null) result))
+  (let* ((*current-path* (ensure-source-path form))
+         (start (instrument-coverage start nil form)))
+    (ir1-error-bailout (start next result form)
+                       (cond ((atom form)
+                              (cond ((and (symbolp form) (not (keywordp form)))
+                                     (ir1-convert-var start next result form))
+                                    ((leaf-p form)
+                                     (reference-leaf start next result form))
+                                    (t
+                                     (reference-constant start next result form))))
+                             (t
+                              (ir1-convert-functoid start next result form)))))
+  (values))
+
+;;; Generate a reference to a manifest constant, creating a new leaf
+;;; if necessary.
+(defun reference-constant (start next result value)
+  (declare (type ctran start next)
+           (type (or lvar null) result))
+  (ir1-error-bailout (start next result value)
+                     (let* ((leaf (find-constant value))
+                            (res (make-ref leaf)))
+                       (push res (leaf-refs leaf))
+                       (link-node-to-previous-ctran res start)
+                       (use-continuation res next result)))
+  (values))
 
 ;;; Add FUNCTIONAL to the COMPONENT-REANALYZE-FUNCTIONALS, unless it's
 ;;; some trivial type for which reanalysis is a trivial no-op.
@@ -786,7 +903,7 @@
   (let* ((op (car form))
          (translator (and (symbolp op) (info :function :ir1-convert op))))
     (if translator
-        (funcall translator start next result form)
+        (funcall translator start next result (proper-list form))
         (multiple-value-bind (res cmacro-fun-name)
                (expand-compiler-macro form)
              (cond ((eq res form)
@@ -901,9 +1018,10 @@
 
 ;;; Convert a bunch of forms, discarding all the values except the
 ;;; last. If there aren't any forms, then translate a NIL.
-(declaim (ftype (sfunction (ctran ctran (or lvar null) list) (values))
-                ir1-convert-progn-body))
 (defun ir1-convert-progn-body (start next result body)
+  (declare (type ctran start next)
+           (type (or lvar null) result)
+           (type list body))
   (if (endp body)
       (reference-constant start next result nil)
       (let ((this-start start)
@@ -922,70 +1040,6 @@
   (values))
 
 
-;;;; code coverage
-
-;;; Check the policy for whether we should generate code coverage
-;;; instrumentation. If not, just return the original START
-;;; ctran. Otherwise insert code coverage instrumentation after
-;;; START, and return the new ctran.
-(defun instrument-coverage (start mode form
-                            &aux (metadata (coverage-metadata *compilation*)))
-  ;; We don't actually use FORM for anything, it's just convenient to
-  ;; have around when debugging the instrumentation.
-  (declare (ignore form))
-  (if (and metadata
-           (policy *lexenv* (> store-coverage-data 0))
-           *allow-instrumenting*)
-      (let ((path (source-path-original-source *current-path*)))
-        (when mode
-          (push mode path))
-        (if (member (ctran-block start)
-                    (gethash path (code-coverage-blocks metadata)))
-            ;; If this source path has already been instrumented in
-            ;; this block, don't instrument it again.
-            start
-            (let ((store
-                   ;; Get an interned record cons for the path. A cons
-                   ;; with the same object identity must be used for
-                   ;; each instrument for the same block.
-                   (ensure-gethash path (code-coverage-records metadata)
-                                   (cons path +code-coverage-unmarked+)))
-                  (next (make-ctran))
-                  (*allow-instrumenting* nil))
-              #+(or x86-64 x86) (declare (ignore store)) ; eval'd for side-effect only
-              (push (ctran-block start)
-                    (gethash path (code-coverage-blocks metadata)))
-              (ir1-convert start next nil
-                           #+(or x86-64 x86)
-                           `(%primitive mark-covered ',path)
-                           #-(or x86-64 x86)
-                           `(locally
-                                (declare (optimize speed
-                                                   (safety 0)
-                                                   (debug 0)
-                                                   (check-constant-modification 0)))
-                              ;; We're being naughty here, and
-                              ;; modifying constant data. That's ok,
-                              ;; we know what we're doing.
-                              (%rplacd ',store t)))
-              next)))
-      start))
-
-;;; In contexts where we don't have a source location for FORM
-;;; e.g. due to it not being a cons, but where we have a source
-;;; location for the enclosing cons, use the latter source location if
-;;; available. This works pretty well in practice, since many PROGNish
-;;; macroexpansions will just directly splice a block of forms into
-;;; some enclosing form with `(progn ,@body), thus retaining the
-;;; EQness of the conses.
-(defun maybe-instrument-progn-like (start forms form)
-  (or (when (and *allow-instrumenting*
-                 (not (get-source-path form)))
-        (let ((*current-path* (get-source-path forms)))
-          (when *current-path*
-            (instrument-coverage start nil form))))
-      start))
-
 ;;;; converting combinations
 
 ;;; Does this form look like something that we should add single-stepping
@@ -1014,15 +1068,18 @@
 ;;; Convert a function call where the function FUN is a LEAF. FORM is
 ;;; the source for the call. We return the COMBINATION node so that
 ;;; the caller can poke at it if it wants to.
-(declaim (ftype (sfunction (ctran ctran (or lvar null) list leaf) combination)
-                ir1-convert-combination))
 (defun ir1-convert-combination (start next result form fun)
+  (declare (type ctran start next)
+           (type (or lvar null) result)
+           (type list form)
+           (type leaf fun)
+           #-sb-xc-host (values combination))
   (let ((ctran (make-ctran))
         (fun-lvar (make-lvar)))
     (ir1-convert start ctran fun-lvar `(the (or function symbol) ,fun))
     (let ((combination
            (ir1-convert-combination-args fun-lvar ctran next result
-                                         (cdr form))))
+                                         (cdr (proper-list form)))))
       (when (step-form-p form)
         ;; Store a string representation of the form in the
         ;; combination node. This will let the IR2 translator know
@@ -1043,7 +1100,7 @@
   (declare (type ctran start next)
            (type lvar fun-lvar)
            (type (or lvar null) result)
-           (list args))
+           (type list args))
   (let ((node (make-combination fun-lvar)))
     (setf (lvar-dest fun-lvar) node)
     (collect ((arg-lvars))
@@ -1068,10 +1125,6 @@
         (setf (combination-args node) (arg-lvars))))
     node))
 
-(defun show-transform (kind name new-form)
-  (let ((*print-right-margin* 100))
-    (format *trace-output* "~&xform (~a) ~S~% -> ~S~%"
-            kind name new-form)))
 ;;; Convert a call to a global function. If not NOTINLINE, then we do
 ;;; source transforms and try out any inline expansion. If there is no
 ;;; expansion, but is INLINE, then give an efficiency note (unless a
@@ -1093,15 +1146,22 @@
                       (struct-fun-transform transform form name))
                 ;; Note that "pass" means fail. Gotta love it.
                 (cond (pass
-                       (ir1-convert-maybe-predicate start next result form var))
+                       (ir1-convert-maybe-predicate start next result
+                                                    (proper-list form)
+                                                    var))
                       (t
                        (unless (policy *lexenv* (zerop store-xref-data))
                          (record-call name (ctran-block start) *current-path*))
-                       (when *show-transforms-p*
+                       (when (show-transform-p *show-transforms-p* name)
                          (show-transform "src" name transformed))
-                       (let ((*transforming* t))
+                       (let ((*transforming* (1+ *transforming*)))
+                         (when (eq (car *current-path*) 'original-source-start)
+                           (setf (ctran-source-path start) *current-path*))
                          (ir1-convert start next result transformed)))))
-              (ir1-convert-maybe-predicate start next result form var))))))
+              (ir1-convert-maybe-predicate start next result
+                                           (proper-list form)
+                                           var))))))
+
 
 ;;; KLUDGE: If we insert a synthetic IF for a function with the PREDICATE
 ;;; attribute, don't generate any branch coverage instrumentation for it.
@@ -1166,7 +1226,6 @@
 
 ;;;; PROCESS-DECLS
 
-#-sb-devel
 (declaim (start-block process-decls make-new-inlinep
                       find-in-bindings
                       process-muffle-decls
@@ -1176,9 +1235,9 @@
 ;;; LAMBDA-VAR for that name, or NIL if it isn't found. We return the
 ;;; *last* variable with that name, since LET* bindings may be
 ;;; duplicated, and declarations always apply to the last.
-(declaim (ftype (sfunction (list symbol) (or lambda-var list))
-                find-in-bindings))
 (defun find-in-bindings (vars name)
+  (declare (list vars) (symbol name)
+           #-sb-xc-host (values (or lambda-var list)))
   (let ((found nil))
     (dolist (var vars)
       (cond ((leaf-p var)
@@ -1631,6 +1690,18 @@
         (make-lexenv
          :default res
          :flushable (cdr spec)))
+       (current-defmethod
+        (destructuring-bind (name qualifiers specializers lambda-list)
+            (cdr spec)
+          (let* ((gfs (or *methods-in-compilation-unit*
+                          (setf *methods-in-compilation-unit*
+                                (make-hash-table :test #'equal))))
+                 (methods (or (gethash name gfs)
+                              (setf (gethash name gfs)
+                                    (make-hash-table :test #'equal)))))
+            (setf (gethash (cons qualifiers specializers) methods)
+                  lambda-list)))
+        res)
        (no-constraints
         (process-no-constraints-decl spec vars)
         res)
@@ -1680,6 +1751,7 @@
         (allow-explicit-check allow-lambda-list)
         (lambda-list (if allow-lambda-list :unspecified nil))
         (optimize-qualities)
+        (local-optimize)
         source-form
         (post-binding-lexenv (if binding-form-p (list nil)))) ; dummy cell
     (flet ((process-it (spec decl)
@@ -1722,6 +1794,12 @@
                    ((equal spec '(top-level-form))) ; ignore
                    ((typep spec '(cons (eql source-form)))
                     (setf source-form (cadr spec)))
+                   ;; Used only for the current function.
+                   ;; E.g. suppressing argument checking without doing
+                   ;; so in all the subforms.
+                   ((typep spec '(cons (eql local-optimize)))
+                    (setf local-optimize spec)
+                    (setf source-form (cadr spec)))
                    (t
                     (multiple-value-bind (new-env new-qualities)
                         (process-1-decl spec lexenv vars fvars
@@ -1737,8 +1815,11 @@
               ;; Kludge: EVAL calls this function to deal with LOCALLY.
               (process-it spec decl)))))
     (warn-repeated-optimize-qualities (lexenv-policy lexenv) optimize-qualities)
+
     (values lexenv result-type (cdr post-binding-lexenv)
-            lambda-list explicit-check source-form)))
+            lambda-list explicit-check source-form
+            (when local-optimize
+              (process-optimize-decl local-optimize (lexenv-policy lexenv))))))
 
 (defun %processing-decls (decls vars fvars ctran lvar binding-form-p fun)
   (multiple-value-bind (*lexenv* result-type post-binding-lexenv)
