@@ -19,8 +19,8 @@
 ;;; For blocks it's a cons with (pred . succ)
 ;;; For labels it maps to the label block
 (defvar *2block-info*)
-(defmacro vop-predecessors (vop) `(car (gethash ,vop *2block-info*)))
-(defmacro vop-successors (vop) `(cdr (gethash ,vop *2block-info*)))
+(defmacro ir2block-predecessors (x) `(car (gethash (the ir2-block ,x) *2block-info*)))
+(defmacro ir2block-successors (x) `(cdr (gethash (the ir2-block ,x) *2block-info*)))
 
 (defun initialize-ir2-blocks-flow-info (component)
   (labels ((block-last-2block (block)
@@ -67,7 +67,7 @@
                 (remove 2block (car info)))))
       (setf (cdr info) succ)
       (dolist (new succ)
-        (pushnew 2block (vop-predecessors new))))))
+        (pushnew 2block (ir2block-predecessors new))))))
 
 ;;;; Conditional move insertion support code
 (defun move-value-target (2block)
@@ -110,6 +110,11 @@
                (values (block-label (car succ-a))
                        target value-a value-b)))))))
 
+#-x86-64
+(defun sb-vm::computable-from-flags-p (res x y flags)
+  (declare (ignorable res x y flags))
+  nil)
+
 ;; To convert a branch to a conditional move:
 ;; 1. Convert both possible values to the chosen common representation
 ;; 2. Execute the conditional VOP
@@ -122,13 +127,24 @@
                          target res
                          flags info
                          label
-                         vop node 2block)
-  (let ((prev (vop-prev vop)))
-    (delete-vop vop)
-    (flet ((reuse-if-eq-arg (value-if vop)
+                         vop node 2block
+                         &aux (prev (vop-prev vop)))
+  (delete-vop vop)
+  (cond
+    ((and (constant-tn-p arg-if)
+          (constant-tn-p arg-else)
+          (sb-vm::computable-from-flags-p
+           res (tn-value arg-if) (tn-value arg-else) flags))
+     (emit-template node 2block (template-or-lose 'sb-vm::compute-from-flags)
+                    (reference-tn-list (list arg-if arg-else) nil)
+                    (reference-tn res t)
+                    (list* flags info)))
+    (t
+     (flet ((reuse-if-eq-arg (value-if vop)
              ;; Most of the time this means:
              ;; if X is already NIL, don't load it again.
-             (when (and (eq (vop-name vop) 'if-eq)
+             (when (and vop
+                        (eq (vop-name vop) 'if-eq)
                         (constant-tn-p value-if))
                (let* ((args (vop-args vop))
                       (x-tn (tn-ref-tn args))
@@ -139,84 +155,74 @@
                             (eq (tn-primitive-type x-tn)
                                 (tn-primitive-type res)))
                    x-tn))))
-           (load-and-coerce (dst src)
+            (load-and-coerce (dst src)
              (when (and dst (neq dst src))
                (emit-and-insert-vop node 2block
                                     (template-or-lose 'move)
                                     (reference-tn src nil)
                                     (reference-tn dst t)
                                     (ir2-block-last-vop 2block)))))
-      (let ((reuse (reuse-if-eq-arg value-if prev)))
-        (if reuse
-            (setf arg-if reuse)
-            (load-and-coerce arg-if   value-if)))
-      (load-and-coerce arg-else value-else))
-    (emit-template node 2block (template-or-lose cmove-vop)
-                   (reference-tn-list (remove nil (list arg-if arg-else))
-                                      nil)
-                   (reference-tn res t)
-                   (list* flags info))
+       (let ((reuse (reuse-if-eq-arg value-if prev)))
+         (if reuse
+             (setf arg-if reuse)
+             (load-and-coerce arg-if   value-if)))
+       (load-and-coerce arg-else value-else))
+     (emit-template node 2block (template-or-lose cmove-vop)
+                    (reference-tn-list (remove nil (list arg-if arg-else))
+                                       nil)
+                    (reference-tn res t)
+                    (list* flags info))))
     (emit-move node 2block res target)
     (vop branch node 2block label)
-    (update-block-succ 2block (list label))))
+    (update-block-succ 2block (list label)))
 
-;; Since conditional branches are always at the end of blocks,
-;; it suffices to look at the last VOP in each block.
-(defun maybe-convert-one-cmov (2block)
-  (let ((vop (or (ir2-block-last-vop 2block)
-                 (return-from maybe-convert-one-cmov))))
-    (unless (eq (vop-name vop) 'branch-if)
-      (return-from maybe-convert-one-cmov))
-    ;; The test and branch-if may be split between two IR1 blocks
-    ;; due to cleanups, can't use bloc-succ of the ir2-block-block
-    (let* ((node (vop-node vop))
-           (succ (block-succ (node-block node)))
-           (a    (first succ))
-           (b    (second succ)))
+(defun maybe-convert-one-cmov (vop)
+  ;; The test and branch-if may be split between two IR1 blocks
+  ;; due to cleanups, can't use bloc-succ of the ir2-block-block
+  (let* ((node (vop-node vop))
+         (succ (block-succ (node-block node)))
+         (a    (first succ))
+         (b    (second succ)))
 
-      (destructuring-bind (jump-target not-p flags) (vop-codegen-info vop)
-        (multiple-value-bind (label target value-a value-b)
-            (cmovp jump-target a b)
-          (unless label
+    (destructuring-bind (jump-target not-p flags) (vop-codegen-info vop)
+      (multiple-value-bind (label target value-a value-b)
+          (cmovp jump-target a b)
+        (unless label
+          (return-from maybe-convert-one-cmov))
+        (multiple-value-bind (cmove-vop arg-a arg-b res info)
+            (convert-conditional-move-p node target value-a value-b)
+          (unless cmove-vop
             (return-from maybe-convert-one-cmov))
-          (multiple-value-bind (cmove-vop arg-a arg-b res info)
-              (convert-conditional-move-p node target value-a value-b)
-            (unless cmove-vop
-              (return-from maybe-convert-one-cmov))
-            (when not-p
-              (rotatef value-a value-b)
-              (rotatef arg-a arg-b))
-            (flet ((safe-coercion-p (from to)
-                     (let ((from (tn-primitive-type from))
-                           (to (tn-primitive-type to)))
-                       ;; These moves will be repositioned before the test VOP,
-                       ;; which may be restricting their type.
-                       ;; Avoid the moves that may touch memory and
-                       ;; thus fail on immediate values.
-                       (not (and (eq from *backend-t-primitive-type*)
-                                 (memq (primitive-type-name to)
-                                       '(#+64-bit sb-vm::unsigned-byte-64
-                                         #+64-bit sb-vm::unsigned-byte-63
-                                         #+64-bit sb-vm::signed-byte-64
-                                         #-64-bit sb-vm::unsigned-byte-32
-                                         #-64-bit sb-vm::unsigned-byte-31
-                                         #-64-bit sb-vm::signed-byte-32
-                                         #-64-bit single-float
-                                         double-float
-                                         complex-single-float
-                                         complex-double-float
-                                         system-area-pointer)))))))
-              (if (and (safe-coercion-p value-a arg-a)
+          (when not-p
+            (rotatef value-a value-b)
+            (rotatef arg-a arg-b))
+          (flet ((safe-coercion-p (from to)
+                   (let ((from (tn-primitive-type from))
+                         (to (tn-primitive-type to)))
+                     ;; These moves will be repositioned before the test VOP,
+                     ;; which may be restricting their type.
+                     ;; Avoid the moves that may touch memory and
+                     ;; thus fail on immediate values.
+                     (not (and (eq from *backend-t-primitive-type*)
+                               (memq (primitive-type-name to)
+                                     '(#+64-bit sb-vm::unsigned-byte-64
+                                       #+64-bit sb-vm::unsigned-byte-63
+                                       #+64-bit sb-vm::signed-byte-64
+                                       #-64-bit sb-vm::unsigned-byte-32
+                                       #-64-bit sb-vm::unsigned-byte-31
+                                       #-64-bit sb-vm::signed-byte-32
+                                       #-64-bit single-float
+                                       double-float
+                                       complex-single-float
+                                       complex-double-float
+                                       system-area-pointer)))))))
+            (when (and (safe-coercion-p value-a arg-a)
                        (safe-coercion-p value-b arg-b))
-                  (convert-one-cmov cmove-vop value-a arg-a
-                                    value-b arg-b
-                                    target  res
-                                    flags info
-                                    label vop node 2block)))))))))
-
-(defun convert-cmovs (component)
-  (do-ir2-blocks (2block component (values))
-    (maybe-convert-one-cmov 2block)))
+              (convert-one-cmov cmove-vop value-a arg-a
+                                value-b arg-b
+                                target  res
+                                flags info
+                                label vop node (vop-block vop)))))))))
 
 (defun delete-unused-ir2-blocks (component)
   (declare (type component component))
@@ -283,7 +289,23 @@
           (let ((x (tn-ref-tn args))
                 (y (tn-ref-tn results)))
             (when (location= x y)
-              (delete-vop vop)))))))))
+              (delete-vop vop)
+              ;; Deleting the copy may make it look like that register
+              ;; is not used anywhere else and some optimizations,
+              ;; like combine-instructions, may incorrectly trigger.
+              ;; FIXME: these tn-refs never go away, so things like
+              ;; combine-instructions should use a different mechanism
+              ;; for checking writes/reads.
+              (let ((x-reads (tn-reads x))
+                    (x-writes (tn-writes x)))
+                (when (tn-reads y)
+                  (reference-tn x nil))
+                (when (tn-writes y)
+                  (reference-tn x t))
+                (when x-reads
+                  (reference-tn y nil))
+                (when x-writes
+                  (reference-tn y t)))))))))))
 
 ;;; Unchain BRANCHes that jump to a BRANCH.
 ;;; Remove BRANCHes that are jumped over by BRANCH-IF
@@ -421,16 +443,34 @@
 
 (defun next-vop (vop)
   (or (vop-next vop)
-      (let ((next-block (ir2-block-next (vop-block vop))))
-        (and (not (or (ir2-block-%trampoline-label next-block)
-                      (ir2-block-%label next-block)))
-             (ir2-block-start-vop next-block)))))
+      (do ((2block (ir2-block-next (vop-block vop))
+                   (ir2-block-next 2block)))
+          ((null 2block) nil)
+        (cond ((or (ir2-block-%trampoline-label 2block)
+                   (ir2-block-%label 2block))
+               (return))
+              ((ir2-block-start-vop 2block)
+               (return (ir2-block-start-vop 2block)))))))
+
+(defun prev-vop (vop)
+  (or (vop-prev vop)
+      (do* ((2block (vop-block vop) prev)
+            (prev (ir2-block-prev 2block)
+                  (ir2-block-prev 2block)))
+           ((null prev) nil)
+        (cond ((or (ir2-block-%trampoline-label 2block)
+                   (ir2-block-%label 2block))
+               (return))
+              ((ir2-block-last-vop prev)
+               (return (ir2-block-last-vop prev)))))))
 
 (defun immediate-templates (fun &optional (constants t))
   (let ((primitive-types (list (primitive-type-or-lose 'character)
                                (primitive-type-or-lose 'fixnum)
                                (primitive-type-or-lose 'sb-vm::positive-fixnum)
+                               #-x86 ;; i387 is weird
                                (primitive-type-or-lose 'double-float)
+                               #-x86
                                (primitive-type-or-lose 'single-float)
                                .
                                #+(or 64-bit 64-bit-registers)
@@ -482,35 +522,41 @@
                args1)
            args2)))
 
-;;; Turn CMP X,Y BRANCH-IF M CMP X,Y BRANCH-IF N
-;;; into CMP X,Y BRANCH-IF M BRANCH-IF N
-;; while it's portable the VOPs are not validated for
-;; compatibility on other backends yet.
-#+(or arm arm64 x86 x86-64)
 (defoptimizer (vop-optimize branch-if) (branch-if)
-  (let ((prev (vop-prev branch-if)))
-    (when (and prev
-               (memq (vop-name prev) *comparison-vops*))
-      (let ((next (next-vop branch-if))
-            transpose)
-        (when (and next
-                   (memq (vop-name next) *comparison-vops*)
-                   (or (vop-args-equal prev next)
-                       (and (or (setf transpose
-                                      (memq (vop-name prev) *commutative-comparison-vops*))
-                                (memq (vop-name next) *commutative-comparison-vops*))
-                            (vop-args-equal prev next t))))
-          (when transpose
-            ;; Could flip the flags for non-commutative operations
-            (loop for tn-ref = (vop-args prev) then (tn-ref-across tn-ref)
-                  for arg in (nreverse (vop-arg-list prev))
-                  do (change-tn-ref-tn tn-ref arg)))
-          (delete-vop next))))))
+  (cond ((boundp '*2block-info*)
+         (maybe-convert-one-cmov branch-if))
+        #+(or arm arm64 x86 x86-64)
+        (t
+         ;; Turn CMP X,Y BRANCH-IF M CMP X,Y BRANCH-IF N
+         ;; into CMP X,Y BRANCH-IF M BRANCH-IF N
+         ;; Run it after CMOVs are converted.
+         ;; While it's portable the VOPs are not validated for
+         ;; compatibility on other backends yet.
+         (let ((prev (vop-prev branch-if)))
+           (when (and prev
+                      (memq (vop-name prev) *comparison-vops*))
+             (let ((next (next-vop branch-if))
+                   transpose)
+               (when (and next
+                          (memq (vop-name next) *comparison-vops*)
+                          (or (vop-args-equal prev next)
+                              (and (or (setf transpose
+                                             (memq (vop-name prev) *commutative-comparison-vops*))
+                                       (memq (vop-name next) *commutative-comparison-vops*))
+                                   (vop-args-equal prev next t))))
+                 (when transpose
+                   ;; Could flip the flags for non-commutative operations
+                   (loop for tn-ref = (vop-args prev) then (tn-ref-across tn-ref)
+                         for arg in (nreverse (vop-arg-list prev))
+                         do (change-tn-ref-tn tn-ref arg)))
+                 (setf (sb-assem::label-comment (car (vop-codegen-info branch-if)))
+                       :merged-ifs)
+                 (delete-vop next))))))))
 
 (defun next-start-vop (block)
   (loop for 2block = block then (ir2-block-next 2block)
         while (and 2block
-                   (= (length (vop-predecessors 2block)) 1))
+                   (= (length (ir2block-predecessors 2block)) 1))
         when (ir2-block-start-vop 2block) return it))
 
 (defun branch-destination (branch &optional (true t))
@@ -520,6 +566,70 @@
         (next-start-vop (ir2-block-next (vop-block branch)))
         (next-start-vop (gethash label *2block-info*)))))
 
+;;; Replace (BOUNDP X) + (BRANCH-IF) + {(SYMBOL-VALUE X) | anything}
+;;; by (FAST-SYMBOL-VALUE X) + (UNBOUND-MARKER-P) + BRANCH-IF
+;;; and delete SYMBOL-VALUE, with the result of FAST-SYMBOL-VALUE flowing
+;;; into the same TN as symbol-value's result.
+;;; This is a valid substitution on any of the architectures, but
+;;; it requires that UNBOUND-MARKER-P return its result in a flag,
+;;; so presently is enabled only for x86-64.
+#+x86-64
+(defoptimizer (vop-optimize boundp) (vop)
+  (let ((sym (tn-ref-tn (vop-args vop)))
+        (next (vop-next vop)))
+    (unless (and (boundp '*2block-info*)
+                 next (eq (vop-name next) 'branch-if))
+      (return-from vop-optimize-boundp-optimizer nil))
+    ;; Only replace if the BOUNDP=T consequent starts with SYMBOL-VALUE so that
+    ;; a bad example such as (IF (BOUNDP '*FOO*) (PRINT 'HI) *FOO*)
+    ;; - which has SYMBOL-VALUE as the wrong consequent of the IF - is unaffected.
+    (let* ((successors (ir2block-successors (vop-block next)))
+           (info (vop-codegen-info next))
+           (label-block (gethash (car info) *2block-info*))
+           (symeval-vop
+            (ir2-block-start-vop
+             (cond ((eq (second info) nil) label-block) ; not negated test.
+                   ;; When negated, the BOUNDP case goes to the "other" successor
+                   ;; block, i.e. whichever is not started by the #<label>.
+                   ((eq label-block (first successors)) (second successors))
+                   (t (first successors))))))
+      (when (and symeval-vop
+                 (or (eq (vop-name symeval-vop) 'symbol-value)
+                     ;; Expect SYMBOL-GLOBAL-VALUE only for variables of kind :GLOBAL.
+                     (and (eq (vop-name symeval-vop) 'symbol-global-value)
+                          (constant-tn-p sym)
+                          (eq (info :variable :kind (tn-value sym)) :global)))
+                 (eq sym (tn-ref-tn (vop-args symeval-vop)))
+                 ;; If the symbol is either a compile-time known symbol,
+                 ;; or can only be the same thing for both vops,
+                 ;; then we're fine to combine.
+                 (or (constant-tn-p sym)
+                     (not (tn-writes sym))
+                     (not (tn-ref-next (tn-writes sym))))
+                 ;; Elide the SYMBOL-VALUE only if there is exactly one way to get there.
+                 ;; Technically we could split the IR2 block and peel off the SYMBOL-VALUE,
+                 ;; and if coming from the BRANCH-IF, jump to the block that contained
+                 ;; everything else except the SYMBOL-VALUE vop.
+                 (not (cdr (ir2block-predecessors (vop-block symeval-vop)))))
+        (let ((replacement (if (eq (vop-name symeval-vop) 'symbol-global-value)
+                               'fast-symbol-global-value
+                               'fast-symbol-value))
+              (result-tn (tn-ref-tn (vop-results symeval-vop))))
+          (emit-and-insert-vop (vop-node vop) (vop-block vop)
+                               (template-or-lose replacement)
+                               (reference-tn sym nil) (reference-tn result-tn t) next)
+          (emit-and-insert-vop (vop-node vop) (vop-block vop)
+                               (template-or-lose 'unbound-marker-p)
+                               (reference-tn result-tn nil) nil next)
+          ;; We need to invert the test since BOUNDP and UNBOUND-MARKER-P have
+          ;; the opposite meaning in the language. But by chance, they specify
+          ;; opposite flags in the vop, which means that the sense of the
+          ;; BRANCH-IF test is automagically correct after substitution.
+          ;; Of course, this is machine-dependent.
+          (delete-vop vop)
+          (delete-vop symeval-vop)
+          next)))))
+
 ;;; Optimize (if x ... nil) to reuse the NIL coming from X.
 (defoptimizer (vop-optimize if-eq) (if-eq)
   (when (boundp '*2block-info*)
@@ -647,7 +757,7 @@
                       (tn-value (tn-ref-tn (tn-ref-across (vop-args conditional))))
                       ;; "-eq-/C" vops take a codegen arg
                       (car (vop-codegen-info conditional))))
-             (else-block-predecessors (vop-predecessors else-block))
+             (else-block-predecessors (ir2block-predecessors else-block))
              (else-vop (ir2-block-start-vop else-block)))
          (push (cons val then-block) chain)
          ;; If ELSE block has more than one predecessor, that's OK,
@@ -697,7 +807,7 @@
     (let ((values (mapcar #'car choices)))
       (cond ((every #'fixnump values)) ; ok
             ((every #'characterp values)
-             (setq values (mapcar #'sb-xc:char-code values)))
+             (setq values (mapcar #'char-code values)))
             (t
              (return-from should-use-jump-table-p nil)))
       (let* ((min (reduce #'min values))
@@ -738,7 +848,7 @@
           (destructuring-bind (clauses else-block test-vop-name) culled-chain
             (let* ((key-type (if (characterp (caar clauses)) 'character 'fixnum))
                    (clause-keyfn (if (eq key-type 'character)
-                                     (lambda (x) (sb-xc:char-code (car x)))
+                                     (lambda (x) (char-code (car x)))
                                      #'car))
                    ;; Sort and unzip the alist
                    (ordered (sort (copy-list clauses) #'< :key clause-keyfn))
@@ -825,13 +935,85 @@
                            (vop-codegen-info vop))
         (delete-vop vop)))))
 
+#+(or arm64 x86-64)
+(defoptimizer (vop-optimize #+arm64
+                            (sb-vm::data-vector-ref/simple-bit-vector-c
+                             sb-vm::data-vector-ref/simple-bit-vector)
+                            #+x86-64
+                            (sb-vm::data-vector-ref-with-offset/simple-bit-vector-c
+                             sb-vm::data-vector-ref-with-offset/simple-bit-vector))
+    (vop)
+  (let* ((next (next-vop vop))
+         (branch (and next
+                      (next-vop next)))
+         plusp)
+    (when (and branch
+               (or
+                (eq (vop-name next) 'sb-vm::fast-if-eq-fixnum/c)
+                (and (eq (vop-name next) 'sb-vm::fast-if->-c/fixnum)
+                     (equal (vop-codegen-info next) '(0))
+                     (setf plusp t)))
+               (eq (vop-name branch)
+                   'branch-if))
+      (let* ((result (tn-ref-tn (vop-results vop)))
+             (value (if plusp
+                        1
+                        (car (vop-codegen-info next)))))
+        (when (and (not (tn-ref-next (tn-reads result)))
+                   (eq result (tn-ref-tn (vop-args next))))
+          (check-type value bit)
+          (let ((template (template-or-lose #+arm64
+                                            (if (eq (vop-name vop) 'sb-vm::data-vector-ref/simple-bit-vector)
+                                                'sb-vm::data-vector-ref/simple-bit-vector-eq
+                                                'sb-vm::data-vector-ref/simple-bit-vector-c-eq)
+                                            #+x86-64
+                                            (if (eq (vop-name vop) 'sb-vm::data-vector-ref-with-offset/simple-bit-vector)
+                                                'sb-vm::data-vector-ref-with-offset/simple-bit-vector-eq
+                                                'sb-vm::data-vector-ref-with-offset/simple-bit-vector-c-eq))))
+            (prog1
+                (emit-and-insert-vop (vop-node vop)
+                                     (vop-block vop)
+                                     template
+                                     (reference-tn-refs (vop-args vop) nil)
+                                     nil
+                                     vop
+                                     (vop-codegen-info vop))
+              ;; copy the condition flag
+              (setf (third (vop-codegen-info branch))
+                    (cdr (template-result-types template)))
+              (when (eq value 1)
+                (setf (second (vop-codegen-info branch))
+                      (not (second (vop-codegen-info branch)))))
+              (delete-vop vop)
+              (delete-vop next))))))))
+
+;;; No need to reset the stack pointer just before returning.
+(defoptimizer (vop-optimize reset-stack-pointer) (vop)
+  (loop for next = (next-vop vop) then (next-vop next)
+        do (cond ((not next)
+                  (return))
+                 ((eq (vop-name next) 'move))
+                 ((memq (vop-name next) '(return-single return known-return
+                                          tail-call tail-call-named
+                                          static-tail-call-named))
+                  (delete-vop vop)
+                  ;; Delete the VOP that saves the stack pointer too.
+                  (let ((tn (tn-ref-tn (vop-args vop))))
+                    (unless (tn-reads tn)
+                      (aver (eq (vop-name (tn-ref-vop (tn-writes tn)))
+                                'current-stack-pointer))
+                      (delete-vop (tn-ref-vop (tn-writes tn)))))
+                  (return))
+                 (t
+                  (return)))))
+
 (defun very-temporary-p (tn)
   (let ((writes (tn-writes tn))
         (reads (tn-reads tn)))
     (and writes reads (not (tn-ref-next writes)) (not (tn-ref-next reads)))))
 
 (defun next-vop-is (vop names)
-  (let ((next (vop-next vop)))
+  (let ((next (next-vop vop)))
     (and next
          (let ((name (vop-name next)))
            (if (atom names) (eq name names) (memq name names)))
@@ -847,8 +1029,6 @@
 ;;; Possibly replace HOWMANY vops starting at FIRST with a vop named REPLACEMENT.
 ;;; Each deleted vop must have exactly one argument and one result.
 ;;; - There must be no way to begin execution in the middle of the pattern.
-;;;   For now we'll require that the vops have the same IR2 block,
-;;;   which is a more restrictive condition.
 ;;; - The argument to each successive vop must be the result of its predecessor.
 ;;; - There must be no other refs to TNs which are to be deleted.
 ;;;
@@ -858,16 +1038,15 @@
 ;;; is in a register (so we get the load as part of the instruction).
 (defun replace-vops (howmany first replacement)
   (flet ((can-replace (vop &aux (result (tn-ref-tn (vop-results vop)))
-                                (next (vop-next vop)))
-           (and (eq (vop-block vop) (vop-block next))
-                (very-temporary-p result)
+                                (next (next-vop vop)))
+           (and (very-temporary-p result)
                 (eq (tn-ref-tn (vop-args next)) result))))
     (let ((last (ecase howmany
                   (2 (when (can-replace first)
-                       (vop-next first)))
+                       (next-vop first)))
                   (3 (when (and (can-replace first)
-                                (can-replace (vop-next first)))
-                       (vop-next (vop-next first)))))))
+                                (can-replace (next-vop first)))
+                       (next-vop (next-vop first)))))))
       (when last
         (let ((new (emit-and-insert-vop (vop-node first)
                                         (vop-block first)
@@ -876,7 +1055,7 @@
                                         (reference-tn (tn-ref-tn (vop-results last)) t)
                                         first))) ; insert before this
           (dotimes (i (1- howmany)) ; if 3, replace 2 "NEXT"'s and then first, etc
-            (delete-vop (vop-next first)))
+            (delete-vop (next-vop first)))
           (delete-vop first)
           new))))) ; return suitable value for RUN-VOP-OPTIMIZERS
 
@@ -891,6 +1070,52 @@
                             (funcall it vop))
                           (vop-next vop)))))))
 
+(defun merge-instance-set-vops (vop)
+  (let ((instance (tn-ref-tn (vop-args vop)))
+        (this vop)
+        (pairs))
+    (loop
+       (let ((index (tn-ref-tn (tn-ref-across (vop-args this)))))
+         (unless (constant-tn-p index) (return))
+         (push (cons (tn-value index) (tn-ref-tn (sb-vm::vop-nth-arg 2 this)))
+               pairs))
+       (let ((next (vop-next this)))
+         (unless (and next
+                      (eq (vop-name next) 'sb-vm::instance-index-set)
+                      (eq (tn-ref-tn (vop-args next)) instance))
+           (return))
+         (setq this next)))
+    (unless (cdr pairs) ; if at least 2
+      (return-from merge-instance-set-vops nil))
+    (setq pairs (nreverse pairs))
+    (let ((new (emit-and-insert-vop
+                (vop-node vop) (vop-block vop)
+                (template-or-lose 'sb-vm::instance-set-multiple)
+                (reference-tn-list (cons instance (mapcar #'cdr pairs)) nil)
+                nil vop (list (mapcar #'car pairs)))))
+      (loop (let ((next (vop-next vop)))
+              (delete-vop vop)
+              (pop pairs)
+              (setq vop next))
+            (unless pairs (return)))
+      new)))
+
+(defun ir2-optimize-stores (component)
+  ;; This runs after representation selection. It's the same as RUN-VOP-OPTIMIZERS,
+  ;; but with hardcoded vop names and function to call.
+  ;; It seems like this should also supplant the #+arm64 hack in GENERATE-CODE.
+  (do-ir2-blocks (block component)
+    (let ((vop (ir2-block-start-vop block)))
+      (loop (unless vop (return))
+            (let ((optimizer
+                   (case (vop-name vop)
+                     (sb-vm::instance-index-set
+                      (when (gethash 'sb-vm::instance-set-multiple
+                                     *backend-parsed-vops*)
+                        'merge-instance-set-vops)))))
+              (setq vop (or (awhen optimizer (funcall it vop))
+                            (vop-next vop))))))))
+
 (defun ir2-optimize (component)
   (let ((*2block-info* (make-hash-table :test #'eq)))
     (initialize-ir2-blocks-flow-info component)
@@ -904,8 +1129,22 @@
     ;; Look for if/else chains before cmovs, because a cmov
     ;; affects whether the last if/else is recognizable.
     #+(or ppc ppc64 x86 x86-64) (convert-if-else-chains component)
-    (convert-cmovs component)
     (run-vop-optimizers component)
     (delete-unused-ir2-blocks component))
 
   (values))
+
+(defun delete-unnecessary-move (vop)
+  (when (and (vop-next vop)
+             (eq (vop-name (vop-next vop)) 'move)
+             ;; the source of the move is the result of this
+             (eq (tn-ref-tn (vop-args (vop-next vop)))
+                 (tn-ref-tn (vop-results vop)))
+             ;; the destination of the move is the same as the input of this
+             (eq (tn-ref-tn (vop-results (vop-next vop)))
+                 (tn-ref-tn (vop-args vop)))
+             ;; there is exactly one write and one read of the intermediate TN
+             (very-temporary-p (tn-ref-tn (vop-results vop))))
+    ;; Change my result ref to the same TN as the input and delete the MOVE
+    (change-tn-ref-tn (vop-results vop) (tn-ref-tn (vop-args vop)))
+    (delete-vop (vop-next vop))))
