@@ -302,6 +302,8 @@
   (more-results nil :type (or operand-parse null))
   ;; a list of all the above together
   (operands nil :type list)
+  ;; Which results can accept :unused TNs
+  (optional-results nil :type list)
   ;; names of variables that should be declared IGNORE
   (ignores () :type list)
   ;; true if this is a :CONDITIONAL VOP. T if a branchful VOP,
@@ -371,7 +373,7 @@
 ;;; Order here is insignificant; it happens to be alphabetical.
 (defglobal vop-parse-slot-names
     '(arg-types args args-var before-load body conditional-p cost guard ignores info-args inherits
-      ltn-policy more-args more-results move-args name node-var note result-types
+      ltn-policy more-args more-results move-args name node-var note optional-results result-types
       results results-var save-p source-location temps translate variant variant-vars vop-var))
 ;; A sanity-check. Of course if this fails, the likelihood is that you can't even
 ;; get this far in cross-compilaion. So it's probably not worth much.
@@ -607,7 +609,7 @@
           :num-results ,num-results
           :ref-ordering ,ordering
           ,@(when (targets)
-              `(:targets ,(sb-xc:coerce (targets) `(vector ,te-type)))))))))
+              `(:targets ,(coerce (targets) `(vector ,te-type)))))))))
 
 (defun make-emit-function-and-friends (parse)
   `(:temps ,(compute-temporaries-description parse)
@@ -780,8 +782,10 @@
                                    `(,dummy)))
                     (ignorable ,n-args ,n-results))
            ,@(loads)
-           (assemble ()
-             ,@(vop-parse-body parse))
+           ;; RETURN-FROM can exit the ASSEMBLE while continuing on with saves.
+           (block ,(vop-parse-name parse)
+             (assemble ()
+               ,@(vop-parse-body parse)))
            ,@(saves))))))
 
 (defvar *parse-vop-operand-count*)
@@ -811,31 +815,35 @@
         (when more
           (error "The MORE operand isn't the last operand: ~S" specs))
         (incf *parse-vop-operand-count*)
+        (incf num)
         (let* ((name (first spec))
                (old (if (vop-parse-inherits parse)
                         (find-operand name
                                       (vop-parse-or-lose
                                        (vop-parse-inherits parse))
-                                      (list kind)
+                                      (list* kind
+                                             (if (eq kind :argument)
+                                                 '(:more-argument)
+                                                 '(:more-result)))
                                       nil)
                         nil))
                (res
-                (nconc (list :kind kind)
-                       (if old
-                        (list
-                         :target (operand-parse-target old)
-                         :born (operand-parse-born old)
-                         :dies (operand-parse-dies old)
-                         :scs (operand-parse-scs old)
-                         :load-tn (operand-parse-load-tn old)
-                         :load (operand-parse-load old))
-                        (ecase kind
-                          (:argument
-                           (list :born (parse-time-spec :load)
-                                 :dies (parse-time-spec `(:argument ,(incf num)))))
-                          (:result
-                           (list :born (parse-time-spec `(:result ,(incf num)))
-                                 :dies (parse-time-spec :save))))))))
+                 (nconc (list :kind kind)
+                        (if old
+                            (list
+                             :target (operand-parse-target old)
+                             :born (operand-parse-born old)
+                             :dies (operand-parse-dies old)
+                             :scs (operand-parse-scs old)
+                             :load-tn (operand-parse-load-tn old)
+                             :load (operand-parse-load old))
+                            (ecase kind
+                              (:argument
+                               (list :born (parse-time-spec :load)
+                                     :dies (parse-time-spec `(:argument ,num))))
+                              (:result
+                               (list :born (parse-time-spec `(:result ,num))
+                                     :dies (parse-time-spec :save))))))))
           (do ((tail (rest spec) (cddr tail)))
               ((null tail))
             (let ((key (first tail))
@@ -896,6 +904,11 @@
     (dolist (name (cddr spec))
       (unless (symbolp name)
         (error "bad temporary name: ~S" name))
+      ;; It's almost always a mistake to have overlaps in the operand names.
+      ;; But I guess that some users think it's fine?
+      #+sb-xc-host
+      (when (member name (vop-parse-temps parse) :key #'operand-parse-name)
+        (warn "temp ~s already exists in ~s" name (vop-parse-name parse)))
       (incf *parse-vop-operand-count*)
       (let ((res (list :born (parse-time-spec :load)
                        :dies (parse-time-spec :save))))
@@ -1037,6 +1050,10 @@
          (setf (vop-parse-save-p parse)
                (vop-spec-arg spec
                              '(member t nil :compute-only :force-to-stack))))
+        (:optional-results
+         (setf (vop-parse-optional-results parse)
+               (append (vop-parse-optional-results parse)
+                       (rest spec))))
         (t
          (error "unknown option specifier: ~S" (first spec)))))
     (values)))
@@ -1094,11 +1111,11 @@
     (values costs load-scs)))
 
 (defconstant-eqx +no-costs+
-    (make-array sb-vm:sc-number-limit :initial-element 0)
+    #.(make-array sb-vm:sc-number-limit :initial-element 0)
   #'equalp)
 
 (defconstant-eqx +no-loads+
-    (make-array sb-vm:sc-number-limit :initial-element t)
+    #.(make-array sb-vm:sc-number-limit :initial-element t)
   #'equalp)
 
 ;;; Pick off the case of operands with no restrictions.
@@ -1139,7 +1156,9 @@
         :more-result-costs
         ',(if (vop-parse-more-results parse)
               (compute-loading-costs-if-any (vop-parse-more-results parse) nil)
-              nil)))))
+              nil)
+        :optional-results ',(loop for name in (vop-parse-optional-results parse)
+                                collect (position name (vop-parse-results parse) :key #'operand-parse-name))))))
 
 ;;;; operand checking and stuff
 
@@ -1222,7 +1241,9 @@
                       (when (sc-allowed-by-primitive-type
                              (sc-or-lose sc)
                              (primitive-type-or-lose ptype))
-                        (return t))))
+                        (return t)))
+                    #+arm64
+                    (eq sc 'sb-vm::zero))
           (warn "~:[Result~;Argument~] ~A to VOP ~S~@
                  has SC restriction ~S which is ~
                  not allowed by the operand type:~%  ~S"
@@ -1385,8 +1406,10 @@
     `(make-vop-info
       :name ',(vop-parse-name parse)
       ,@(make-vop-info-types parse)
-      :guard ,(when (vop-parse-guard parse)
-                `(lambda () ,(vop-parse-guard parse)))
+      :guard ,(awhen (vop-parse-guard parse)
+                (if (typep it '(cons (eql lambda)))
+                    it
+                    `(lambda (node) (declare (ignore node)) ,it)))
       :note ',(vop-parse-note parse)
       :info-arg-count ,(length (vop-parse-info-args parse))
       :ltn-policy ',(vop-parse-ltn-policy parse)
@@ -1546,6 +1569,8 @@
 ;;;     Specifies a Form that is evaluated in the global environment.
 ;;;     If form returns NIL, then emission of this VOP is prohibited
 ;;;     even when all other restrictions are met.
+;;;     As an additional possibility, if Form is a lambda expression,
+;;;     then it is funcalled with the node under consideration.
 ;;;
 ;;; :VOP-VAR Name
 ;;; :NODE-VAR Name
@@ -1558,7 +1583,7 @@
 ;;; :MOVE-ARGS {NIL | :FULL-CALL | :LOCAL-CALL | :KNOWN-RETURN}
 ;;;     Indicates if and how the more args should be moved into a
 ;;;     different frame.
-(defmacro define-vop ((name &optional inherits) &body specs)
+(defmacro define-vop ((&optional name inherits) &body specs)
   (declare (type symbol name))
   ;; Parse the syntax into a VOP-PARSE structure, and then expand into
   ;; code that creates the appropriate VOP-INFO structure at load time.
@@ -1570,6 +1595,11 @@
                     (copy-vop-parse inherited-parse)
                     (make-vop-parse)))
          (n-res (gensym)))
+    (unless name
+      (let ((clause (assoc :translate specs)))
+        (when (singleton-p (cdr clause))
+          (setf name (cadr clause)))))
+    (aver (typep name '(and symbol (not null))))
     (setf (vop-parse-name parse) name)
     (setf (vop-parse-inherits parse) inherits)
 
@@ -1713,8 +1743,27 @@
       (try-coalescing vop-info-temps)
       (try-coalescing vop-info-ref-ordering)
       (try-coalescing vop-info-targets)))
+  ;; vop rdefinition should be allowed, but a dup in the cross-compiler
+  ;; is probably a mistake. REGISTER-VOP-PARSE is the wrong place
+  ;; to check this, because parsing has both compile-time and load-time
+  ;; effects, since inheritance is computed at compile-time.
+  ;; And there are false positives with any DEFINE-VOP in an assembler file
+  ;; because those are processed twice. I don't know what to do.
+  #+nil (when (gethash (vop-info-name vop-info) *backend-template-names*)
+                 (warn "Duplicate vop name: ~s" vop-info))
   (setf (gethash (vop-info-name vop-info) *backend-template-names*)
         vop-info))
+
+(defun undefine-vop (name)
+  (let ((parse (gethash name *backend-parsed-vops*)))
+    (dolist (translate (vop-parse-translate parse))
+      (let ((info (info :function :info translate)))
+        (setf (fun-info-templates info)
+              (delete name (fun-info-templates info)
+                      :key #'vop-info-name))
+        (format t "~&~s has ~d templates~%" translate (length (fun-info-templates info)))))
+    (remhash name *backend-parsed-vops*)
+    (remhash name *backend-template-names*)))
 
 ;;;; emission macros
 
@@ -1777,8 +1826,8 @@
          (result-count (length (vop-parse-results parse)))
          (info-count (length (vop-parse-info-args parse)))
          (noperands (+ arg-count result-count info-count))
-         (n-node (gensym))
-         (n-block (gensym))
+         (n-node (sb-xc:gensym))
+         (n-block (sb-xc:gensym))
          (n-template (gensym)))
 
     (when (or (vop-parse-more-args parse) (vop-parse-more-results parse))
@@ -1795,7 +1844,7 @@
         (collect ((ibinds)
                   (ivars))
           (dolist (info (subseq operands arg-count (+ arg-count info-count)))
-            (let ((temp (gensym)))
+            (let ((temp (sb-xc:gensym)))
               (ibinds `(,temp ,info))
               (ivars temp)))
 
@@ -1837,7 +1886,7 @@
          (fixed-args (butlast args))
          (fixed-results (butlast results))
          (n-node (gensym))
-         (n-block (gensym))
+         (n-block (sb-xc:gensym))
          (n-template (gensym)))
 
     (unless (or (vop-parse-more-args parse)
@@ -1952,17 +2001,18 @@
                    (when (and ,tn-var (not (eq ,tn-var :more)))
                      (,bod ,tn-var)))))))))))
 
-;;; Iterate over all the IR2 blocks in PHYSENV, in emit order.
-(defmacro do-physenv-ir2-blocks ((block-var physenv &optional result)
-                                 &body body)
-  (once-only ((n-physenv physenv))
-    (once-only ((n-first `(lambda-block (physenv-lambda ,n-physenv))))
+;;; Iterate over all the IR2 blocks in the environment ENV, in emit
+;;; order.
+(defmacro do-environment-ir2-blocks ((block-var env &optional result)
+                                     &body body)
+  (once-only ((n-env env))
+    (once-only ((n-first `(lambda-block (environment-lambda ,n-env))))
       (once-only ((n-tail `(block-info
                             (component-tail
                              (block-component ,n-first)))))
         `(do ((,block-var (block-info ,n-first)
                           (ir2-block-next ,block-var)))
              ((or (eq ,block-var ,n-tail)
-                  (not (eq (ir2-block-physenv ,block-var) ,n-physenv)))
+                  (not (eq (ir2-block-environment ,block-var) ,n-env)))
               ,result)
            ,@body)))))
