@@ -11,6 +11,8 @@
 
 (in-package "SB-VM")
 
+(defconstant-eqx +fixup-kinds+ #(:absolute :jmp :lui :addi) #'equalp)
+
 
 ;;;; Registers
 
@@ -61,9 +63,10 @@
   (defreg l1 24) ; tagged temporary 1
   (defreg alloc 25) ; ALLOC pointer
   ;; 26 and 27 are used by the system kernel.
-  ;; 28 is the global pointer of our C runtime, and used for
-  ;; jump/branch relaxation in Lisp.
-  (defreg gp 28)
+  (defreg k0 26)
+  (defreg k1 27)
+  ;; 28 is the global pointer of our C runtime
+  (defreg gp-do-not-use 28)
   (defreg nsp 29) ; number (native) stack pointer
   ;; C frame pointer, or additional saved register.
   (defreg code 30) ; current function object
@@ -79,7 +82,6 @@
   (defregset *register-arg-offsets*
       a0 a1 a2 a3 a4 a5)
 
-  ;; OAOOM: Same as runtime/mips-lispregs.h
   (defregset boxed-regs
       a0 a1 a2 a3 a4 a5 fdefn lexenv
       nfp ocfp lra l0 l1 code)
@@ -291,7 +293,7 @@
      (if (static-symbol-p value)
          immediate-sc-number
          nil))
-    ((or (integer #.sb-xc:most-negative-fixnum #.sb-xc:most-positive-fixnum)
+    ((or (integer #.most-negative-fixnum #.most-positive-fixnum)
          character)
      immediate-sc-number)
     #-sb-xc-host ; There is no such object type in the host
