@@ -12,13 +12,129 @@
 
 (in-package "SB-C")
 
-;;; The front-end data structure (IR1) is composed of nodes,
-;;; representing actual evaluations. Linear sequences of nodes in
-;;; control-flow order are combined into blocks (but see
-;;; JOIN-SUCCESSOR-IF-POSSIBLE for precise conditions); control
-;;; transfers inside a block are represented with CTRANs and between
-;;; blocks -- with BLOCK-SUCC/BLOCK-PRED lists; data transfers are
-;;; represented with LVARs.
+(declaim (special *lexenv*))
+
+;;; The LEXENV represents the lexical environment used for IR1 conversion.
+;;; (This is also what shows up as an ENVIRONMENT value in macroexpansion.)
+(declaim (inline internal-make-lexenv))
+(defstruct (lexenv
+            (:include abstract-lexenv)
+             (:print-function
+              (lambda (lexenv stream depth)
+                (if (null-lexenv-p lexenv)
+                    (print-unreadable-object (lexenv stream)
+                      (write-string "NULL-LEXENV" stream))
+                    (default-structure-print lexenv stream depth))))
+             (:copier nil)
+             (:constructor make-null-lexenv ())
+             (:constructor make-almost-null-lexenv (%policy handled-conditions
+                                                    flushable lambda parent))
+             (:constructor make-package-lock-lexenv
+                           (disabled-package-locks %policy
+                            &aux (handled-conditions nil)))
+             (:constructor internal-make-lexenv
+                           (funs vars blocks tags
+                            type-restrictions
+                            flushable
+                            lambda cleanup handled-conditions
+                            disabled-package-locks %policy user-data
+                            parent)))
+  ;; an alist of (NAME . WHAT), where WHAT is either a FUNCTIONAL (a
+  ;; local function), a DEFINED-FUN, representing an
+  ;; INLINE/NOTINLINE declaration, or a list (MACRO . <function>) (a
+  ;; local macro, with the specifier expander). Note that NAME may be
+  ;; a (SETF <name>) list, not necessarily a single symbol.
+  (funs nil :type list)
+  ;; an alist translating variable names to LEAF structures. A special
+  ;; binding is indicated by a :SPECIAL GLOBAL-VAR leaf. Each special
+  ;; binding within the code gets a distinct leaf structure, as does
+  ;; the current "global" value on entry to the code compiled.
+  ;; (locally (special ...)) is handled by adding the most recent
+  ;; special binding to the front of the list.
+  ;;
+  ;; If the CDR is (MACRO . <exp>), then <exp> is the expansion of a
+  ;; symbol macro.
+  (vars nil :type list)
+  ;; BLOCKS and TAGS are alists from block and go-tag names to 2-lists
+  ;; of the form (<entry> <continuation>), where <continuation> is the
+  ;; continuation to exit to, and <entry> is the corresponding ENTRY
+  ;; node.
+  (blocks nil :type list)
+  (tags nil :type list)
+  ;; an alist (THING . CTYPE) which is used to keep track of
+  ;; "pervasive" type declarations. When THING is a leaf, this is for
+  ;; type declarations that pertain to the type in a syntactic extent
+  ;; which does not correspond to a binding of the affected name.
+  (type-restrictions nil :type list)
+  ;; the lexically enclosing lambda, if any
+  (lambda nil :type (or clambda null))
+  ;; the lexically enclosing cleanup, or NIL if none enclosing within LAMBDA
+  (cleanup nil :type (or cleanup null))
+  ;; condition types we handle with a handler around the compiler
+  (handled-conditions *handled-conditions*)
+  ;; lexically disabled package locks (list of symbols)
+  (disabled-package-locks *disabled-package-locks*)
+  ;; the current OPTIMIZE policy. this is null in the null environment,
+  ;; and the global policy is stored in *POLICY*. (Because we want to
+  ;; be able to affect it from :WITH-COMPILATION-UNIT.) NIL here also
+  ;; works as a convenient null-lexenv identifier.
+  (%policy nil :type (or null policy))
+  ;; A list associating extra user info to symbols.  The entries
+  ;; are of the form (:declare name . value),
+  ;; (:variable name key . value), or (:function name key . value)
+  (user-data nil :type list)
+  (parent nil)
+  ;; Cache of all visible variables, including the ones coming from
+  ;; (call-lexenv lambda)
+  ;; Used for LEAF-VISIBLE-TO-DEBUGGER-P
+  (var-cache nil :type (or null hash-table))
+  ;; A list of functions that can be removed when unused.
+  ;; Similar to the FLUSHABLE attribute in DEFKNOWN, but can applied
+  ;; locally to things that are generally not flushable but can be
+  ;; flushed in some circumstances.
+  (flushable nil :type list))
+
+(defun lexenv-policy (lexenv)
+  (or (lexenv-%policy lexenv) *policy*))
+
+(defun null-lexenv-p (lexenv)
+  (not (lexenv-%policy lexenv)))
+
+;;; an object suitable for input to standard functions that accept
+;;; "environment objects" (of the ANSI glossary)
+(def!type lexenv-designator () '(or abstract-lexenv null))
+
+;;; support for the idiom (in MACROEXPAND and elsewhere) that NIL is
+;;; to be taken as a null lexical environment.
+;;; Of course this is a mostly pointless "idiom" because NIL *is*
+;;; an environment, as far as most environment inquiry functions care.
+(defun coerce-to-lexenv (x)
+  (etypecase x
+    (null (make-null-lexenv))
+    (lexenv x)
+    #+(and sb-fasteval (not sb-xc-host))
+    (sb-interpreter:basic-env (sb-interpreter:lexenv-from-env x))))
+
+;;; The front-end data structure (IR1) is composed of nodes and
+;;; continuations. The general idea is that continuations contain
+;;; top-down information and nodes contain bottom-up, derived
+;;; information. A continuation represents a place in the code, while
+;;; a node represents code that does something.
+;;;
+;;; This representation is more of a flow-graph than an augmented
+;;; syntax tree. The evaluation order is explicitly represented in the
+;;; linkage by continuations, rather than being implicit in the nodes
+;;; which receive the the results of evaluation. This allows us to
+;;; decouple the flow of results from the flow of control. A
+;;; continuation represents both, but the continuation can represent
+;;; the case of a discarded result by having no DEST.
+
+;;; Note: Continuations have been split into CTRANs and LVARs. Control
+;;; transfers inside a block are represented with CTRANs; data
+;;; transfers are represented with LVARs. However, many of the
+;;; comments and names have not been updated to reflect this, and it
+;;; is easy to find references to the old way of doing things with
+;;; continuations throughout the compiler.
 
 ;;; "Lead-in" Control TRANsfer [to some node]
 (defstruct (ctran (:constructor make-ctran) (:copier nil))
@@ -48,8 +164,9 @@
   ;; the basic block this continuation is in. This is null only in
   ;; :UNUSED continuations.
   (block nil :type (or cblock null))
-  ;; Entries created by the BLOCK special operator
-  (entries nil :type list))
+  ;; Use for reporting notes for the following node,
+  ;; which can be transformed and lose its original source code.
+  (source-path nil :type list))
 
 (defmethod print-object ((x ctran) stream)
   (print-unreadable-object (x stream :type t :identity t)
@@ -207,9 +324,9 @@
   ;; top level form containing the original source.
   (source-path *current-path* :type list)
   ;; If this node is in a tail-recursive position, then this is set to
-  ;; T. At the end of IR1 (in physical environment analysis) this is
-  ;; computed for all nodes (after cleanup code has been emitted).
-  ;; Before then, a non-null value indicates that IR1 optimization has
+  ;; T. At the end of IR1 (in environment analysis) this is computed
+  ;; for all nodes (after cleanup code has been emitted).  Before
+  ;; then, a non-null value indicates that IR1 optimization has
   ;; converted a tail local call to a direct transfer.
   ;;
   ;; If the back-end breaks tail-recursion for some reason, then it
@@ -328,13 +445,13 @@
                    (:copier nil)
                    (:conc-name block-)
                    (:predicate block-p))
-    ;; a list of all the blocks that are predecessors/successors of this
-    ;; block. In well-formed IR1, most blocks will have one successor.
-    ;; The only exceptions are:
-    ;;  1. component head blocks (any number)
-    ;;  2. blocks ending in an IF (1 or 2)
-    ;;  3. blocks with DELETE-P set (zero)
-    (pred nil :type list)
+  ;; a list of all the blocks that are predecessors/successors of this
+  ;; block. In well-formed IR1, most blocks will have one successor.
+  ;; The only exceptions are:
+  ;;  1. component head blocks (any number)
+  ;;  2. blocks ending in an IF (1 or 2)
+  ;;  3. blocks with DELETE-P set (zero)
+  (pred nil :type list)
   (succ nil :type list)
   ;; the ctran which heads this block (a :BLOCK-START), or NIL when we
   ;; haven't made the start ctran yet (and in the dummy component head
@@ -380,9 +497,9 @@
   ;; what macroexpansions and source transforms happened "in" this block, used
   ;; for xref
   (xrefs nil :type list)
-  ;; Cache the physenv of a block during lifetime analysis. :NONE if
-  ;; no cached value has been stored yet.
-  (physenv-cache :none :type (or null physenv (member :none))))
+  ;; Cache the environment of a block during lifetime analysis. :NONE
+  ;; if no cached value has been stored yet.
+  (environment-cache :none :type (or null environment (member :none))))
 (defmethod print-object ((cblock cblock) stream)
   (if (boundp '*compilation*)
       (print-unreadable-object (cblock stream :type t :identity t)
@@ -461,9 +578,9 @@
   ;; Entry/exit points have these blocks as their
   ;; predecessors/successors. The start and return from each
   ;; non-deleted function is linked to the component head and
-  ;; tail. Until physical environment analysis links NLX entry stubs
-  ;; to the component head, every successor of the head is a function
-  ;; start (i.e. begins with a BIND node.)
+  ;; tail. Until environment analysis links NLX entry stubs to the
+  ;; component head, every successor of the head is a function start
+  ;; (i.e. begins with a BIND node.)
   (head (missing-arg) :type cblock)
   (tail (missing-arg) :type cblock)
   ;; New blocks are inserted before this.
@@ -515,7 +632,7 @@
   (reanalyze-functionals nil :type list)
   (delete-blocks nil :type list)
   (nlx-info-generated-p nil :type boolean)
-  ;; this is filled by physical environment analysis
+  ;; this is filled by environment analysis
   (dx-lvars nil :type list)
   ;; The default LOOP in the component.
   (outer-loop (missing-arg) :type cloop)
@@ -567,62 +684,38 @@
   (mess-up nil :type (or node null))
   ;; For all kinds, except :DYNAMIC-EXTENT: a list of all the NLX-INFO
   ;; structures whose NLX-INFO-CLEANUP is this cleanup. This is filled
-  ;; in by physical environment analysis.
+  ;; in by environment analysis.
   ;;
   ;; For :DYNAMIC-EXTENT: a list of all DX LVARs, preserved by this
   ;; cleanup. This is filled when the cleanup is created (now by
-  ;; locall call analysis) and is rechecked by physical environment
-  ;; analysis. (For closures this is a list of the allocating node -
-  ;; during IR1, and a list of the argument LVAR of the allocator -
-  ;; after physical environment analysis.)
-  (info nil :type list)
-  ;; Used by propagate-ref-dx to check that the new ref is inside the
-  ;; original let
-  (lexenv nil :type (or null lexenv)))
+  ;; locall call analysis) and is rechecked by environment
+  ;; analysis. (For closures this is a list of the enclose node during
+  ;; IR1, and a list of the LVAR of the enclose after environment
+  ;; analysis.)
+  (info nil :type list))
 (defprinter (cleanup :identity t)
   kind
   mess-up
   (info :test info))
 
-;;; A PHYSENV represents the result of physical environment analysis.
-;;;
-;;; As far as I can tell from reverse engineering, this IR1 structure
-;;; represents the physical environment (which is probably not the
-;;; standard Lispy term for this concept, but I dunno what is the
-;;; standard term): those things in the lexical environment which a
-;;; LAMBDA actually interacts with. Thus in
-;;;   (DEFUN FROB-THINGS (THINGS)
-;;;     (DOLIST (THING THINGS)
-;;;       (BLOCK FROBBING-ONE-THING
-;;;         (MAPCAR (LAMBDA (PATTERN)
-;;;                   (WHEN (FITS-P THING PATTERN)
-;;;                     (RETURN-FROM FROB-THINGS (LIST :FIT THING PATTERN))))
-;;;                 *PATTERNS*))))
-;;; the variables THINGS, THING, and PATTERN and the block names
-;;; FROB-THINGS and FROBBING-ONE-THING are all in the inner LAMBDA's
-;;; lexical environment, but of those only THING, PATTERN, and
-;;; FROB-THINGS are in its physical environment. In IR1, we largely
-;;; just collect the names of these things; in IR2 an IR2-PHYSENV
-;;; structure is attached to INFO and used to keep track of
-;;; associations between these names and less-abstract things (like
-;;; TNs, or eventually stack slots and registers). -- WHN 2001-09-29
-(defstruct (physenv (:copier nil))
-  ;; the function that allocates this physical environment
+;;; The ENVIRONMENT structure represents the result of environment analysis.
+(defstruct (environment (:copier nil))
+  ;; the function that allocates this environment
   (lambda (missing-arg) :type clambda :read-only t)
   ;; This ultimately converges to a list of all the LAMBDA-VARs and
   ;; NLX-INFOs needed from enclosing environments by code in this
-  ;; physical environment. In the meantime, it may be
+  ;; environment. In the meantime, it may be
   ;;   * NIL at object creation time
   ;;   * a superset of the correct result, generated somewhat later
   ;;   * smaller and smaller sets converging to the correct result as
   ;;     we notice and delete unused elements in the superset
   (closure nil :type list)
   ;; a list of NLX-INFO structures describing all the non-local exits
-  ;; into this physical environment
+  ;; into this environment
   (nlx-info nil :type list)
   ;; some kind of info used by the back end
   (info nil))
-(defprinter (physenv :identity t)
+(defprinter (environment :identity t)
   lambda
   (closure :test closure)
   (nlx-info :test nlx-info))
@@ -657,11 +750,10 @@
 ;;; An NLX-INFO structure is used to collect various information about
 ;;; non-local exits. This is effectively an annotation on the
 ;;; continuation, although it is accessed by searching in the
-;;; PHYSENV-NLX-INFO.
+;;; ENVIRONMENT-NLX-INFO.
 (defstruct (nlx-info
             (:copier nil)
-            (:constructor make-nlx-info
-                (cleanup exit &aux (block (first (block-succ (node-block exit)))))))
+            (:constructor make-nlx-info (cleanup block)))
   ;; the cleanup associated with this exit. In a catch or
   ;; unwind-protect, this is the :CATCH or :UNWIND-PROTECT cleanup,
   ;; and not the cleanup for the escape block. The CLEANUP-KIND of
@@ -670,7 +762,7 @@
   (cleanup (missing-arg) :type cleanup)
   ;; the ``continuation'' exited to (the block, succeeding the EXIT
   ;; nodes). If this exit is from an escape function (CATCH or
-  ;; UNWIND-PROTECT), then physical environment analysis deletes the
+  ;; UNWIND-PROTECT), then environment analysis deletes the
   ;; escape function and instead has the %NLX-ENTRY use this
   ;; continuation.
   ;;
@@ -679,9 +771,9 @@
   ;; ENTRY must also be used to disambiguate, since exits to different
   ;; places may deliver their result to the same continuation.
   (block (missing-arg) :type cblock)
-  ;; the entry stub inserted by physical environment analysis. This is
-  ;; a block containing a call to the %NLX-ENTRY funny function that
-  ;; has the original exit destination as its successor. Null only
+  ;; the entry stub inserted by environment analysis. This is a block
+  ;; containing a call to the %NLX-ENTRY funny function that has the
+  ;; original exit destination as its successor. Null only
   ;; temporarily.
   (target nil :type (or cblock null))
   ;; for a lexical exit it determines whether tag existence check is
@@ -770,8 +862,9 @@
   (leaf-%source-name leaf))
 
 ;;; The CONSTANT structure is used to represent known constant values.
-;;; When compiling to a file, named named and anonymous constants with the
-;;; same value will not necessarily share the same leaf.
+;;; If NAME is not null, then it is the name of the named constant
+;;; which this leaf corresponds to, otherwise this is an anonymous
+;;; constant.
 (defstruct (constant (:constructor make-constant (value
                                                   &optional
                                                   (type (ctype-of value))
@@ -914,8 +1007,8 @@
   ;;
   ;;    :ASSIGNMENT
   ;;    similar to a LET (as per FUNCTIONAL-SOMEWHAT-LETLIKE-P), but
-  ;;    can have other than one call as long as there is at most
-  ;;    one non-tail call.
+  ;;    can have more than one call as long as the calls all return to
+  ;;    the same place.
   ;;
   ;;    :OPTIONAL
   ;;    a lambda that is an entry point for an OPTIONAL-DISPATCH.
@@ -989,15 +1082,15 @@
   ;; INLINEP will always be NIL as well.)
   (inline-expansion nil :type list)
   ;; the lexical environment that the INLINE-EXPANSION should be converted in
-  (lexenv *lexenv* :type lexenv :read-only t)
+  (lexenv *lexenv* :type lexenv)
   ;; the original function or macro lambda list, or :UNSPECIFIED if
   ;; this is a compiler created function
   (arg-documentation nil :type (or list (member :unspecified)))
   ;; the documentation string for the lambda
   (documentation nil :type (or null string))
-  ;; Node, allocating closure for this lambda. May be NIL when we are
-  ;; sure that no closure is needed.
-  (allocator nil :type (or null combination))
+  ;; the enclose node allocating the closure for this lambda. May be
+  ;; NIL when we are sure that no closure is needed.
+  (enclose nil :type (or null enclose))
   ;; various rare miscellaneous info that drives code generation & stuff
   (plist () :type list)
   ;; xref information for this functional (only used for functions with an
@@ -1122,9 +1215,9 @@
   (tail-set nil :type (or tail-set null))
   ;; the structure which represents the phsical environment that this
   ;; function's variables are allocated in. This is filled in by
-  ;; physical environment analysis. In a LET, this is EQ to our home's
-  ;; physical environment.
-  (physenv nil :type (or physenv null))
+  ;; environment analysis. In a LET, this is EQ to our home's
+  ;; environment.
+  (environment nil :type (or environment null))
   ;; In a LET, this is the NODE-LEXENV of the combination node. We
   ;; retain it so that if the LET is deleted (due to a lack of vars),
   ;; we will still have caller's lexenv to figure out which cleanup is
@@ -1273,26 +1366,26 @@
 ;;; lambda arguments which may ultimately turn out not to be simple
 ;;; and lexical.
 ;;;
-;;; LAMBDA-VARs with no REFs are considered to be deleted; physical
-;;; environment analysis isn't done on these variables, so the back
-;;; end must check for and ignore unreferenced variables. Note that a
-;;; deleted LAMBDA-VAR may have sets; in this case the back end is
-;;; still responsible for propagating the SET-VALUE to the set's CONT.
+;;; LAMBDA-VARs with no REFs are considered to be deleted; environment
+;;; analysis isn't done on these variables, so the back end must check
+;;; for and ignore unreferenced variables. Note that a deleted
+;;; LAMBDA-VAR may have sets; in this case the back end is still
+;;; responsible for propagating the SET-VALUE to the set's CONT.
 (!def-boolean-attribute lambda-var
   ;; true if this variable has been declared IGNORE
   ignore
-  ;; This is set by physical environment analysis if it chooses an
-  ;; indirect (value cell) representation for this variable because it
-  ;; is both set and closed over.
+  ;; This is set by environment analysis if it chooses an indirect
+  ;; (value cell) representation for this variable because it is both
+  ;; set and closed over.
   indirect
   ;; true if the last reference has been deleted (and new references
   ;; should not be made)
   deleted
-  ;; This is set by physical environment analysis if, should it be an
-  ;; indirect lambda-var, an actual value cell object must be
-  ;; allocated for this variable because one or more of the closures
-  ;; that refer to it are not dynamic-extent.  Note that both
-  ;; attributes must be set for the value-cell object to be created.
+  ;; This is set by environment analysis if, should it be an indirect
+  ;; lambda-var, an actual value cell object must be allocated for
+  ;; this variable because one or more of the closures that refer to
+  ;; it are not dynamic-extent.  Note that both attributes must be set
+  ;; for the value-cell object to be created.
   explicit-value-cell
   ;; Do not propagate constraints for this var
   no-constraints
@@ -1329,9 +1422,6 @@
   (private-constraints     nil :type (or null (array t 1)))
   (equality-constraints    nil :type (or null (array t 1)))
 
-  ;; The FOP handle of the lexical variable represented by LAMBDA-VAR
-  ;; in the fopcompiler.
-  (fop-value nil)
   source-form)
 (defprinter (lambda-var :identity t)
   %source-name
@@ -1430,8 +1520,8 @@
 (defstruct (basic-combination (:include valued-node)
                               (:constructor nil)
                               (:copier nil))
-    ;; LVAR for the function
-    (fun (missing-arg) :type lvar)
+  ;; LVAR for the function
+  (fun (missing-arg) :type lvar)
   ;; list of LVARs for the args. In a local call, an argument lvar may
   ;; be replaced with NIL to indicate that the corresponding variable
   ;; is unreferenced, and thus no argument value need be passed.
@@ -1547,12 +1637,16 @@
   asserted-type
   type-to-check)
 
-;;; A filter to help order the value semantics of MULTIPLE-VALUE-PROG1
-(defstruct (vestigial-exit-cast (:include cast
-                                          (%type-check nil)
-                                          (asserted-type *wild-type*)
-                                          (type-to-check *wild-type*))
-                                (:copier nil)))
+;;; The DELAY node is interposed between a VALUE's USE and its DEST in
+;;; order to allow the value to be immediately used. This is necessary
+;;; for implementing multiple-use unknown values LVARs, as otherwise,
+;;; a non-moveable dynamic extent object may be allocated between the
+;;; DEST and one of the LVAR's uses but not the others.
+(defstruct (delay (:include cast
+                   (%type-check nil)
+                   (asserted-type *wild-type*)
+                   (type-to-check *wild-type*))
+                  (:copier nil)))
 
 ;;; A cast that always follows %check-bound and they are deleted together.
 ;;; Created via BOUND-CAST ir1-translator by chaining it together with %check-bound.
@@ -1594,8 +1688,8 @@
 ;;; continuation and the exit continuation's DEST. Instead of using
 ;;; the returned value being delivered directly to the exit
 ;;; continuation, it is delivered to our VALUE lvar. The original exit
-;;; lvar is the exit node's LVAR; physenv analysis also makes it the
-;;; lvar of %NLX-ENTRY call.
+;;; lvar is the exit node's LVAR; environment analysis also makes it
+;;; the lvar of %NLX-ENTRY call.
 (defstruct (exit (:include valued-node)
                  (:copier nil))
   ;; the ENTRY node that this is an exit for. If null, this is a
@@ -1611,8 +1705,45 @@
   (entry :test entry)
   (value :test value))
 
-(defstruct (no-op (:include node)
-                  (:copier nil)))
+;;; The ENCLOSE node marks the place at which closure allocation code
+;;; would be emitted, if necessary.
+(defstruct (enclose (:include valued-node) ; this node uses a dummy lvar for dx analysis
+                    (:copier nil))
+  ;; the list of functionals that this ENCLOSE node allocates.
+  (funs nil :type list))
+(defprinter (enclose :identity t)
+  funs)
+
+
+;;;; miscellaneous IR1 structures
+
+(defstruct (undefined-warning
+            (:print-object (lambda (x s)
+                             (print-unreadable-object (x s :type t)
+                               (prin1 (undefined-warning-name x) s))))
+            (:copier nil))
+  ;; the name of the unknown thing
+  (name nil :type (or symbol list))
+  ;; the kind of reference to NAME
+  (kind (missing-arg) :type (member :function :type :variable))
+  ;; the number of times this thing was used
+  (count 0 :type unsigned-byte)
+  ;; a list of COMPILER-ERROR-CONTEXT structures describing places
+  ;; where this thing was used. Note that we only record the first
+  ;; *UNDEFINED-WARNING-LIMIT* calls.
+  (warnings () :type list))
+
+(declaim (freeze-type undefined-warning))
+
+(defstruct (argument-mismatch-warning
+            (:print-object (lambda (x s)
+                             (print-unreadable-object (x s :type t)
+                               (prin1 (argument-mismatch-warning-name x) s))))
+            (:copier nil))
+  (name nil :type (or symbol list))
+  ;; a list of (KEYS . COMPILER-ERROR-CONTEXT)
+  (warnings () :type list))
+
 
 ;;; a helper for the POLICY macro, defined late here so that the
 ;;; various type tests can be inlined
@@ -1642,4 +1773,4 @@
 ;;;; Freeze some structure types to speed type testing.
 
 (declaim (freeze-type node lexenv ctran lvar cblock component cleanup
-                      physenv tail-set nlx-info leaf))
+                      environment tail-set nlx-info leaf))
