@@ -22,7 +22,6 @@ modification. Defaults to SAFETY.")
     ;; places which might want to use this instead -- or
     ;; some other derived policy.
     (cond ((= safety 0) 0)
-          ((and (< safety 2) (< safety speed)) 2)
           (t 3))
   ("no" "maybe" "weak" "full")
   "Control the way to perform runtime type checking:
@@ -127,6 +126,10 @@ debugger.")
 (define-optimization-quality insert-array-bounds-checks
     (if (= safety 0) 0 3)
   ("no" "yes" "yes" "yes"))
+(define-optimization-quality aref-trapping
+    #-ubsan (if (= safety 3) 3 0) ; equiv. to safety unless expressed otherwise
+    #+ubsan 2 ; default to yes
+  ("no" "yes" "yes" "yes"))
 
 (define-optimization-quality store-xref-data
     (if (= space 3)
@@ -143,10 +146,10 @@ debugger.")
   ("no" "no" "yes" "yes"))
 
 #+sb-safepoint
-(define-optimization-quality inhibit-safepoints
-    0
-  ("no" "no" "yes" "yes")
-  "When disabled, the compiler will insert safepoints at strategic
+(define-optimization-quality insert-safepoints
+    1
+  ("no" "yes" "yes" "yes")
+  "When enabled, the compiler will insert safepoints at strategic
 points (loop edges, function prologues) to ensure that potentially
 long-running code can be interrupted.
 
