@@ -502,7 +502,7 @@
       (inst cmpwi amount 0)
       (inst neg ndesc amount)
       (inst bge positive)
-      (inst cmpwi ndesc 31)
+      (inst cmplwi ndesc 31)
       (inst srw result number ndesc)
       (inst ble done)
       (move result zero-tn)
@@ -549,7 +549,7 @@
          (inst cmpwi amount 0)
          (inst neg ndesc amount)
          (inst bge positive)
-         (inst cmpwi ndesc 31)
+         (inst cmplwi ndesc 31)
          (inst sraw result number ndesc)
          (inst ble done)
          (inst srawi result number 31)
@@ -1013,7 +1013,7 @@
   (:results (value :scs (unsigned-reg)))
   (:result-types unsigned-num))
 
-(define-vop (bignum-set word-index-set-nr)
+(define-vop (bignum-set word-index-set)
   (:variant bignum-digits-offset other-pointer-lowtag)
   (:translate #+bignum-assertions sb-bignum:%%bignum-set
               #-bignum-assertions sb-bignum:%bignum-set)
