@@ -9,20 +9,21 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 (defun %test-fixnum (value temp target not-p)
   (assemble ()
     (inst andi. temp value fixnum-tag-mask)
     (inst b? (if not-p :ne :eq) target)))
 
-(defun %test-fixnum-and-headers (value temp target not-p headers)
+(defun %test-fixnum-and-headers (value temp target not-p headers &key value-tn-ref)
   (let ((drop-through (gen-label)))
     (assemble ()
       (inst andi. temp value fixnum-tag-mask)
       (inst beq (if not-p drop-through target)))
     (%test-headers value temp target not-p nil headers
-                   :drop-through drop-through)))
+                   :drop-through drop-through
+                   :value-tn-ref value-tn-ref)))
 
 (defun %test-immediate (value temp target not-p immediate)
   (assemble ()
@@ -37,8 +38,9 @@
     (inst b? (if not-p :ne :eq) target)))
 
 (defun %test-headers (value temp target not-p function-p headers
-                      &key (drop-through (gen-label)))
-    (let ((lowtag (if function-p fun-pointer-lowtag other-pointer-lowtag)))
+                      &key (drop-through (gen-label)) value-tn-ref)
+  (declare (ignore value-tn-ref))
+  (let ((lowtag (if function-p fun-pointer-lowtag other-pointer-lowtag)))
     (multiple-value-bind (when-true when-false)
         (if not-p
             (values drop-through target)
