@@ -164,7 +164,6 @@
   (:generator 2
     (inst lr res  (make-fixup foreign-symbol :foreign))))
 
-#+linkage-table
 (define-vop (foreign-symbol-dataref-sap)
   (:translate foreign-symbol-dataref-sap)
   (:policy :fast-safe)
@@ -173,10 +172,9 @@
   (:info foreign-symbol)
   (:results (res :scs (sap-reg)))
   (:result-types system-area-pointer)
-  (:temporary (:scs (non-descriptor-reg)) addr)
   (:generator 2
-    (inst lr addr (make-fixup foreign-symbol :foreign-dataref))
-    (loadw res addr)))
+    (inst lr res (make-fixup foreign-symbol :foreign-dataref))
+    (loadw res res)))
 
 (define-vop (call-out)
   (:args (function :scs (sap-reg) :target cfunc)
@@ -414,9 +412,7 @@
               ;; And make the call.
               (load-address-into
                r0
-               (foreign-symbol-address
-                #-sb-thread "funcall3"
-                #+sb-thread "callback_wrapper_trampoline"))
+               (foreign-symbol-address "callback_wrapper_trampoline"))
               (inst mtlr r0)
               (inst blrl)
 
@@ -450,7 +446,7 @@
 
         ;; Now that the segment is done, convert it to a static
         ;; vector we can point foreign code to.
-        (let* ((buffer (sb-assem::segment-buffer segment))
+        (let* ((buffer (sb-assem:segment-buffer segment))
                (vector (make-static-vector (length buffer)
                                            :element-type '(unsigned-byte 8)
                                            :initial-contents buffer))
