@@ -11,6 +11,20 @@
 ;;;; files for more information.
 
 (in-package "SB-VM")
+
+(defun emit-gc-store-barrier (object cell-address temps &optional value-tn-ref value-tn)
+  (aver (neq (car temps) cell-address)) ; LD would clobber the cell-address
+  (when (require-gc-store-barrier-p object value-tn-ref value-tn)
+    ;; (inst ld (car temps) thread-base-tn (ash thread-card-table-slot word-shift))
+    ;; RLIDCL dest, source, (64-rightshift), (64-indexbits)
+    (inst rldicl (car temps) (or cell-address object) (- 64 gencgc-card-shift)
+          (make-fixup nil :gc-barrier))
+    ;; THREAD-TN's low byte is 0.  NL5 is the card table address.
+    (inst stbx thread-base-tn
+          (make-random-tn :kind :normal
+                          :sc (sc-or-lose 'non-descriptor-reg) :offset nl5-offset)
+          (car temps))))
+
 
 ;;; Cell-Ref and Cell-Set are used to define VOPs like CAR, where the offset to
 ;;; be read or written is a property of the VOP used.
@@ -28,40 +42,50 @@
          (value :scs (descriptor-reg any-reg)))
   (:variant-vars offset lowtag)
   (:policy :fast-safe)
+  (:vop-var vop)
+  (:temporary (:sc non-descriptor-reg) t1)
   (:generator 4
+    (emit-gc-store-barrier object nil (list t1) (vop-nth-arg 1 vop) value)
     (storew value object offset lowtag)))
 
 ;;;; Indexed references:
 
 ;;; Define some VOPs for indexed memory reference.
 
+(define-vop (descriptor-word-index-set)
+  (:args (object :scs (descriptor-reg))
+         (index :scs (any-reg immediate))
+         (value :scs (any-reg descriptor-reg)))
+  (:arg-types * tagged-num *)
+  (:temporary (:scs (non-descriptor-reg)) temp)
+  (:variant-vars offset lowtag)
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 5
+   (emit-gc-store-barrier object nil (list temp) (vop-nth-arg 2 vop) value)
+   (sc-case index
+    ((immediate)
+     (let ((offset (- (ash (+ (tn-value index) offset) word-shift) lowtag)))
+       (cond ((and (typep offset '(signed-byte 16)) (not (logtest offset #b11)))
+              (inst std value object offset))
+             (t
+              (inst lr temp offset)
+              (inst stdx value object temp)))))
+    (t
+     (inst addi temp index (- (ash offset word-shift) lowtag))
+     (inst stdx value object temp)))))
+
 ;;; Due to the encoding restrictione that doubleword accesses can not displace
 ;;; from the base register by an arbitrarily aligned value, but only an even
-;;; multiple of 4, it would behoove us to invent a different lowtag scheme,
-;;; something like:
+;;; multiple of 4. By using a certain arrangement of lowtags we can get two of
+;;; the pointer types to get lowtag subtraction for free:
+;;;    #b0100 list - good
 ;;;    #b0110 function
-;;;    #b0100 instance
-;;;    #b1000 list
-;;;    #b1100 other
-;;; All lowtags except FUN-POINTER-LOWTAG become 4-byte-aligned.
-;;; We would need to increase N-FIXNUM-TAG-BITS to 4 for this to work, and it
-;;; may cause some breakage due to assumptions that lowtags are spaced evenly
-;;; and that N-FIXNUM-TAG-BITS is not more than WORD-SHIFT.
-;;; Both those constraints may be impossible to remove - I don't know.
-;;; The alternatives are far worse - constantly computing into the LIP
-;;; register, or else allowing native pointers to pin objects.
-;;;
-;;; Potentially it will much simpler to have N-FIXNUM-TAG-BITS be 3,
-;;; and then we have two lowtags that allow use of single-instruction load/store
-;;; forms, and two that do not; something like this:
-;;;    #b0110 function
-;;;    #b0100 instance - good
-;;;    #b1100 list     - good
-;;;    #b1111 other
+;;;    #b1100 instance - good
+;;;    #b1110 other
 ;;; The misalignment of OTHER is not a huge problem, because array access requires
 ;;; calculating the displacement anyway, and the remaining OTHER pointer types
 ;;; like VALUE-CELL are not as prevalent. SYMBOL remains a bit of a problem.
-;;; Also, we can't load static symbol values using the NULL register as a base.
 
 ;;; NB: this macro uses an inverse of the 'shift' convention in the 32-bit file.
 ;;; Here is the convention is that 'scale' how much to left-shift a natural
@@ -76,13 +100,12 @@
   `(define-vop (,name)
      (:args (object :scs (descriptor-reg))
             (index :scs (any-reg immediate))
-            ,@(when write-p
-                '((value :scs (any-reg descriptor-reg) :target result))))
+            ,@(when write-p '((value :scs (any-reg descriptor-reg)))))
      (:arg-types * tagged-num ,@(when write-p '(*)))
      (:temporary (:scs (non-descriptor-reg)) temp)
-     (:results (,(if write-p 'result 'value)
-                :scs (any-reg descriptor-reg)))
-     (:result-types *)
+     ,@(unless write-p
+         `((:results (value :scs (any-reg descriptor-reg)))
+           (:result-types *)))
      (:variant-vars offset lowtag)
      (:policy :fast-safe)
      (:generator 5
@@ -108,20 +131,19 @@
                 (- (ash offset word-shift) lowtag))
           (inst ,rr-op value object temp)))
        ,@(when sign-extend-byte
-           `((inst extsb value value)))
-       ,@(when write-p
-           '((move result value))))))
+           `((inst extsb value value))))))
 
 (define-indexer word-index-ref           3 nil ld  ldx) ;; Word means Lisp Word
-(define-indexer word-index-set           3 t   std stdx)
 (define-indexer 32-bits-index-ref        2 nil lwz lwzx)
 (define-indexer signed-32-bits-index-ref 2 nil lwa lwax :multiple-of-four t)
-(define-indexer 32-bits-index-set        2 t   stw stwx)
 (define-indexer 16-bits-index-ref        1 nil lhz lhzx)
 (define-indexer signed-16-bits-index-ref 1 nil lha lhax)
-(define-indexer 16-bits-index-set        1 t   sth sthx)
 (define-indexer byte-index-ref           0 nil lbz lbzx)
 (define-indexer signed-byte-index-ref    0 nil lbz lbzx :sign-extend-byte t)
+
+(define-indexer word-index-set           3 t   std stdx)
+(define-indexer 32-bits-index-set        2 t   stw stwx)
+(define-indexer 16-bits-index-set        1 t   sth sthx)
 (define-indexer byte-index-set           0 t   stb stbx)
 
 (define-vop (word-index-cas)
@@ -135,12 +157,23 @@
   (:result-types *)
   (:variant-vars offset lowtag)
   (:policy :fast-safe)
+  (:vop-var vop)
   (:generator 5
+    (let ((ea
+           (ecase lowtag
+             (#.instance-pointer-lowtag nil)
+             (#.other-pointer-lowtag ; has to be (SETF SVREF)
+              (cond ((sc-is index immediate)
+                     (let ((offset (- (ash (+ (tn-value index) offset) word-shift) lowtag)))
+                       (inst lr temp offset)))
+                    (t
+                     (inst addi temp index (- (ash offset word-shift) lowtag))))
+              (inst add temp object temp)
+              temp))))
+      (emit-gc-store-barrier object ea (list result temp) (vop-nth-arg 3 vop) new-value))
     (sc-case index
       ((immediate)
-       (let ((offset (- (+ (ash (tn-value index) word-shift)
-                           (ash offset word-shift))
-                        lowtag)))
+       (let ((offset (- (ash (+ (tn-value index) offset) word-shift) lowtag)))
          (inst lr temp offset)))
       (t
        (inst sldi temp index (- word-shift n-fixnum-tag-bits))
@@ -154,3 +187,16 @@
     (inst bne LOOP)
     EXIT
     (inst isync)))
+
+(define-vop (set-instance-hashed)
+  (:args (object :scs (descriptor-reg)))
+  (:temporary (:sc non-descriptor-reg) baseptr header)
+  (:generator 5
+    (inst addi baseptr object (- instance-pointer-lowtag))
+    (inst sync)
+    LOOP
+    (inst ldarx header 0 baseptr)
+    (inst ori header header (ash 1 stable-hash-required-flag))
+    (inst stdcx. header 0 baseptr)
+    (inst bne LOOP)
+    (inst isync)))
