@@ -50,29 +50,6 @@
 (defconstant pseudo-atomic-interrupted-flag 1)
 (defconstant pseudo-atomic-flag 4)
 
-(defconstant float-sign-shift 31)
-
-(defconstant single-float-bias 126)
-(defconstant-eqx single-float-exponent-byte (byte 8 23) #'equalp)
-(defconstant-eqx single-float-significand-byte (byte 23 0) #'equalp)
-(defconstant single-float-normal-exponent-min 1)
-(defconstant single-float-normal-exponent-max 254)
-(defconstant single-float-hidden-bit (ash 1 23))
-
-(defconstant double-float-bias 1022)
-(defconstant-eqx double-float-exponent-byte (byte 11 20) #'equalp)
-(defconstant-eqx double-float-significand-byte (byte 20 0) #'equalp)
-(defconstant double-float-normal-exponent-min 1)
-(defconstant double-float-normal-exponent-max #x7FE)
-(defconstant double-float-hidden-bit (ash 1 20))
-
-(defconstant single-float-digits
-  (+ (byte-size single-float-significand-byte) 1))
-
-(defconstant double-float-digits
-  (+ (byte-size double-float-significand-byte) 32 1))
-
-
 (defconstant float-inexact-trap-bit (ash 1 0))
 (defconstant float-divide-by-zero-trap-bit (ash 1 1))
 (defconstant float-underflow-trap-bit (ash 1 2))
@@ -120,8 +97,11 @@
   (defconstant linkage-table-space-end   #x0b000000))
 
 ;;; While on gencgc we don't.
-#+gencgc (!gencgc-space-setup #x04000000 :dynamic-space-start #x1000000000)
+#+gencgc (!gencgc-space-setup #x04000000
+                              :read-only-space-size 0
+                              :dynamic-space-start #x1000000000)
 
+(defconstant linkage-table-growth-direction :up)
 (defconstant linkage-table-entry-size #+little-endian 28 #+big-endian 24)
 
 #+linux
