@@ -197,7 +197,7 @@
   (:results (result :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:note "inline ASH")
-  (:generator 3
+  (:generator 1
     (cond ((< (- n-word-bits) amount n-word-bits)
            (if (plusp amount)
                (inst slli result number amount)
@@ -214,7 +214,7 @@
   (:results (result :scs (signed-reg)))
   (:result-types signed-num)
   (:note "inline ASH")
-  (:generator 3
+  (:generator 1
     (cond ((< (- n-word-bits) amount n-word-bits)
            (if (plusp amount)
                (inst slli result number amount)
@@ -230,22 +230,19 @@
          (amount))
   (:results (result))
   (:policy :fast-safe)
-  ;; ANY-REG is fine because N-WORD-BITS is even.
-  (:temporary (:sc any-reg) temp)
   (:temporary (:sc non-descriptor-reg) ndesc)
   (:variant-vars variant)
   (:generator 3
     (inst bge amount zero-tn positive)
-    (inst sub ndesc zero-tn amount)
-    (inst li temp n-word-bits)
-    (inst blt ndesc temp no-overflow)
-    ;; KLUDGE: Even though this gives the wrong answer for 64-bit
-    ;; values with a shift less than -63, I think that this VOP cannot
-    ;; be selected in that case, since any positive shift amount would
-    ;; overflow. In that case, a more correct VOP would be
-    ;; selected. More testing/thinking required.
-    (inst subi ndesc temp 1)
+    (inst li ndesc (- n-word-bits))
+    (inst blt ndesc amount no-overflow)
+    (ecase variant
+      (:signed (inst srai result number (1- n-word-bits)))
+      (:unsigned (move result zero-tn)))
+    (inst j done)
+
     NO-OVERFLOW
+    (inst sub ndesc zero-tn amount)
     (ecase variant
       (:signed (inst sra result number ndesc))
       (:unsigned (inst srl result number ndesc)))
@@ -378,9 +375,8 @@
       ;; optimize this.
       (loop for masker in maskers
             for shift = 1 then (ash shift 1) do
-              (inst li mask masker) ; NL1
+              (inst li mask masker)
               (let ((input (if (= shift 1) arg num)))
-                ; TEMP = NL2 ARG = NL3 NUM = NL0
                 (inst srli temp input shift)
                 (inst and num input mask))
               (inst and temp temp mask)
@@ -535,26 +531,19 @@
 
 ;;;; Logical operations
 
-(define-vop (shift-towards-someplace)
-  (:policy :fast-safe)
-  (:args (num :scs (unsigned-reg))
-         (amount :scs (signed-reg)))
-  (:arg-types unsigned-num tagged-num)
-  (:results (r :scs (unsigned-reg)))
-  (:result-types unsigned-num))
-
-(define-vop (shift-towards-start shift-towards-someplace)
-  (:translate shift-towards-start)
-  (:note "SHIFT-TOWARDS-START")
-  (:generator 1
-    (inst srl r num amount)))
-
-(define-vop (shift-towards-end shift-towards-someplace)
-  (:translate shift-towards-end)
-  (:note "SHIFT-TOWARDS-END")
-  (:generator 1
-    (inst sll r num amount)))
-
+(macrolet ((define (translate operation)
+             `(define-vop ()
+                (:translate ,translate)
+                (:note ,(string translate))
+                (:policy :fast-safe)
+                (:args (num :scs (unsigned-reg))
+                       (amount :scs (signed-reg)))
+                (:arg-types unsigned-num tagged-num)
+                (:results (r :scs (unsigned-reg)))
+                (:result-types unsigned-num)
+                (:generator 1 (inst ,operation r num amount)))))
+  (define shift-towards-start srl)
+  (define shift-towards-end   sll))
 
 ;;;; Modular arithmetic
 (defmacro define-mod-binop ((name prototype) function)
@@ -612,7 +601,7 @@
 
   (deftransform ash-left-mod32 ((integer count)
                                 ((unsigned-byte 32) (unsigned-byte 5)))
-    (when (sb-c::constant-lvar-p count)
+    (when (sb-c:constant-lvar-p count)
       (sb-c::give-up-ir1-transform))
     '(%primitive fast-ash-left-mod32/unsigned=>unsigned integer count)))
 
@@ -627,7 +616,7 @@
 
   (deftransform ash-left-mod64 ((integer count)
                                 ((unsigned-byte 64) (unsigned-byte 6)))
-    (when (sb-c::constant-lvar-p count)
+    (when (sb-c:constant-lvar-p count)
       (sb-c::give-up-ir1-transform))
     '(%primitive fast-ash-left-mod64/unsigned=>unsigned integer count)))
 
