@@ -19,7 +19,7 @@
   (:generator 1
     (move res csp-tn)))
 
-(define-vop ()
+(define-vop (current-fp-sap)
   (:translate current-fp)
   (:policy :fast-safe)
   (:results (res :scs (sap-reg)))
@@ -48,18 +48,15 @@
   (:policy :fast-safe)
   (:args (object :scs (sap-reg) :target sap)
          (offset :scs (any-reg))
-         (value :scs (descriptor-reg) :target result))
+         (value :scs (descriptor-reg)))
   (:arg-types system-area-pointer positive-fixnum *)
-  (:results (result :scs (descriptor-reg)))
-  (:result-types *)
   (:temporary (:scs (sap-reg) :from (:argument 1)) sap)
    #+#.(cl:if sb-vm::fixnum-as-word-index-needs-temp '(and) '(or))
   (:temporary (:sc non-descriptor-reg) temp)
   (:generator 2
     (with-fixnum-as-word-index (offset temp)
       (inst add sap object offset))
-    (storew value sap 0)
-    (move result value)))
+    (storew value sap 0)))
 
 (define-vop (code-from-mumble)
   (:policy :fast-safe)
