@@ -12,26 +12,18 @@
 (in-package "SB-VM")
 
 ;;;; LIST and LIST*
-(define-vop (list-or-list*)
+(define-vop (list)
   (:args (things :more t))
   (:temporary (:scs (descriptor-reg)) ptr)
   (:temporary (:scs (descriptor-reg)) temp)
   (:temporary (:scs (descriptor-reg) :to (:result 0) :target result)
               res)
   (:temporary (:scs (non-descriptor-reg)) alloc-temp)
-  (:info num)
+  (:info star cons-cells)
   (:results (result :scs (descriptor-reg)))
-  (:variant-vars star)
-  (:policy :safe)
   (:node-var node)
   (:generator 0
-    (cond ((zerop num)
-           (move result null-tn))
-          ((and star (= num 1))
-           (move result (tn-ref-tn things)))
-          (t
-           (macrolet
-               ((maybe-load (tn)
+    (macrolet ((maybe-load (tn)
                   (once-only ((tn tn))
                     `(sc-case ,tn
                        ((any-reg descriptor-reg zero null)
@@ -39,11 +31,10 @@
                        (control-stack
                         (load-stack-tn temp ,tn)
                         temp)))))
-             (let* ((dx-p (node-stack-allocate-p node))
-                    (cons-cells (if star (1- num) num))
-                    (alloc (* (pad-data-block cons-size) cons-cells)))
-               (pseudo-atomic ()
-                 (allocation res alloc list-pointer-lowtag
+      (let ((dx-p (node-stack-allocate-p node))
+            (alloc (* (pad-data-block cons-size) cons-cells)))
+        (pseudo-atomic ()
+                 (allocation 'list alloc list-pointer-lowtag res
                              :stack-p dx-p
                              :temp-tn alloc-temp)
                  (move ptr res)
@@ -61,14 +52,7 @@
                              (maybe-load (tn-ref-tn (tn-ref-across things)))
                              null-tn)
                          ptr cons-cdr-slot list-pointer-lowtag))
-               (move result res)))))))
-
-(define-vop (list list-or-list*)
-  (:variant nil))
-
-(define-vop (list* list-or-list*)
-  (:variant t))
-
+        (move result res)))))
 
 ;;;; Special purpose inline allocators.
 
@@ -96,7 +80,7 @@
     (let* ((size (+ length closure-info-offset))
            (alloc-size (pad-data-block size)))
       (pseudo-atomic ()
-        (allocation result alloc-size fun-pointer-lowtag
+        (allocation nil alloc-size fun-pointer-lowtag result
                     :stack-p stack-allocate-p
                     :temp-tn temp)
         (inst li temp (logior (ash (1- size) n-widetag-bits) closure-widetag))
@@ -132,15 +116,18 @@
 (define-vop (fixed-alloc)
   (:args)
   (:info name words type lowtag stack-allocate-p)
-  (:ignore name stack-allocate-p)
+  (:ignore name)
   (:results (result :scs (descriptor-reg)))
   (:temporary (:scs (non-descriptor-reg)) temp)
   (:generator 4
+    ;; I don't see why fixed allocations need to be pseudo-atomic.
+    ;; The other precisely GC'd backends have it this way.
     (pseudo-atomic ()
-      (allocation result (pad-data-block words) lowtag :temp-tn temp)
-      (when type
-        (inst li temp (logior (ash (1- words) n-widetag-bits) type))
-        (storew temp result 0 lowtag)))))
+      (allocation nil (pad-data-block words) lowtag result :temp-tn temp
+                  :stack-p stack-allocate-p)
+        (when type
+          (inst li temp (logior (ash (1- words) (length-field-shift type)) type))
+          (storew temp result 0 lowtag)))))
 
 (define-vop (var-alloc)
   (:args (extra :scs (any-reg)))
@@ -153,14 +140,14 @@
   (:temporary (:scs (non-descriptor-reg)) temp)
   (:generator 6
     (inst add bytes extra (* (1+ words) n-word-bytes))
-    (inst sll header bytes (- n-widetag-bits 2))
+    (inst sll header bytes (- (length-field-shift type) 2))
     ;; The specified EXTRA value is the exact value placed in the header
     ;; as the word count when allocating code.
     (cond ((= type code-header-widetag)
            (inst add header header type))
           (t
-           (inst add header header (+ (ash -2 n-widetag-bits) type))
+           (inst add header header (+ (ash -2 (length-field-shift type)) type))
            (inst and bytes (lognot lowtag-mask))))
     (pseudo-atomic ()
-      (allocation result bytes lowtag :temp-tn temp)
+      (allocation nil bytes lowtag result :temp-tn temp)
       (storew header result 0 lowtag))))
