@@ -31,7 +31,7 @@
       ;; Compute the encoded rank. See ENCODE-ARRAY-RANK.
       (inst sub ndescr rank (fixnumize 1))
       (inst and ndescr ndescr (fixnumize array-rank-mask))
-      (inst sll ndescr ndescr array-rank-byte-pos)
+      (inst sll ndescr ndescr array-rank-position)
       (inst or ndescr ndescr type)
       ;; Remove the extraneous fixnum tag bits because TYPE and RANK
       ;; were fixnums
@@ -101,9 +101,7 @@
        (:arg-types ,type positive-fixnum ,element-type)
        (:args (object :scs (descriptor-reg))
               (index :scs (any-reg zero immediate))
-              (value :scs ,scs))
-       (:results (result :scs ,scs))
-       (:result-types ,element-type)))))
+              (value :scs ,scs))))))
 
   (def-data-vector-frobs simple-base-string byte-index
     character character-reg)
@@ -192,10 +190,8 @@
          (:policy :fast-safe)
          (:args (object :scs (descriptor-reg))
                 (index :scs (unsigned-reg) :target shift)
-                (value :scs (unsigned-reg zero immediate) :target result))
+                (value :scs (unsigned-reg zero immediate)))
          (:arg-types ,type positive-fixnum positive-fixnum)
-         (:results (result :scs (unsigned-reg)))
-         (:result-types positive-fixnum)
          (:temporary (:scs (non-descriptor-reg)) temp old offset)
          (:temporary (:scs (non-descriptor-reg) :from (:argument 1)) shift)
          (:generator 25
@@ -222,23 +218,16 @@
                 (inst and temp value ,(1- (ash 1 bits)))))
              (inst sll temp shift)
              (inst or old temp))
-           (inst st old object offset)
-           (sc-case value
-             (immediate
-              (inst li result (tn-value value)))
-             (t
-              (move result value)))))
+           (inst st old object offset)))
        (define-vop (,(symbolicate "DATA-VECTOR-SET-C/" type))
          (:translate data-vector-set)
          (:policy :fast-safe)
          (:args (object :scs (descriptor-reg))
-                (value :scs (unsigned-reg zero immediate) :target result))
+                (value :scs (unsigned-reg zero immediate)))
          (:arg-types ,type
                      (:constant index)
                      positive-fixnum)
          (:info index)
-         (:results (result :scs (unsigned-reg)))
-         (:result-types positive-fixnum)
          (:temporary (:scs (non-descriptor-reg)) offset-reg temp old)
          (:generator 20
            (multiple-value-bind (word extra) (floor index ,elements-per-word)
@@ -280,12 +269,7 @@
                   (inst or old temp)))
                (if (typep offset '(signed-byte 13))
                    (inst st old object offset)
-                   (inst st old object offset-reg)))
-             (sc-case value
-               (immediate
-                (inst li result (tn-value value)))
-               (t
-                (move result value))))))))))
+                   (inst st old object offset-reg))))))))))
 
   (def-small-data-vector-frobs simple-bit-vector 1)
   (def-small-data-vector-frobs simple-array-unsigned-byte-2 2)
@@ -314,18 +298,14 @@
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg))
-         (value :scs (single-reg) :target result))
+         (value :scs (single-reg)))
   (:arg-types simple-array-single-float positive-fixnum single-float)
-  (:results (result :scs (single-reg)))
-  (:result-types single-float)
   (:temporary (:scs (non-descriptor-reg)) offset)
   (:generator 5
     (inst add offset index
           (- (* vector-data-offset n-word-bytes)
              other-pointer-lowtag))
-    (inst stf value object offset)
-    (unless (location= result value)
-      (inst fmovs result value))))
+    (inst stf value object offset)))
 
 (define-vop (data-vector-ref/simple-array-double-float)
   (:note "inline array access")
@@ -349,18 +329,14 @@
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg))
-         (value :scs (double-reg) :target result))
+         (value :scs (double-reg)))
   (:arg-types simple-array-double-float positive-fixnum double-float)
-  (:results (result :scs (double-reg)))
-  (:result-types double-float)
   (:temporary (:scs (non-descriptor-reg)) offset)
   (:generator 20
     (inst sll offset index 1)
     (inst add offset (- (* vector-data-offset n-word-bytes)
                         other-pointer-lowtag))
-    (inst stdf value object offset)
-    (unless (location= result value)
-      (move-double-reg result value))))
+    (inst stdf value object offset)))
 
 #+long-float
 (define-vop (data-vector-ref/simple-array-long-float)
@@ -416,9 +392,7 @@
   (:arg-types simple-array-signed-byte-8 positive-fixnum tagged-num)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg zero immediate))
-         (value :scs (signed-reg)))
-  (:results (result :scs (signed-reg)))
-  (:result-types tagged-num))
+         (value :scs (signed-reg))))
 
 
 (define-vop (data-vector-ref/simple-array-signed-byte-16
@@ -437,9 +411,7 @@
   (:arg-types simple-array-signed-byte-16 positive-fixnum tagged-num)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg zero immediate))
-         (value :scs (signed-reg)))
-  (:results (result :scs (signed-reg)))
-  (:result-types tagged-num))
+         (value :scs (signed-reg))))
 
 
 ;;; Complex float arrays.
@@ -468,29 +440,21 @@
   (:note "inline array store")
   (:translate data-vector-set)
   (:policy :fast-safe)
-  (:args (object :scs (descriptor-reg) :to :result)
+  (:args (object :scs (descriptor-reg))
          (index :scs (any-reg))
-         (value :scs (complex-single-reg) :target result))
+         (value :scs (complex-single-reg)))
   (:arg-types simple-array-complex-single-float positive-fixnum
               complex-single-float)
-  (:results (result :scs (complex-single-reg)))
-  (:result-types complex-single-float)
   (:temporary (:scs (non-descriptor-reg) :from (:argument 1)) offset)
   (:generator 5
-    (let ((value-real (complex-single-reg-real-tn value))
-          (result-real (complex-single-reg-real-tn result)))
+    (let ((value-real (complex-single-reg-real-tn value)))
       (inst sll offset index 1)
       (inst add offset (- (* vector-data-offset n-word-bytes)
                           other-pointer-lowtag))
-      (inst stf value-real object offset)
-      (unless (location= result-real value-real)
-        (inst fmovs result-real value-real)))
-    (let ((value-imag (complex-single-reg-imag-tn value))
-          (result-imag (complex-single-reg-imag-tn result)))
+      (inst stf value-real object offset))
+    (let ((value-imag (complex-single-reg-imag-tn value)))
       (inst add offset n-word-bytes)
-      (inst stf value-imag object offset)
-      (unless (location= result-imag value-imag)
-        (inst fmovs result-imag value-imag)))))
+      (inst stf value-imag object offset))))
 
 (define-vop (data-vector-ref/simple-array-complex-double-float)
   (:note "inline array access")
@@ -516,29 +480,21 @@
   (:note "inline array store")
   (:translate data-vector-set)
   (:policy :fast-safe)
-  (:args (object :scs (descriptor-reg) :to :result)
+  (:args (object :scs (descriptor-reg))
          (index :scs (any-reg))
-         (value :scs (complex-double-reg) :target result))
+         (value :scs (complex-double-reg)))
   (:arg-types simple-array-complex-double-float positive-fixnum
               complex-double-float)
-  (:results (result :scs (complex-double-reg)))
-  (:result-types complex-double-float)
   (:temporary (:scs (non-descriptor-reg) :from (:argument 1)) offset)
   (:generator 20
-    (let ((value-real (complex-double-reg-real-tn value))
-          (result-real (complex-double-reg-real-tn result)))
+    (let ((value-real (complex-double-reg-real-tn value)))
       (inst sll offset index 2)
       (inst add offset (- (* vector-data-offset n-word-bytes)
                           other-pointer-lowtag))
-      (inst stdf value-real object offset)
-      (unless (location= result-real value-real)
-        (move-double-reg result-real value-real)))
-    (let ((value-imag (complex-double-reg-imag-tn value))
-          (result-imag (complex-double-reg-imag-tn result)))
+      (inst stdf value-real object offset))
+    (let ((value-imag (complex-double-reg-imag-tn value)))
       (inst add offset (* 2 n-word-bytes))
-      (inst stdf value-imag object offset)
-      (unless (location= result-imag value-imag)
-        (move-double-reg result-imag value-imag)))))
+      (inst stdf value-imag object offset))))
 
 #+long-float
 (define-vop (data-vector-ref/simple-array-complex-long-float)
@@ -607,6 +563,4 @@
          (index :scs (any-reg zero immediate))
          (value :scs (unsigned-reg)))
   (:arg-types * tagged-num unsigned-num)
-  (:results (result :scs (unsigned-reg)))
-  (:result-types unsigned-num)
   (:variant vector-data-offset other-pointer-lowtag))
