@@ -222,7 +222,7 @@
                  ((xmm-register-p r)
                   0))))
     (let ((l (cond (l)
-                   (reg
+                   ((xmm-register-p reg)
                     (xmm-size reg))
                    ((xmm-register-p thing)
                     (xmm-size thing))
@@ -753,14 +753,12 @@
                 (:emitter
                  (cond ((and (xmm-register-p dst)
                              (ea-p src))
-                        (emit-avx2-inst segment dst src ,prefix #x10 :l 0))
+                        (emit-avx2-inst segment src dst ,prefix #x10 :l 0))
                        ((xmm-register-p dst)
-                        (emit-avx2-inst segment dst src2 ,prefix #x10 :vvvv src
-                                                                      :l 0))
+                        (emit-avx2-inst segment dst src2 ,prefix #x10 :vvvv src :l 0))
                        (t
                         (aver (xmm-register-p src))
-                        (emit-avx2-inst segment src dst ,prefix #x11
-                                        :l 0)))))))
+                        (emit-avx2-inst segment dst src ,prefix #x11 :l 0)))))))
   (def vmovsd #xf2)
   (def vmovss #xf3))
 
