@@ -133,8 +133,8 @@
 
 
 (deftransform %alien-funcall ((function type &rest args) * * :node node)
-  (aver (sb-c::constant-lvar-p type))
-  (let* ((type (sb-c::lvar-value type))
+  (aver (sb-c:constant-lvar-p type))
+  (let* ((type (sb-c:lvar-value type))
          (env (sb-c::node-lexenv node))
          (arg-types (alien-fun-type-arg-types type))
          (result-type (alien-fun-type-result-type type)))
@@ -202,8 +202,8 @@
 
 (defoptimizer (sign-extend derive-type) ((x size))
   (declare (ignore x))
-  (when (sb-c::constant-lvar-p size)
-    (specifier-type `(signed-byte ,(sb-c::lvar-value size)))))
+  (when (sb-c:constant-lvar-p size)
+    (specifier-type `(signed-byte ,(sb-c:lvar-value size)))))
 
 (define-vop (sign-extend)
   (:translate sign-extend)
@@ -247,7 +247,7 @@
    (inst mov res (ea (make-fixup foreign-symbol :foreign-dataref)))))
 
 #+sb-safepoint
-(defconstant thread-saved-csp-offset -1)
+(defconstant thread-saved-csp-offset (- (1+ sb-vm::thread-header-slots)))
 
 (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
   (defun destroyed-c-registers ()
@@ -264,10 +264,10 @@
       (append
        (loop for gpr in gprs
              collect `(:temporary (:sc any-reg :offset ,gpr :from :eval :to :result)
-                                  ,(car (push (gensym) vars))))
+                                  ,(car (push (sb-xc:gensym) vars))))
        (loop for float to 15
              collect `(:temporary (:sc single-reg :offset ,float :from :eval :to :result)
-                                  ,(car (push (gensym) vars))))
+                                  ,(car (push (sb-xc:gensym) vars))))
        `((:ignore ,@vars))))))
 
 (define-vop (call-out)
@@ -281,13 +281,17 @@
   (:temporary (:sc unsigned-reg :offset rax-offset :to :result) rax)
   #+sb-safepoint
   (:temporary (:sc unsigned-stack :from :eval :to :result) pc-save)
+  #+win32
+  (:temporary (:sc unsigned-reg :offset r15-offset :from :eval :to :result) r15)
   (:ignore results)
   (:vop-var vop)
   (:generator 0
     (move rbx function)
     (emit-c-call vop rax rbx args
                  sb-alien::*alien-fun-type-varargs-default*
-                 #+sb-safepoint pc-save))
+                 #+sb-safepoint pc-save
+                 #+win32 rbx))
+  #+win32 (:ignore r15)
   . #.(destroyed-c-registers))
 
 ;;; Calls to C can generally be made without loading a register
@@ -299,13 +303,26 @@
   (:temporary (:sc unsigned-reg :offset rax-offset :to :result) rax)
   #+sb-safepoint
   (:temporary (:sc unsigned-stack :from :eval :to :result) pc-save)
+  #+win32
+  (:temporary (:sc unsigned-reg :offset r15-offset :from :eval :to :result) r15)
+  #+win32
+  (:ignore r15)
+  #+win32
+  (:temporary (:sc unsigned-reg :offset rbx-offset :from :eval :to :result) rbx)
   (:ignore results)
   (:vop-var vop)
   (:generator 0
-    (emit-c-call vop rax c-symbol args varargsp #+sb-safepoint pc-save))
+    (emit-c-call vop rax c-symbol args varargsp
+                 #+sb-safepoint pc-save
+                 #+win32 rbx))
   . #.(destroyed-c-registers))
 
-(defun emit-c-call (vop rax fun args varargsp #+sb-safepoint pc-save)
+#+win32
+(defconstant win64-seh-direct-thunk-addr win64-seh-data-addr)
+#+win32
+(defconstant win64-seh-indirect-thunk-addr (+ win64-seh-data-addr 8))
+
+(defun emit-c-call (vop rax fun args varargsp #+sb-safepoint pc-save #+win32 rbx)
   (declare (ignorable varargsp))
   ;; Current PC - don't rely on function to keep it in a form that
   ;; GC understands
@@ -332,15 +349,15 @@
   ;; for vararg calls.
   (when varargsp
     (move-immediate rax
-                  (loop for tn-ref = args then (tn-ref-across tn-ref)
-                        while tn-ref
-                        count (eq (sb-name (sc-sb (tn-sc (tn-ref-tn tn-ref))))
-                                  'float-registers))))
+                    (loop for tn-ref = args then (tn-ref-across tn-ref)
+                          while tn-ref
+                          count (eq (sb-name (sc-sb (tn-sc (tn-ref-tn tn-ref))))
+                                    'float-registers))))
 
   ;; Store SP in thread struct, unless the enclosing block says not to
   #+sb-safepoint
   (when (policy (sb-c::vop-node vop) (/= sb-c:insert-safepoints 0))
-    (storew rsp-tn thread-base-tn thread-saved-csp-offset))
+    (inst mov (thread-slot-ea thread-saved-csp-offset) rsp-tn))
 
   #+win32 (inst sub rsp-tn #x20)       ;MS_ABI: shadow zone
 
@@ -349,9 +366,24 @@
   ;; where ea is the address of the linkage table entry's operand.
   ;; So while the former is a jump to a jump, we can optimize out
   ;; one jump in a statically linked executable.
+  #-win32
   (inst call (cond ((tn-p fun) fun)
                    ((sb-c::code-immobile-p vop) (make-fixup fun :foreign))
                    (t (ea (make-fixup fun :foreign 8)))))
+  ;; On win64, we don't support immobile space (yet) and calls go through one of
+  ;; the thunks defined in set_up_win64_seh_data(). If the linkage table is
+  ;; involved, RBX either points to a linkage table trampoline or to the linkage
+  ;; table operand; this simplifies UNDEFINED-ALIEN-TRAMP's job.
+  #+win32
+  (cond ((tn-p fun)
+         (move rbx fun)
+         (inst mov rax win64-seh-direct-thunk-addr)
+         (inst call rax))
+        (t
+         (inst mov rbx (make-fixup fun :foreign 8))
+         (inst mov rax win64-seh-indirect-thunk-addr)
+         (inst call rax)))
+
   ;; For the undefined alien error
   (note-this-location vop :internal-error)
   #+win32 (inst add rsp-tn #x20)       ;MS_ABI: remove shadow space
@@ -359,8 +391,7 @@
   ;; Zero the saved CSP, unless this code shouldn't ever stop for GC
   #+sb-safepoint
   (when (policy (sb-c::vop-node vop) (/= sb-c:insert-safepoints 0))
-    (inst xor (object-slot-ea thread-base-tn thread-saved-csp-offset 0)
-          rsp-tn)))
+    (inst xor (thread-slot-ea thread-saved-csp-offset) rsp-tn)))
 
 (define-vop (alloc-number-stack-space)
   (:info amount)
@@ -456,7 +487,7 @@
                      ;; stack location to a temporary register.
                      (unless gpr
                        (incf stack-argument-count)
-                       (setf gpr temp-reg-tn)
+                       (setf gpr rax)
                        (inst mov gpr stack-arg-tn))
                      ;; Copy from either argument register or temporary
                      ;; register to target.
@@ -473,8 +504,8 @@
                             ;; temporary (general purpose) register, and
                             ;; from there to the target location.
                             (incf stack-argument-count)
-                            (inst mov temp-reg-tn stack-arg-tn)
-                            (inst mov target-tn temp-reg-tn)))))
+                            (inst mov rax stack-arg-tn)
+                            (inst mov target-tn rax)))))
                   (t
                    (bug "Unknown alien floating point type: ~S" type)))))
 
