@@ -19,7 +19,7 @@
   (:generator 1
     (move res rsp-tn)))
 
-(define-vop ()
+(define-vop (current-fp-sap)
   (:translate current-fp)
   (:policy :fast-safe)
   (:results (res :scs (sap-reg sap-stack)))
@@ -51,19 +51,16 @@
   (:policy :fast-safe)
   (:args (sap :scs (sap-reg) :to :eval)
          (offset :scs (any-reg) :target temp)
-         (value :scs (descriptor-reg) :to :result :target result))
+         (value :scs (descriptor-reg)))
   (:arg-types system-area-pointer positive-fixnum *)
-  (:temporary (:sc unsigned-reg :from (:argument 1) :to :result) temp)
-  (:results (result :scs (descriptor-reg)))
-  (:result-types *)
+  (:temporary (:sc unsigned-reg :from (:argument 1)) temp)
   (:generator 9
     (move temp offset)
     (inst neg temp)
     (inst mov
           (ea (frame-byte-offset 0) sap
               temp (ash 1 (- word-shift n-fixnum-tag-bits)))
-          value)
-    (move result value)))
+          value)))
 
 (define-vop ()
   (:translate fun-code-header)
