@@ -77,10 +77,8 @@
   (:conditional :e)
   (:generator 1
     (inst cmp :dword
-          (ea (+ (ash (+ (get-dsd-index layout sb-kernel::id-word0)
-                         instance-slots-offset)
-                      word-shift)
-                 (ash (- (layout-depthoid test) 2) 2)
+          (ea (+ (id-bits-offset)
+                 (ash (- (wrapper-depthoid test) 2) 2)
                  (- instance-pointer-lowtag))
               x)
           ;; Small layout-ids can only occur for layouts made in genesis.
@@ -95,12 +93,12 @@
               (make-fixup test :layout-id)))))
 
 #+compact-instance-header
-;; ~20 instructions vs. 35
-(define-vop (layout-of) ; no translation
+(progn
+;; ~17 instructions vs. 35
+(define-vop ()
     (:policy :fast-safe)
-    (:translate layout-of)
-    (:args (object :scs (descriptor-reg))
-           #+nil (layouts :scs (constant)))
+    (:translate wrapper-of)
+    (:args (object :scs (descriptor-reg)))
     (:temporary (:sc unsigned-reg :offset rax-offset) rax)
     (:results (result :scs (descriptor-reg)))
     (:generator 6
@@ -128,13 +126,6 @@
       (inst jmp  :eq NULL)
       (inst movzx '(:byte :dword) rax object)
       LOAD-FROM-VECTOR
-      #+nil ;; old way
-      (progn
-        (inst mov  result layouts)
-        (inst mov  :dword result
-              (ea (+ (ash vector-data-offset word-shift) (- other-pointer-lowtag))
-                  result rax 8)))
-      ;; new way
       (inst mov :dword result
             (ea (make-fixup '**primitive-object-layouts**
                            :symbol-value
@@ -144,7 +135,22 @@
       (inst jmp  done)
       NULL
       (inst mov  result (make-fixup 'null :layout))
-      DONE))
+      DONE
+      #+metaspace (inst mov result (ea +5 result)))) ; layout->wrapper
+(define-vop ()
+    (:policy :fast-safe)
+    (:translate %instanceoid-layout)
+    (:args (object :scs (descriptor-reg) :to :save))
+    (:temporary (:sc unsigned-reg) temp)
+    (:results (result :scs (descriptor-reg)))
+    (:generator 6
+      (inst lea temp (ea -3 object))
+      (inst and temp (lognot #b1000))
+      (inst mov :dword result (make-fixup 't :layout))
+      (inst test :byte temp 15)
+      (inst jmp :ne DONE)
+      (inst mov :dword result (ea 4 temp))
+      DONE)))
 
 (macrolet ((load-type (target source lowtag)
              `(inst movzx '(:byte :dword) ,target (ea (- ,lowtag) ,source))))
@@ -155,8 +161,8 @@
   (:results (result :scs (unsigned-reg)))
   (:result-types positive-fixnum)
   (:generator 1 (load-type result object other-pointer-lowtag)))
-(define-vop (fun-subtype)
-  (:translate fun-subtype)
+(define-vop ()
+  (:translate %fun-pointer-widetag)
   (:policy :fast-safe)
   (:args (function :scs (descriptor-reg)))
   (:results (result :scs (unsigned-reg)))
@@ -175,58 +181,72 @@
 
 ;;; This operation is racy with GC and therefore slightly dangerous, especially
 ;;; on objects in immobile space which reserve byte 3 of the header for GC.
-(define-vop (set-header-data)
+(define-vop ()
   (:translate set-header-data)
   (:policy :fast-safe)
-  (:args (x :scs (descriptor-reg) :target res :to (:result 0))
+  (:args (x :scs (descriptor-reg) :to :eval)
          (data :scs (any-reg) :target temp))
   (:arg-types * positive-fixnum)
-  (:results (res :scs (descriptor-reg)))
-  (:temporary (:sc unsigned-reg :from (:argument 1) :to (:result 0)) temp)
+  (:temporary (:sc unsigned-reg :from (:argument 1)) temp)
   (:generator 6
     (move temp data)
     (inst shl temp (- n-widetag-bits n-fixnum-tag-bits))
     ;; merge in the widetag. We should really preserve bit 63 as well
     ;; which could be a GC mark bit, but it's not concurrent at the moment.
     (inst mov :byte temp (ea (- other-pointer-lowtag) x))
-    (storew temp x 0 other-pointer-lowtag)
+    (storew temp x 0 other-pointer-lowtag)))
+(define-vop (logior-header-bits)
+  (:translate logior-header-bits)
+  (:policy :fast-safe)
+  (:args (x :scs (descriptor-reg))
+         (bits :scs (unsigned-reg immediate)))
+  (:arg-types * positive-fixnum)
+  (:results (res :scs (descriptor-reg)))
+  (:generator 1
+    (if (sc-is bits immediate)
+        (let ((bits (tn-value bits)))
+          (cond ((typep bits '(unsigned-byte 8))
+                 (inst or :byte (ea (- 1 other-pointer-lowtag) x) bits))
+                ((not (logtest bits #xff))
+                 (inst or :byte (ea (- 2 other-pointer-lowtag) x) (ash bits -8)))
+                (t
+                 (inst or :word (ea (- 1 other-pointer-lowtag) x) bits))))
+        (inst or :word (ea (- 1 other-pointer-lowtag) x) bits))
     (move res x)))
-
-;;; These next 3 vops are for manipulating and reading the flag bits of
-;;; arrays headers, or more generally any OTHER-POINTER object.
-(define-vop (set-header-bits)
-  (:translate set-header-bits)
+(define-vop ()
+  (:translate assign-vector-flags)
   (:policy :fast-safe)
   (:args (x :scs (descriptor-reg)))
-  (:arg-types t (:constant t))
   (:info bits)
+  (:arg-types t (:constant (unsigned-byte 8)))
   (:generator 1
-    (if (typep bits '(unsigned-byte 8))
-        (inst or :byte (ea (- 1 other-pointer-lowtag) x) bits)
-        (inst or :dword (ea (- other-pointer-lowtag) x) (ash bits n-widetag-bits)))))
-(define-vop (unset-header-bits)
-  (:translate unset-header-bits)
+    (inst mov :byte (ea (- (/ array-flags-position n-word-bytes) other-pointer-lowtag) x)
+      bits)))
+(define-vop ()
+  (:translate reset-header-bits)
   (:policy :fast-safe)
   (:args (x :scs (descriptor-reg)))
-  (:arg-types t (:constant t))
+  (:arg-types t (:constant (unsigned-byte 16)))
   (:info bits)
   (:generator 1
-    (if (typep bits '(unsigned-byte 8))
-        (inst and :byte (ea (- 1 other-pointer-lowtag) x) (logandc1 bits #xff))
-        (inst and :dword (ea (- other-pointer-lowtag) x)
-              (lognot (ash bits n-widetag-bits))))))
+    (let ((byte 1))
+      (when (> bits #xff)
+        (setf bits (ash bits -8))
+        (setf byte 2))
+      (inst and :byte (ea (- byte other-pointer-lowtag) x) (logandc1 bits #xff)))))
 (define-vop (test-header-bit)
   (:translate test-header-bit)
   (:policy :fast-safe)
   (:args (array :scs (descriptor-reg)))
-  (:arg-types t (:constant t))
   (:info mask)
+  (:arg-types t (:constant t))
   (:conditional :ne)
   (:generator 1
-    ;; Assert that the mask is in header-data byte index 0
-    ;; which is byte index 1 of the whole header word.
-    (aver (typep mask '(unsigned-byte 8)))
-    (inst test :byte (ea (- 1 other-pointer-lowtag) array) mask)))
+    (let ((byte 1))
+      (when (> mask #xff)
+        (setf mask (ash mask -8))
+        (setf byte 2))
+      (inst test :byte (ea (- byte other-pointer-lowtag) array) mask))))
 
 (define-vop (pointer-hash)
   (:translate pointer-hash)
@@ -315,39 +335,27 @@
                result))))
 
 ;;;; symbol frobbing
-(defun load-symbol-info-vector (result symbol)
+(defun load-symbol-dbinfo (result symbol)
   (loadw result symbol symbol-info-slot other-pointer-lowtag)
   ;; If RES has list-pointer-lowtag, take its CDR. If not, use it as-is.
   ;; This CMOV safely reads from memory when it does not move, because if
-  ;; there is an info-vector in the slot, it has at least one element.
+  ;; there is a PACKED-INFO in the slot, it has at least 4 data words in total
+  ;; - the header, at least one info descriptor, and at least one datum.
+  ;; And since 3 is odd, that would be aligned up to 4.
+  ;; Use bit index 2 of the lowtag to distinguish list from instance.
+  ;; An instance will have a 0 in that bit.
   ;; This would compile to almost the same code without a VOP,
   ;; but using a jmp around a mov instead.
-  (aver (= (logior list-pointer-lowtag #b1000) other-pointer-lowtag))
-  (inst test :byte result #b1000)
-  (inst cmov :e result
-        (object-slot-ea result cons-cdr-slot list-pointer-lowtag)))
+  (aver (= (logior instance-pointer-lowtag #b0100) list-pointer-lowtag))
+  (inst test :byte result #b0100)
+  (inst cmov :nz result (object-slot-ea result cons-cdr-slot list-pointer-lowtag)))
 
-(define-vop (symbol-info-vector)
+(define-vop (symbol-dbinfo)
   (:policy :fast-safe)
-  (:translate symbol-info-vector)
+  (:translate symbol-dbinfo)
   (:args (x :scs (descriptor-reg)))
   (:results (res :scs (descriptor-reg)))
-  (:generator 1 (load-symbol-info-vector res x)))
-
-(define-vop (symbol-plist)
-  (:policy :fast-safe)
-  (:translate symbol-plist)
-  (:args (x :scs (descriptor-reg)))
-  (:results (res :scs (descriptor-reg)))
-  (:temporary (:sc unsigned-reg) temp)
-  (:generator 1
-    (loadw res x symbol-info-slot other-pointer-lowtag)
-    ;; Instruction pun: (CAR x) is the same as (VECTOR-LENGTH x)
-    ;; so if the info slot holds a vector, this gets a fixnum- it's not a plist.
-    (loadw res res cons-car-slot list-pointer-lowtag)
-    (inst mov temp nil-value)
-    (inst test :byte res fixnum-tag-mask)
-    (inst cmov :e res temp)))
+  (:generator 1 (load-symbol-dbinfo res x)))
 
 ;;;; other miscellaneous VOPs
 
@@ -373,13 +381,14 @@
   (:results (sap :scs (sap-reg)))
   (:result-types system-area-pointer)
   (:translate current-thread-offset-sap)
-  (:args (n :scs (any-reg) :target sap))
+  (:args (index :scs (any-reg) :target sap))
   (:arg-types tagged-num)
   (:policy :fast-safe)
   (:generator 2
-    (inst mov sap
-          (ea thread-base-tn
-              n (ash 1 (- word-shift n-fixnum-tag-bits)))))))
+    (let (#+gs-seg (thread-tn nil))
+      (inst mov sap
+            (ea thread-segment-reg thread-tn
+                index (ash 1 (- word-shift n-fixnum-tag-bits))))))))
 
 (define-vop (halt)
   (:generator 1
