@@ -275,6 +275,9 @@
 ;;; A (SIGNED-BYTE 64) can be represented with either fixnum or a bignum with
 ;;; exactly one digit.
 
+(defmacro bignum-header-for-length (n)
+  (logior (ash n n-widetag-bits) bignum-widetag))
+
 (define-vop (signed-byte-64-p type-predicate)
   (:translate signed-byte-64-p)
   (:generator 45
@@ -295,8 +298,7 @@
            (inst and :byte temp lowtag-mask)
            (inst cmp :byte temp other-pointer-lowtag)))
       (inst jmp :ne nope)
-      (inst cmp :qword (ea (- other-pointer-lowtag) value)
-            (+ (ash 1 n-widetag-bits) bignum-widetag))
+      (inst cmp :qword (ea (- other-pointer-lowtag) value) (bignum-header-for-length 1))
       (inst jmp (if not-p :ne :e) target))
     NOT-TARGET))
 
@@ -325,11 +327,11 @@
         ;; Get the header.
         (loadw temp value 0 other-pointer-lowtag)
         ;; Is it one?
-        (inst cmp temp (+ (ash 1 n-widetag-bits) bignum-widetag))
+        (inst cmp temp (bignum-header-for-length 1))
         (inst jmp :e single-word)
         ;; If it's other than two, we can't be an (unsigned-byte 64)
         ;: Leave TEMP holding 0 in the affirmative case.
-        (inst sub temp (+ (ash 2 n-widetag-bits) bignum-widetag))
+        (inst sub temp (bignum-header-for-length 2))
         (inst jmp :ne nope)
         ;; Compare the second digit to zero (in TEMP).
         (inst cmp (object-slot-ea value (1+ bignum-digits-offset) other-pointer-lowtag)
@@ -397,31 +399,87 @@
        (inst jmp (if not-p :a :be) target)
        (emit-label skip))))
 
+;;; SINGLE-FLOAT-P, CHARACTERP, UNBOUND-MARKER-P produce a flag result
+;;; and never need a temporary.
+(macrolet ((define (name widetag)
+             `(define-vop (,name simple-type-predicate)
+                (:translate ,name)
+                (:info)
+                (:conditional :z)
+                (:generator 1 (inst cmp :byte value ,widetag)))))
+  (define single-float-p single-float-widetag)
+  (define characterp character-widetag)
+  (define unbound-marker-p unbound-marker-widetag))
+
 (define-vop (pointerp)
-  (:args (value :scs (any-reg descriptor-reg) :target temp))
+  (:args (value :scs (any-reg descriptor-reg)))
   (:temporary (:sc unsigned-reg :from (:argument 0)) temp)
-  (:conditional)
-  (:info target not-p)
+  (:conditional :z)
   (:policy :fast-safe)
   (:translate pointerp)
   (:generator 3
-    (inst lea :dword temp (ea -3 value))
-    (inst test :byte temp #b11)
-    (inst jmp (if not-p :nz :z) target)))
+    (if (location= temp value) (inst sub :dword value 3) (inst lea :dword temp (ea -3 value)))
+    (inst test :byte temp #b11)))
+
+;;; FUNCTIONP, LISTP, %INSTANCEP, %OTHER-POINTER-P produce a flag result
+(macrolet ((define (name lowtag)
+             `(define-vop (,name pointerp)
+                (:translate ,name)
+                (:generator 2
+                  (if (location= temp value)
+                      (inst sub :dword value ,lowtag)
+                      (inst lea :dword temp (ea (- ,lowtag) value)))
+                  (inst test :byte temp lowtag-mask)))))
+  (define functionp fun-pointer-lowtag)
+  (define listp list-pointer-lowtag)
+  (define %instancep instance-pointer-lowtag)
+  (define %other-pointer-p other-pointer-lowtag))
+
+(macrolet ((define (name widetag)
+             `(define-vop (,name type-predicate)
+                (:translate ,name)
+                (:info)
+                (:conditional :z)
+                (:generator 4
+                  (inst lea temp (ea (- fun-pointer-lowtag) value))
+                  (inst test :byte temp lowtag-mask)
+                  (inst jmp :ne out)
+                  (inst cmp :byte (ea temp) ,widetag)
+                  out))))
+  (define closurep closure-widetag)
+  (define simple-fun-p simple-fun-widetag)
+  (define funcallable-instance-p funcallable-instance-widetag))
 
 ;;;; list/symbol types
 ;;;
 ;;; symbolp (or symbol (eq nil))
 ;;; consp (and list (not (eq nil)))
 
+(defun test-other-ptr (arg arg-ref widetag temp label)
+  (inst cmp :byte
+        (cond ((other-pointer-tn-ref-p arg-ref)
+               (ea (- other-pointer-lowtag) arg))
+              (t
+               (%lea-for-lowtag-test temp arg other-pointer-lowtag :qword)
+               (inst test :byte temp lowtag-mask)
+               (inst jmp :ne label)
+               (ea temp)))
+        widetag))
+
 (define-vop (symbolp type-predicate)
   (:translate symbolp)
-  (:generator 12
-    (let ((is-symbol-label (if not-p DROP-THRU target)))
-      (inst cmp value nil-value)
-      (inst jmp :e is-symbol-label)
-      (test-type value temp target not-p (symbol-widetag)))
-    DROP-THRU))
+  (:info)
+  (:conditional :z)
+  (:generator 5
+    ;; SYMBOLP would have been IR1-transformed to NON-NULL-SYMBOL-P when possible
+    (inst cmp value nil-value)
+    (inst jmp :e out)
+    (test-other-ptr value args symbol-widetag temp out)
+    out))
+
+(define-vop (non-null-symbol-p symbolp)
+  (:translate non-null-symbol-p)
+  (:generator 3 (test-other-ptr value args symbol-widetag temp out) out))
 
 (define-vop (consp type-predicate)
   (:translate consp)
@@ -443,41 +501,74 @@
   (:generator 2
    (inst cmp :byte (ea (- other-pointer-lowtag) x) widetag)))
 
-;;; TRANSFORM-INSTANCE-TYPEP checks for this vop by name and will try to use it,
-;;; so don't define it if inapplicable.
 #+compact-instance-header
 (progn
-(defknown layout-eq (instance t) boolean (flushable))
-(define-vop (layout-eq)
-  (:translate layout-eq)
+ (define-vop ()
+   (:translate %instance-layout)
+   (:policy :fast-safe)
+   (:args (object :scs (descriptor-reg)))
+   (:results (res :scs (descriptor-reg)))
+   (:variant-vars lowtag)
+   (:variant instance-pointer-lowtag)
+   (:generator 1
+    (inst mov :dword res (ea (- 4 lowtag) object))))
+ (define-vop ()
+   (:translate %set-instance-layout)
+   (:policy :fast-safe)
+   (:args (object :scs (descriptor-reg))
+          (value :scs (any-reg descriptor-reg)))
+   (:vop-var vop)
+   (:temporary (:sc unsigned-reg) temp)
+   (:generator 1
+     (emit-gc-store-barrier object nil temp (vop-nth-arg 1 vop) value)
+     (inst mov :dword (ea (- 4 instance-pointer-lowtag) object) value)))
+ (define-vop (%fun-layout %instance-layout)
+   (:translate %fun-layout)
+   (:variant fun-pointer-lowtag))
+ (define-vop (%set-fun-layout %set-instance-layout)
+   (:translate %set-fun-layout)
+   (:generator 1
+     (emit-gc-store-barrier object nil temp (vop-nth-arg 1 vop) value)
+     (inst mov :dword (ea (- 4 fun-pointer-lowtag) object) value)))
+ (define-vop ()
+  (:translate sb-c::layout-eq)
   (:policy :fast-safe)
   (:conditional :e)
-  (:args (instance :scs (descriptor-reg))
-         (layout :scs (descriptor-reg immediate)))
+  (:args (object :scs (descriptor-reg))
+         (layout :scs (descriptor-reg immediate #+metaspace constant)))
+  (:arg-types * * (:constant t))
+  (:info lowtag)
   (:generator 1
-    (inst cmp :dword
-          (ea (- 4 instance-pointer-lowtag) instance)
-          (if (sc-is layout immediate)
+    ;; With metaspace, the layout argument is actually a #<WRAPPER>
+    ;; which does not have IMMEDIATE sc, but rather CONSTANT sc.
+    ;; But we use a layout fixup which stuffs in the pointer to the layout.
+    (inst cmp :dword (ea (- 4 lowtag) object)
+          (if (sc-is layout immediate constant)
               (make-fixup (tn-value layout) :layout)
               layout)))))
 
-;;; Return the DISP part of an EA based on MEM-OP, which is comprised
-;;; of a vop name and its codegen info.
-;;; Return NIL if the value can't be encoded in a machine instruction.
-(defun valid-memref-byte-disp (mem-op)
-  (ecase (car mem-op)
-    ((instance-index-ref-c
-      raw-instance-ref-c/word
-      raw-instance-ref-c/signed-word)
-     (destructuring-bind (index) (cdr mem-op)
+;;; Return the DISP part of an EA based on MEM-OP,
+;;; which is a memory access vop such as INSTANCE-REF.
+;;; Return NIL if the access can't be absorbed into a following instruction.
+(defun valid-memref-byte-disp (mem-op &aux (info (vop-codegen-info mem-op)))
+  (ecase (vop-name mem-op)
+    (instance-index-ref-c
+     ;; for historical reasons, this has a "-C" variant which takes an info arg
+     (destructuring-bind (index) info
        (- (ash (+ index instance-slots-offset) word-shift)
           instance-pointer-lowtag)))
+    ((%raw-instance-ref/word %raw-instance-ref/signed-word)
+     ;; raw slot vops accept an immediate TN, not a codegen arg
+     (let ((index (tn-ref-tn (tn-ref-across (vop-args mem-op)))))
+       (when (sc-is index immediate)
+         (- (ash (+ (tn-value index) instance-slots-offset) word-shift)
+            instance-pointer-lowtag))))
     (slot
-     (destructuring-bind (name index lowtag) (cdr mem-op)
+     (destructuring-bind (name index lowtag) info
        (declare (ignore name))
        (- (ash index word-shift) lowtag)))
     (data-vector-ref-with-offset/simple-vector-c
-     (destructuring-bind (index offset) (cdr mem-op)
+     (destructuring-bind (index offset) info
        (let ((disp (- (ash (+ vector-data-offset index offset) word-shift)
                       other-pointer-lowtag)))
          (if (typep disp '(signed-byte 32)) disp))))))
@@ -486,12 +577,27 @@
   (:translate fixnump)
   (:args-var arg-ref)
   (:args (value :scs (any-reg descriptor-reg) :load-if (tn-ref-memory-access arg-ref)))
+  (:info)
+  (:conditional :z)
+  ;; the compiler is very sensitive to this cost here as regards boxing. DON'T TOUCH !!!
   (:generator 3
    (awhen (tn-ref-memory-access arg-ref)
      (setq value (ea (cdr it) value)))
-   (test-type value nil target not-p
-              (even-fixnum-lowtag odd-fixnum-lowtag pad0-lowtag pad1-lowtag
-               pad2-lowtag pad3-lowtag pad4-lowtag pad5-lowtag))))
+   (generate-fixnum-test value)))
+
+(macrolet
+    ((define-simple-array-type-vops ()
+         `(progn
+           ,@(map 'list
+                  (lambda (saetp &aux (primtype (saetp-primitive-type-name saetp))
+                                      (name (symbolicate primtype "-P")))
+                    `(define-vop (,name symbolp)
+                       (:translate ,name)
+                       (:generator 4
+                        (test-other-ptr value args ,(saetp-typecode saetp) temp out)
+                        out)))
+                  *specialized-array-element-type-properties*))))
+  (define-simple-array-type-vops))
 
 ;;; Try to absorb a memory load into FIXNUMP.
 (defoptimizer (sb-c::vop-optimize fixnump) (vop)
@@ -500,20 +606,30 @@
   ;; to (memref+fixnump (memref x)), it would seem to allow matching of the pattern
   ;; again if this optimizer is reapplied, because the "new" fixnump vop is superficially
   ;; the same, except for the attachment of extra data to its input.
-  (unless (tn-ref-memory-access (sb-c::vop-args vop))
+  (unless (tn-ref-memory-access (vop-args vop))
     (let ((prev (sb-c::previous-vop-is
                  vop
                  '(instance-index-ref-c slot
+                   ;; FIXME: could we also handle the non "-C" vop?
+                   ;; The problem is that because FIXNUMP only takes one arg,
+                   ;; an additional TN would have to be grafted into the data flow
+                   ;; to convey the INDEX arg. So it's not the same fixnump vop
+                   ;; any more - it's like a two-arg variant of fixnump.
                    data-vector-ref-with-offset/simple-vector-c))))
-      (aver (not (sb-c::vop-results vop))) ; is a :CONDITIONAL vop
+      ;; Inhibit the optimization on simple-vector if we need to trap uninitialized reads.
+      ;; #+ubsan always inhibits, otherwise it's policy-based
+      (when (and prev
+                 (eq (vop-name prev) 'data-vector-ref-with-offset/simple-vector-c)
+                 #-ubsan (sb-c::policy (sb-c::vop-node vop) (= safety 3)))
+        (return-from vop-optimize-fixnump-optimizer nil))
+      (aver (not (vop-results vop))) ; is a :CONDITIONAL vop
       (when (and prev (eq (vop-block prev) (vop-block vop)))
-        (let ((arg (sb-c::vop-args vop)))
-          (when (and (eq (tn-ref-tn (sb-c::vop-results prev)) (tn-ref-tn arg))
+        (let ((arg (vop-args vop)))
+          (when (and (eq (tn-ref-tn (vop-results prev)) (tn-ref-tn arg))
                      (sb-c::very-temporary-p (tn-ref-tn arg)))
-            (binding* ((mem-op (cons (vop-name prev) (vop-codegen-info prev)))
-                       (disp (valid-memref-byte-disp mem-op) :exit-if-null)
+            (binding* ((disp (valid-memref-byte-disp prev) :exit-if-null)
                        (arg-ref
-                        (sb-c::reference-tn (tn-ref-tn (sb-c::vop-args prev)) nil))
+                        (sb-c:reference-tn (tn-ref-tn (vop-args prev)) nil))
                        (new (sb-c::emit-and-insert-vop
                              (sb-c::vop-node vop) (vop-block vop) (sb-c::vop-info vop)
                              arg-ref nil prev (vop-codegen-info vop))))
