@@ -13,6 +13,80 @@
 
 ;;;; register specs
 
+;;; Observe that [R12 + displacement] needs an extra byte of encoding in
+;;; certain addressing modes, as does [R13 + index]
+;;;     49895B08         MOV [R11+8], RBX
+;;;     49895C2408       MOV [R12+8], RBX
+;;;     49895D08         MOV [R13+8], RBX
+;;;     49891C03         MOV [R11+RAX], RBX
+;;;     49891C04         MOV [R12+RAX], RBX
+;;;     49895C0500       MOV [R13+RAX], RBX
+;;; This is because 12 and 13 are equal-mod-8 to 4 and 5, the indices of RSP
+;;; and RBP respectively, which have different behaviors in the ModRegR/M byte.
+;;; So due to lack of perfectly orthogonal encodings, it would be slightly
+;;; beneficial to hint to the register allocator that it should pick registers
+;;; 12 and 13 only when all other registers are unavailable.
+;;; And you might think that it should "work" simply to list the usable offsets
+;;; in the SC in the order in which they should be chosen when the register
+;;; allocator needs to pick the next available register, but unfortunately
+;;; it does not work to do that, because it uses bitmaps from which to pick,
+;;; and not an ordered list.
+
+;;; A register that's never used by the code generator, and can therefore
+;;; be used as an assembly temporary in cases where a VOP :TEMPORARY can't
+;;; be used.
+;;; This is only enabled for certain build configurations that need
+;;; a scratch register in various random places.
+#-ubsan (eval-when (:compile-toplevel :load-toplevel :execute)
+          (defconstant global-temp-reg nil))
+#+ubsan (progn (define-symbol-macro temp-reg-tn r11-tn)
+               (defconstant global-temp-reg 11))
+
+#+gs-seg
+(progn
+  ;; There is no permanent THREAD-TN. There are a few problems in trying
+  ;; cleverly to make PSEUDO-ATOMIC agnostic of whether you're using a global
+  ;; THREAD-TN or a local one (a global symbol macro vs. a let binding)
+  ;; 1) ECL seems to think that it is an error to LET bind a global symbol macro.
+  ;;    > (define-symbol-macro thread-tn 'wat)
+  ;;    > (defun f (&optional (thread-tn thread-tn)) (format t "Hi: ~a~%" thread-tn))
+  ;;    > (compile'f)
+  ;;    ;;; Error:
+  ;;    ;;;   * The constant THREAD-TN is being bound.
+  ;; 2) It actually doesn't completely work completely anyway,
+  ;;    because pseudo-atomic needs to know not to use a segment override,
+  ;;    and TNs don't indicate a memory segment.
+  ;; While the second problem could be solved, the first can't,
+  ;; and it's not clear to me whether ECL is wrong to say it's an error,
+  ;; or SBCL is remiss in choosing not to style-warn (which would annoy me
+  ;; for that reason).
+  ;; So we wire the thread structure to r15 within some vops when convenient.
+  ;; Wiring to r15 is not strictly necessary, but I want to see if sb-aprof
+  ;; can be made to work slightly easier regardless of #+gs-seg.
+  ;; And with 15 being the highest register number, it is the least likely
+  ;; to be picked by the register allocator for anything else.
+  ;; Additionally I'd like to have an optimization pass that avoids reloading
+  ;; r15 when consecutive allocation sequences occur in straight-line code.
+  (define-symbol-macro thread-segment-reg :gs)
+  (define-symbol-macro thread-reg nil))
+#-gs-seg
+(progn
+  (define-symbol-macro thread-segment-reg :cs)
+  ;; r13 is preferable to r12 because 12 is an alias of 4 in ModRegRM, which
+  ;; implies use of a SIB byte with no index register for fixed displacement.
+  (define-symbol-macro thread-reg 13)
+  (define-symbol-macro thread-tn r13-tn))
+
+;;; Kludge needed for decoding internal error args - we assume that the
+;;; shadow memory is pointed to by this register (RAX).
+(defconstant msan-temp-reg-number 0)
+
+;;; The encoding anomaly for r12 makes it a perfect choice for the card table base.
+;;; It will never be used with a constant displacement.
+(define-symbol-macro card-table-reg 12)
+(define-symbol-macro gc-card-table-reg-tn r12-tn)
+(define-symbol-macro card-index-mask (make-fixup nil :gc-barrier))
+
 (macrolet ((defreg (name offset size)
              (declare (ignore size))
              `(eval-when (:compile-toplevel :load-toplevel :execute)
@@ -45,8 +119,9 @@
                              array))))
                 (defglobal ,offsets-list
                     (remove-if (lambda (x)
-                                 (member x `(,r11-offset ; temp reg
-                                             ,r13-offset ; thread base
+                                 (member x `(,global-temp-reg ; if there is one
+                                             ,thread-reg      ; if using a GPR
+                                             ,card-table-reg
                                              ,rsp-offset
                                              ,rbp-offset)))
                                (loop for i below 16 collect i))))))
@@ -54,7 +129,7 @@
   (define-gprs t *qword-regs* +qword-register-names+
     #("RAX" "RCX" "RDX" "RBX" "RSP" "RBP" "RSI" "RDI"
       "R8"  "R9"  "R10" "R11" "R12" "R13" "R14" "R15"))
-  (define-gprs t *dword-regs* +dword-register-names+
+  (define-gprs nil *dword-regs* +dword-register-names+
     #("EAX" "ECX" "EDX"  "EBX"  "ESP"  "EBP"  "ESI"  "EDI"
       "R8D" "R9D" "R10D" "R11D" "R12D" "R13D" "R14D" "R15D"))
   (define-gprs nil *word-regs* +word-register-names+
@@ -96,9 +171,9 @@
   (eval-when (:compile-toplevel :load-toplevel :execute)
     (defparameter *register-arg-names* '(rdx rdi rsi)))
   (defregset    *register-arg-offsets* rdx rdi rsi)
-  #!-win32
+  #-win32
   (defregset    *c-call-register-arg-offsets* rdi rsi rdx rcx r8 r9)
-  #!+win32
+  #+win32
   (defregset    *c-call-register-arg-offsets* rcx rdx r8 r9))
 
 ;;;; SB definitions
@@ -132,12 +207,12 @@
   (fp-complex-single-immediate immediate-constant)
   (fp-complex-double-immediate immediate-constant)
 
-  #!+sb-simd-pack (int-sse-immediate immediate-constant)
-  #!+sb-simd-pack (double-sse-immediate immediate-constant)
-  #!+sb-simd-pack (single-sse-immediate immediate-constant)
-  #!+sb-simd-pack-256 (int-avx2-immediate immediate-constant)
-  #!+sb-simd-pack-256 (double-avx2-immediate immediate-constant)
-  #!+sb-simd-pack-256 (single-avx2-immediate immediate-constant)
+  #+sb-simd-pack (int-sse-immediate immediate-constant)
+  #+sb-simd-pack (double-sse-immediate immediate-constant)
+  #+sb-simd-pack (single-sse-immediate immediate-constant)
+  #+sb-simd-pack-256 (int-avx2-immediate immediate-constant)
+  #+sb-simd-pack-256 (double-avx2-immediate immediate-constant)
+  #+sb-simd-pack-256 (single-avx2-immediate immediate-constant)
   (immediate immediate-constant)
 
   ;;
@@ -156,17 +231,17 @@
   (double-stack stack)
   (complex-single-stack stack)  ; complex-single-floats
   (complex-double-stack stack :element-size 2)  ; complex-double-floats
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (int-sse-stack stack :element-size 2)
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (double-sse-stack stack :element-size 2)
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (single-sse-stack stack :element-size 2)
-  #!+sb-simd-pack-256
+  #+sb-simd-pack-256
   (int-avx2-stack stack :element-size 4)
-  #!+sb-simd-pack-256
+  #+sb-simd-pack-256
   (double-avx2-stack stack :element-size 4)
-  #!+sb-simd-pack-256
+  #+sb-simd-pack-256
   (single-avx2-stack stack :element-size 4)
 
   ;;
@@ -252,44 +327,44 @@
                       :alternate-scs (complex-double-stack))
 
   ;; temporary only
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (sse-reg float-registers
            :locations #.*float-regs*)
   ;; regular values
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (int-sse-reg float-registers
                :locations #.*float-regs*
                :constant-scs (int-sse-immediate)
                :save-p t
                :alternate-scs (int-sse-stack))
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (double-sse-reg float-registers
                   :locations #.*float-regs*
                   :constant-scs (double-sse-immediate)
                   :save-p t
                   :alternate-scs (double-sse-stack))
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (single-sse-reg float-registers
                   :locations #.*float-regs*
                   :constant-scs (single-sse-immediate)
                   :save-p t
                   :alternate-scs (single-sse-stack))
-  #!+avx2
-  (avx2-reg float-registers
-   :locations #.*float-regs*)
-  #!+sb-simd-pack-256
+  (ymm-reg float-registers :locations #.*float-regs*)
+  ;; These next 3 should probably be named to YMM-{INT,SINGLE,DOUBLE}-REG
+  ;; but I think there are 3rd-party libraries that expect these names.
+  #+sb-simd-pack-256
   (int-avx2-reg float-registers
                :locations #.*float-regs*
                :constant-scs (int-avx2-immediate)
                :save-p t
                :alternate-scs (int-avx2-stack))
-  #!+sb-simd-pack-256
+  #+sb-simd-pack-256
   (double-avx2-reg float-registers
                   :locations #.*float-regs*
                   :constant-scs (double-avx2-immediate)
                   :save-p t
                   :alternate-scs (double-avx2-stack))
-  #!+sb-simd-pack-256
+  #+sb-simd-pack-256
   (single-avx2-reg float-registers
                   :locations #.*float-regs*
                   :constant-scs (single-avx2-immediate)
@@ -311,20 +386,20 @@
 (defparameter *double-sc-names* '(double-reg double-stack))
 (defparameter *complex-sc-names* '(complex-single-reg complex-single-stack
                                    complex-double-reg complex-double-stack))
-#!+sb-simd-pack
+#+sb-simd-pack
 (defparameter *oword-sc-names* '(sse-reg int-sse-reg single-sse-reg double-sse-reg
                                  int-sse-stack single-sse-stack double-sse-stack))
-#!+sb-simd-pack-256
-(defparameter *hword-sc-names* '(avx2-reg int-avx2-reg single-avx2-reg double-avx2-reg
+#+sb-simd-pack-256
+(defparameter *hword-sc-names* '(ymm-reg int-avx2-reg single-avx2-reg double-avx2-reg
                                    int-avx2-stack single-avx2-stack double-avx2-stack))
 ) ; EVAL-WHEN
 (!define-storage-classes
   . #.(mapcar (lambda (class-spec)
                 (let ((size
                         (case (car class-spec)
-                          #!+sb-simd-pack
+                          #+sb-simd-pack
                           (#.*oword-sc-names*   :oword)
-                          #!+sb-simd-pack-256
+                          #+sb-simd-pack-256
                           (#.*hword-sc-names*   :hword)
                           (#.*qword-sc-names*   :qword)
                           (#.*float-sc-names*   :float)
@@ -372,11 +447,10 @@
 (defun xmm-tn-p (thing)
   (and (tn-p thing)
        (eq (sb-name (sc-sb (tn-sc thing))) 'float-registers)))
-
-;; A register that's never used by the code generator, and can therefore
-;; be used as an assembly temporary in cases where a VOP :TEMPORARY can't
-;; be used.
-(define-symbol-macro temp-reg-tn r11-tn)
+;;; Return true if THING is on the stack (in whatever storage class).
+(defun stack-tn-p (thing)
+  (and (tn-p thing)
+       (eq (sb-name (sc-sb (tn-sc thing))) 'stack)))
 
 ;;; TNs for registers used to pass arguments
 ;;; This can't be a DEFCONSTANT-EQX, for a similar reason to above, but worse.
@@ -389,23 +463,19 @@
             (symbol-value (symbolicate register-arg-name "-TN")))
           *register-arg-names*))
 
-;; r13 is preferable to r12 because 12 is an alias of 4 in ModRegRM, which
-;; implies use of a SIB byte with no index register for fixed displacement.
-(define-symbol-macro thread-base-tn r13-tn)
-
 ;;; If value can be represented as an immediate constant, then return
 ;;; the appropriate SC number, otherwise return NIL.
 (defun immediate-constant-sc (value)
   (typecase value
-    ((or (integer #.sb-xc:most-negative-fixnum #.sb-xc:most-positive-fixnum)
+    ((or (integer #.most-negative-fixnum #.most-positive-fixnum)
          character)
      immediate-sc-number)
     (symbol ; Symbols in static and immobile space are immediate
-     (when (or ;; With #!+immobile-symbols, all symbols are in immobile-space.
+     (when (or ;; With #+immobile-symbols, all symbols are in immobile-space.
                ;; And the cross-compiler always uses immobile-space if enabled.
-               #!+(or immobile-symbols (and immobile-space (host-feature sb-xc-host))) t
+               #+(or immobile-symbols (and immobile-space sb-xc-host)) t
 
-               ;; Otherwise, if #!-immobile-symbols, and the symbol was present
+               ;; Otherwise, if #-immobile-symbols, and the symbol was present
                ;; in the initial core image as indicated by the symbol header, then
                ;; it's in immobile-space. There is a way in which the bit can be wrong,
                ;; but it's highly unlikely - the symbol would have to be uninterned from
@@ -413,39 +483,38 @@
                ;; initial package. All without breaking anything. Hence, unlikely.
                ;; Also note that if compiling to memory, the symbol's current address
                ;; is used to determine whether it's immediate.
-               #!+(and (not (host-feature sb-xc-host)) immobile-space (not immobile-symbols))
+               #+(and (not sb-xc-host) immobile-space (not immobile-symbols))
                (or (logbitp +initial-core-symbol-bit+ (get-header-data value))
                    (and (sb-c::core-object-p sb-c::*compile-object*)
                         (immobile-space-obj-p value)))
 
                (static-symbol-p value))
        immediate-sc-number))
-    #!+immobile-space
-    (layout
-       immediate-sc-number)
+    #+metaspace (sb-vm:layout (bug "Can't reference layout as a constant"))
+    #+(and immobile-space (not metaspace)) (wrapper immediate-sc-number)
     (single-float
-       (if (eql value 0f0) fp-single-zero-sc-number fp-single-immediate-sc-number))
+       (if (eql value $0f0) fp-single-zero-sc-number fp-single-immediate-sc-number))
     (double-float
-       (if (eql value 0d0) fp-double-zero-sc-number fp-double-immediate-sc-number))
+       (if (eql value $0d0) fp-double-zero-sc-number fp-double-immediate-sc-number))
     ((complex single-float)
-       (if (eql value #c(0f0 0f0))
+       (if (eql value (complex $0f0 $0f0))
             fp-complex-single-zero-sc-number
             fp-complex-single-immediate-sc-number))
     ((complex double-float)
-       (if (eql value #c(0d0 0d0))
+       (if (eql value (complex $0d0 $0d0))
             fp-complex-double-zero-sc-number
             fp-complex-double-immediate-sc-number))
-    #!+(and sb-simd-pack (not (host-feature sb-xc-host)))
+    #+(and sb-simd-pack (not sb-xc-host))
     ((simd-pack double-float) double-sse-immediate-sc-number)
-    #!+(and sb-simd-pack (not (host-feature sb-xc-host)))
+    #+(and sb-simd-pack (not sb-xc-host))
     ((simd-pack single-float) single-sse-immediate-sc-number)
-    #!+(and sb-simd-pack (not (host-feature sb-xc-host)))
+    #+(and sb-simd-pack (not sb-xc-host))
     (simd-pack int-sse-immediate-sc-number)
-    #!+(and sb-simd-pack-256 (not (host-feature sb-xc-host)))
+    #+(and sb-simd-pack-256 (not sb-xc-host))
     ((simd-pack-256 double-float) double-avx2-immediate-sc-number)
-    #!+(and sb-simd-pack-256 (not (host-feature sb-xc-host)))
+    #+(and sb-simd-pack-256 (not sb-xc-host))
     ((simd-pack-256 single-float) single-avx2-immediate-sc-number)
-    #!+(and sb-simd-pack-256 (not (host-feature sb-xc-host)))
+    #+(and sb-simd-pack-256 (not sb-xc-host))
     (simd-pack-256 int-avx2-immediate-sc-number)))
 
 (defun boxed-immediate-sc-p (sc)
@@ -458,9 +527,9 @@
           (integer  (if tag (fixnumize val) val))
           (symbol   (if (static-symbol-p val)
                         (+ nil-value (static-symbol-offset val))
-                        (make-fixup val :immobile-object)))
-          #!+immobile-space
-          (layout
+                        (make-fixup val :immobile-symbol)))
+          #+(and immobile-space (not metaspace))
+          (wrapper
            (make-fixup val :layout))
           (character (if tag
                          (logior (ash (char-code val) n-widetag-bits)
@@ -502,7 +571,16 @@
          (sb (sb-name (sc-sb sc)))
          (offset (tn-offset tn)))
     (ecase sb
-      (registers (reg-name (tn-reg tn)))
+      (registers
+       (concatenate 'string
+                    (reg-name (tn-reg tn))
+                    (case (sc-name (tn-sc tn))
+                      (descriptor-reg "(d)")
+                      (any-reg "(a)")
+                      (unsigned-reg "(u)")
+                      (signed-reg "(s)")
+                      (sap-reg "(p)")
+                      (t "(?)"))))
       (float-registers (format nil "FLOAT~D" offset))
       (stack (format nil "S~D" offset))
       (constant (format nil "Const~D" offset))
@@ -530,16 +608,15 @@
          (t
           (values :default nil))))
       (logbitp
-       (cond
-         ((or (and (valid-funtype '#.`((integer 0 ,(- 63 n-fixnum-tag-bits))
-                                       fixnum) '*)
-                   (sb-c::constant-lvar-p
-                    (first (sb-c::basic-combination-args node))))
-              (valid-funtype '((integer 0 63) (signed-byte 64)) '*)
-              (valid-funtype '((integer 0 63) (unsigned-byte 64)) '*))
-          (values :transform '(lambda (index integer)
-                               (%logbitp integer index))))
-         (t
-          (values :default nil))))
+       (if (or
+            (valid-funtype '((mod 64) word) '*)
+            (valid-funtype '((mod 64) signed-word) '*))
+           (values :transform '(lambda (index integer) (%logbitp index integer)))
+           (values :default nil)))
       (t
        (values :default nil)))))
+
+(defvar *register-names* +qword-register-names+)
+
+(defmacro unbound-marker-bits ()
+  (logior (+ sb-vm:static-space-start #x100) unbound-marker-widetag))
