@@ -130,8 +130,8 @@
 
 
 (deftransform %alien-funcall ((function type &rest args) * * :node node)
-  (aver (sb-c::constant-lvar-p type))
-  (let* ((type (sb-c::lvar-value type))
+  (aver (sb-c:constant-lvar-p type))
+  (let* ((type (sb-c:lvar-value type))
          (env (sb-c::node-lexenv node))
          (arg-types (alien-fun-type-arg-types type))
          (result-type (alien-fun-type-result-type type)))
@@ -231,7 +231,6 @@
   (:generator 2
    (inst mov res (make-fixup foreign-symbol :foreign))))
 
-#+linkage-table
 (define-vop (foreign-symbol-dataref-sap)
   (:translate foreign-symbol-dataref-sap)
   (:policy :fast-safe)
@@ -454,7 +453,7 @@ pointer to the arguments."
     (finalize-segment segment)
     ;; Now that the segment is done, convert it to a static
     ;; vector we can point foreign code to.
-    (let ((buffer (sb-assem::segment-buffer segment)))
+    (let ((buffer (sb-assem:segment-buffer segment)))
       (make-static-vector (length buffer)
                           :element-type '(unsigned-byte 8)
                           :initial-contents buffer))))
