@@ -61,29 +61,6 @@
 ;;; address space)
 (defconstant n-machine-word-bits 32)
 
-(defconstant float-sign-shift 31)
-
-;;; comment from CMU CL:
-;;;   These values were taken from the alpha code. The values for
-;;;   bias and exponent min/max are not the same as shown in the 486 book.
-;;;   They may be correct for how Python uses them.
-(defconstant single-float-bias 126)    ; Intel says 127.
-(defconstant-eqx single-float-exponent-byte    (byte 8 23) #'equalp)
-(defconstant-eqx single-float-significand-byte (byte 23 0) #'equalp)
-;;; comment from CMU CL:
-;;;   The 486 book shows the exponent range -126 to +127. The Lisp
-;;;   code that uses these values seems to want already biased numbers.
-(defconstant single-float-normal-exponent-min 1)
-(defconstant single-float-normal-exponent-max 254)
-(defconstant single-float-hidden-bit (ash 1 23))
-
-(defconstant double-float-bias 1022)
-(defconstant-eqx double-float-exponent-byte    (byte 11 20) #'equalp)
-(defconstant-eqx double-float-significand-byte (byte 20 0)  #'equalp)
-(defconstant double-float-normal-exponent-min 1)
-(defconstant double-float-normal-exponent-max #x7FE)
-(defconstant double-float-hidden-bit (ash 1 20))
-
 (defconstant long-float-bias 16382)
 (defconstant-eqx long-float-exponent-byte    (byte 15 0) #'equalp)
 (defconstant-eqx long-float-significand-byte (byte 31 0) #'equalp)
@@ -91,12 +68,6 @@
 (defconstant long-float-normal-exponent-max #x7FFE)
 (defconstant long-float-hidden-bit (ash 1 31))         ; actually not hidden
 
-(defconstant single-float-digits
-  (+ (byte-size single-float-significand-byte) 1))
-
-(defconstant double-float-digits
-  (+ (byte-size double-float-significand-byte) n-word-bits 1))
-
 (defconstant long-float-digits
   (+ (byte-size long-float-significand-byte) n-word-bits 1))
 
@@ -128,7 +99,7 @@
 
 ;;; where to put the different spaces
 ;;;
-;;; Note: Mostly these values are black magic, inherited from CMU CL
+;;; Note: Mostly these values are magic, inherited from CMU CL
 ;;; without any documentation. However, there were a few explanatory
 ;;; comments in the CMU CL sources:
 ;;;   * On Linux,
@@ -191,17 +162,21 @@
 ;;; NetBSD configuration used to have this comment regarding the linkage
 ;;; table: "In CMUCL: 0xB0000000->0xB1000000"
 
-#+win32     (!gencgc-space-setup #x22000000)
-#+linux     (!gencgc-space-setup #x01000000 :dynamic-space-start #x09000000)
-#+sunos     (!gencgc-space-setup #x20000000 :dynamic-space-start #x48000000)
-#+freebsd   (!gencgc-space-setup #x01000000 :dynamic-space-start #x58000000)
-#+dragonfly (!gencgc-space-setup #x01000000 :dynamic-space-start #x58000000)
-#+openbsd   (!gencgc-space-setup #x11000000 :dynamic-space-start #x8d000000)
-#+netbsd    (!gencgc-space-setup #x20000000 :dynamic-space-start #x60000000)
-#+darwin    (!gencgc-space-setup #x04000000 :dynamic-space-start #x10000000)
+(defmacro space-setup (arg &rest more)
+  `(!gencgc-space-setup ,arg #-win32 :read-only-space-size #-win32 0 ,@more))
+
+#+win32     (space-setup #x22000000)
+#+linux     (space-setup #x01000000 :dynamic-space-start #x09000000)
+#+sunos     (space-setup #x20000000 :dynamic-space-start #x48000000)
+#+freebsd   (space-setup #x01000000 :dynamic-space-start #x58000000)
+#+dragonfly (space-setup #x01000000 :dynamic-space-start #x58000000)
+#+openbsd   (space-setup #x11000000 :dynamic-space-start #x8d000000)
+#+netbsd    (space-setup #x20000000 :dynamic-space-start #x60000000)
+#+darwin    (space-setup #x04000000 :dynamic-space-start #x10000000)
 
 ;;; Size of one linkage-table entry in bytes.
 (defconstant linkage-table-entry-size 8)
+(defconstant linkage-table-growth-direction :up)
 
 
 (defenum (:start 8)
