@@ -48,14 +48,6 @@
                                       word-shift)
                                  instance-pointer-lowtag)
                        :base layout)))
-  (define-vop ()
-    (:translate sb-c::layout-depthoid-gt)
-    (:policy :fast-safe)
-    (:args (layout :scs (descriptor-reg)))
-    (:info k)
-    (:arg-types * (:constant (unsigned-byte 16)))
-    (:conditional :g)
-    (:generator 1 (inst cmp (read-depthoid) (fixnumize k))))
   (define-vop ()
     (:translate sb-c::layout-depthoid-ge)
     (:policy :fast-safe)
@@ -75,10 +67,8 @@
   (:generator 1
     (inst cmp
           (make-ea :dword
-                   :disp (+ (ash (+ (get-dsd-index layout sb-kernel::id-word0)
-                                    instance-slots-offset)
-                                 word-shift)
-                            (ash (- (layout-depthoid test) 2) 2)
+                   :disp (+ (id-bits-offset)
+                            (ash (- (wrapper-depthoid test) 2) 2)
                             (- instance-pointer-lowtag))
                    :base x)
           (if (or (typep (layout-id test) '(and (signed-byte 8) (not (eql 0))))
@@ -97,8 +87,8 @@
                                       :disp (- other-pointer-lowtag)))))
 
 
-(define-vop (fun-subtype)
-  (:translate fun-subtype)
+(define-vop ()
+  (:translate %fun-pointer-widetag)
   (:policy :fast-safe)
   (:args (function :scs (descriptor-reg)))
   (:results (result :scs (unsigned-reg)))
@@ -120,18 +110,15 @@
 (define-vop (set-header-data)
   (:translate set-header-data)
   (:policy :fast-safe)
-  (:args (x :scs (descriptor-reg) :target res :to (:result 0))
+  (:args (x :scs (descriptor-reg) :to :eval)
          (data :scs (any-reg) :target eax))
   (:arg-types * positive-fixnum)
-  (:results (res :scs (descriptor-reg)))
-  (:temporary (:sc unsigned-reg :offset eax-offset
-                   :from (:argument 1) :to (:result 0)) eax)
+  (:temporary (:sc unsigned-reg :offset eax-offset :from (:argument 1)) eax)
   (:generator 6
     (move eax data)
-    (inst shl eax (- n-widetag-bits 2))
+    (inst shl eax (- n-widetag-bits n-fixnum-tag-bits))
     (load-type al-tn x (- other-pointer-lowtag))
-    (storew eax x 0 other-pointer-lowtag)
-    (move res x)))
+    (storew eax x 0 other-pointer-lowtag)))
 
 (define-vop (test-header-bit)
   (:translate test-header-bit)
@@ -246,39 +233,6 @@
                    :disp (- fun-pointer-lowtag
                             (* simple-fun-insts-offset n-word-bytes))))))
 
-;;;; symbol frobbing
-
-(define-vop (symbol-info-vector)
-  (:policy :fast-safe)
-  (:translate symbol-info-vector)
-  (:args (x :scs (descriptor-reg)))
-  (:results (res :scs (descriptor-reg)))
-  (:temporary (:sc unsigned-reg :offset eax-offset) eax)
-  (:generator 1
-    (loadw res x symbol-info-slot other-pointer-lowtag)
-    ;; If RES has list-pointer-lowtag, take its CDR. If not, use it as-is.
-    ;; This CMOV safely reads from memory when it does not move, because if
-    ;; there is an info-vector in the slot, it has at least one element.
-    ;; This would compile to almost the same code without a VOP,
-    ;; but using a jmp around a mov instead.
-    (inst lea eax (make-ea :dword :base res :disp (- list-pointer-lowtag)))
-    (emit-optimized-test-inst eax lowtag-mask)
-    (inst cmov :e res
-          (object-slot-ea res cons-cdr-slot list-pointer-lowtag))))
-(define-vop (symbol-plist)
-  (:policy :fast-safe)
-  (:translate symbol-plist)
-  (:args (x :scs (descriptor-reg)))
-  (:results (res :scs (descriptor-reg)))
-  (:temporary (:sc unsigned-reg) temp)
-  (:generator 1
-    (loadw res x symbol-info-slot other-pointer-lowtag)
-    ;; Instruction pun: (CAR x) is the same as (VECTOR-LENGTH x)
-    ;; so if the info slot holds a vector, this gets a fixnum- it's not a plist.
-    (loadw res res cons-car-slot list-pointer-lowtag)
-    (inst mov temp nil-value)
-    (emit-optimized-test-inst res fixnum-tag-mask)
-    (inst cmov :e res temp)))
 
 ;;;; other miscellaneous VOPs
 
