@@ -12,7 +12,7 @@
 (in-package "SB-VM")
 
 (define-vop (reset-stack-pointer)
-  (:args (ptr :scs (any-reg)))
+  (:args (ptr :scs (any-reg control-stack)))
   (:generator 1
     (move esp-tn ptr)))
 
@@ -25,6 +25,7 @@
             )
   (:temporary (:sc any-reg :offset esi-offset) esi)
   (:temporary (:sc any-reg :offset edi-offset) edi)
+  (:temporary (:sc descriptor-reg) temp-dword)
   (:ignore r-moved-ptrs)
   (:generator 1
     (move edi last-nipped-ptr)
@@ -33,12 +34,13 @@
     (inst sub edi n-word-bytes)
     (inst cmp esp-tn esi)
     (inst jmp :a done)
-    (inst std)
     LOOP
-    (inst movs :dword)
+    (inst mov temp-dword (make-ea :dword :base esi))
+    (inst mov (make-ea :dword :base edi) temp-dword)
+    (inst sub esi n-word-bytes)
+    (inst sub edi n-word-bytes)
     (inst cmp esp-tn esi)
     (inst jmp :be loop)
-    (inst cld)
     DONE
     (inst lea esp-tn (make-ea :dword :base edi :disp n-word-bytes))
     (inst sub edi esi)
@@ -54,7 +56,8 @@
 ;;; bogus SC that reflects the costs of the memory-to-memory moves for each
 ;;; operand, but this seems unworthwhile.
 (define-vop (push-values)
-  (:args (vals :more t))
+  (:args (vals :more t
+               :scs (descriptor-reg)))
   (:results (start :from :load) (count))
   (:info nvals)
   (:generator 20
@@ -130,7 +133,8 @@
 
     (move loop-index count)
     (inst mov start esp-tn)
-    (inst jecxz done)  ; check for 0 count?
+    (inst test ecx-tn ecx-tn)
+    (inst jmp :z done)  ; check for 0 count?
 
     (inst sub esp-tn count)
     (inst sub src count)
