@@ -503,27 +503,36 @@
           (setf (info :function :type name) :generic-function))))))
 
 (defun compute-gf-ftype (name)
-  (let ((gf (and (fboundp name) (fdefinition name))))
-    (if (generic-function-p gf)
-        (let* ((ll (generic-function-lambda-list gf))
-               ;; If the GF has &REST without &KEY then we don't augment
-               ;; the FTYPE with keywords, so as not to complain about keywords
-               ;; which seem not to be accepted.
-               (type (sb-c::ftype-from-lambda-list
-                      (if (and (member '&rest ll) (not (member '&key ll)))
-                          ll
-                          (generic-function-pretty-arglist gf)))))
-          ;; It would be nice if globaldb were transactional,
-          ;; so that either both updates or neither occur.
-          (setf (info :function :type name) type
-                (info :function :where-from name) :defined-method)
-          type)
-        ;; The defaulting expression for (:FUNCTION :TYPE) does not store
-        ;; the default. For :GENERIC-FUNCTION that is not FBOUNDP we also
-        ;; don't, however this branch should never be reached because the
-        ;; info only stores :GENERIC-FUNCTION when methods are loaded.
-        ;; Maybe AVER that it does not happen?
-        (sb-impl::ftype-from-fdefn name))))
+  (let ((gf (and (fboundp name) (fdefinition name)))
+        (methods-in-compilation-unit (and (boundp 'sb-c::*methods-in-compilation-unit*)
+                                          sb-c::*methods-in-compilation-unit*
+                                          (gethash name sb-c::*methods-in-compilation-unit*))))
+    (cond ((generic-function-p gf)
+           (let* ((ll (generic-function-lambda-list gf))
+                  ;; If the GF has &REST without &KEY then we don't augment
+                  ;; the FTYPE with keywords, so as not to complain about keywords
+                  ;; which seem not to be accepted.
+                  (type (sb-c::ftype-from-lambda-list
+                         (if (and (member '&rest ll) (not (member '&key ll)))
+                             ll
+                             (generic-function-pretty-arglist gf methods-in-compilation-unit)))))
+
+             ;; It would be nice if globaldb were transactional,
+             ;; so that either both updates or neither occur.
+             (setf (info :function :where-from name) :defined-method
+                   (info :function :type name) type)))
+          (methods-in-compilation-unit
+           (setf (info :function :where-from name) :defined-method
+                 (info :function :type name)
+                 (sb-c::ftype-from-lambda-list
+                  (gf-merge-arglists methods-in-compilation-unit))))
+          (t
+           ;; The defaulting expression for (:FUNCTION :TYPE) does not store
+           ;; the default. For :GENERIC-FUNCTION that is not FBOUNDP we also
+           ;; don't, however this branch should never be reached because the
+           ;; info only stores :GENERIC-FUNCTION when methods are loaded.
+           ;; Maybe AVER that it does not happen?
+           (sb-impl::ftype-from-fdefn name)))))
 
 (defun real-add-method (generic-function method &optional skip-dfun-update-p)
   (flet ((similar-lambda-lists-p (old-method new-lambda-list)
@@ -1702,10 +1711,16 @@
 ;;; The compiler uses this for type-checking that callers pass acceptable
 ;;; keywords, so don't make this do anything fancy like looking at effective
 ;;; methods without also fixing the compiler.
-(defmethod generic-function-pretty-arglist ((gf standard-generic-function))
+(defmethod generic-function-pretty-arglist ((gf standard-generic-function) &optional methods-in-compilation-unit)
   (let ((gf-lambda-list (generic-function-lambda-list gf))
         (methods (generic-function-methods gf)))
-    (flet ((canonize (k)
+    (flet ((lambda-list (m)
+             (or (and methods-in-compilation-unit
+                      (gethash (cons (method-qualifiers m)
+                                     (unparse-specializers gf (method-specializers m)))
+                               methods-in-compilation-unit))
+                 (method-lambda-list m)))
+           (canonize (k)
              (multiple-value-bind (kw var)
                  (parse-key-arg-spec k)
                (if (and (eql (symbol-package kw) *keyword-package*)
@@ -1714,15 +1729,54 @@
                    (list (list kw var))))))
       (multiple-value-bind (llks required optional rest keys)
           (parse-lambda-list gf-lambda-list :silent t)
-        (collect ((keys (mapcar #'canonize keys)))
-        ;; Possibly extend the keyword parameters of the gf by
-        ;; additional key parameters of its methods:
-          (dolist (m methods
-                     (make-lambda-list llks nil required optional rest (keys)))
-            (binding* (((m.llks nil nil nil m.keys)
-                        (parse-lambda-list (method-lambda-list m) :silent t)))
-              (setq llks (logior llks m.llks))
-              (dolist (k m.keys)
-                (unless (member (parse-key-arg-spec k) (keys)
-                                :key #'parse-key-arg-spec :test #'eq)
-                  (keys (canonize k)))))))))))
+        (if (or (ll-kwds-keyp llks)
+                (ll-kwds-restp llks))
+            (collect ((keys (mapcar #'canonize keys)))
+              ;; Possibly extend the keyword parameters of the gf by
+              ;; additional key parameters of its methods:
+              (flet ((process (lambda-list)
+                       (binding* (((m.llks nil nil nil m.keys)
+                                   (parse-lambda-list lambda-list :silent t)))
+                         (setq llks (logior llks m.llks))
+                         (dolist (k m.keys)
+                           (unless (member (parse-key-arg-spec k) (keys)
+                                           :key #'parse-key-arg-spec :test #'eq)
+                             (keys (canonize k)))))))
+                (dolist (m methods)
+                  (process (lambda-list m))))
+              (make-lambda-list llks nil required optional rest (keys)))
+            (make-lambda-list llks nil required optional))))))
+
+(defun gf-merge-arglists (methods-in-compilation-unit)
+  (flet ((canonize (k)
+           (multiple-value-bind (kw var)
+               (parse-key-arg-spec k)
+             (if (and (eql (symbol-package kw) *keyword-package*)
+                      (string= kw var))
+                 var
+                 (list (list kw var))))))
+    (with-hash-table-iterator (iterator methods-in-compilation-unit)
+      (multiple-value-bind (llks required optional rest keys)
+          (parse-lambda-list (nth-value 2 (iterator)) :silent t)
+        (if (or (ll-kwds-keyp llks)
+                (ll-kwds-restp llks))
+            (collect ((keys (mapcar #'canonize keys)))
+              ;; Possibly extend the keyword parameters of the gf by
+              ;; additional key parameters of its methods:
+              (flet ((process (lambda-list)
+                       (binding* (((m.llks nil nil nil m.keys)
+                                   (parse-lambda-list lambda-list :silent t)))
+                         (setq llks (logior llks m.llks))
+                         (dolist (k m.keys)
+                           (unless (member (parse-key-arg-spec k) (keys)
+                                           :key #'parse-key-arg-spec :test #'eq)
+                             (keys (canonize k)))))))
+
+                (loop
+                 (multiple-value-bind (more key value) (iterator)
+                   (declare (ignore key))
+                   (unless more
+                     (return))
+                   (process value)))
+                (make-lambda-list llks nil required optional rest (keys))))
+            (make-lambda-list llks nil required optional))))))
