@@ -1,4 +1,4 @@
-;;;; Bits and pieces of the wrapper machninery. This used to live in cache.lisp,
+;;;; Bits and pieces of the wrapper machinery. This used to live in cache.lisp,
 ;;;; but doesn't really logically belong there.
 
 ;;;; This software is part of the SBCL system. See the README file for
@@ -27,7 +27,7 @@
 (in-package "SB-PCL")
 
 (defmacro wrapper-class (wrapper)
-  `(classoid-pcl-class (layout-classoid ,wrapper)))
+  `(classoid-pcl-class (wrapper-classoid ,wrapper)))
 
 ;;; This is called in BRAID when we are making wrappers for classes
 ;;; whose slots are not initialized yet, and which may be built-in
@@ -36,7 +36,7 @@
   (let ((found (find-classoid name nil)))
     (cond
      (found
-      (let ((layout (classoid-layout found)))
+      (let ((layout (classoid-wrapper found)))
         (aver layout)
         layout))
      (t
@@ -62,7 +62,7 @@
             (classoid
              (let ((owrap (class-wrapper class)))
                (cond (owrap
-                      (layout-classoid owrap))
+                      (wrapper-classoid owrap))
                      ((or (*subtypep (class-of class) *the-class-standard-class*)
                           (*subtypep (class-of class) *the-class-funcallable-standard-class*)
                           (typep class 'forward-referenced-class))
@@ -76,7 +76,7 @@
                      :invalid nil :length length :flags +pcl-object-layout-flag+)))
     (t
      (let* ((found (find-classoid (slot-value class 'name)))
-            (layout (classoid-layout found)))
+            (layout (classoid-wrapper found)))
        (unless (classoid-pcl-class found)
          (setf (classoid-pcl-class found) class))
        (aver (eq (classoid-pcl-class found) class))
@@ -99,7 +99,7 @@
 ;;; This "simple" function hides a horrible inconsistency: we don't have a single
 ;;; canonical atomically checkable validity indicator. We carry around invalid layouts
 ;;; with nonzero hashes. Why else would there be so may places that can do:
-;;;   (setf (layout-invalid layout) nil
+;;;   (setf (wrapper-invalid layout) nil
 ;;; whilst the layout in question already has a "valid" hash?
 ;;; It's hard to know what the right thing is, but since internal code uses
 ;;; this test to decide how to handle layouts that have been invalidated,
@@ -107,11 +107,11 @@
 ;;; even though logically that *should* be the canonical test.
 (declaim (inline invalid-wrapper-p))
 (defun invalid-wrapper-p (wrapper)
-  (not (null (layout-invalid wrapper))))
+  (not (null (wrapper-invalid wrapper))))
 
 ;;; The portable code inherited from the ancient PCL sources used an extremely
 ;;; clever mechanism to store a mapping from new layouts ("wrappers") to old.
-;;; Old layouts point to new in a straighforward way - via the INVALID slot -
+;;; Old layouts point to new in a straightforward way - via the INVALID slot -
 ;;; but the invalidation logic wants backpointers from new to old as well.
 ;;; It arranges so that if the following state exists:
 ;;;   layout_A -> (:flush layout_B)
@@ -120,7 +120,7 @@
 ;;; the state of layout_A should be changed to (:flush layout_C)
 ;;; at the same time that the state of layout_B is changed.
 ;;;
-;;; In general, alterating layout_B might might have to perform more than one
+;;; In general, altering layout_B might might have to perform more than one
 ;;; update to older versions of the layout, so it's a one-to-many relation.
 ;;; The same "transitivity" could have been implemented at the time of flushing
 ;;; layout_A by chasing an extra pointer (the B -> C pointer) and recording that
@@ -128,12 +128,12 @@
 ;;; improves anything, as it merely performs work eagerly that could have
 ;;; been performed lazily. (PCL generally prefers lazy updates.)
 ;;;
-;;; Anyway the extreme cleverness (and premature optimization, imho), was almost,
+;;; Anyway the extreme cleverness (and premature optimization, IMHO), was almost,
 ;;; but not quite, clever enough - it leaked three conses per redefinition,
 ;;; which it pretty much had to in order to remain portable.
 ;;; We can at least replicate it in a way that doesn't leak conses,
 ;;; or at least theoretically doesn't. A likely scenario is that old caches may
-;;; never get adquately flushed to drop all references to obsolete layouts.
+;;; never get adequately flushed to drop all references to obsolete layouts.
 ;;; If you call a GF once and not again, it won't flush; thus causing a retained
 ;;; old layout no matter what else is done.
 ;;;
@@ -150,17 +150,17 @@
 ;;; The bug is that it was possible for an observer to see (:FLUSH new-thing)
 ;;; in an older layout when it should actually have seen (:OBSOLETE newer-thing).
 ;;; On the other hand, it's still possible for an observer to see the
-;;; wrong value in the INVALID slot itself. i.e. if it reads the slot
+;;; wrong value in the INVALID slot itself. I.e. if it reads the slot
 ;;; before %invalidate-wrapper performs the transitivity operation,
 ;;; hence the possibility that this optimization is not only premature,
 ;;; but also pretty much not correct.
 (defun %invalidate-wrapper (owrapper state nwrapper)
   (aver (member state '(:flush :obsolete)))
-  #+sb-thread (aver (sb-thread:holding-mutex-p sb-c::**world-lock**))
-  (let ((classoid (layout-classoid nwrapper))
+  #+sb-thread (aver (sb-thread:holding-mutex-p sb-kernel::**world-lock**))
+  (let ((classoid (wrapper-classoid nwrapper))
         (new-previous ())
         (new-state (cons state nwrapper)))
-    (aver (eq (layout-classoid owrapper) classoid))
+    (aver (eq (wrapper-classoid owrapper) classoid))
     ;; First off, a previous call to INVALIDATE-WRAPPER may have
     ;; recorded OWRAPPER as an NWRAPPER to update to. Since OWRAPPER
     ;; is about to be invalid, it no longer makes sense to update to
@@ -172,8 +172,8 @@
     (dolist (weak-pointer (sb-kernel::standard-classoid-old-layouts classoid))
       (let ((previous (weak-pointer-value weak-pointer)))
         (when previous
-          (setf (layout-invalid previous)
-                (cond ((and (eq (car (layout-invalid previous)) :obsolete)
+          (setf (wrapper-invalid previous)
+                (cond ((and (eq (car (wrapper-invalid previous)) :obsolete)
                             (eq state :flush))
                        ;; :obsolete must stay :obsolete,
                        ;; requiring the protocol for obsolete instances.
@@ -187,11 +187,12 @@
     ;; accessing the wrapper at the same time from outside the lock?
     ;; Inform readers of the reason for wrapper invalidity before marking
     ;; the wrapper as invalid.
-    (setf (layout-invalid owrapper) new-state)
+    (setf (wrapper-invalid owrapper) new-state)
     ;; Ensure that the INVALID slot conveying ancillary data describing the
     ;; invalidity reason is published before causing the invalid layout trap.
     (sb-thread:barrier (:write))
-    (setf (layout-clos-hash owrapper) 0)
+    #+metaspace (setf (layout-clos-hash (wrapper-friend owrapper)) 0)
+    (setf (wrapper-clos-hash owrapper) 0)
     (push (make-weak-pointer owrapper) new-previous)
     ;; This function is called for effect; return value is arbitrary.
     (setf  (sb-kernel::standard-classoid-old-layouts classoid)
@@ -202,8 +203,8 @@
 ;;; (or the names of our callees.)
 (defun check-wrapper-validity (instance)
   (with-world-lock ()
-    (let* ((owrapper (layout-of instance))
-           (state (layout-invalid owrapper)))
+    (let* ((owrapper (wrapper-of instance))
+           (state (wrapper-invalid owrapper)))
       (aver (not (eq state :uninitialized)))
       (cond ((not state)
              owrapper)
@@ -230,7 +231,7 @@
                ;; Error message here is trying to figure out a bit more about the
                ;; situation, since we don't have anything approaching a test-case
                ;; for the bug.
-               (let ((new-state (layout-invalid (layout-of instance))))
+               (let ((new-state (wrapper-invalid (wrapper-of instance))))
                  (when (eq new-state t)
                    (cerror "Nevermind and recurse." 'bug
                            :format-control "~@<~4IProblem forcing cache flushes. Please report ~
@@ -239,31 +240,31 @@
                                             ~% Wrapper-of: ~S~
                                             ~% Class-wrapper: ~S~%~:@>"
                            :format-arguments (mapcar (lambda (x)
-                                                       (cons x (layout-invalid x)))
+                                                       (cons x (wrapper-invalid x)))
                                                      (list owrapper
-                                                           (layout-of instance)
+                                                           (wrapper-of instance)
                                                            (class-wrapper class)))))))
              (check-wrapper-validity instance))
             ((consp state)
-             (ecase (car state)
-               (:flush
-                (let ((new (cdr state)))
+             (let ((new (the wrapper (cdr state))))
+               (ecase (car state)
+                 (:flush
                   (cond ((std-instance-p instance)
-                         (setf (%instance-layout instance) new))
+                         (setf (%instance-wrapper instance) new))
                         ((fsc-instance-p instance)
-                         (setf (%fun-layout instance) new))
+                         (setf (%fun-wrapper instance) new))
                         (t
-                         (bug "unrecognized instance type")))))
-               (:obsolete
-                (%obsolete-instance-trap owrapper (cdr state) instance))))))))
+                         (bug "unrecognized instance type"))))
+                 (:obsolete
+                  (%obsolete-instance-trap owrapper new instance)))))))))
 
 (declaim (inline check-obsolete-instance))
 (defun check-obsolete-instance (instance)
-  (when (invalid-wrapper-p (layout-of instance))
+  (when (invalid-wrapper-p (wrapper-of instance))
     (check-wrapper-validity instance)))
 
 (defun valid-wrapper-of (instance)
-  (let ((wrapper (layout-of instance)))
+  (let ((wrapper (wrapper-of instance)))
     (if (invalid-wrapper-p wrapper)
         (check-wrapper-validity instance)
         wrapper)))
@@ -353,7 +354,7 @@
                   `((class *the-class-t*)
                     (type t))))
          (unless (eq mt t)
-           (setq wrapper (layout-of arg))
+           (setq wrapper (wrapper-of arg))
            (when (invalid-wrapper-p wrapper)
              (setq ,invalid-wrapper-p t)
              (setq wrapper (check-wrapper-validity arg)))
