@@ -22,30 +22,14 @@
 #include "alloc.h"
 #include "breakpoint.h"
 #include "thread.h"
-#include "genesis/code.h"
+#include "code.h"
 #include "genesis/fdefn.h"
 
-#ifdef LISP_FEATURE_X86_64
 #define REAL_LRA_SLOT 0
-#define KNOWN_RETURN_P_SLOT 2
-#define BOGUS_LRA_CONSTANTS 3
-#elif defined(LISP_FEATURE_X86)
-#define REAL_LRA_SLOT 1
-#define KNOWN_RETURN_P_SLOT 3
-#define BOGUS_LRA_CONSTANTS 4
-#else
-#define REAL_LRA_SLOT 0
-#define KNOWN_RETURN_P_SLOT 1
-#define BOGUS_LRA_CONSTANTS 2
-#endif
 
 static void *compute_pc(lispobj code_obj, int pc_offset)
 {
-    struct code *code;
-
-    code = (struct code *)native_pointer(code_obj);
-    return (void *)((char *)code + code_header_words(code->header)*sizeof(lispobj)
-                    + pc_offset);
+    return code_text_start((struct code*)native_pointer(code_obj)) + pc_offset;
 }
 
 unsigned int breakpoint_install(lispobj code_obj, int pc_offset)
@@ -82,18 +66,23 @@ lispobj find_code(os_context_t *context)
     lispobj code = *os_context_register_addr(context, reg_CODE);
     lispobj header;
 
+#ifdef LISP_FEATURE_PPC64
+    if (lowtag_of(code) == 0)
+        code |= OTHER_POINTER_LOWTAG;
+#endif
+
     if (lowtag_of(code) != OTHER_POINTER_LOWTAG)
         return NIL;
 
     header = *(lispobj *)(code-OTHER_POINTER_LOWTAG);
 
-    if (widetag_of(header) == CODE_HEADER_WIDETAG)
+    if (header_widetag(header) == CODE_HEADER_WIDETAG)
         return code;
     else
         return code - HeaderValue(header)*sizeof(lispobj);
 #else
     lispobj codeptr =
-        (lispobj)component_ptr_from_pc((lispobj *)(*os_context_pc_addr(context)));
+        (lispobj)component_ptr_from_pc((char *)(*os_context_pc_addr(context)));
 
     if (codeptr == 0)
         return NIL;
@@ -104,29 +93,16 @@ lispobj find_code(os_context_t *context)
 
 static long compute_offset(os_context_t *context, lispobj code)
 {
-    if (code == NIL)
-        return 0;
-    else {
-        uword_t code_start;
-        struct code *codeptr = (struct code *)native_pointer(code);
-#ifdef LISP_FEATURE_HPPA
-        uword_t pc = *os_context_pc_addr(context) & ~3;
-#else
+  if (code != NIL) {
         uword_t pc = *os_context_pc_addr(context);
-#endif
-
-        code_start = (uword_t)codeptr
-            + code_header_words(codeptr->header)*sizeof(lispobj);
-        if (pc < code_start)
-            return 0;
-        else {
-            uword_t offset = pc - code_start;
-            if (offset >= (uword_t)fixnum_value(codeptr->code_size))
-                return 0;
-            else
-                return make_fixnum(offset);
-        }
+        struct code *codeptr = (struct code *)native_pointer(code);
+        uword_t code_start = (uword_t)code_text_start(codeptr);
+        int offset;
+        if (pc >= code_start &&
+            ((offset = pc - code_start) < code_text_size(codeptr)))
+            return make_fixnum(offset);
     }
+    return 0;
 }
 
 void handle_breakpoint(os_context_t *context)
@@ -137,7 +113,7 @@ void handle_breakpoint(os_context_t *context)
     fake_foreign_function_call(context);
 
 #ifndef LISP_FEATURE_SB_SAFEPOINT
-    unblock_gc_signals(0, 0);
+    unblock_gc_signals();
 #endif
     code = find_code(context);
 
@@ -164,7 +140,7 @@ void *handle_fun_end_breakpoint(os_context_t *context)
     fake_foreign_function_call(context);
 
 #ifndef LISP_FEATURE_SB_SAFEPOINT
-    unblock_gc_signals(0, 0);
+    unblock_gc_signals();
 #endif
 
     code = find_code(context);
@@ -183,7 +159,7 @@ void *handle_fun_end_breakpoint(os_context_t *context)
 
     lra = codeptr->constants[REAL_LRA_SLOT];
 
-#ifdef LISP_FEATURE_PPC
+#if defined LISP_FEATURE_PPC || defined LISP_FEATURE_PPC64
     /* PPC now passes LRA objects in reg_LRA during return.  Other
      * platforms should as well, but haven't been fixed yet. */
     *os_context_register_addr(context, reg_LRA) = lra;
