@@ -84,28 +84,31 @@ extern struct weak_pointer *weak_pointer_chain; /* in gc-common.c */
  * - With smarter macros it ought to be possible to avoid 8-byte loads and shifts.
  *   They would need to be endian-aware, which I didn't want to do just yet.
  */
-#define vector_flagp(header, val) ((int)header & (flag_##val << N_WIDETAG_BITS))
-#define vector_flags_zerop(header) ((int)(header) & 0x0700) == 0
+#define vector_flagp(header, val) ((int)header & (flag_##val << ARRAY_FLAGS_POSITION))
+#define vector_flags_zerop(header) ((int)(header) & 0x07 << ARRAY_FLAGS_POSITION) == 0
 // True if flags are zero, also testing the widetag at the same time.
-#define ordinary_simple_vector_p(header) ((int)(header) & 0x07ff) == SIMPLE_VECTOR_WIDETAG
+#define ordinary_simple_vector_p(header) \
+    ((int)(header) & (0x07 << ARRAY_FLAGS_POSITION | 0xffff >> (16-ARRAY_FLAGS_POSITION))) \
+    == SIMPLE_VECTOR_WIDETAG
 // Return true if vector is a weak vector that is not a hash-table <k,v> vector.
 #define vector_is_weak_not_hashing_p(header) \
-  ((int)(header) & ((flag_VectorWeak|flag_VectorHashing) << N_WIDETAG_BITS)) == \
-    (flag_VectorWeak << N_WIDETAG_BITS)
+  ((int)(header) & ((flag_VectorWeak|flag_VectorHashing) << ARRAY_FLAGS_POSITION)) == \
+    (flag_VectorWeak << ARRAY_FLAGS_POSITION)
 
 // Mask out the fullcgc mark bit when asserting header validity
 #define UNSET_WEAK_VECTOR_VISITED(v) \
-  gc_assert((v->header & 0xffff) == \
-    (((flag_VectorWeakVisited|flag_VectorWeak) << N_WIDETAG_BITS) \
+  gc_assert((v->header & (0xff << ARRAY_FLAGS_POSITION | 0xff)) ==     \
+    (((flag_VectorWeakVisited|flag_VectorWeak) << ARRAY_FLAGS_POSITION) \
      | SIMPLE_VECTOR_WIDETAG)); \
-  v->header ^= flag_VectorWeakVisited << N_WIDETAG_BITS
+  v->header ^= flag_VectorWeakVisited << ARRAY_FLAGS_POSITION
 
 /* values for the *_alloc_* parameters, also see the commentary for
  * struct page in gencgc-internal.h. These constants are used in gc-common,
  * so they can't easily be made gencgc-only */
 #define FREE_PAGE_FLAG        0
 #define PAGE_TYPE_MASK        7 // mask out the 'single-object flag'
-/* Note: MAP-ALLOCATED-OBJECTS expects this value to be 1 */
+/* Note: lisp's CLOSE-CURRENT-GC-REGION expects BOXED_PAGE_FLAG = 1.
+ * (probably should do a foreign call rather than kludge it) */
 #define BOXED_PAGE_FLAG       1
 #define UNBOXED_PAGE_FLAG     2
 /* CONS_PAGE_FLAG doesn't get stored in the page table, though I am considering
@@ -161,6 +164,4 @@ extern int simple_fun_index(struct code*, struct simple_fun*);
 
 extern lispobj fdefn_callee_lispobj(struct fdefn *fdefn);
 
-boolean valid_widetag_p(unsigned char widetag);
-
 #endif /* _GC_INTERNAL_H_ */
