@@ -9,6 +9,8 @@
 
 /* Abstract out the data for an allocation region allowing a single
  * routine to be used for allocation and closing. */
+/* Caution: if you change this, you may have to change compiler/generic/objdef
+ * (for the THREAD object), all the backends' allocators, and room.lisp */
 struct alloc_region {
 
     /* These two are needed for quick allocation. */
@@ -27,7 +29,11 @@ struct alloc_region {
 
 // One region for each of {BOXED,UNBOXED,CODE}_PAGE_FLAG
 extern struct alloc_region  gc_alloc_region[3];
+#define boxed_region   gc_alloc_region[BOXED_PAGE_FLAG-1]
+#define unboxed_region gc_alloc_region[UNBOXED_PAGE_FLAG-1]
+#define code_region    gc_alloc_region[CODE_PAGE_TYPE-1]
+
 extern generation_index_t from_space, new_space;
-extern struct weak_pointer *weak_pointers;
+extern int gencgc_alloc_profiler;
 
 #endif /*  _GENCGC_ALLOC_REGION_H_ */
