@@ -25,6 +25,10 @@
 #include "genesis/code.h"
 #include "hopscotch.h"
 
+#ifndef GENCGC_IS_PRECISE
+#error "GENCGC_IS_PRECISE must be #defined as 0 or 1"
+#endif
+
 extern char *page_address(page_index_t);
 int gencgc_handle_wp_violation(void *);
 
@@ -53,13 +57,6 @@ int gencgc_handle_wp_violation(void *);
   typedef unsigned short page_bytes_t;
 #endif
 
-/* Define this as 0 on the cc invocation if you need to test without it.
- * While the x86 requires object-based pins, the precise backends don't,
- * though should generally prefer object pinning over page pinning */
-#ifndef PIN_GRANULARITY_LISPOBJ
-#define PIN_GRANULARITY_LISPOBJ 1
-#endif
-
 /* Note that this structure is also used from Lisp-side in
  * src/code/room.lisp, and the Lisp-side structure layout is currently
  * not groveled from C code but hardcoded. Any changes to the
@@ -111,11 +108,7 @@ struct page {
          *
          * If the page is free, all the following fields are zero. */
         type :5,
-        /* This is set when the page is write-protected. This should
-         * always reflect the actual write_protect status of a page.
-         * (If the page is written into, we catch the exception, make
-         * the page writable, and clear this flag.) */
-        write_protected :1,
+        padding :1,
         /* This flag is set when the above write_protected flag is
          * cleared by the SIGBUS handler (or SIGSEGV handler, for some
          * OSes). This is useful for re-scavenging pages that are
@@ -134,12 +127,35 @@ struct page {
 extern struct page *page_table;
 #ifdef LISP_FEATURE_BIG_ENDIAN
 # define WP_CLEARED_FLAG      (1<<1)
-# define WRITE_PROTECTED_FLAG (1<<2)
 #else
-# define WRITE_PROTECTED_FLAG (1<<5)
 # define WP_CLEARED_FLAG      (1<<6)
 #endif
 
+/* When computing a card index we never subtract the heap base, which simplifies
+ * code generation. Because there is no alignment constraint beyond being card-aligned,
+ * the low bits can wraparound from all 1s to all 0s such that lowest numbered
+ * page index in linear order may have a higher card index.
+ * As a small example of the distinction between page index and card index:
+ *   heap base = 0xEB00, card size = 256 bytes, total cards = 8, mask = #b111
+ *
+ *     page     page      card
+ *    index     addr     index
+ *       0      EB00        3
+ *       1      EC00        4
+ *       2      ED00        5
+ *       3      EE00        6
+ *       4      EF00        7
+ *       5      F000        0
+ *       6      F100        1
+ *       7      F200        2
+ */
+extern char * gc_card_mark;
+extern int gc_card_table_mask;
+#define addr_to_card_index(addr) ((((uword_t)addr)>>GENCGC_CARD_SHIFT) & gc_card_table_mask)
+#define page_to_card_index(n) addr_to_card_index(page_address(n))
+#define PAGE_WRITEPROTECTED_P(n) (gc_card_mark[page_to_card_index(n)] & 1)
+#define SET_PAGE_PROTECTED(n,val) gc_card_mark[page_to_card_index(n)] = val
+
 struct __attribute__((packed)) corefile_pte {
   uword_t sso; // scan start offset
   page_bytes_t bytes_used;
@@ -164,9 +180,14 @@ static inline void ensure_region_closed(struct alloc_region *alloc_region,
 
 static inline void gc_set_region_empty(struct alloc_region *region)
 {
+    /* Free-pointer has to be not equal to 0 because it's undefined behavior
+     * to add any value whatsoever to the null pointer.
+     * Annoying, isn't it.  http://c-faq.com/null/machexamp.html */
+    region->free_pointer = region->end_addr = (void*)0x1000;
+    /* Start 0 is the indicator of closed-ness. */
+    region->start_addr = 0;
     /* last_page is not reset. It can be used as a hint where to resume
      * allocating after closing and re-opening the region */
-    region->start_addr = region->free_pointer = region->end_addr = 0;
 }
 
 static inline void gc_init_region(struct alloc_region *region)
@@ -197,16 +218,16 @@ find_page_index(void *addr)
 
 #define SINGLE_OBJECT_FLAG (1<<4)
 #define page_single_obj_p(page) ((page_table[page].type & SINGLE_OBJECT_FLAG)!=0)
-#ifdef PIN_GRANULARITY_LISPOBJ
-#ifndef GENCGC_IS_PRECISE
-#error "GENCGC_IS_PRECISE must be #defined as 0 or 1"
-#endif
+
 #define page_has_smallobj_pins(page) \
   (page_table[page].pinned && !page_single_obj_p(page))
 static inline boolean pinned_p(lispobj obj, page_index_t page)
 {
     extern struct hopscotch_table pinned_objects;
-    gc_dcheck(compacting_p());
+    // FIXME: this gets called if !compacting_p,
+    // but most people don't run with extra debug assertions,
+    // and if you enable them, you'll pretty quickly crash here.
+    // gc_dcheck(compacting_p());
 #if !GENCGC_IS_PRECISE
     return page_has_smallobj_pins(page)
         && hopscotch_containsp(&pinned_objects, obj);
@@ -223,9 +244,6 @@ static inline boolean pinned_p(lispobj obj, page_index_t page)
     return hopscotch_containsp(&pinned_objects, obj);
 #endif
 }
-#else
-#  define pinned_p(obj, page) (0)
-#endif
 
 // Return true only if 'obj' must be *physically* transported to survive gc.
 // Return false if obj is in the immobile space regardless of its generation.
@@ -253,10 +271,8 @@ struct fixedobj_page { // 12 bytes per page
       int packed;
       struct {
         unsigned char flags;
-        /* space per object in Lisp words. Can exceed obj_size
-           to align on a larger boundary */
-        unsigned char obj_align;
-        unsigned char obj_size; /* in Lisp words, incl. header */
+        unsigned char obj_align; // object spacing expressed in lisp words
+        unsigned char unused1;
         /* Which generations have data on this page */
         unsigned char gens_; // a bitmap
       } parts;
@@ -268,7 +284,6 @@ struct fixedobj_page { // 12 bytes per page
 };
 extern struct fixedobj_page *fixedobj_pages;
 #define fixedobj_page_obj_align(i) (fixedobj_pages[i].attr.parts.obj_align<<WORD_SHIFT)
-#define fixedobj_page_obj_size(i) fixedobj_pages[i].attr.parts.obj_size
 #endif
 
 extern page_index_t next_free_page;
