@@ -32,49 +32,9 @@ extern int ffs(int);
 #include "genesis/vector.h"
 #include "murmur_hash.h"
 
-#ifdef LISP_FEATURE_USE_SYS_MMAP
-///
-///                **********************************
-///                *  GC MUST NOT ACQUIRE ANY LOCKS *
-///                **********************************
-///
-/// It's normally fine to use the mmap() system call to obtain memory for the
-/// hopscotch tables, unless your C runtime intercepts mmap() and causes it
-/// to sometimes (or always?) need a spinlock. That lock may be owned already,
-/// so GC will patiently wait forever; meanwhile the lock owner is also
-/// waiting forever on GC to finish.
-/// So bypass the C library routine and call the OS directly
-/// in case of a non-signal-safe interceptor such as
-///   https://chromium.googlesource.com/chromium/src/third_party/tcmalloc/chromium/+/refs/heads/master/src/malloc_hook_mmap_linux.h#146
-///
-static inline void* sys_mmap(void* addr, size_t length, int prot, int flags,
-                             int fd, off_t offset) {
-    // "linux-os.h" brings in <syscall.h>, others may need something different.
-    // mmap2 allows large file access with 32-bit off_t. We don't care about that,
-    // but _usually_ only one or the other of the syscalls exists depending on,
-    // various factors. Basing it on word size will pick the right one.
-#ifdef LISP_FEATURE_64_BIT
-    void* result = (void*)syscall(SYS_mmap, addr, length, prot, flags, fd, offset);
-#else
-    void* result = (void*)syscall(SYS_mmap2, addr, length, prot, flags, fd, offset);
-#endif
-    if (result == MAP_FAILED) lose("mmap failed in GC");
-    return result;
-}
-static inline int sys_munmap(void* addr, size_t length) {
-    return syscall(__NR_munmap, addr, length);
-}
-#define hopscotch_allocate(nbytes) \
-  sys_mmap(0, nbytes, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS, 0, 0)
-#define hopscotch_deallocate(addr,length) sys_munmap(addr, length)
-
-#else
-
 #define hopscotch_allocate(nbytes) os_allocate(nbytes)
 #define hopscotch_deallocate(addr,length) os_deallocate(addr, length)
 
-#endif
-
 typedef struct hopscotch_table* tableptr;
 void hopscotch_integrity_check(tableptr,char*,int);
 
@@ -320,7 +280,7 @@ uword_t sxhash_simple_string(struct vector* string)
     unsigned int* char_string = (unsigned int*)(string->data);
 #endif
     unsigned char* base_string = (unsigned char*)(string->data);
-    sword_t len = fixnum_value(string->length);
+    sword_t len = vector_len(string);
     uword_t result = 0;
     sword_t i;
     switch (widetag_of(&string->header)) {
@@ -374,7 +334,10 @@ static boolean vector_eql(uword_t arg1, uword_t arg2)
 
     int widetag1 = header_widetag(header1);
     sword_t nwords = sizetab[widetag1](obj1);
-    if (widetag1 < SIMPLE_ARRAY_UNSIGNED_BYTE_2_WIDETAG)
+    // OMGWTF! Widetags have been rearranged so many times, I am not sure
+    // how to keep this code from regressing.
+    // ASSUMPTION: the range of number widetags ends with (COMPLEX DOUBLE-FLOAT)
+    if (widetag1 <= COMPLEX_DOUBLE_FLOAT_WIDETAG)
         // All words must match exactly. Start by comparing the length
         // (as encoded in the header) since we don't yet know that obj2
         // occupies the correct number of words.
@@ -383,8 +346,14 @@ static boolean vector_eql(uword_t arg1, uword_t arg2)
 
     // Vector elements must have already been coalesced
     // when comparing simple-vectors for similarity.
-    return (obj1[1] == obj2[1]) // same length vectors
-        && !memcmp(obj1 + 2, obj2 + 2, (nwords-2) << WORD_SHIFT);
+    // Note that only vectors marked "shareable" will get here, so no
+    // hash-table storage vectors or anything of that nature.
+    struct vector *v1 = (void*)obj1;
+    struct vector *v2 = (void*)obj2;
+    return (vector_len(v1) == vector_len(v2)) // same length vectors
+        && !memcmp(v1->data, v2->data,
+                   // ASSUMPTION: exactly 2 non-data words
+                   (nwords-2) << WORD_SHIFT);
 }
 
 /* Initialize 'ht' for first use, which entails zeroing the counters
