@@ -18,14 +18,13 @@
 
 extern void sigset_tostring(const sigset_t *sigset, char* result, int result_length);
 
-/* Set all deferrable signals into *s. */
-extern void sigaddset_deferrable(sigset_t *s);
 /* Set all blockable signals into *s. */
 extern void sigaddset_blockable(sigset_t *s);
 
 extern sigset_t deferrable_sigset;
 extern sigset_t blockable_sigset;
 extern sigset_t gc_sigset;
+extern sigset_t thread_start_sigset;
 
 extern boolean deferrables_blocked_p(sigset_t *sigset);
 
@@ -139,7 +138,7 @@ extern void lower_thread_control_stack_guard_page(struct thread *th);
 extern void reset_thread_control_stack_guard_page(struct thread *th);
 
 #if defined(LISP_FEATURE_SB_SAFEPOINT) && !defined(LISP_FEATURE_WIN32)
-# ifdef LISP_FEATURE_SB_THRUPTION
+# ifdef LISP_FEATURE_SB_SAFEPOINT
 void thruption_handler(int signal, siginfo_t *info, os_context_t *context);
 # endif
 #endif
