@@ -23,7 +23,6 @@
 #include "genesis/symbol.h"
 #include "genesis/static-symbols.h"
 #include "thread.h"
-#include "sbcl.h"
 #include "os.h"
 #include "arch.h"
 #include "interr.h"
@@ -74,7 +73,7 @@ os_zero(os_vm_address_t addr, os_vm_size_t length)
          * zero-filled. */
 
         os_invalidate(block_start, block_size);
-        addr = os_validate(NOT_MOVABLE, block_start, block_size);
+        addr = os_validate(NOT_MOVABLE, block_start, block_size, 0, 0);
 
         if (addr == NULL || addr != block_start)
             lose("os_zero: block moved! %p ==> %p", block_start, addr);
@@ -82,10 +81,21 @@ os_zero(os_vm_address_t addr, os_vm_size_t length)
 }
 #endif
 
+#include "sys_mmap.inc"
+#ifdef LISP_FEATURE_USE_SYS_MMAP
+os_vm_address_t os_allocate(os_vm_size_t len) {
+    void* answer = sbcl_mmap(0, len, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS, 0, 0);
+    if (answer == MAP_FAILED) return 0;
+    return answer;
+}
+void os_deallocate(os_vm_address_t addr, os_vm_size_t len) {
+    sbcl_munmap(addr, len);
+}
+#else
 os_vm_address_t
 os_allocate(os_vm_size_t len)
 {
-    return os_validate(MOVABLE, (os_vm_address_t)NULL, len);
+    return os_validate(MOVABLE, (os_vm_address_t)NULL, len, 0, 0);
 }
 
 void
@@ -93,6 +103,7 @@ os_deallocate(os_vm_address_t addr, os_vm_size_t len)
 {
     os_invalidate(addr,len);
 }
+#endif
 
 int
 os_get_errno(void)
@@ -100,9 +111,7 @@ os_get_errno(void)
     return errno;
 }
 
-#if defined LISP_FEATURE_SB_THREAD && !defined LISP_FEATURE_SB_SAFEPOINT \
-  && !defined CANNOT_USE_POSIX_SEM_T
-
+#if defined LISP_FEATURE_SB_THREAD && defined LISP_FEATURE_UNIX && !defined USE_DARWIN_GCD_SEMAPHORES
 void
 os_sem_init(os_sem_t *sem, unsigned int value)
 {
@@ -171,7 +180,7 @@ void os_link_runtime()
         return; // Linkage was already performed by coreparse
 
     struct vector* symbols = VECTOR(SymbolValue(REQUIRED_FOREIGN_SYMBOLS,0));
-    lisp_linkage_table_n_prelinked = fixnum_value(symbols->length);
+    lisp_linkage_table_n_prelinked = vector_len(symbols);
     for (j = 0 ; j < lisp_linkage_table_n_prelinked ; ++j)
     {
         lispobj item = symbols->data[j];
@@ -208,6 +217,9 @@ gc_managed_heap_space_p(lispobj addr)
             addr < DYNAMIC_0_SPACE_START + dynamic_space_size)
         || (DYNAMIC_1_SPACE_START <= addr &&
             addr < DYNAMIC_1_SPACE_START + dynamic_space_size)
+#endif
+#ifdef LISP_FEATURE_DARWIN_JIT
+        || (STATIC_CODE_SPACE_START <= addr && addr < STATIC_CODE_SPACE_END)
 #endif
         )
         return 1;
@@ -218,17 +230,31 @@ gc_managed_heap_space_p(lispobj addr)
 
 /* Remap a part of an already existing memory mapping from a file,
  * and/or create a new mapping as need be */
-void* load_core_bytes(int fd, os_vm_offset_t offset, os_vm_address_t addr, os_vm_size_t len)
+void* load_core_bytes(int fd, os_vm_offset_t offset, os_vm_address_t addr, os_vm_size_t len,
+                      int __attribute__((unused)) execute)
 {
     int fail = 0;
     os_vm_address_t actual;
+#ifdef LISP_FEATURE_64_BIT
+    actual = sbcl_mmap(addr, len,
+#else
+    /* FIXME: why does using sbcl_mmap cause failure here? I would guess that it can't
+     * pass 'offset' correctly if LARGEFILE is mandatory, which it isn't on 64-bit.
+     * Deadlock should be impossible this early in core loading, I suppose, hence
+     * on one hand I don't care; but on the other, it would be nice to not to see
+     * any use of a potentially hooked mmap() API within this file. */
     actual = mmap(addr, len,
+#endif
                   // If mapping to a random address, then the assumption is
                   // that we're not going to execute the core; nor should we write to it.
                   // However, the addr=0 case is for 'editcore' which unfortunately _does_
                   // write the memory. I'd prefer that it not,
                   // but that's not the concern here.
+#ifdef LISP_FEATURE_DARWIN_JIT
+                  OS_VM_PROT_READ | (execute ?  OS_VM_PROT_EXECUTE : OS_VM_PROT_WRITE),
+#else
                   addr ? OS_VM_PROT_ALL : OS_VM_PROT_READ | OS_VM_PROT_WRITE,
+#endif
                   // Do not pass MAP_FIXED with addr of 0, because most OSes disallow that.
                   MAP_PRIVATE | (addr ? MAP_FIXED : 0),
                   fd, (off_t) offset);
@@ -242,6 +268,33 @@ void* load_core_bytes(int fd, os_vm_offset_t offset, os_vm_address_t addr, os_vm
         lose("load_core_bytes(%d,%zx,%p,%zx) failed", fd, offset, addr, len);
     return (void*)actual;
 }
+
+#ifdef LISP_FEATURE_DARWIN_JIT
+void* load_core_bytes_jit(int fd, os_vm_offset_t offset, os_vm_address_t addr, os_vm_size_t len)
+{
+    ssize_t count;
+
+    lseek(fd, offset, SEEK_SET);
+
+    size_t n_bytes = 65536;
+    char* buf = malloc(n_bytes);
+
+    while (len) {
+        count = read(fd, buf, n_bytes);
+
+        if (count <= -1) {
+            perror("read");
+        }
+
+        memcpy(addr, buf, count);
+        addr += count;
+        len -= count;
+    }
+    free(buf);
+    return (void*)0;
+}
+#endif
+
 #endif
 
 boolean
@@ -261,5 +314,3 @@ gc_managed_addr_p(lispobj addr)
     }
     return 0;
 }
-
-
