@@ -19,7 +19,7 @@
 #include <sys/file.h>
 
 #include "sbcl.h"
-#if defined(LISP_FEATURE_WIN32) && defined(LISP_FEATURE_SB_THREAD)
+#ifdef LISP_FEATURE_WIN32
 #include "pthreads_win32.h"
 #else
 #include <signal.h>
@@ -195,7 +195,12 @@ output_space(FILE *file, int id, lispobj *addr, lispobj *end,
     words = end - addr;
     write_lispobj(words, file);
 
-    bytes = words * sizeof(lispobj);
+#ifdef LISP_FEATURE_METASPACE
+    if (id == READ_ONLY_CORE_SPACE_ID)
+        bytes = (READ_ONLY_SPACE_END - READ_ONLY_SPACE_START);
+    else
+#endif
+        bytes = words * sizeof(lispobj);
 
 #ifdef LISP_FEATURE_CHENEYGC
     /* KLUDGE: cheneygc can not restart a saved core if the dynamic space is empty,
@@ -254,7 +259,7 @@ void unwind_binding_stack()
     int i;
     if (!sym || !simple_vector_p(value = ((struct symbol*)sym)->value))
         fprintf(stderr, "warning: bad value in %s\n", symbol_name);
-    else for(i=fixnum_value(VECTOR(value)->length)-1; i>=0; --i)
+    else for(i=vector_len(VECTOR(value))-1; i>=0; --i)
         SYMBOL(VECTOR(value)->data[i])->value = UNBOUND_MARKER_WIDETAG;
     if (verbose) printf("done]\n");
 }
@@ -315,6 +320,14 @@ save_to_filehandle(FILE *file, char *filename, lispobj init_function,
                  static_space_free_pointer,
                  core_start_pos,
                  core_compression_level);
+#ifdef LISP_FEATURE_DARWIN_JIT
+    output_space(file,
+                 STATIC_CODE_CORE_SPACE_ID,
+                 (lispobj *)STATIC_CODE_SPACE_START,
+                 static_code_space_free_pointer,
+                 core_start_pos,
+                 core_compression_level);
+#endif
     output_space(file,
                  DYNAMIC_CORE_SPACE_ID,
                  current_dynamic_space,
@@ -355,7 +368,8 @@ save_to_filehandle(FILE *file, char *filename, lispobj init_function,
         memset(data + aligned_size - N_WORD_BYTES, 0, N_WORD_BYTES);
         gc_store_corefile_ptes((struct corefile_pte*)data);
         write_lispobj(PAGE_TABLE_CORE_ENTRY_TYPE_CODE, file);
-        write_lispobj(5, file); // 5 = # of words in this core header entry
+        write_lispobj(6, file); // number of words in this core header entry
+        write_lispobj(gc_card_table_nbits, file);
         write_lispobj(next_free_page, file);
         write_lispobj(aligned_size, file);
         sword_t offset = write_bytes(file, data, aligned_size, core_start_pos,
