@@ -209,7 +209,7 @@ static char* NO_SANITIZE_MEMORY deduce_thread_pc(struct thread* th, void** addr)
     uword_t* fp = __builtin_frame_address(0);
     char* return_pc = 0;
 
-    if (th != arch_os_get_current_thread()) {
+    if (th != get_sb_vm_thread()) {
         int i = fixnum_value(read_TLS(FREE_INTERRUPT_CONTEXT_INDEX,th));
         os_context_t *c = nth_interrupt_context(i-1, th);
         fp = (uword_t*)*os_context_register_addr(c,reg_FP);
@@ -243,7 +243,7 @@ deduce_thread(void (*context_scanner)(), uword_t pointer, char** pc)
     for_each_thread(th) {
         void **esp=(void **)-1;
         sword_t i,free;
-        if (th == arch_os_get_current_thread())
+        if (th == get_sb_vm_thread())
             esp = (void **)((void *)&pointer);
         else {
             void **esp1;
@@ -350,7 +350,7 @@ static lispobj examine_threads(struct hopscotch_table* targets,
                 *thread_pc = 0;
                 // Scan just the current thread's stack
                 // (We don't know where the other stack pointers are)
-                th = arch_os_get_current_thread();
+                th = get_sb_vm_thread();
                 void **esp = __builtin_frame_address(0);
                 void **where;
                 for (where = ((void **)th->control_stack_end)-1; where >= esp;  --where)
@@ -703,7 +703,7 @@ static boolean ignorep(lispobj* base_ptr,
                        lispobj ignored_objects)
 {
     int i;
-    for (i = fixnum_value(VECTOR(ignored_objects)->length)-1; i >= 0; --i)
+    for (i = vector_len(VECTOR(ignored_objects))-1; i >= 0; --i)
       if (native_pointer(VECTOR(ignored_objects)->data[i]) == base_ptr)
           return 1;
     return 0;
@@ -768,8 +768,8 @@ static uword_t build_refs(lispobj* where, lispobj* end,
                 // If not a hashtable, just skip it.
                 if (!vector_flagp(*where, VectorHashing)) continue;
                 lispobj* data = where + 2;
-                int kv_vector_length = fixnum_value(where[1]);
-                lispobj lhash_table = data[kv_vector_length-1];
+                int kv_vector_len = vector_len((struct vector*)where);
+                lispobj lhash_table = data[kv_vector_len-1];
                 gc_assert(instancep(lhash_table));
                 struct hash_table* hash_table =
                   (struct hash_table *)native_pointer(lhash_table);
@@ -966,7 +966,8 @@ static int trace_paths(void (*context_scanner)(),
     hopscotch_create(&targets, HASH_FUNCTION, 0, 32, 0);
     i = 0;
     do {
-        gc_assert(make_fixnum(i) <= VECTOR(paths)->length);
+        // Oh dear, is this really supposed to be '<=' (vs '<') ?
+        gc_assert(i <= vector_len(VECTOR(paths)));
         lispobj car = CONS(weak_pointers)->car;
         lispobj value = ((struct weak_pointer*)native_pointer(car))->value;
         weak_pointers = CONS(weak_pointers)->cdr;
@@ -979,7 +980,9 @@ static int trace_paths(void (*context_scanner)(),
                                   &targets, &visited,
                                   inverted_heap, &scratchpad,
                                   n_pins, pins, context_scanner, criterion);
-            if ((VECTOR(paths)->data[i] = path) != 0) ++n_found;
+            lispobj* elt = VECTOR(paths)->data + i;
+            ensure_ptr_word_writable(elt);
+            if ((*elt = path) != 0) ++n_found;
         }
         ++i;
     } while (weak_pointers != NIL);
