@@ -50,9 +50,9 @@ static const int should_allocate_low =
 #endif
 
 static void
-ensure_space(int attributes, uword_t start, uword_t size)
+ensure_space(int attributes, uword_t start, uword_t size, int execute, int jit)
 {
-    if (os_validate(attributes, (os_vm_address_t)start, (os_vm_size_t)size)==NULL) {
+    if (os_validate(attributes, (os_vm_address_t)start, (os_vm_size_t)size, execute, jit)==NULL) {
         fprintf(stderr,
                 "ensure_space: failed to allocate %lu bytes at %p\n",
                 (long unsigned)size, (void*)start);
@@ -63,23 +63,26 @@ ensure_space(int attributes, uword_t start, uword_t size)
 }
 
 os_vm_address_t undefined_alien_address = 0;
+/* As contrasted with the useless os_vm_page_size which is identical to
+ * the constant BACKEND_PAGE_BYTES that we define for various other purposes
+ * such as core file loading and generational GC,
+ * this is the number the OS tells us */
+int os_reported_page_size;
+
+#ifdef LISP_FEATURE_WIN32
+#include <sysinfoapi.h>
+#endif
 
 static void
 ensure_undefined_alien(void) {
-    os_vm_address_t start = os_validate(MOVABLE|IS_GUARD_PAGE, NULL,
-#ifndef LISP_FEATURE_WIN32
-    /* We can/should disregard our 'os_vm_page_size' constant which tends to be
-     * larger than the granularity that the OS will allow you to manipulate via
-     * mprotect(). e.g. on x86-64-linux we use a page size of 32K but in reality
-     * the protection granularity is 4K.
-     * Moreover, since the memory protection is not changed after allocation,
-     * the granuarity that mprotect() operates on is immaterial. As such, it
-     * probably would work to put N_WORD_BYTES here since that's all we need. */
-                                        getpagesize()
-#else // Use the same value as does contrib/sb-posix/interface.lisp
-                                        4096
+#ifdef LISP_FEATURE_WIN32
+    SYSTEM_INFO info;
+    GetSystemInfo(&info);
+    os_reported_page_size = info.dwPageSize;
+#else
+    os_reported_page_size = getpagesize();
 #endif
-                                        );
+    os_vm_address_t start = os_validate(MOVABLE|IS_GUARD_PAGE, NULL, os_reported_page_size, 0, 0);
     if (start) {
         undefined_alien_address = start;
     } else {
@@ -96,23 +99,32 @@ boolean allocate_hardwired_spaces(boolean hard_failp)
     struct {
         uword_t start;
         unsigned size;
+        int execute;
+        int jit;
     } preinit_spaces[] = {
-      { READ_ONLY_SPACE_START, READ_ONLY_SPACE_SIZE },
-      { STATIC_SPACE_START, STATIC_SPACE_SIZE },
-#ifdef LISP_FEATURE_LINKAGE_TABLE
-      { LINKAGE_TABLE_SPACE_START, LINKAGE_TABLE_SPACE_SIZE },
+        { READ_ONLY_SPACE_START, READ_ONLY_SPACE_SIZE, 1, 0},
+        { LINKAGE_TABLE_SPACE_START, LINKAGE_TABLE_SPACE_SIZE, 1, 2},
+        { STATIC_SPACE_START, STATIC_SPACE_SIZE, 0, 0},
+#ifdef LISP_FEATURE_DARWIN_JIT
+        { STATIC_CODE_SPACE_START, STATIC_CODE_SPACE_SIZE, 1, 1},
 #endif
     };
     int i;
     int n_spaces = sizeof preinit_spaces / sizeof preinit_spaces[0];
     boolean success = 1;
     for (i = 0; i< n_spaces; ++i) {
+        if (!preinit_spaces[i].size) continue;
         if (hard_failp)
             ensure_space(NOT_MOVABLE | should_allocate_low,
-                         preinit_spaces[i].start, preinit_spaces[i].size);
+                         preinit_spaces[i].start,
+                         preinit_spaces[i].size,
+                         preinit_spaces[i].execute,
+                         preinit_spaces[i].jit);
         else if (!os_validate(NOT_MOVABLE | should_allocate_low,
                               (os_vm_address_t)preinit_spaces[i].start,
-                              preinit_spaces[i].size)) {
+                              preinit_spaces[i].size,
+                              preinit_spaces[i].execute,
+                              preinit_spaces[i].jit)) {
             success = 0;
             break;
         }
@@ -138,28 +150,26 @@ allocate_lisp_dynamic_space(boolean did_preinit)
 }
 
 static inline void
-protect_page(void *page, int protect_p, os_vm_prot_t flags) {
+protect_guard_page(void *page, int protect_p, os_vm_prot_t flags) {
     os_protect(page, os_vm_page_size, protect_p ?
-               flags : OS_VM_PROT_ALL);
+               flags : OS_VM_PROT_READ | OS_VM_PROT_WRITE);
 }
 
 #define DEF_PROTECT_PAGE(name,page_name,flags)                          \
     void                                                                \
     protect_##name(int protect_p, struct thread *thread) {              \
         if (!thread)                                                    \
-            thread = arch_os_get_current_thread();                      \
-        protect_page(page_name(thread), protect_p, flags);              \
+            thread = get_sb_vm_thread();                      \
+        protect_guard_page(page_name(thread), protect_p, flags);        \
     }
 
 DEF_PROTECT_PAGE(control_stack_hard_guard_page,
                  CONTROL_STACK_HARD_GUARD_PAGE,
                  OS_VM_PROT_NONE)
 DEF_PROTECT_PAGE(control_stack_guard_page,
-                 CONTROL_STACK_GUARD_PAGE,
-                 OS_VM_PROT_READ|OS_VM_PROT_EXECUTE)
+                 CONTROL_STACK_GUARD_PAGE, OS_VM_PROT_READ)
 DEF_PROTECT_PAGE(control_stack_return_guard_page,
-                 CONTROL_STACK_RETURN_GUARD_PAGE,
-                 OS_VM_PROT_READ|OS_VM_PROT_EXECUTE)
+                 CONTROL_STACK_RETURN_GUARD_PAGE, OS_VM_PROT_READ)
 
 DEF_PROTECT_PAGE(binding_stack_hard_guard_page,
                  BINDING_STACK_HARD_GUARD_PAGE,
