@@ -36,7 +36,6 @@
 #include "os.h"
 #include "arch.h"
 #include "globals.h"
-#include "sbcl.h"
 #include "interrupt.h"
 #include "interr.h"
 #include "lispregs.h"
@@ -59,6 +58,7 @@
 #include "validate.h"
 #include "thread.h"
 #include "align.h"
+#include "unaligned.h"
 
 #include "gc.h"
 #include "gencgc-internal.h"
@@ -74,9 +74,6 @@
 #define EH_EXIT_UNWIND 0x04
 #endif
 
-/* Tired of writing arch_os_get_current_thread each time. */
-#define this_thread (arch_os_get_current_thread())
-
 /* Documented limit for ReadConsole/WriteConsole is 64K bytes.
    Real limit observed on W2K-SP3 is somewhere in between 32KiB and 64Kib...
 */
@@ -132,14 +129,14 @@ intptr_t win_aver(intptr_t value, char* comment, char* file, int line,
             fprintf(stderr, report_template,
                     file, line,
                     comment, value,
-                    this_thread,
+                    get_sb_vm_thread(),
                     (unsigned)errorCode, errorMessage,
                     posixerrno, posixstrerror);
         } else {
             lose(report_template,
                     file, line,
                     comment, value,
-                    this_thread,
+                    get_sb_vm_thread(),
                     (unsigned)errorCode, errorMessage,
                     posixerrno, posixstrerror);
         }
@@ -220,30 +217,21 @@ EXCEPTION_DISPOSITION handle_exception(EXCEPTION_RECORD *,
  * exception frame on nested funcall()s also points to it.
  */
 
-
-void *base_seh_frame;
-
 HMODULE runtime_module_handle = 0u;
 
+#ifdef LISP_FEATURE_X86
 static void *get_seh_frame(void)
 {
     void* retval;
-#ifdef LISP_FEATURE_X86
     asm volatile ("mov %%fs:0,%0": "=r" (retval));
-#else
-    asm volatile ("mov %%gs:0,%0": "=r" (retval));
-#endif
     return retval;
 }
 
 static void set_seh_frame(void *frame)
 {
-#ifdef LISP_FEATURE_X86
     asm volatile ("mov %0,%%fs:0": : "r" (frame));
-#else
-    asm volatile ("mov %0,%%gs:0": : "r" (frame));
-#endif
 }
+#endif
 
 #if defined(LISP_FEATURE_SB_THREAD)
 
@@ -572,6 +560,7 @@ void* os_dlsym_default(char* name)
 {
     unsigned int i;
     void* result = 0;
+    buildTimeImages[0] = (void*)runtime_module_handle;
     if (buildTimeImageCount == 0) {
         buildTimeImageCount =
             1 + os_get_build_time_shared_libraries(15u,
@@ -671,7 +660,7 @@ int os_preinit(char *argv[], char *envp[])
             if (TlsGetValue(key)!=NULL)
                 lose("TLS slot assertion failed: fresh slot value is not NULL");
             TlsSetValue(OUR_TLS_INDEX, (void*)(intptr_t)0xFEEDBAC4);
-            if ((intptr_t)(void*)arch_os_get_current_thread()!=(intptr_t)0xFEEDBAC4)
+            if ((intptr_t)(void*)get_sb_vm_thread()!=(intptr_t)0xFEEDBAC4)
                 lose("TLS slot assertion failed: TIB layout change detected");
             TlsSetValue(OUR_TLS_INDEX, NULL);
             break;
@@ -728,11 +717,131 @@ intptr_t win32_get_module_handle_by_address(os_vm_address_t addr)
                       ? result : 0);
 }
 
+/*
+ * x86-64 exception handling around foreign function calls.
+ *
+ * On x86-64, when an exception is raised, the Windows kernel looks up the RIP
+ * to retrieve an associated UNWIND_INFO object. This object informs the kernel
+ * whether there's an exception handler and how to find the start of the current
+ * frame. If there's no exception handler or it has declined to handle the
+ * exception, the frame's return address is looked up and the process is
+ * repeated, walking the stack until an exception handler handles the exception.
+ * If the exception goes unhandled, the process is usually terminated. Our goal,
+ * then, is to associate an appropriate UNWIND_INFO object with return addresses
+ * that point to Lisp code. We don't need to inform the kernel how to walk a
+ * Lisp frame, since our exception handler will always handle the exception.
+ *
+ * UNWIND_INFO objects are mapped onto their respective address ranges by
+ * RUNTIME_FUNCTION objects. These objects are usually stored in the .pdata
+ * section of an executable file, but can also be mapped dynamically via
+ * RtlAddFunctionTable().
+ *
+ * A major constraint imposed by UNWIND_INFO and RUNTIME_FUNCTION is that the
+ * code's address range is specified by 32-bit relative addresses, which means
+ * we can only establish a frame-based handler for a 4 GB region at most. This
+ * is problematic because most Lisp code lives in dynamic space which can be
+ * much larger than 4 GB. Furthermore, the exception handler is also specified
+ * as a 32-bit relative address, so we can't directly reference our
+ * handle_exception() function.
+ *
+ * We work around this constraint by allocating these data structures in a fixed
+ * memory region at WIN64_SEH_DATA_ADDR and map an UNWIND_INFO to this region
+ * using RtlAddFunctionTable(). Foreign calls (see EMIT-C-CALL) place the target
+ * address in RBX and call a thunk within this region. The thunk swaps out the
+ * return address with the thunk's return address thereby establishing a
+ * frame-based exception handler. The original address (pointing into Lisp code)
+ * is stored in a non-volatile, callee-saved register, allowing the thunk to
+ * jump back to Lisp. The exception handler is a trampoline to
+ * handle_exception().
+ *
+ * In the future, should we segregate code objects into dedicated code areas
+ * smaller than 4 GB, these thunks could be removed, and UNWIND_INFO could be
+ * associated directly with such areas.
+ */
+
+#ifdef LISP_FEATURE_X86_64
+typedef struct _UNWIND_INFO {
+  uint8_t Version : 3;
+  uint8_t Flags : 5;
+  uint8_t SizeOfProlog;
+  uint8_t CountOfCodes;
+  uint8_t FrameRegister : 4;
+  uint8_t FrameOffset : 4;
+  ULONG ExceptionHandler;
+  ULONG ExceptionData[1];
+} UNWIND_INFO;
+
+struct win64_seh_data {
+    uint8_t direct_thunk[8];
+    uint8_t indirect_thunk[8];
+    uint8_t handler_trampoline[16];
+    UNWIND_INFO ui; // needs to be DWORD-aligned
+    RUNTIME_FUNCTION rt;
+};
+
+static void
+set_up_win64_seh_thunk(size_t page_size)
+{
+    if (page_size < sizeof(struct win64_seh_data))
+        lose("Not enough space to allocate struct win64_seh_data");
+
+    AVER(VirtualAlloc(WIN64_SEH_DATA_ADDR, page_size,
+                      MEM_RESERVE | MEM_COMMIT, PAGE_EXECUTE_READWRITE));
+
+    struct win64_seh_data *seh_data = (void *) WIN64_SEH_DATA_ADDR;
+    DWORD64 base = (DWORD64) seh_data;
+
+    uint8_t *dthunk = seh_data->direct_thunk;
+    dthunk[0] = 0x41; // pop r15
+    dthunk[1] = 0x5F;
+    dthunk[2] = 0xFF; // call rbx
+    dthunk[3] = 0xD3;
+    dthunk[4] = 0x41; // push r15
+    dthunk[5] = 0x57;
+    dthunk[6] = 0xC3; // ret
+    dthunk[7] = 0x90; // nop (padding)
+
+    uint8_t *ithunk = seh_data->indirect_thunk;
+    ithunk[0] = 0x41; // pop r15
+    ithunk[1] = 0x5F;
+    ithunk[2] = 0xFF; // call qword ptr [rbx]
+    ithunk[3] = 0x13;
+    ithunk[4] = 0x41; // push r15
+    ithunk[5] = 0x57;
+    ithunk[6] = 0xC3; // ret
+    ithunk[7] = 0x90; // nop (padding)
+
+    uint8_t *tramp = seh_data->handler_trampoline;
+    tramp[0] = 0xFF; // jmp qword ptr [rip+2]
+    tramp[1] = 0x25;
+    UNALIGNED_STORE32((tramp+2), 2);
+    tramp[6] = 0x66; // 2-byte nop
+    tramp[7] = 0x90;
+    *(void **)(tramp+8) = handle_exception;
+
+    UNWIND_INFO *ui = &seh_data->ui;
+    ui->Version = 1;
+    ui->Flags = UNW_FLAG_EHANDLER;
+    ui->SizeOfProlog = 0;
+    ui->CountOfCodes = 0;
+    ui->FrameRegister = 0;
+    ui->FrameOffset = 0;
+    ui->ExceptionHandler = (DWORD64) tramp - base;
+    ui->ExceptionData[0] = 0;
+
+    RUNTIME_FUNCTION *rt = &seh_data->rt;
+    rt->BeginAddress = 0;
+    rt->EndAddress = 16;
+    rt->UnwindData = (DWORD64) ui - base;
+
+    AVER(RtlAddFunctionTable(rt, 1, base));
+}
+#endif
+
 static LARGE_INTEGER lisp_init_time;
 static double qpcMultiplier;
 
-void os_init(char __attribute__((__unused__)) *argv[],
-             char __attribute__((__unused__)) *envp[])
+void os_init()
 {
 #ifdef LISP_FEATURE_64_BIT
     LARGE_INTEGER qpcFrequency;
@@ -756,7 +865,9 @@ void os_init(char __attribute__((__unused__)) *argv[],
 #endif
     os_number_of_processors = system_info.dwNumberOfProcessors;
 
-    base_seh_frame = get_seh_frame();
+#ifdef LISP_FEATURE_X86_64
+    set_up_win64_seh_thunk(os_vm_page_size);
+#endif
 
     resolve_optional_imports();
     runtime_module_handle = (HMODULE)win32_get_module_handle_by_address(&runtime_module_handle);
@@ -773,7 +884,8 @@ uword_t get_monotonic_time()
 #endif
 
 os_vm_address_t
-os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len)
+os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len,
+            int __attribute__((unused)) execute, int __attribute__((unused)) jit)
 {
     if (!addr) {
         int protection = attributes & IS_GUARD_PAGE ? PAGE_NOACCESS : PAGE_EXECUTE_READWRITE;
@@ -827,7 +939,6 @@ void* os_commit_memory(os_vm_address_t addr, os_vm_size_t len)
 
 void os_revalidate_bzero(os_vm_address_t addr,  os_vm_size_t len) {
     AVERLAX(VirtualFree(addr, len, MEM_DECOMMIT));
-    AVERLAX(VirtualAlloc(addr, len, MEM_COMMIT, PAGE_EXECUTE_READWRITE));
 }
 
 void
@@ -847,7 +958,8 @@ os_invalidate(os_vm_address_t addr, os_vm_size_t len)
  * thing to maintain).
  */
 
-void* load_core_bytes(int fd, os_vm_offset_t offset, os_vm_address_t addr, os_vm_size_t len)
+void* load_core_bytes(int fd, os_vm_offset_t offset, os_vm_address_t addr, os_vm_size_t len,
+                      int __attribute__((unused)) execute)
 {
     os_commit_memory(addr, len);
 #ifdef LISP_FEATURE_64_BIT
@@ -896,26 +1008,6 @@ extern boolean internal_errors_enabled;
 
 extern void exception_handler_wrapper();
 
-void
-c_level_backtrace(const char* header, int depth)
-{
-    void* frame;
-    int n = 0;
-    void** lastseh;
-
-    for (lastseh = get_seh_frame(); lastseh && (lastseh!=(void*)-1);
-         lastseh = *lastseh);
-
-    fprintf(stderr, "Backtrace: %s (thread %p)\n", header, this_thread);
-    for (frame = __builtin_frame_address(0); frame; frame=*(void**)frame)
-    {
-        if ((n++)>depth)
-            return;
-        fprintf(stderr, "[#%02d]: ebp = %p, ret = %p\n",n,
-                frame, ((void**)frame)[1]);
-    }
-}
-
 #ifdef LISP_FEATURE_X86
 #define voidreg(ctxptr,name) ((void*)((ctxptr)->E##name))
 #else
@@ -1032,22 +1124,18 @@ handle_access_violation(os_context_t *ctx,
         return 0;
     }
 
-    if (1+(lispobj*)fault_address == (lispobj*)self) {
+    if ((1+THREAD_HEADER_SLOTS)+(lispobj*)fault_address == (lispobj*)self) {
         thread_in_safety_transition(ctx);
         return 0;
     }
 #endif
 
     /* dynamic space */
-    if (DYNAMIC_SPACE_START <= (lispobj)fault_address &&
-        (lispobj)fault_address < (DYNAMIC_SPACE_START + dynamic_space_size)) {
-        MEMORY_BASIC_INFORMATION mem_info;
-        if (AVERLAX(VirtualQuery(fault_address, &mem_info, sizeof mem_info)) &&
-            mem_info.State == MEM_RESERVE) {
-            os_commit_memory(PTR_ALIGN_DOWN(fault_address, os_vm_page_size),
-                             os_vm_page_size);
-            return 0;
-        }
+    page_index_t page = find_page_index(fault_address);
+    if (page != -1 && !PAGE_WRITEPROTECTED_P(page)) {
+        os_commit_memory(PTR_ALIGN_DOWN(fault_address, os_vm_page_size),
+                         os_vm_page_size);
+        return 0;
     }
     if (gencgc_handle_wp_violation(fault_address)) {
         return 0;
@@ -1138,18 +1226,11 @@ signal_internal_error_or_lose(os_context_t *ctx,
     lose("Exception too early in cold init, cannot continue.");
 }
 
-/*
- * A good explanation of the exception handling semantics is
- *   http://win32assembly.online.fr/Exceptionhandling.html (possibly defunct)
- * or:
- *   http://www.microsoft.com/msj/0197/exception/exception.aspx
- */
-
-EXCEPTION_DISPOSITION
-handle_exception(EXCEPTION_RECORD *exception_record,
-                 struct lisp_exception_frame *exception_frame,
-                 CONTEXT *win32_context,
-                 void __attribute__((__unused__)) *dispatcher_context)
+static EXCEPTION_DISPOSITION
+handle_exception_ex(EXCEPTION_RECORD *exception_record,
+                    struct lisp_exception_frame *exception_frame,
+                    CONTEXT *win32_context,
+                    BOOL continue_search_on_unhandled_access_violation)
 {
     if (!win32_context)
         /* Not certain why this should be possible, but let's be safe... */
@@ -1160,7 +1241,7 @@ handle_exception(EXCEPTION_RECORD *exception_record,
 
         /* Undo any dynamic bindings. */
         unbind_to_here(exception_frame->bindstack_pointer,
-                       arch_os_get_current_thread());
+                       get_sb_vm_thread());
         return ExceptionContinueSearch;
     }
 
@@ -1176,7 +1257,7 @@ handle_exception(EXCEPTION_RECORD *exception_record,
     DWORD lastError = GetLastError();
     DWORD lastErrno = errno;
 
-    struct thread* self = arch_os_get_current_thread();
+    struct thread* self = get_sb_vm_thread();
 
     os_context_t context, *ctx = &context;
     context.win32_context = win32_context;
@@ -1209,10 +1290,14 @@ handle_exception(EXCEPTION_RECORD *exception_record,
      * isn't happy: */
 
     int rc;
+    EXCEPTION_DISPOSITION disp = ExceptionContinueExecution;
     switch (code) {
     case EXCEPTION_ACCESS_VIOLATION:
-        rc = handle_access_violation(
-            ctx, exception_record, fault_address, self);
+        rc = handle_access_violation(ctx, exception_record, fault_address, self);
+        if (rc && continue_search_on_unhandled_access_violation) {
+            rc = 0;
+            disp = ExceptionContinueSearch;
+        }
         break;
 
     case SBCL_EXCEPTION_BREAKPOINT:
@@ -1236,7 +1321,35 @@ handle_exception(EXCEPTION_RECORD *exception_record,
 
     errno = lastErrno;
     SetLastError(lastError);
-    return ExceptionContinueExecution;
+    return disp;
+}
+
+/*
+ * A good explanation of the exception handling semantics is
+ *   https://web.archive.org/web/20120628151428/http://win32assembly.online.fr/Exceptionhandling.html
+ *   (x86-specific)
+ * or:
+ *   https://docs.microsoft.com/en-us/windows/win32/debug/structured-exception-handling
+ * or:
+ *   James McNellis's CppCon 2018 talk: "Unwinding the Stack: Exploring How C++
+ *   Exceptions Work on Windows"
+ *
+ * On x86, this function is always invoked by frame-based exception handlers,
+ * either the one established by call_into_lisp in x86-assem.S or the top-level
+ * handler established by wos_install_interrupt_handlers().
+ *
+ * On x86-64, this function may be invoked by the frame-based exception handler
+ * established by set_up_win64_seh_thunk(). Our vectored exception handler,
+ * veh(), invokes handle_exception_ex directly.
+ */
+
+EXCEPTION_DISPOSITION
+handle_exception(EXCEPTION_RECORD *exception_record,
+                 struct lisp_exception_frame *exception_frame,
+                 CONTEXT *win32_context,
+                 void __attribute__((__unused__)) *dispatcher_context)
+{
+    return handle_exception_ex(exception_record, exception_frame, win32_context, FALSE);
 }
 
 #ifdef LISP_FEATURE_X86_64
@@ -1245,26 +1358,52 @@ handle_exception(EXCEPTION_RECORD *exception_record,
     int sbcl__lastErrno = errno;                                \
     RUN_BODY_ONCE(restoring_errno, errno = sbcl__lastErrno)
 
+/*
+ * This vectored exception handler runs before frame-based handlers, so we only
+ * want to handle exceptions triggered by Lisp code. Exceptions raised by
+ * foreign function calls are handled by the frame-based handler established by
+ * set_up_win64_seh_thunk().
+ *
+ * Access violation exceptions can be triggered by the runtime as well, and
+ * that's neither Lisp code nor is it covered the SEH thunk, so we handle those
+ * exceptions differently.
+ */
 LONG
 veh(EXCEPTION_POINTERS *ep)
 {
-    EXCEPTION_DISPOSITION disp;
+    EXCEPTION_DISPOSITION disp = ExceptionContinueSearch;
 
     RESTORING_ERRNO() {
-        if (!arch_os_get_current_thread())
+        if (!get_sb_vm_thread())
             return EXCEPTION_CONTINUE_SEARCH;
     }
 
-    disp = handle_exception(ep->ExceptionRecord,0,ep->ContextRecord,0);
+    DWORD64 rip = ep->ContextRecord->Rip;
+    DWORD code = ep->ExceptionRecord->ExceptionCode;
+    BOOL from_lisp =
+        (rip >= READ_ONLY_SPACE_START &&
+         rip <= READ_ONLY_SPACE_END) ||
+        (rip >= DYNAMIC_SPACE_START &&
+         rip <= DYNAMIC_SPACE_START+dynamic_space_size);
 
-    switch (disp)
-    {
+    if (code == EXCEPTION_ACCESS_VIOLATION ||
+        code == STATUS_HEAP_CORRUPTION ||
+        from_lisp)
+        disp = handle_exception_ex(ep->ExceptionRecord, 0, ep->ContextRecord,
+                                   // continue search on unhandled memory faults
+                                   // if not in Lisp code. This gives foreign
+                                   // handlers a chance to handle the
+                                   // exception. If nothing handles it, our
+                                   // frame-based handler will.
+                                   !from_lisp);
+
+    switch (disp) {
     case ExceptionContinueExecution:
         return EXCEPTION_CONTINUE_EXECUTION;
     case ExceptionContinueSearch:
         return EXCEPTION_CONTINUE_SEARCH;
     default:
-        fprintf(stderr,"Exception handler is mad\n");
+        fprintf(stderr, "Exception handler is mad\n"); fflush(stderr);
         ExitProcess(0);
     }
 }
@@ -1273,7 +1412,7 @@ veh(EXCEPTION_POINTERS *ep)
 os_context_register_t
 carry_frame_pointer(os_context_register_t default_value)
 {
-    struct thread* self = arch_os_get_current_thread();
+    struct thread* self = get_sb_vm_thread();
     os_context_register_t bp = thread_extra_data(self)->carried_base_pointer;
     return bp ? bp : default_value;
 }
@@ -1341,11 +1480,11 @@ char *dirname(char *path)
 
 // 0 - not a socket or other error, 1 - has input, 2 - has no input
 int
-socket_input_available(HANDLE socket)
+socket_input_available(HANDLE socket, long time, long utime)
 {
     int count = 0;
     int wsaErrno = GetLastError();
-    TIMEVAL timeout = {0, 0};
+    TIMEVAL timeout = {time, utime};
     fd_set readfds, errfds;
 
     FD_ZERO(&readfds);
@@ -1384,6 +1523,7 @@ boolean io_begin_interruptible(HANDLE handle)
     if (!ptr_CancelIoEx)
         return 1;
 
+    struct thread* this_thread = get_sb_vm_thread();
     if (!__sync_bool_compare_and_swap(&thread_extra_data(this_thread)->synchronous_io_handle_and_flag,
                                       0, handle)) {
         ResetEvent(thread_private_events(this_thread,0));
@@ -1402,7 +1542,7 @@ io_end_interruptible(HANDLE handle)
     if (!ptr_CancelIoEx)
         return;
     thread_mutex_lock(&interrupt_io_lock);
-    __sync_bool_compare_and_swap(&thread_extra_data(this_thread)->synchronous_io_handle_and_flag,
+    __sync_bool_compare_and_swap(&thread_extra_data(get_sb_vm_thread())->synchronous_io_handle_and_flag,
                                  handle, 0);
     thread_mutex_unlock(&interrupt_io_lock);
 }
@@ -1715,7 +1855,7 @@ win32_unix_write(HANDLE handle, void * buf, int count)
 {
     DWORD written_bytes;
     OVERLAPPED overlapped;
-    struct thread * self = arch_os_get_current_thread();
+    struct thread * self = get_sb_vm_thread();
     BOOL waitInGOR;
     LARGE_INTEGER file_position;
     BOOL seekable;
@@ -1790,7 +1930,7 @@ win32_unix_read(HANDLE handle, void * buf, int count)
 {
     OVERLAPPED overlapped = {.Internal=0};
     DWORD read_bytes = 0;
-    struct thread * self = arch_os_get_current_thread();
+    struct thread * self = get_sb_vm_thread();
     DWORD errorCode = 0;
     BOOL waitInGOR = FALSE;
     BOOL ok = FALSE;
@@ -1920,7 +2060,7 @@ DWORD
 win32_wait_object_or_signal(HANDLE waitFor)
 {
 #ifdef LISP_FEATURE_SB_THREAD
-    struct thread *self = arch_os_get_current_thread();
+    struct thread *self = get_sb_vm_thread();
     HANDLE handles[] = {waitFor, thread_private_events(self,1)};
     return
         WaitForMultipleObjects(2,handles, FALSE, INFINITE);
@@ -1933,7 +2073,7 @@ DWORD
 win32_wait_for_multiple_objects_or_signal(HANDLE *handles, DWORD count)
 {
 #ifdef LISP_FEATURE_SB_THREAD
-    struct thread *self = arch_os_get_current_thread();
+    struct thread *self = get_sb_vm_thread();
     handles[count] = thread_private_events(self,1);
     return
         WaitForMultipleObjects(count + 1, handles, FALSE, INFINITE);
@@ -1993,7 +2133,7 @@ int
 futex_wait(int *lock_word, int oldval, long sec, unsigned long usec)
 {
   DWORD timeout = sec < 0 ? INFINITE : (sec * 1000) + (usec / 1000);
-  struct thread* th = arch_os_get_current_thread();
+  struct thread* th = get_sb_vm_thread();
   thread_extra_data(th)->waiting_on_address = lock_word;
   int result = WaitOnAddress(lock_word, &oldval, 4, timeout);
   thread_extra_data(th)->waiting_on_address = 0;
@@ -2015,9 +2155,9 @@ futex_wake(PVOID lock_word, int n)
 }
 #endif
 
-int _sbcl_pthread_sigmask(int how, const sigset_t *set, sigset_t *oldset)
+int sb_pthread_sigmask(int how, const sigset_t *set, sigset_t *oldset)
 {
-  struct extra_thread_data* self = thread_extra_data(arch_os_get_current_thread());
+  struct extra_thread_data* self = thread_extra_data(get_sb_vm_thread());
   if (oldset)
     *oldset = self->blocked_signal_set;
   if (set) {
@@ -2044,7 +2184,7 @@ int sb_pthr_kill(struct thread* thread, int signum)
 
 int sigpending(sigset_t *set)
 {
-    struct extra_thread_data* data = thread_extra_data(arch_os_get_current_thread());
+    struct extra_thread_data* data = thread_extra_data(get_sb_vm_thread());
     *set = InterlockedCompareExchange((volatile LONG*)&data->pending_signal_set,
                                       0, 0);
     return 0;
