@@ -16,7 +16,6 @@
 #include "globals.h"
 #include "validate.h"
 #include "os.h"
-#include "sbcl.h"
 #include "arch.h"
 #include "lispregs.h"
 #include "signal.h"
@@ -28,12 +27,13 @@
 #include "getallocptr.h"
 #include "unaligned.h"
 #include "search.h"
-#include "globals.h" // for asm_routines_start,end
+#include "var-io.h"
 
 #include "genesis/static-symbols.h"
 #include "genesis/symbol.h"
 #include "forwarding-ptr.h"
 #include "core.h"
+#include "gc-private.h"
 
 #define INT3_INST 0xCC
 #define INTO_INST 0xCE
@@ -71,6 +71,21 @@ static void xgetbv(unsigned *eax, unsigned *edx)
 
 #define VECTOR_FILL_T "VECTOR-FILL/T"
 
+void set_alloc_tramp_vectors(int ymm_enable)
+{
+    struct code* code = (struct code*)asm_routines_start;
+    lispobj* instructions = (lispobj*)code + code_header_words(code);
+    int index;
+    get_asm_routine_by_name("FPR-SAVE", &index);
+    if (index)
+        instructions[index] =
+            (lispobj)get_asm_routine_by_name(ymm_enable ? "SAVE-YMM" : "SAVE-XMM", 0);
+    get_asm_routine_by_name("FPR-RESTORE", &index);
+    if (index)
+        instructions[index] =
+            (lispobj)get_asm_routine_by_name(ymm_enable ? "RESTORE-YMM" : "RESTORE-XMM", 0);
+}
+
 // Poke in a byte that changes an opcode to enable faster vector fill.
 // Using fixed offsets and bytes is no worse than what we do elsewhere.
 void tune_asm_routines_for_microarch(void)
@@ -94,6 +109,7 @@ void tune_asm_routines_for_microarch(void)
             }
         }
     }
+    set_alloc_tramp_vectors(avx_supported);
     int our_cpu_feature_bits = 0;
     // avx2_supported gets copied into bit 1 of *CPU-FEATURE-BITS*
     if (avx2_supported) our_cpu_feature_bits |= 1;
@@ -120,6 +136,7 @@ void untune_asm_routines_for_microarch(void)
 {
     asm_routine_poke(VECTOR_FILL_T, 0x12, 0xEB); // Change JL to JMP
     SetSymbolValue(CPU_FEATURE_BITS, 0, 0);
+    set_alloc_tramp_vectors(0);
 }
 
 #ifndef _WIN64
@@ -226,20 +243,20 @@ arch_internal_error_arguments(os_context_t *context)
 boolean
 arch_pseudo_atomic_atomic(os_context_t __attribute__((unused)) *context)
 {
-    return get_pseudo_atomic_atomic(arch_os_get_current_thread());
+    return get_pseudo_atomic_atomic(get_sb_vm_thread());
 }
 
 void
 arch_set_pseudo_atomic_interrupted(os_context_t __attribute__((unused)) *context)
 {
-    struct thread *thread = arch_os_get_current_thread();
+    struct thread *thread = get_sb_vm_thread();
     set_pseudo_atomic_interrupted(thread);
 }
 
 void
 arch_clear_pseudo_atomic_interrupted(os_context_t __attribute__((unused)) *context)
 {
-    struct thread *thread = arch_os_get_current_thread();
+    struct thread *thread = get_sb_vm_thread();
     clear_pseudo_atomic_interrupted(thread);
 }
 
@@ -359,6 +376,12 @@ sigtrap_handler(int __attribute__((unused)) signal,
                 siginfo_t __attribute__((unused)) *info,
                 os_context_t *context)
 {
+#ifdef LISP_FEATURE_LINUX
+    // ICEBP instruction = handle-pending-interrupt following pseudo-atomic
+    if (((unsigned char*)*os_context_pc_addr(context))[-1] == 0xF1)
+        return interrupt_handle_pending(context);
+#endif
+
     unsigned int trap;
 
     if (single_stepping) {
@@ -368,7 +391,7 @@ sigtrap_handler(int __attribute__((unused)) signal,
 
     /* This is just for info in case the monitor wants to print an
      * approximation. */
-    access_control_stack_pointer(arch_os_get_current_thread()) =
+    access_control_stack_pointer(get_sb_vm_thread()) =
         (lispobj *)*os_context_sp_addr(context);
 
     /* On entry %rip points just after the INT3 byte and aims at the
@@ -692,7 +715,15 @@ static void record_pc(char* pc, unsigned int index, boolean sizedp)
        v->data[index] = v->data[index+1] = NIL;
        index += 2;
     }
+    // Wasn't the point of code serial# that you don't store
+    // code blob pointers into the various profiling buffers? (FIXME?)
     if (code) {
+#ifdef LISP_FEATURE_SOFT_CARD_MARKS
+        page_index_t page = find_page_index(&v->data[index]);
+        // technically this only needs to be unprotected if the generation
+        // of the code is younger, but the KISS principle pertains.
+        if (page >= 0) unprotect_page_index(page);
+#endif
         v->data[index] = make_lispobj(code, OTHER_POINTER_LOWTAG);
         v->data[index+1] = make_fixnum((lispobj)pc - (lispobj)code);
     } else {
@@ -711,9 +742,10 @@ allocation_tracker_counted(uword_t* sp)
         if (index == 0)
             index = 2; // reserved overflow counter for fixed-size alloc
         uword_t disp = index * 8;
-        // rewrite call into: LOCK INC QWORD PTR, [R11+n] ; opcode = 0xFF / 0
-        uword_t new_inst =
-          0xF0 | (0x49 << 8) | (0xFF << 16) | (0x83L << 24) | (disp << 32);
+        int base_reg = word_at_pc >> 56;
+        // rewrite call into: LOCK INC QWORD PTR, [Rbase+n] ; opcode = 0xFF / 0
+        uword_t new_inst = 0xF0 | ((0x48|(base_reg>>3)) << 8) // w + possibly 'b'
+            | (0xFF << 16) | ((0x80L+(base_reg&7)) << 24) | (disp << 32);
         // Ensure atomicity of the write. A plain store would probably do,
         // but since this is self-modifying code, the most stringent memory
         // order is prudent.
@@ -733,24 +765,17 @@ allocation_tracker_sized(uword_t* sp)
     if (instrumentp(sp, &pc, &word_at_pc)) {
         int index = claim_index(2);
         uword_t word_after_pc = pc[1];
-        unsigned char prefix = word_after_pc & 0xFF;
-        unsigned char opcode = (word_after_pc >>  8) & 0xFF;
-        unsigned char modrm  = (word_after_pc >> 16) & 0xFF;
-        if ((prefix == 0x48 || prefix == 0x4D) && /* REX w=1 or w=r=b=1 */
-            (opcode == 0x85) && /* TEST */
-            (modrm & 0xC0) == 0xC0 && /* register-direct mode */
-            ((modrm >> 3) & 0x7) == (modrm & 0x7)) { /* same register */
-        } else {
-            lose("Can't decode instruction @ pc %p", pc);
-        }
+        int pair = word_at_pc >> 56;
+        int base_reg = pair & 0xf;
+        int size_reg = pair >> 4;
         // rewrite call into:
-        //  LOCK INC QWORD PTR, [R11+n] ; opcode = 0xFF / 0
+        //  LOCK INC QWORD PTR, [Rbase+n] ; opcode = 0xFF / 0
         uword_t disp = index * 8;
-        uword_t new_inst1 =
-          0xF0 | (0x49 << 8) | (0xFF << 16) | (0x83L << 24) | (disp << 32);
-        //  LOCK ADD [R11+n], Rxx ; opcode = 0x01
-        prefix = 0x49 | ((prefix & 1) << 2); // 'b' bit becomes 'r' bit
-        modrm  = 0x83 | (modrm & (7<<3)); // copy 'reg' into new modrm byte
+        uword_t new_inst1 = 0xF0 | ((0x48 | (base_reg>>3)) << 8) // w + b
+            | (0xFF << 16) | ((0x80L+(base_reg&7)) << 24) | (disp << 32);
+        //  LOCK ADD [Rbase+n+8], Rsize ; opcode = 0x01
+        int prefix = 0x48 | ((size_reg >> 3) << 2) | (base_reg >> 3); // w + r + b
+        int modrm  = 0x80 | ((size_reg & 7) << 3) | (base_reg & 7);
         disp = (1 + index) * 8;
         uword_t new_inst2 =
           0xF0 | (prefix << 8) | (0x01 << 16) | ((long)modrm << 24) | (disp << 32);
@@ -765,3 +790,5 @@ allocation_tracker_sized(uword_t* sp)
     int __attribute__((unused)) ret = thread_mutex_unlock(&alloc_profiler_lock);
     gc_assert(ret == 0);
 }
+
+#include "x86-arch-shared.inc"
