@@ -63,7 +63,7 @@ int arch_os_thread_init(struct thread *thread) {
 #endif
 #ifdef MEMORY_SANITIZER
     extern __thread unsigned long __msan_param_tls[];
-    thread->msan_param_tls = (uword_t)&__msan_param_tls[0];
+    ((lispobj*)thread)[THREAD_MSAN_PARAM_TLS_SLOT] = (uword_t)&__msan_param_tls[0];
 #endif
     return 1;
 }
@@ -107,7 +107,7 @@ void sb_dump_mcontext(char *reason, ucontext_t* context)
 #define REMAINING sizeof obuf - ptr
     ptr = 0;
     ptr = snprintf(obuf, REMAINING,
-                   "Thread %d CPU state (%s): PC=%lx Flags={%s}\n",
+                   THREAD_ID_LABEL" CPU state (%s): PC=%lx Flags={%s}\n",
                    THREAD_ID_VALUE, reason,
                    (uword_t)context->uc_mcontext.gregs[REG_RIP],
                    smallbuf+1);
