@@ -13,8 +13,7 @@ rm -fr sandbox/scratch
                   --eval '(in-package :cl-test)' \
                   --eval '(disable-note :nil-vectors-are-strings)' \
                   --eval '(time (do-tests))' \
-                  --eval '(let* ((expected (list* "APROPOS-LIST.ERROR.2" "APROPOS.ERROR.2" "BOTH-CASE-P.2" "CHAR-DOWNCASE.2"
- "CHAR-UPCASE.2" "COMPILE-FILE.2"
+                  --eval '(let* ((expected (list* "APROPOS-LIST.ERROR.2" "APROPOS.ERROR.2" "COMPILE-FILE.2"
  "DEFINE-COMPILER-MACRO.8" "DESTRUCTURING-BIND.ERROR.10"
  "EXP.ERROR.10" "EXP.ERROR.11" "EXP.ERROR.8"
  "EXP.ERROR.9" "EXPT.ERROR.10" "EXPT.ERROR.11" "EXPT.ERROR.8" "EXPT.ERROR.9"
@@ -31,7 +30,7 @@ rm -fr sandbox/scratch
  "FORMATTER.X.29" "LOOP.1.39" "LOOP.1.40" "LOOP.1.41" "LOOP.1.42" "LOOP.1.43"
  "MACROLET.36" "MAKE-CONDITION.3" "MAKE-CONDITION.4"
  "MAKE-PATHNAME-ERROR-ABSOLUTE-WILD-INFERIORS-BACK"
- "MAKE-PATHNAME-ERROR-RELATIVE-WILD-INFERIORS-BACK" "MAP.48"
+ "MAKE-PATHNAME-ERROR-RELATIVE-WILD-INFERIORS-BACK"
  "PPRINT-LOGICAL-BLOCK.ERROR.1" "PPRINT-LOGICAL-BLOCK.ERROR.1-UNSAFE"
  "PPRINT-LOGICAL-BLOCK.ERROR.3" "PPRINT-LOGICAL-BLOCK.ERROR.3-UNSAFE"
  "PRINT-LEVEL.8" "PRINT-LEVEL.9" "PRINT.BACKQUOTE.RANDOM.1"
@@ -47,7 +46,15 @@ rm -fr sandbox/scratch
                        "EXPT.ERROR.4" "EXPT.ERROR.5" "EXPT.ERROR.6" "EXPT.ERROR.7"
                        "PROBE-FILE.4" "OPEN.OUTPUT.23" "OPEN.IO.22" "OPEN.IO.23")
          #+arm64 (list "EXP.ERROR.4" "EXP.ERROR.5" "EXP.ERROR.6" "EXP.ERROR.7" "EXPT.ERROR.4"
-                       "EXPT.ERROR.5" "EXPT.ERROR.6" "EXPT.ERROR.7"))))
+                       "EXPT.ERROR.5" "EXPT.ERROR.6" "EXPT.ERROR.7")
+         #-sb-unicode (list "MISC.638")
+         (if (member :sb-fasteval sb-impl:+internal-features+)
+             (list "INTERSECTION.FOLD.1" "UNION.FOLD.1" "SET-DIFFERENCE.FOLD.1" 
+                   "SET-EXCLUSIVE-OR.FOLD.1" "ASH.ERROR.5" 
+                   "ALL-STRUCTURE-CLASSES-ARE-SUBTYPES-OF-STRUCTURE-OBJECT.2" "TRACE.8")
+             (list "MAP.48"))
+         
+         #+sb-unicode (list "BOTH-CASE-P.2" "CHAR-DOWNCASE.2" "CHAR-UPCASE.2"))))
                          (failing (remove "FORMAT.E.26"
                                           (mapcar (function string) regression-test:*failed-tests*)
                                           :test (function equal)))
