@@ -126,3 +126,21 @@
       `(lambda (x)
          (typep x #.(find-class 'symbol)))
     (('x) t)))
+
+(with-test (:name :slot-value-on-not-slot-object)
+  (checked-compile-and-assert ()
+                              `(lambda (x)
+                                 (slot-value x 'm))
+    ((nil) (condition 'sb-pcl::missing-slot)))
+  (checked-compile-and-assert ()
+                              `(lambda (x)
+                                 (slot-boundp x 's))
+    ((1) (condition 'sb-pcl::missing-slot)))
+  (checked-compile-and-assert ()
+                              `(lambda (x)
+                                 (setf (slot-value x 'j) 30))
+    ((1.0) (condition 'sb-pcl::missing-slot)))
+  (checked-compile-and-assert ()
+                              `(lambda (x)
+                                 (slot-makunbound x 'l))
+    ((#\a) (condition 'sb-pcl::missing-slot))))
