@@ -19,6 +19,15 @@
 (when (eq sb-ext:*evaluator-mode* :interpret)
   (sb-ext:exit :code 104))
 
+#+(or x86 x86-64)
+(with-test (:name :legal-bpt-lra-object)
+  (sb-int:binding* ((code (sb-kernel:fun-code-header #'read))
+                    (sap (sb-sys:sap+ (sb-kernel:code-instructions code) 13)) ; random
+                    ((bpt-sap bpt-code-obj) (sb-di::make-bpt-lra sap)))
+    (declare (ignore bpt-sap))
+    ;; This was causing heap corruption
+    (assert (zerop (sb-kernel:code-jump-table-words bpt-code-obj)))))
+
 
 ;;;; Check that we get debug arglists right.
 
@@ -441,7 +450,8 @@
 
 (with-test (:name (:print-frame-call :respect *debug-print-variable-alist*
                    *print-length* :bug-1261646))
-  (let* ((printed (print-backtrace-to-string/debug-print-variable-alist (make-array 200)))
+  (let* ((printed (print-backtrace-to-string/debug-print-variable-alist
+                   (make-array 200 :initial-element 0)))
          (call "(PRINT-BACKTRACE-TO-STRING/DEBUG-PRINT-VARIABLE-ALIST ")
          (position (+ (search call printed) (length call))))
     (assert (eql position (search "#(0 0 0 0 0 ...)" printed :start2 position)))))
@@ -654,7 +664,7 @@
     (assert (= count 1))))
 
 (with-test (:name :properly-tagged-p-internal
-            :fails-on (not (or :x86 :x86-64)))
+            :fails-on (not (or :x86 :x86-64 :arm64)))
   ;; Pick a code component that has a ton of restarts.
   (let* ((code (sb-kernel:fun-code-header #'sb-impl::update-package-with-variance))
          (n (sb-kernel:code-n-entries code)))
@@ -700,3 +710,9 @@
           (assert (= (file-length fasl1) (file-length fasl2)))
           (loop repeat (file-length fasl1)
                 do (assert (= (read-byte fasl1) (read-byte fasl2)))))))))
+
+;; lp#1901781
+(defun ll-unknown (x y) (declare (optimize (debug 0))) (+ x y))
+(compile 'll-unknown)
+(with-test (:name :unknown-lambda-list)
+  (assert (eq (sb-kernel:%fun-lambda-list #'ll-unknown) :unknown)))
