@@ -338,8 +338,13 @@
   (assert (string/= (documentation 'you-cant-use-this 'function)
                     (documentation 'function.defun.late 'function))))
 
+;;; FIXME: this was supposed to print a warning about using the deprecated QUIT
+;;; function, but it instead warns about is an incorrect call to UNIX-EXIT.
+;;; This was broken by rev b2c767cea2008282 which un-deprecated QUIT without
+;;; appropriately fixing the test, but instead blindly changing the call
+;;; to its "correct" replacement, thereby losing the meaning of this test.
 (with-test (:name :load-time-deprecation-warning)
-  (let ((source "load-test.tmp") fasl)
+  (let ((source (scratch-file-name "tmp")) fasl)
     (with-open-file (f source :direction :output
                        :if-does-not-exist :create :if-exists :supersede)
       (write-string "(defun a () (sb-unix:unix-exit))" f)
