@@ -1,7 +1,6 @@
 ;;; -*- coding: utf-8 -*-
 ;;; enc-cn.impure.lisp: test case for enc-cn.lisp and enc-cn-tbl.lisp
-#-sb-unicode
-(sb-ext:exit :code 104)
+#+(or (not sb-unicode) unicode-lite) (sb-ext:exit :code 104)
 
 (let ((str (coerce '(#\u0031 #\u0041 #\uff21 #\u3042 #\u80e1 #\u73a5 #\u59ee)
                    'string))
