@@ -277,4 +277,52 @@
 (with-test (:name (:source-path typep :invalid-type-specifier))
   (assert-condition-source-paths
    (typep 1 'undefined-type)
-   (2)))
+   ;; both the style-warning and the note count
+   (2) (2)))
+
+(with-test (:name :dead-code-note-after-transforms)
+  (assert
+   (typep (nth-value 4
+                     (checked-compile
+                      `(lambda (x)
+                         (when nil
+                           (funcall x)))))
+          '(cons sb-ext:code-deletion-note null))))
+
+(with-test (:name :ignore-deleted-subforms)
+  (assert-condition-source-paths
+   (lambda (x m)
+     (when nil
+       (funcall x
+                (if m
+                    (print 20)
+                    (print x)))))
+   (2 2)))
+
+(with-test (:name :ignore-deleted-subforms.2)
+  (assert-condition-source-paths
+   (lambda ()
+     (when nil
+       (let ((z (print 10)))
+         z)))
+   (2 2)))
+
+(with-test (:name :ignore-deleted-subforms.3)
+  (assert-condition-source-paths
+   (lambda (x)
+     (when x
+       (unless x
+         (let ((z (print 10)))
+           (if z
+               10
+               (funcall x))))))
+   (2 2 2)))
+
+(with-test (:name :ignore-deleted-subforms.4)
+  (assert-condition-source-paths
+   (lambda (x)
+     (when nil
+       (if (print 10)
+           10
+           x)))
+   (2 2)))
