@@ -35,7 +35,7 @@
     (assert (= (aref a 4) 18))
     ;; broken cells are the cons, string, bignum, hash-table, plus one NIL
     ;; cell that was never assigned into
-    (assert (= (count nil *weak-vect*) 5))))
+    *weak-vect*))
 
 ;;; Make sure MAP-REFERENCING-OBJECTS doesn't spuriously treat raw bits as
 ;;; potential pointers. Also make sure it sees the SYMBOL-INFO slot.
@@ -44,7 +44,7 @@
 (with-test (:name :map-referencing-objs)
   (sb-vm::map-referencing-objects (lambda (x) (assert (not (typep x 'afoo))))
                                   :dynamic '*posix-argv*)
-  (let ((v (sb-kernel:symbol-info 'satisfies)) referers)
+  (let ((v (sb-kernel:symbol-%info 'satisfies)) referers)
     (sb-vm::map-referencing-objects (lambda (referer) (push referer referers))
                                     #+gencgc :dynamic #-gencgc :static v)
     #+immobile-space
@@ -152,7 +152,8 @@
                 (sb-kernel:get-lisp-obj-address string-one)
                 (sb-kernel:get-lisp-obj-address string-two)))))
 #+gencgc
-(with-test (:name :pin-all-code-with-gc-enabled)
+(with-test (:name :pin-all-code-with-gc-enabled
+                  :skipped-on :interpreter)
   #+sb-thread (sb-thread:join-thread (sb-thread:make-thread #'make-some-objects))
   #-sb-thread (progn (make-some-objects) (sb-sys:scrub-control-stack))
   (sb-sys:with-code-pages-pinned (:dynamic) (gc))
@@ -268,8 +269,8 @@
 #+gencgc
 (defun ensure-code/data-separation ()
   (let* ((n-bits (+ sb-vm:next-free-page 10))
-         (code-bits (make-array n-bits :element-type 'bit))
-         (data-bits (make-array n-bits :element-type 'bit))
+         (code-bits (make-array n-bits :element-type 'bit :initial-element 0))
+         (data-bits (make-array n-bits :element-type 'bit :initial-element 0))
          (total-code-size 0))
     (sb-vm:map-allocated-objects
      (lambda (obj type size)
@@ -362,7 +363,8 @@
 
 (defvar *foo*)
 #+gencgc
-(with-test (:name (sb-ext:search-roots :simple-fun))
+(with-test (:name (sb-ext:search-roots :simple-fun)
+            :broken-on (and :darwin :arm64))
   ;; Tracing a path to a simple fun wasn't working at some point
   ;; because of failure to employ fun_code_header in the right place.
   (setq *foo* (compile nil '(lambda () 42)))
@@ -388,14 +390,13 @@
     (sb-thread:join-thread gc-thread)))
 
 (defun parse-address-range (line)
-  ;; I hope nothing preceding the match of "00-" could be a false positive.
+  ;; I hope nothing preceding the match of "-" could be a false positive.
   ;; If there is, I suspect we should parse the legend that contains
   ;;  "REGION TYPE                      START - END" to determine the column
   ;; with a #\- which appears consistently in the same place on each following line.
-  (let ((pos (search "00-" line)))
-    (assert pos)
-    (let* ((separator (+ pos 2)) ; position of the #\-
-           (start separator))
+  (let ((separator (position #\- line)))
+    (assert separator)
+    (let* ((start separator))
       (loop (if (digit-char-p (char line (1- start)) 16) (decf start) (return)))
       (values (parse-integer line :start start :end separator :radix 16)
               (multiple-value-bind (value end)
@@ -451,6 +452,7 @@
                      (prevsym "")
                      (nsyms 0))
                  (loop for addr from start to end by 8
+                       repeat 100
                        do (let ((sym (sb-sys:sap-foreign-symbol (sb-sys:int-sap addr))))
                             (when (and sym (string/= sym prevsym))
                               (incf nsyms)
@@ -466,17 +468,6 @@
     (setq working nil)
     (sb-thread:join-thread gc-thread)))
 
-(with-test (:name :no-conses-on-large-object-pages)
-  (let* ((fun (checked-compile '(lambda (&rest params) params)))
-         (list (make-list #+gencgc (/ sb-vm:large-object-size
-                                      (sb-ext:primitive-object-size '(1))
-                                      1/2)
-                          #-gencgc 16384))
-         (rest (apply fun list)))
-    (sb-sys:with-pinned-objects (rest)
-      (sb-ext:gc :full t)
-      (assert (and (equal list rest) t)))))
-
 (defun use-up-thread-region ()
   ;; cons until the thread-local allocation buffer uses up a page
   (loop
