@@ -1,14 +1,6 @@
 #!/bin/sh
 
-(cd ../../src/runtime ; make libsbcl.a)
-
-# Specify that some symbols are undefined so that the complete
-# contents of 'wrap.o' and 'largefile.o' get pulled in.
-# FIXME: maybe need '-m32' flag and whatnot
-cc -o test-sbcl -g \
-  -Wl,-ufstat_wrapper -Wl,-uget_timezone -Wl,-ulseek_largefile -Wl,-uspawn \
-  -Wl,--export-dynamic -no-pie fake-mman.c ../../src/runtime/libsbcl.a \
-  -ldl -lpthread -lm
+source ./build-test-sbcl
 
 export SBCL_FAKE_MMAP_INSTRUCTION_FILE=`pwd`/fakemap
 
@@ -18,6 +10,10 @@ export SBCL_FAKE_MMAP_INSTRUCTION_FILE=`pwd`/fakemap
   --eval '(gc :full t)' \
   --eval '(exit)'
 
+# TODO:
+# 1. this needs to be run as part of the regression suite
+# 2. and split into one test per shell file for parallelization
+
 # There is a slight problem in that one of the tests in core.test.sh
 # asserts something about dynamic space size, which due to relocation
 # might come out one GC page too small.
@@ -27,5 +23,5 @@ export SBCL_FAKE_MMAP_INSTRUCTION_FILE=`pwd`/fakemap
 # The base gets aligned up to 0x2000010000 and one page is subtracted
 # from dynamic space size so not to overrun the memory.
 # And this script does not actually exit with the correct status anyway.
-(cd .. ; TEST_SBCL_RUNTIME=`pwd`/heap-reloc/test-sbcl ./run-tests.sh core.test.sh)
+(cd .. ; TEST_SBCL_RUNTIME=`pwd`/heap-reloc/test-sbcl ./run-tests.sh save*.test.sh)
 rm test-sbcl
