@@ -24,9 +24,14 @@ TEST_DIRECTORY=$junkdir SBCL_HOME=../obj/sbcl-home exec ../src/runtime/sbcl \
   --noinform --core ../output/sbcl.core \
   --no-userinit --no-sysinit --noprint --disable-debugger $* << EOF
 (pop *posix-argv*)
+(setf (extern-alien "gc_allocate_dirty" char) 1)
 (require :sb-posix)
+(require :sb-sprof)
 (let ((*evaluator-mode* :compile))
   (with-compilation-unit () (load"run-tests")))
+#+(and x86-64 linux sb-thread)
+  (unless (find :gs-seg sb-impl:+internal-features+)
+    (push :test-aprof *features*))
 (in-package run-tests)
 (import '(sb-alien:alien-funcall sb-alien:extern-alien
           sb-alien:int sb-alien:c-string sb-alien:unsigned))
@@ -50,6 +55,43 @@ TEST_DIRECTORY=$junkdir SBCL_HOME=../obj/sbcl-home exec ../src/runtime/sbcl \
             (cond ((< posn-a posn-b) t)
                   ((> posn-a posn-b) nil)
                   (t (string< a b)))))))
+(defun summarize-gc-times ()
+  (let (observations)
+    (flet ((parse-triple (string pos)
+             (sb-int:binding* (((int1 end) (parse-integer string :start (1+ pos)
+                                                                 :junk-allowed t))
+                               ((int2 end) (parse-integer string :start (1+ end)
+                                                                 :junk-allowed t))
+                               ((int3) (parse-integer string :start (1+ end)
+                                                             :junk-allowed t)))
+               (list int1 int2 int3))))
+      (dolist (pn (directory "$logdir/*.*"))
+        (with-open-file (f pn)
+          (let ((legend "GC: time-to-stw"))
+            (loop
+             (let ((line (read-line f nil)))
+               (unless line (return))
+               (when (and (> (length line) (length legend))
+                          (string= line legend :end1 (length legend)))
+                 (let* ((p1 (position #\= line))
+                        (p2 (position #\= line :start (1+ p1)))
+                        (stw-dur (parse-triple line p1))
+                        (gc-dur (parse-triple line p2))
+                        (count (parse-integer line :start (+ (search "over " line) 5)
+                                              :junk-allowed t)))
+                   (let ((name (concatenate 'string (pathname-name pn) "."
+                                            (pathname-type pn))))
+                     (push (list count stw-dur gc-dur name) observations))))))))))
+    (let ((fmt "  ~5d (~{~10d~^ ~}) (~{~7d~^ ~}) ~a~%"))
+      (format t "~&Top 15 worst by max time to stop-the-world:~%")
+      (let ((list (sort (copy-list observations) #'> :key (lambda (x) (third (second x))))))
+        (dotimes (i 15) (apply #'format t fmt (pop list))))
+      (format t "~&Top 15 worst by avg GC duration (excluding STW delay):~%")
+      (let ((list (sort (copy-list observations) #'> :key (lambda (x) (second (third x))))))
+        (dotimes (i 15) (apply #'format t fmt (pop list))))
+      (format t "~&Top 15 worst by max GC duration (excluding STW delay):~%")
+      (let ((list (sort (copy-list observations) #'> :key (lambda (x) (third (third x))))))
+        (dotimes (i 15) (apply #'format t fmt (pop list)))))))
 (defun parallel-execute-tests (files max-jobs vop-summary-stats-p)
   (format t "Using ~D processes~%" max-jobs)
   ;; Interleave the order in which all tests are launched rather than
@@ -105,6 +147,9 @@ TEST_DIRECTORY=$junkdir SBCL_HOME=../obj/sbcl-home exec ../src/runtime/sbcl \
           (wait))
         (let ((pid (sb-posix:fork)))
           (when (zerop pid)
+            #+(and linux sb-thread 64-bit)
+            (sb-alien:alien-funcall (sb-alien:extern-alien "reset_gc_stats"
+                                    (function sb-alien:void)))
             ;; FILE is (filename . test-iteration)
             (with-open-file (stream (format nil "$logdir/~a~@[-~d~]" (car file) (cdr file))
                                     :direction :output :if-exists :supersede)
@@ -112,6 +157,10 @@ TEST_DIRECTORY=$junkdir SBCL_HOME=../obj/sbcl-home exec ../src/runtime/sbcl \
                              (sb-sys:fd-stream-fd stream) 1)
               (alien-funcall (extern-alien "dup2" (function int int int)) 1 2))
             (setq file (car file))
+            #+test-aprof
+            (unless (search "allocator.pure" file)
+              (sb-aprof::aprof-start)
+              (proclaim '(optimize sb-c:instrument-consing)))
             ;; Send this to the log file, not the terminal
             (setq *debug-io* (make-two-way-stream (make-concatenated-stream)
                                                   *error-output*))
@@ -119,13 +168,14 @@ TEST_DIRECTORY=$junkdir SBCL_HOME=../obj/sbcl-home exec ../src/runtime/sbcl \
                    (let ((shell (or #+sunos (posix-getenv "SHELL") "/bin/sh")))
                      ;; exec the shell with the test and we'll pick up its exit code
                      (alien-funcall (extern-alien "execl" (function int c-string c-string
-                                                                    c-string unsigned))
+                                                                    &optional c-string unsigned))
                                     shell shell
                                     (concatenate 'string file ".sh") 0))
                    ;; if exec fails, just exit with a wrong (not 104) status
                    (alien-funcall (extern-alien "_exit" (function (values) int)) 0))
                   (t
-                   (setq sb-c::*static-vop-usage-counts* (make-hash-table))
+                   (sb-sprof:start-profiling :sample-interval .001)
+                   (setq sb-c::*static-vop-usage-counts* (make-hash-table :synchronized t))
                    (let ((*features* (cons :parallel-test-runner *features*)))
                      (pure-runner (list (concatenate 'string file ".lisp"))
                                   (if (search "-cload" file) 'cload-test 'load-test)
@@ -138,6 +188,12 @@ TEST_DIRECTORY=$junkdir SBCL_HOME=../obj/sbcl-home exec ../src/runtime/sbcl \
                        (let ((*print-pretty* nil))
                          (sb-int:dohash ((name count) sb-c::*static-vop-usage-counts*)
                            (format output "~7d ~s~%" count name)))))
+                   (sb-sprof:stop-profiling)
+                   #+test-aprof (progn (sb-aprof::aprof-stop) (sb-aprof:aprof-show))
+                   (when (member :allocator-metrics sb-impl:+internal-features+)
+                     (format t "~2&Allocator histogram:~%")
+                     (funcall (intern "PRINT-ALLOCATOR-HISTOGRAM" "SB-THREAD")))
+                   (sb-sprof:report :type :flat)
                    (exit :code (if (unexpected-failures) 1 104)))))
           (format t "~A: pid ~d~@[ (trial ~d)~]~%" (car file) pid (cdr file))
           (incf subprocess-count)
@@ -180,7 +236,8 @@ TEST_DIRECTORY=$junkdir SBCL_HOME=../obj/sbcl-home exec ../src/runtime/sbcl \
                                 (impure-cload-files)
                                 (sh-files)))))
        jobs
-       t))
+       t)
+      #+(and linux sb-thread 64-bit) (summarize-gc-times))
     ;; long form
     (let ((jobs 4)
           (runs-per-test 1)
