@@ -17,7 +17,7 @@
 ;;;; are order-preserving - unless they are of the form (CONS (EQL x)).
 ;;;; This is not a requirement in general, but is quite reasonable.
 (with-test (:name :pprint-dispatch-order-preserving)
-  (let ((tbl (sb-pretty::make-pprint-dispatch-table nil nil nil)))
+  (let ((tbl (sb-pretty::make-pprint-dispatch-table #() nil nil)))
     (handler-bind ((warning #'muffle-warning)) ; nonexistent types
       ;; use EVAL because there are *two* warnings to muffle: first time
       ;; is when the compiler sees a symbol used as an unknown type-specifier,
@@ -29,7 +29,7 @@
       (set-pprint-dispatch (eval ''fool) #'pprint-fill 0 tbl)
       (set-pprint-dispatch (eval ''foo2) #'pprint-fill 5 tbl))
     (let ((entries (sb-pretty::pp-dispatch-entries tbl)))
-      (assert (equal (mapcar #'sb-pretty::pprint-dispatch-entry-type entries)
+      (assert (equal (map 'list #'sb-pretty::pprint-dispatch-entry-type entries)
                      '(foo1 foo2 fool))))))
 
 ;;;; tests for former BUG 99, where pretty-printing was pretty messed
@@ -427,7 +427,7 @@
 (with-test (:name :closure-macro-arglist)
   ;; assert correct test setup - MACDADDY is a closure if compiling,
   ;; or a funcallable-instance if not
-  (assert (eq (sb-kernel:fun-subtype (macro-function 'macdaddy))
+  (assert (eq (sb-kernel:%fun-pointer-widetag (macro-function 'macdaddy))
               #-interpreter sb-vm:closure-widetag
               #+interpreter sb-vm:funcallable-instance-widetag))
   ;; MACRO-INDENTATION used %simple-fun-arglist instead of %fun-arglist.
