@@ -19,13 +19,13 @@ chmod u+x "$tmpcore"
   (save-lisp-and-die "$tmpcore" :executable t :save-runtime-options t)
 EOF
 chmod u+x "$tmpcore"
-./"$tmpcore" --no-userinit --no-sysinit --noprint --version --eval '(exit)' <<EOF
-  (when (equal *posix-argv* '("./$tmpcore" "--version" "--eval" "(exit)"))
+./"$tmpcore" --no-userinit --no-sysinit --noprint --versions --eval '(exit)' <<EOF
+  (when (equal *posix-argv* '("./$tmpcore" "--versions" "--eval" "(exit)"))
     (exit :code 42))
 EOF
 status=$?
 rm "$tmpcore"
-if [ $status != 42 ]; then
+if [ $status -ne 42 ]; then
     echo "saving runtime options from executable failed"
     exit 1
 fi
