@@ -31,16 +31,16 @@
 (assert (streamp *str*))
 (defclass astream (fundamental-output-stream) (x y))
 (with-test (:name :update-stream-layout)
-  (assert (sb-kernel:layout-invalid (sb-kernel:%instance-layout *str*)))
+  (assert (sb-kernel:wrapper-invalid (sb-kernel:%instance-wrapper *str*)))
   (assert (streamp *str*))
-  (assert (/= 0 (sb-kernel:layout-clos-hash (sb-kernel:%instance-layout *str*))))
+  (assert (/= 0 (sb-kernel:wrapper-clos-hash (sb-kernel:%instance-wrapper *str*))))
   (defclass astream () (x y))
-  (assert (sb-kernel:layout-invalid (sb-kernel:%instance-layout *str*)))
-  (assert (= 0 (sb-kernel:layout-clos-hash (sb-kernel:%instance-layout *str*))))
+  (assert (sb-kernel:wrapper-invalid (sb-kernel:%instance-wrapper *str*)))
+  (assert (= 0 (sb-kernel:wrapper-clos-hash (sb-kernel:%instance-wrapper *str*))))
   (assert (not (streamp *str*)))
-  (assert (/= 0 (sb-kernel:layout-clos-hash (sb-kernel:%instance-layout *str*))))
+  (assert (/= 0 (sb-kernel:wrapper-clos-hash (sb-kernel:%instance-wrapper *str*))))
   (defclass astream (fundamental-output-stream) (x y))
-  (assert (sb-kernel:layout-invalid (sb-kernel:%instance-layout *str*)))
+  (assert (sb-kernel:wrapper-invalid (sb-kernel:%instance-wrapper *str*)))
   (assert (streamp *str*)))
 
 ;;; type errors for inappropriate stream arguments, fixed in
