@@ -32,6 +32,7 @@
            #:get-simple-fun-instruction-model
 
            #:scratch-file-name
+           #:*scratch-file-prefix*
            #:with-scratch-file
            #:opaque-identity
            #:runtime #:split-string #:integer-sequence #:shuffle))
@@ -125,13 +126,7 @@
   ;; needs to wait, to have a facsimile of the situation prior to implementation
   ;; of the so-called pauseless thread start feature.
   ;; Wouldn't you know, it's just reintroducing a startup semaphore.
-  ;; And interruption tests are even more likely to fail with sb-thruption
-  ;; because sb-thruption is flawed: it presumes that there is enough synchronization
-  ;; between sender/receiver that checking the INVOKED variable (shared via a closure)
-  ;; makes any sense at all, which it doesn't. (In addition, it supposes that merely
-  ;; by polling at safepoints, interrupts somehow become safe, which is not true
-  ;; in general - it is only true of the GC "interrupt" delivered by the kernel
-  ;; when the safepoint page trap is hit.)
+  ;; And interruption tests are even more likely to fail with :sb-safepoint.
   ;; Noneless, this tries to be robust enough to pass.
   (let* ((sem (sb-thread:make-semaphore))
          (child  (make-kill-thread
@@ -172,7 +167,12 @@
                             (if (expected-failure-p fails-on)
                                 (fail-test :expected-failure name error)
                                 (fail-test :unexpected-failure name error))
-                            (return-from run-test))))
+                            (return-from run-test)))
+                   (timeout (lambda (error)
+                              (if (expected-failure-p fails-on)
+                                  (fail-test :expected-failure name error t)
+                                  (fail-test :unexpected-failure name error t))
+                              (return-from run-test))))
       ;; Non-pretty is for cases like (with-test (:name (let ...)) ...
       (log-msg/non-pretty *trace-output* "Running ~S" name)
       (funcall test-function)
@@ -185,10 +185,6 @@
         (setf threads (union (union *threads-to-kill*
                                     *threads-to-join*)
                              threads))
-        #+(and sb-safepoint-strictly (not win32))
-        (dolist (thread (sb-thread:list-all-threads))
-          (when (typep thread 'sb-thread:signal-handling-thread)
-            (ignore-errors (sb-thread:join-thread thread))))
         (dolist (thread (sb-thread:list-all-threads))
           (unless (or (not (sb-thread:thread-alive-p thread))
                       (eql (the sb-thread:thread thread)
@@ -294,7 +290,7 @@
       (enable-debugger)
       (invoke-debugger condition))))
 
-(defun fail-test (type test-name condition)
+(defun fail-test (type test-name condition &optional backtrace)
   (if (stringp condition)
       (log-msg *trace-output* "~@<~A ~S ~:_~A~:>"
                type test-name condition)
@@ -303,7 +299,8 @@
   (push (list type *test-file* (or test-name *test-count*))
         *failures*)
   (unless (stringp condition)
-    ;; (sb-debug:print-backtrace :from :interrupted-frame)
+    (when backtrace
+      (sb-debug:print-backtrace :from :interrupted-frame))
     (when (or (and *break-on-failure*
                    (not (eq type :expected-failure)))
               *break-on-expected-failure*)
@@ -865,12 +862,13 @@
 ;;; Return a random file name to avoid writing into the source tree.
 ;;; We can't use any of the interfaces provided in libc because those are inadequate
 ;;; for purposes of COMPILE-FILE. This is not trying to be robust against attacks.
+(defvar *scratch-file-prefix* "sbcl-scratch")
 (defun scratch-file-name (&optional extension)
   (let ((a (make-array 10 :element-type 'character)))
     (dotimes (i 10)
       (setf (aref a i) (code-char (+ (char-code #\a) (random 26)))))
     (let ((dir (posix-getenv #+win32 "TMP" #+unix "TMPDIR"))
-          (file (format nil "sbcl~d~a~@[.~a~]"
+          (file (format nil "~a~d~a~@[.~a~]" *scratch-file-prefix*
                         (sb-unix:unix-getpid) a extension)))
       (if dir
           (namestring
@@ -926,3 +924,21 @@
             (when (>= (sb-disassem:dstate-cur-offs dstate) (sb-disassem:seg-length segment))
               (return)))
       (result))))
+
+;;; If a test file is not very tolerant of the statistical profiler, then
+;;; it should call this. There seem to be at least 2 common categories of failure:
+;;;  * the stack exhaustion test can get a profiling signal when you're already
+;;;    near exhaustion, and then the profiler exhausts the stack, which isn't
+;;;    handled very well. Example:
+;;;    Control stack exhausted, fault: 0xd76b9ff8, PC: 0x806cad2
+;;;       0: fp=0xd76ba008 pc=0x806cad2 Foreign function search_dynamic_space
+;;;       1: fp=0xd76ba028 pc=0x80633d1 Foreign function search_all_gc_spaces
+;;;       2: fp=0xd76ba048 pc=0x805647e Foreign function component_ptr_from_pc
+;;;       3: fp=0xd76baa18 pc=0x8065dfd Foreign function (null)
+;;;       4: fp=0xd76baa98 pc=0x806615a Foreign function record_backtrace_from_context
+;;;       5: fp=0xd76baab8 pc=0x806625e Foreign function sigprof_handler
+;;;  * debugger-related tests. I'm not sure why.
+(defun disable-profiling ()
+  (when (find-package "SB-SPROF")
+    (format t "INFO: disabling SB-SPROF~%")
+    (funcall (intern "STOP-PROFILING" "SB-SPROF"))))
