@@ -887,8 +887,8 @@
     (let ((result (sb-kernel:type-specifier
                    (sb-kernel:specifier-type
                     `(or list ,@(huge-union (lambda (x) `(array ,x (1 1 1)))))))))
-      (assert (or (equal result '(or cons null (array * (1 1 1))))
-                  (equal result '(or null cons (array * (1 1 1)))))))
+      (assert (or (equal result '(or list (array * (1 1 1))))
+                  (equal result '(or (array * (1 1 1)) list)))))
 
     ;; And unions of unions of distinct array types should reduce.
     (assert
@@ -928,7 +928,8 @@
   (let* ((hair (sb-kernel:specifier-type '(sb-kernel:unboxed-array 1)))
          (xform (sb-c::source-transform-union-typep 'myobj hair)))
     (assert (equal xform
-                   '(or (typep myobj '(and vector (not (array t))))))))
+                   '(or (typep myobj
+                               '(and vector (not (array t)) (not (array nil))))))))
 
   ;; Exclude one subtype at a time and make sure they all work.
   (dotimes (i (length sb-vm:*specialized-array-element-type-properties*))
@@ -1014,3 +1015,11 @@
 (compile 'd8-p)
 (with-test (:name :deep-structure-is-a)
   (assert (d8-p (opaque-identity (make-d8)))))
+
+(with-test (:name :intersection-complex-=)
+  (let ((unk (sb-kernel:specifier-type '(and unknown unknown2))))
+    (assert-tri-eq nil nil (sb-kernel:type= (sb-kernel:specifier-type t) unk))
+    (assert-tri-eq nil nil (sb-kernel:type= (sb-kernel:specifier-type 'integer) unk))
+    (assert-tri-eq nil nil (sb-kernel:type= (sb-kernel:specifier-type 'float) unk))
+    (assert-tri-eq nil nil (sb-kernel:type= (sb-kernel:specifier-type 'pathname) unk))
+    (assert-tri-eq nil nil (sb-kernel:type= (sb-kernel:specifier-type 'sequence) unk))))
