@@ -15,14 +15,24 @@
   (flet ((try (f) (assert-error (funcall f 'hash-table 3))))
     (mapc #'try '(typexpand-1 typexpand typexpand-all))))
 
+(with-test (:name :no-*-as-t) ; lp#1860919
+  (assert-signal (sb-kernel:specifier-type '(function (*) t)) warning)
+  (dolist (f '((lambda (x) (the * x))
+               (lambda (x) (declare (* x)) x)
+               (lambda (x) (declare (type * x)) x)))
+    (multiple-value-bind (f warn err)
+        (let ((*error-output* (make-broadcast-stream))) (compile nil f))
+      (declare (ignore f))
+      (assert (and warn err)))))
+
 (with-test (:name :stream-layout-bits)
-  (loop for layout being each hash-value
+  (loop for wrapper being each hash-value
         of (sb-kernel:classoid-subclasses (sb-kernel:find-classoid 't))
         do (flet ((check-bit (bit ancestor-type)
                     (let ((ancestor (sb-kernel:find-layout ancestor-type)))
-                      (when (or (eq layout ancestor)
-                                (find ancestor (sb-kernel:layout-inherits layout)))
-                        (assert (logtest bit (sb-kernel:layout-flags layout)))))))
+                      (when (or (eq wrapper ancestor)
+                                (find ancestor (sb-kernel:wrapper-inherits wrapper)))
+                        (assert (logtest bit (sb-kernel:wrapper-flags wrapper)))))))
               (check-bit sb-kernel:+stream-layout-flag+ 'stream)
               (check-bit sb-kernel:+string-stream-layout-flag+ 'string-stream)
               (check-bit sb-kernel:+file-stream-layout-flag+ 'file-stream))))
@@ -169,7 +179,7 @@
   (assert-tri-eq t   t (subtypep '(function ()) '(function (&rest t))))
   (assert-tri-eq nil t (subtypep '(function (&rest t)) '(function ())))
   (assert-tri-eq t   t (subtypep '(function)
-                                 '(function (&optional * &rest t))))
+                                 '(function (&optional t &rest t))))
   (assert-tri-eq nil t (subtypep '(function) '(function (t &rest t))))
   (assert-tri-eq t   t (subtypep 'function '(function)))
   (assert-tri-eq t   t (subtypep '(function) 'function)))
@@ -551,7 +561,9 @@
 
 ;; lp#1333731
 (with-test (:name (adjust-array :changes type-of))
-  (let ((a (make-array 10 :adjustable t)))
+  ;; I think adjusting an array to enlarge it must read all the old data,
+  ;; which would be undefined behavior if you hadn't initialized the array.
+  (let ((a (make-array 10 :adjustable t :initial-element 0)))
     (assert (equal (type-of a) '(vector t 10)))
     (adjust-array a 20)
     (assert (equal (type-of a) '(vector t 20)))))
@@ -636,7 +648,7 @@
                (values bit &optional *)
                (values bit &rest *)
                (values bit &rest *)))
-    (assert-error (sb-kernel:values-specifier-type x))))
+    (assert-signal (sb-kernel:values-specifier-type x) warning)))
 
 (with-test (:name :classoids-as-type-specifiers)
   (dolist (classoid (list (find-classoid 'integer)
@@ -674,6 +686,17 @@
   (assert (eq (make-numeric-type :class 'integer :low '(4) :high '(5))
               *empty-type*)))
 
+(with-test (:name :prettier-union-types :skipped-on (not :sb-unicode))
+  ;; (OR STRING BIGNUM) used to unparse as
+  ;; (OR (VECTOR CHARACTER) BASE-STRING (INTEGER * -4611686018427387905)
+  ;;     (INTEGER 4611686018427387904)) etc
+  (dolist (other '(float real bignum))
+    (let* ((spec `(or string ,other))
+           (parse (sb-kernel:specifier-type spec))
+           (unparse (sb-kernel:type-specifier parse)))
+      (assert (or (equal unparse `(or string ,other))
+                  (equal unparse `(or ,other string)))))))
+
 (with-test (:name :unparse-string)
   (assert (equal (type-specifier (specifier-type '(string 10)))
                  '(#+sb-unicode string #-sb-unicode base-string 10)))
@@ -718,6 +741,8 @@
                                           ,(sb-kernel:find-layout what))))))
 
 (with-test (:name :type-of-empty-instance)
+  (assert (eq (type-of (eval '(sb-kernel:%make-funcallable-instance 6)))
+              'sb-kernel:funcallable-instance))
   (assert (eq (type-of (eval '(sb-kernel:%make-instance 12)))
               'sb-kernel:instance)))
 
@@ -726,3 +751,55 @@
                   (sb-kernel:make-numeric-type :low '(-79106810381456307)))
                  '(or (double-float (-7.91068103814563d16)) (single-float (-7.910681e16))
                    (rational (-79106810381456307))))))
+
+(with-test (:name (:cons-union :lp1912863))
+  (let ((c (cons 2 4)))
+    (assert (not (typep c '(or (cons (integer 0 8) (integer 5 15))
+                               (cons (integer 3 15) (integer 4 14))))))))
+
+(with-test (:name (:rational-union :equivalent-to-t))
+  (let ((type '(or (integer * -1) (rational -1/2 1/2) (integer 1) (not integer))))
+    (assert-tri-eq t t (subtypep t type))))
+
+(with-test (:name (:rational-union :no-integers-in-rational))
+  (let ((type '(or (integer 1 1) (rational 1/2 1/2))))
+    (assert-tri-eq t t (subtypep type 'rational))
+    (assert-tri-eq nil t (subtypep 'rational type))
+    (assert-tri-eq nil t (subtypep type 'integer))
+    (assert-tri-eq nil t (subtypep 'integer type))
+    (assert (typep 1 type))
+    (assert (typep 1/2 type))
+    (assert (not (typep 3/4 type)))))
+
+(with-test (:name (:rational-union :open-bounds-closed))
+  (let ((t1 '(rational -1 1))
+        (t2 '(or (integer -1 1) (rational (-1) (1)))))
+    (assert-tri-eq t t (subtypep t1 t2))
+    (assert-tri-eq t t (subtypep t2 t1))))
+
+(with-test (:name (:rational-union :lp1912863 :bug039))
+  (flet ((bug039 ()
+           (let ((t1 'cons)
+                 (t2 '(or (not (cons t (real -1 1)))
+                       (not (cons sequence (eql 2))))))
+             (assert-tri-eq t t (subtypep t1 t2))
+             (assert-tri-eq t t (subtypep `(not ,t2) `(not ,t1))))))))
+
+(with-test (:name (:rational-union :lp1912863 :bug041))
+  (flet ((bug041 ()
+           (let ((t1 '(not (cons t integer)))
+                 (t2 '(not (cons (array nil) (eql 0))))
+                 (t3 '(cons simple-array t)))
+             (assert-tri-eq t t (subtypep t1 t2))
+             (assert-tri-eq t t (subtypep `(not (or ,t2 ,t3)) `(not ,t1)))
+             (assert-tri-eq t t (subtypep `(and (not ,t2) (not ,t3)) `(not ,t1))))))))
+
+(with-test (:name (:lp1916040 :answer))
+  (let* ((t1 '(cons sequence short-float))
+         (t2 '(or (cons t atom) (cons function t)))
+         (answer (multiple-value-list (subtypep t1 t2))))
+    (assert (member answer '((nil nil) (t t)) :test 'equal))))
+
+(with-test (:name (:lp1916233))
+  (assert-tri-eq t t (subtypep '(cons (or (simple-array ratio) simple-array) nil) nil))
+  (assert-tri-eq t t (subtypep '(or (array ratio) sequence) t)))
