@@ -998,7 +998,7 @@ sb-vm::(define-vop (cl-user::test)
 (with-test (:name :sap-set-does-not-cons)
   (loop for (type accessor telltale) in
         '((sb-vm:word sb-sys:sap-ref-word "ALLOC-UNSIGNED-BIGNUM")
-          (double-float sb-sys:sap-ref-double "CONS"))
+          (double-float sb-sys:sap-ref-double "ALLOC-TRAMP"))
         do (let* ((positive-test
                    (compile nil `(lambda (sap) (,accessor sap 0))))
                   (negative-test
@@ -1099,3 +1099,45 @@ sb-vm::(define-vop (cl-user::test)
     ;; 2 of them are are CONS->RNN and CONS->R11
     ;; the other is ENABLE-ALLOC-COUNTER which may or may not be present
     (assert (<= (length fixups) 3))))
+
+(defstruct submarine x y z)
+(defstruct (moreslots (:include submarine)) a b c)
+(declaim (ftype (function () double-float) get-dbl))
+(with-test (:name :write-combining-instance-set)
+  (let* ((f (compile nil '(lambda (s)
+                            (setf (submarine-x (truly-the submarine s)) 0
+                                  (submarine-y s) 0
+                                  (submarine-z s) 0))))
+         (lines (disassembly-lines f)))
+    (assert (= 1 (loop for line in lines count (search "MOVUPD" line))))
+    (assert (= 1 (loop for line in lines count (search "MOVSD" line)))))
+  (let* ((f (compile nil '(lambda (s)
+                            (setf (moreslots-a (truly-the moreslots s)) 0
+                                  (submarine-x s) 0
+                                  (moreslots-c s) 0))))
+         (lines (disassembly-lines f)))
+    (assert (= 3 (loop for line in lines count (search "MOVSD" line)))))
+  ;; This was crashing in the MOV emitter (luckily) because it received
+  ;; an XMM register due to omission of a MOVE-FROM-DOUBLE to heap-allocate.
+  (compile nil
+           '(lambda (sub a)
+             (declare (submarine sub))
+             (let ((fooval (+ (get-dbl) 23d0)))
+               (setf (submarine-x sub) a
+                     (submarine-y sub) fooval)
+               a))))
+
+#+immobile-code
+(with-test (:name :no-static-linkage-if-notinline)
+  ;; The normal state of the image has no "static" calls to FIND-PACKAGE
+  ;; but also has no globally proclaimed NOTINLINE, because that would
+  ;; suppress the optimization for CACHED-FIND-PACKAGE on a constant string.
+  (assert (not (sb-vm::fdefn-has-static-callers (sb-kernel::find-fdefn 'find-package))))
+  (assert (not (sb-int:info :function :inlinep 'find-package))))
+
+(sb-vm::define-vop (trythis)
+  (:generator 1
+   (sb-vm::inst and sb-vm::rax-tn (sb-c:make-fixup nil :gc-barrier))))
+(defun zook ()
+  (sb-sys:%primitive trythis)
+  nil)
