@@ -1,5 +1,7 @@
 (in-package "SB-COLD")
 
+(declaim (optimize debug))
+
 ;;; Common functions
 
 (defvar *output-directory*
@@ -13,6 +15,7 @@
 (defmacro with-input-txt-file ((s name) &body body)
   `(with-open-file (,s (make-pathname :name ,name :type "txt"
                                       :defaults *unicode-character-database*))
+     (setf (gethash (format nil "tools-for-build/~A.txt" ,name) *ucd-inputs*) 'used)
      ,@body))
 
 (defmacro with-output-dat-file ((s name) &body body)
@@ -20,6 +23,7 @@
                                       :defaults *output-directory*)
                        :direction :output :element-type '(unsigned-byte 8)
                        :if-exists :supersede :if-does-not-exist :create)
+     (setf (gethash (format nil "output/~A.dat" ,name) *ucd-outputs*) 'made)
      ,@body))
 
 (defmacro with-ucd-output-syntax (&body body)
@@ -34,8 +38,9 @@
                                       :defaults *output-directory*)
                        :direction :output :element-type 'character
                        :if-exists :supersede :if-does-not-exist :create)
+     (setf (gethash (format nil "output/~A.lisp-expr" ,name) *ucd-outputs*) 'made)
      (with-ucd-output-syntax
-       ,@body)))
+         ,@body)))
 
 (defun split-string (line character)
   (loop for prev-position = 0 then (1+ position)
@@ -65,6 +70,7 @@
 (defvar *slurped-random-constants*
   (with-open-file (f (make-pathname :name "more-ucd-consts" :type "lisp-expr"
                                     :defaults *unicode-character-database*))
+    (setf (gethash "tools-for-build/more-ucd-consts.lisp-expr" *ucd-inputs*) 'used)
     (read f)))
 
 (defun init-indices (symbol &aux (strings
@@ -185,7 +191,7 @@ Length should be adjusted when the standard changes.")
 "Table of scripts. Used in the creation of misc entries.")
 
 (defparameter *line-break-class-table*
-  (with-input-txt-file (s "LineBreakProperty")
+  (with-input-txt-file (s "LineBreak")
     (loop with hash = (make-hash-table)
        for line = (read-line s nil nil) while line
        unless (or (not (position #\# line)) (= 0 (position #\# line)))
@@ -554,7 +560,50 @@ Length should be adjusted when the standard changes.")
              (setf (ucd-misc (gethash code-point *ucd-entries*)) new-misc))))))
 
 (defun fixup-casefolding ()
-  (with-input-txt-file (s "CaseFolding")
+  ;; KLUDGE: CaseFolding.txt as distributed by Unicode contains a
+  ;; non-ASCII character, an eszet, within a comment to act as an
+  ;; example.  We can't in general assume that our host lisp will let
+  ;; us read that, and we can't portably write that we don't care
+  ;; about the text content of anything on a line after a hash because
+  ;; text decoding happens at a lower level.  So here we rewrite the
+  ;; CaseFolding.txt file to exclude the UTF-8 sequence corresponding
+  ;; to the eszet character.
+  (with-open-file (in (make-pathname :name "CaseFolding" :type "txt"
+                                     :defaults *unicode-character-database*)
+                      :element-type '(unsigned-byte 8))
+    (setf (gethash "tools-for-build/CaseFolding.txt" *ucd-inputs*) 'used)
+    (with-open-file (out (make-pathname :name "CaseFolding" :type "txt"
+                                        :defaults *output-directory*)
+                         :direction :output
+                         :if-exists :supersede
+                         :if-does-not-exist :create
+                         :element-type '(unsigned-byte 8))
+      (setf (gethash "output/CaseFolding.txt" *ucd-outputs*) 'made)
+      ;; KLUDGE: it's inefficient, though simple, to do the I/O
+      ;; byte-by-bite.
+      (do ((inbyte (read-byte in nil nil) (read-byte in nil nil))
+           (eszet (map '(vector (unsigned-byte 8)) 'char-code "<eszet>"))
+           (eszet-count 0)
+           (filename "CaseFolding.txt"))
+          ((null inbyte)
+           (unless (= eszet-count 1)
+             (error "Unexpected number of eszets in ~A: ~D"
+                    filename eszet-count)))
+        (cond
+          ((= inbyte #xc3)
+           (let ((second (read-byte in nil nil)))
+              (cond
+                ((null second)
+                 (error "No continuation after #xc3 in ~A" filename))
+                ((= second #x9f) (incf eszet-count) (write-sequence eszet out))
+                (t (error "Unexpected continuation after #xc3 in ~A: #x~X"
+                          filename second)))))
+          ((>= inbyte #x7f)
+           (error "Unexpected octet in ~A: #x~X" filename inbyte))
+          (t (write-byte inbyte out))))))
+  (with-open-file (s (make-pathname :name "CaseFolding" :type "txt"
+                                    :defaults *output-directory*))
+    (setf (gethash "tools-for-build/CaseFolding.txt" *ucd-inputs*) 'used)
     (loop for line = (read-line s nil nil)
        while line
        unless (or (not (position #\; line)) (equal (position #\# line) 0))
@@ -707,7 +756,7 @@ Length should be adjusted when the standard changes.")
     (values code-points ret))))
 
 (defparameter *collation-table*
-  (with-input-txt-file (stream "Allkeys70")
+  (with-input-txt-file (stream "allkeys")
     (loop with hash = (make-hash-table :test #'equal)
        for line = (read-line stream nil nil) while line
        unless (eql 0 (position #\# line))
@@ -910,7 +959,12 @@ Used to look up block data.")
   (with-output-lisp-expr-file (*standard-output* "other-collation-info")
     (write-string ";;; The highest primary variable collation index")
     (terpri)
-    (prin1 *maximum-variable-key*) (terpri)))
+    (prin1 *maximum-variable-key*) (terpri))
+  (with-output-lisp-expr-file (*standard-output* "n-collation-entries")
+    (write-string ";;; The number of entries in the collation table")
+    (terpri)
+    (prin1 (hash-table-count *collation-table*))
+    (terpri)))
 
 (defun output (&optional (*output-directory* *output-directory*))
   (output-misc-data)
